# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

class AnimHelper(object):
# ============================================================
    def __init__(self):
        self.ser_name = 'AnimHelper'
        self.drawer = None
        self.signaler = None
        self.signal_name = ''
        self.groups = {}        # списки действий для групп по именам
        self.current_group_name = ''
        self.current_group = []
        self.current_index = 0  # индекс анимации в группе
        self.current_anim_times = 0 # сколько раз осталось проиграть текущую анимацию
        self.current_animation = '' # название текущей анимации
        self.is_hidden = True
# ============================================================
    #def on_update(self, ms):
    #    pass
# ============================================================
# [ ['anim1', 3], [anim2, ('random', 3, 5)] ]
    def initialize(self, drawer, signaler, signal_name = 'ah_signal'):
        '''Задает связку с LayeredDrawer объекта'''
        self.drawer = drawer
        self.signaler = signaler
        self.signal_name = signal_name
        self.set_animations('stop')     # дефолтный пустой для остановки анимаций
# ============================================================
    def set_animations(self, group_name, actions = []):
        '''Передает список действий-'сигналов' для LayeredDrawer
        group_name: название группы действий, например 'generating', 'stop'...
        actions: анимации, которые необходимо показывать'''
        self.groups[group_name] = actions[:]
# ============================================================
    def stop(self):
        self.start_group('stop')
# ============================================================
    def start_group(self, group_name):
        self.signaler.Cancel(self.signal_name)  # на всякий случай приостановим
        self.current_group_name = group_name
        if group_name in self.groups:
            self.current_group = self.groups[group_name]
            self.restart_current_group()
        else:   # 'stop'
            trace('ERROR: no group "' + group_name + '" in AnimHelper')
            self.current_group = []
            self.current_index = 0
# ============================================================
    def restart_current_group(self):
        self.current_index = 0
        self._start_anim()
# ============================================================
    def _start_anim(self):
        if self.current_group:
            _anim = self.current_group[self.current_index]
            self.current_animation = _anim[0]
            self.current_anim_times = self.get_count(_anim[1])
            if self.current_anim_times > 0: # нельзя передавать пустые анимации
                self.start_next_animation()
# ============================================================
    def start_next_animation(self):
        '''Запускает следующую анимацию'''
        if self.current_group:  # если есть анимации
            self.is_hidden = False
            if self.current_anim_times > 0:
                self.signaler.Cancel(self.signal_name)  # на всякий случай приостановим
                self.drawer.ShowImages(self.current_animation, True)
                self.drawer.StopAnimation(self.current_animation, 0)
                self.drawer.PlayAnimationOnce(self.current_animation, 'signal_msg', self.signal_name)
                self.current_anim_times -= 1
            else:
                self.drawer.ShowImages(self.current_animation, False)   # прячем предыдущую анимацию
                self.current_index += 1 # переходим к следующей анимации в группе
                anims_count = len(self.current_group)
                if self.current_index >= anims_count:   # если это была последняя анимация
                    self.restart_current_group()        # перезапустим текущую группу (т.е. бесконечно играем)
                else:
                    self._start_anim()
# ============================================================
    def get_count(self, _count):
        '''Возвращает кол-во повторов для анимации'''
        if istype(_count, 'list'):
            cmd = _count[0]
            if cmd == 'random':
                _from = _count[1]
                _till = _count[2]
                return tools.Rand(_from, _till)
        elif istype(_count, 'number'):
            return _count
# ============================================================
    def hide(self):
        if self.current_animation:
            self.drawer.ShowImages(self.current_animation, False)
        if self.current_group:
            for _anim in self.current_group:
                _animation = _anim[0]
                self.drawer.ShowImages(_animation, False)
        self.is_hidden = True
# ============================================================
    def hide_group(self, group_name):
        if group_name in self.groups:
            _current_group = self.groups[group_name]
            if _current_group:
                for _anim in self.current_group:
                    _animation = _anim[0]
                    self.drawer.ShowImages(_animation, False)
# ============================================================
    def show_last_anim(self):
        '''Чтобы не исчезла анимация в switch_animation'''
        if self.current_animation:
            self.drawer.ShowImages(self.current_animation, not self.is_hidden)
# ============================================================
# ============================================================
