# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import fsm_state
import toolz

class BoatyardBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        self.show_ship = False
        self.show_gold = False  # нужно показать золото при выходе за экран
        self.gold_shown = False # показано ли золото
        self.show_unit = False  # показывать ли юнита на верфи
        self.is_blocked = False
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'BoatyardBuilding'
        self.SetIsHome(False)
        self.inflate_started = False
        self.hack_for_upgrade = True
        self.faked_upgrade_complete = False
        EMgr.add_event_listener(self)
# ============================================================
    # @override
    def init_fsm(self):
        base_building.BaseBuilding.init_fsm(self)
        self.fsm_methods.states[StateRepairing.name] = StateRepairing(self)
        self.fsm_methods.states[StateUpgrading.name] = StateUpgrading(self)
        self.fsm_methods.states[StateBuilding.name] = StateBuilding(self)
# ============================================================
    # @override
    def dispose(self):
        EMgr.remove_event_listener(self)
# ============================================================
    # @override
    def switch_animation(self):
        base_building.BaseBuilding.switch_animation(self)
        drawer = self.Drawer
        drawer.ShowImages('unit', self.show_unit)
        if self.show_ship:
            drawer.ShowImages('ship', True)
        drawer.ShowImages('gold', self.gold_shown)
        if self.is_blocked:
            drawer.ShowImages('button', False)
# ============================================================
    # @override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        '''Обработчик всех обычных сообщений (которые не были обработаны в on_msg)'''
        if msg_type == 'hammer_event':
            self.PlaySound('SND_HAMMER_HIT_1')
# ============================================================
    # @override
    def on_create(self):
        base_building.BaseBuilding.on_create(self)
        _uid = self.GetAttrStr('unit_linked_uid')
        self.linked_unit = BMgr.get_item(_uid, self.get('locind'))
# ============================================================
    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        if name == 'questman_complete':            # юнит получил золото
            self.show_gold = True
            if self.is_offscreen:
                self.gold_shown = True
            self.switch_animation()
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'offscreen':
            if self.is_offscreen: return    # чтобы не дублировать
            self.is_offscreen = True
            #if self.faked_upgrade_complete and not self.has_next_level:  # достигнут максимальный вариант...
            #    if self.show_unit and self.linked_unit:
            #        self.linked_unit.ready_to_show(self)
            #        self.linked_unit.Signals('try_to_show', 1)
            if self.show_gold:
                self.gold_shown = True
                self.switch_animation()
        elif signal == 'onscreen':
            self.is_offscreen = False
        elif signal == 'faked_upgrade_complete':
            #self.switch_level_after_fake()
            #self.SM.SetState('normal')
            #self.show_unit = False
            self.reset_cache()
            self.switch_level('boat_build')
            self.faked_upgrade_complete = True
            TMgr.remove_updateable_item(self)
            # ## затычка, чтобы только после постройки лодки считать квест выполненным
            # оставим на всякий случай:
            building_info = {'type':self.get('type'), 'subtype':self.get('stype')}
            EMgr.send_event( 'build_repair_boat_complete', self, building_info )
            #if self.is_offscreen:        # если уже за экраном
            if not self.has_next_level:  # достигнут максимальный вариант...
                if self.linked_unit:
                    self.linked_unit.ready_to_show(self, True)
        else:
            base_building.BaseBuilding.on_signal(self, signal)
# ============================================================
#    #@override
#    def on_state_change(self, cur_state, next_state):
#        base_building.BaseBuilding.on_state_change(self, cur_state, next_state)
#        if next_state == 'normal':
#            if cur_state in ['build', 'upgrade']:
#                self.is_blocked = True  # отныне и впредь
#        if cur_state != next_state:
#            if next_state == 'upgrade':       # затычка для особого поведения верфи, там анимация должна проигрываться
#                self.Drawer.RestartEffect('build')
# ============================================================
    #@override
    def on_building_create(self):
        base_building.BaseBuilding.on_building_create(self)
        self.is_deactivated = self.GetAttrBool('deactivated')
# ============================================================
    #@override
    def switch_level_to_ship(self):
        '''Здесь происходит "апгрейд" здания, смена варианта. Здесь должны (!) проверяться все необходимые требования, кроме кол-ва ресурсов'''
        var_name = 'level_2'
        if var_name in self.variants_switch:
            next_var_name = self.variants_switch[var_name]
            trace('switch_level_to_ship:')
            trace(' switch_level_to_ship:', self.get('name'), self.get('uid'))
            self.SwitchLevel(next_var_name)   # переключаемся на следующий вариант
            self.update_next_res()
            self.show_ship = True
            self.show_unit = False
            self.switch_animation()
            self.unreserve_action_cells()
            #if self.linked_unit:
                #self.linked_unit.show_as_possible()
        self.update_res_enough_state()
# ============================================================
    #@override
    def switch_level_to_goldship(self):
        '''Здесь происходит "апгрейд" здания, смена варианта. Здесь должны (!) проверяться все необходимые требования, кроме кол-ва ресурсов'''
        var_name = 'level_3'
        if var_name in self.variants_switch:
            next_var_name = self.variants_switch[var_name]
            trace('switch_level_to_goldship:')
            trace(' switch_level_to_goldship:', self.get('name'), self.get('uid'))
            self.SwitchLevel(next_var_name)   # переключаемся на следующий вариант
            self.update_next_res()
            self.show_ship = True
            self.switch_animation()
            self.unreserve_action_cells()
        self.update_res_enough_state()
# ============================================================
    def hide_unit(self):
        self.switch_level_to_ship()
# ============================================================
    #@override
    def scale_bump(self):
        if not self.show_unit:
            base_building.BaseBuilding.scale_bump(self)
# ============================================================
    #@override
    def _switch_animation_extra(self, place):
        if place == 'upgrade':
            self.Drawer.ShowImages('build')     # для верфи
            self.Drawer.SetProgressedAnimation('build')
            self.Drawer.ShowImages('repair')     # для верфи
            #TMgr.add_task( task_callworker.CallWorkerTask(self) )
            #self.Drawer.SetFramesProgress('build', self.build_progress/self.build_time)
# ============================================================
    def on_update(self, ms):
        quant = ms * self.get_boost_multiplier()
        self.build_progress += quant
        if self.build_progress <= self.build_time:
            self.Drawer.SetFramesProgress('build', self.build_progress/self.build_time) # сменяет кадры постройки, если изображение указано как анимированное
            _progress = self.build_progress/self.build_time
            if not self.inflate_started:
                if _progress > 0.3:
                    self.PlaySound('SND_BALLOON_INFLATE')
                    self.inflate_started = True
# ============================================================
    #@override
    def on_worker_in(self, unit):
        '''We need to start faked upgrade'''
        base_building.BaseBuilding.on_worker_in(self, unit)
        # #self.Drawer.SetProgressedAnimation('build')
        self.Drawer.PlayAnimationOnce('build', self.build_time)
        self.Signals.Restart('faked_upgrade_complete', self.build_time)
        TMgr.add_updateable_item(self)
        #self.SM.SetState('upgrade')
        if self.get('name') != 'ballsyard1':
            self.PlaySound('SND_BOATYARD_WORK')
        self.Drawer.SetAnimationEvent('unit', 50, 'hammer_event')
        self.show_unit = True
# ============================================================
    #@override
    def is_active(self, skip_priority_check = False):
        return base_building.BaseBuilding.is_active(self, skip_priority_check) and not self.is_blocked
# ============================================================
    #@override
    def _get_tips_extra(self, place, result, by_click):
        if place == 'before_name':
            if self.is_mouse_over_zone == 'ship':
                result.append( ('text', 'COLOR_HEADER') )
                ship_text = self.GetAttrStr('ship_text')
                result.append( ('text', ship_text) )
                return False
        return True
# ============================================================
    #@override
    def Get_tip_position(self, tip_size, camera_pos, window_rect, zoom):
        '''Возвращает координаты типса от левого-верхнего угла картинки типса'''
        if self.is_mouse_over_zone == 'ship':
            return toolz.Tools.get_tip_position(self, tip_size, camera_pos, window_rect, 'notip_ship_rect')
        else:
            return toolz.Tools.get_tip_position(self, tip_size, camera_pos, window_rect)
# ============================================================
# ============================================================
# ============================================================
class StateRepairing(fsm_state.StateRepairing):
    def __init__(self, item = None):
        fsm_state.StateRepairing.__init__(self, item)
        self.ser_name = 'BoatyardStateRepairing'

    #@override
    def on_enter(self, evID):
        fsm_state.StateRepairing.on_enter(self, evID)
        self.item.Drawer.RestartEffect('build')
    #@override
    def on_exit(self, next_state):
        fsm_state.StateRepairing.on_exit(self, next_state)
        self.item.is_blocked = True  # отныне и впредь
# ============================================================
class StateUpgrading(fsm_state.StateUpgrading):
    def __init__(self, item = None):
        fsm_state.StateUpgrading.__init__(self, item)
        self.ser_name = 'BoatyardStateUpgrading'

    #@override
    def on_enter(self, evID):
        fsm_state.StateUpgrading.on_enter(self, evID)
        self.item.Drawer.RestartEffect('build')
    #@override
    def on_exit(self, next_state):
        fsm_state.StateUpgrading.on_exit(self, next_state)
        self.item.is_blocked = True  # отныне и впредь
# ============================================================
class StateBuilding(fsm_state.StateBuilding):
    def __init__(self, item = None):
        fsm_state.StateBuilding.__init__(self, item)
        self.ser_name = 'BoatyardStateBuilding'

    #@override
    def on_exit(self, next_state):
        fsm_state.StateBuilding.on_exit(self, next_state)
        self.item.is_blocked = True  # отныне и впредь
# ============================================================