# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import task_doctor

class DoctorBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'DoctorBuilding'
        self.SetIsHome(False)
        self.is_internal_free_units = True
        self.unit_boost = self.GetAttrInt('unit_boost')
        self.doctor = None
        self.is_unit_inside = False
        self.is_unit_inside_visible = False
# ============================================================
    def need_base_resource(self):
        return False
# ============================================================
    def on_switch_level_complete(self):
        base_building.BaseBuilding.on_switch_level_complete(self)
        self.unit_boost = self.GetAttrInt('unit_boost')
# ============================================================
    #@override
    def need_to_show_unit(self):
        #if self.is_unit_inside:
        #    return self.is_under_attack == 0
        return False
# ============================================================
    #@override
    def need_no_action(self):
        return False
# ============================================================
    #@override
    def need_pers_no_action(self):
        #if self.is_unit_inside:
        #    return self.is_under_attack == 0
        return False
# ============================================================
    def init_anim(self):
        self.Drawer.SetAnimationEvent('unit', 10, 'worker_event')
        settings = []
        settings.append( ['unit', ('random', 2, 4)] )
        settings.append( ['pers_no_action', 5] )
        self.anim_helper.set_animations('default', settings)
# ============================================================
    #@override
    def on_worker_in(self, unit):
        '''Чтобы здание могло отловить когда пришел юнит-работник в здание'''
        base_building.BaseBuilding.on_worker_in(self, unit)
        _len = len(self._internal_free_units)
        if _len > 0:
            self.doctor = self._internal_free_units[0]
            self.mark_unit_inside(True)
        self.switch_animation()
# ============================================================
    def on_upgrade_complete(self, building_info):
        base_building.BaseBuilding.on_upgrade_complete(self, building_info)
        self.Drawer.SetAnimationEvent('unit', 10, 'worker_event')
        EMgr.send_event('doctor_building_event', self)
# ============================================================
    #@override
    def on_awaiting_action(self):
        self.anim_helper.hide()
        self.anim_helper.stop()
# ============================================================
    #@override
    def return_worker_home(self):
        #base_building.BaseBuilding.action_hit_me(self)
        _return_unit = None
        if self.is_unit_inside:
            _return_unit = base_building.BaseBuilding.return_worker_home(self)
            if self._internal_free_units:
                __unit = self._internal_free_units.pop(0)
                if self.doctor:
                    self.doctor = None
            #has_workers = len(self.worker_units)
            self.mark_unit_inside(False)
        self.switch_animation()
        return _return_unit
# ============================================================
    #@override
    def has_workers(self):
        return len(self._internal_free_units) > 0
# ============================================================
    # @override
    def _start_resource_generation(self):
        #self.anim_helper.start_group('default')
        pass
# ============================================================
    #@override
    def on_functional(self):
        LBoard.Core.append_building(self)
        self.update_buildings_count()
        if self.is_unit_inside:
            self.mark_unit_inside(True)
# ============================================================
    #@override
    def on_functional_internal(self):
        base_building.BaseBuilding.on_functional_internal(self)
        _g = UMgr.get_general()
        _g.try_call_doctor()
        EMgr.send_event('doctor_building_event', self)
        #if self.is_unit_inside:
        #    self.mark_unit_inside(True)
# ============================================================
    #@override
    def on_non_functional(self):
        if not self.doctor:
            LBoard.Core.remove_building(self)   # for cases of barabrians/raptors attacks
            self.update_buildings_count()
        EMgr.send_event('doctor_building_event', self)
        if self.is_unit_inside:
            self.mark_unit_inside(False)
# ============================================================
    def update_buildings_count(self):
        h_buildings = BMgr.get_buildings_raw(-1, 'hospital')
        for b in h_buildings:
            _is_functional = b.is_functional()
            if _is_functional:
                UStats.SetLevelValue('HAS_DOCTORS', 1)
                return
        UStats.SetLevelValue('HAS_DOCTORS', 0)
# ============================================================
    def on_repair_complete(self):
        base_building.BaseBuilding.on_repair_complete(self)
        #if self.is_unit_inside:
        #    self.mark_unit_inside(True)
        LBoard.Core.try_open_help(1,2, 11, self, '5208')
# ============================================================
    # @override
#    def on_repair_complete(self, building_info):
#        base_building.BaseBuilding.on_repair_complete(self, building_info)
        #if LBoard.Core.episode == 1 and LBoard.Core.level == 5:
        #    _need_help = LBoard.Core.is_help_needed(10)
        #    if _need_help:
        #        _uid = self.get('uid')
        #        if _uid == '600':
        #            Camera.MoveTo(self)
        #            UI.open_help(10)
# ============================================================
    #@override
    def get_worker(self):
        _has_workers = self.has_workers()
        if _has_workers and self.is_unit_inside:
            return self.worker_units[0]
            #return self._internal_free_units[0]
        else:
            return None
# ============================================================
    #@override
    def pop_doctor(self):
        _has_workers = self.has_workers()
        if _has_workers and self.is_unit_inside:
            _internal_count = len(self._internal_free_units)
            if _internal_count:
                _doctor = self._internal_free_units.pop(0)
                _doctor.work_boost = self.unit_boost / 100
                self.mark_unit_inside(False)
                return _doctor
        else:
            return None
# ============================================================
    # @override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'worker_event':
            self.PlaySound('SND_HOSPITAL_WORK')
        pass
# ============================================================
    def mark_unit_inside(self, is_inside):
        if not self.is_unit_inside:
            if is_inside:
                self.is_unit_inside = True
        if self.is_unit_inside_visible != is_inside:
            self.is_unit_inside_visible = is_inside
            self.switch_animation()
            if self.is_unit_inside_visible:
                self.init_anim()
                self.anim_helper.start_group('default')
            #    mbox('inside')
            else:
                self.anim_helper.hide()
            #    mbox('outside')
# ============================================================
    #@override
    def _on_predestroy(self):
        if not self.is_unit_inside: # если юнит еще где-то бегает
            _free_count = len(self._internal_free_units)
            if _free_count > 0:
                _unit = self._internal_free_units.pop(0)
                TMgr.cancel_doctor_tasks(_unit)
                if _unit.task:
                    _unit.task.cancel() # отменяем таск - пусть бежит в штаб
                else:       # уже бежит к домику
                    _unit.stop_walk()
                    _unit.parent_home = None # чтобы юнит бежал в штаб/домик
                    UMgr.add_free_unit(_unit)
# ============================================================

