# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import base_building_tips
import fsm_state
# ============================================================
# ============================================================
class FarmTips(base_building_tips.BuildingTips):
    # ============================================================
    def __init__(self, parent = None):
        base_building_tips.BuildingTips.__init__(self, parent)
        self.ser_name = 'FarmTips'
    # ============================================================
    #@override
    def Get_tip_image(self):
        '''Возвращает ID изображения, которое используется для бэка типса'''
        # чтобы обновить кадр селекта:
        _parent = self.parent
        _frame = 0
        _is_green_state = _parent.is_green_state()
        if not _is_green_state: _frame = 1
        _parent.Drawer.SetFrame('select', _frame)
        _parent.Drawer.SetSequence('arrow', _frame)    # номер последовательности сейчас совпадает с номером кадра
        for field in _parent.fields:
            if field:
                field.Drawer.SetFrame('select', _frame)
                field.Drawer.SetSequence('arrow', _frame)    # номер последовательности сейчас совпадает с номером кадра
        # ---
        if not _is_green_state:   # если хватает ресурсов на следующий апгрейд
            return 'IMG_I_TIP_BG_RED'
        return 'IMG_I_TIP_BG'
# ============================================================
# ============================================================
class FarmBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'FarmBuilding'
        self.tips = FarmTips(self)
        self.SetIsHome(False)
        self.to_building_subtype = 'bakery'
        self.fields = [None, None, None, None]
        self.pre_fields = [None, None, None, None]  # для апгрейдов
        self.field_cells = ['field1_cell', 'field2_cell', 'field3_cell', 'field4_cell']
        self.grown_stack = []
        self.is_unit_working = False
        self.has_places_for_fields = True
# ============================================================
    #@override
    def extra_init(self):
        base_building.BaseBuilding.extra_init(self)
        # зарезервируем места для полей:`
        for i in range(0,4):
            c_name = self.field_cells[i]
            _cell = self.GetAttrCellFixed(c_name)
            if self.IsPlaceFreeFor(_cell, 'farm_field1', True):
                self.ReservePlaceFor(_cell, 'farm_field1')
# ============================================================
    # @override
    def init_fsm(self):
        base_building.BaseBuilding.init_fsm(self)
        self.fsm_methods.states[fsm_state.StateRemovingFinal.name] = StateRemovingFinal(self)
        self.fsm_methods.states[fsm_state.StateRemoving.name] = StateRemoving(self)
# ============================================================
    def need_base_resource(self):
        return False
# ============================================================
    def on_res_gen_started(self):
        self.PlaySound('SND_FARM_WORK')
# ============================================================
    # @override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'field_grown':
            self.on_field_grown()
# ============================================================
    # @override
    def on_repair_complete(self):
        '''Для обработки ситуации окончания repair'''
        self.is_generating_res = False
        self.is_unit_working = False
        for field in self.fields:
            if field:
                if field.get('variant_name') == 'broken_1': # если поле было "сломано"
                    field.reinit_field_effectiveness_params(self)
                    field.FSM.HandleEvent('field_repaired_event')
                    field.switch_level()
                field.start_wheat_growth()
        LBoard.Core.try_open_help(2,1, 22, self, '3840')
# ============================================================
    # @override
    def on_extinguish_complete(self):
        '''Для обработки ситуации окончания extinguishing'''
        self.is_generating_res = False
        self.is_unit_working = False
        for field in self.fields:
            if field:
                if field.get('variant_name') == 'burning_1': # если поле горело
                    field.switch_level()
                    field.FSM.HandleEvent('field_extinguished_event')
                    field.switch_animation()
# ============================================================
    def on_build_started(self):
        for field in self.fields:
            if field:
                field.switch_level()
                field.switch_animation()
# ============================================================
    def on_cancel_build(self):
        for field in self.fields:
            if field:
                BMgr.RemoveItemFromMap(field)
        self.pre_fields = []
# ============================================================
    #@override
    def get_raw_button_frame(self):
        _frame = base_building.BaseBuilding.get_raw_button_frame(self)
        #if _frame >= 0:
            # #if self.status == 'normal' and (self.is_generated_res or self.is_generating_res):
            #_can_build = self.has_free_field_places()
            #if not _can_build:
            #    _frame = -1
        return _frame
# ============================================================
    #@override
    def is_active(self, skip_priority_check = False):
        '''Возможны ли действия над объектом в данный момент. Метод может быть переопределен'''
        _is_active = base_building.BaseBuilding.is_active(self, skip_priority_check)
        if _is_active:
            if not self.has_places_for_fields:
                return False
            return True
        else:
            return False
# ============================================================
    #@override
    def _start_resource_generation(self):
        #res = self.GetRes("normal_gives_1")
        #_is_res_empty = True
        #if res: _is_res_empty = res.Empty()
        #if _is_res_empty:     # если ресурс не задан, то здесь делать нечего
        #    return
        #res_name = res.GetSingleName()
        #if self.has_generating_res_progress() or not res_name or not self.res_generation_time or self.status != 'normal':
        #    return
        self.is_generating_res = True
        self.on_res_gen_started()
        self.Drawer.ShowIn('unit')
        self.switch_animation()
# ============================================================
    #@override
    def on_upgrade_complete(self, building_info):
        for field in self.pre_fields:
            if field:
                field.reinit_field_effectiveness_params(self)
                field.FSM.HandleEvent('prenormal_event')
                field.on_farm_build_complete()
        self.pre_fields = []
# ============================================================
    #@override
    def on_upgrade_started(self):
        for field in self.pre_fields:
            if field:
                _need_clean = False
                if field.status != 'normal': _need_clean = True
                field.switch_level()
                field.switch_animation()
                if _need_clean: field.clean_field_group()
# ============================================================
    #@override
    def on_build_complete(self, building_info):
        for field in self.fields:
            if field:
                field.reinit_field_effectiveness_params(self)
                field.FSM.HandleEvent('prenormal_event')
                field.on_farm_build_complete()
        self.pre_fields = []
# ============================================================
    #@override
    def update_effectiveness(self):
        '''У самой фермы эффективности нет и она не падает'''
        pass
# ============================================================
    #@override
    def give_resource(self):
        res = base_building.BaseBuilding.give_resource(self)
        self.check_grown_stack()
        return res
# ============================================================
    #@override
    def on_create(self):
        base_building.BaseBuilding.on_create(self)
        if self.status == 'broken':
            self.append_field('broken_1', True)
        elif self.status == 'burning':
            self.append_field('burning_1', True)
        elif self.status == 'place':
           self.append_field('place')
        else:
            self.append_field()
# ============================================================
    #@override
    def on_before_building_complete(self):
        base_building.BaseBuilding.on_before_building_complete(self)
        if self.status == 'normal': # сразу после апргрейда и строительства/ремонта?
            self.append_field('place')
            for field in self.fields:
                if field:
                    if field.status in ['place', '__none__']:
                        self.pre_fields.append(field)
# ============================================================
    #@override
    def on_upgrade_cancel(self):
        if self.status == 'normal':
            for field in self.pre_fields:
                BMgr.RemoveItemFromMap(field)
                for i in range(len(self.fields)):
                    if self.fields[i] == field:
                        self.fields[i] = None
        self.pre_fields = []
# ============================================================
    def append_field(self, field_variant = 'level_1', even_in_fog = False):
        result = []
        _need_fields = 0
        if self.status == 'normal':
            _need_fields = self.variants_indexes[self.get('variant_name')] + 1
        elif self.status in ['broken', 'burning', 'place']:
            _need_fields = self.variants_indexes[self.get('variant_name')] + 1
        _total = 0
        for i in range(0,4):
            field = self.fields[i]
            if field: _total += 1
        if _total >= _need_fields:
            #mbox(self.status + '-' + str(_total)+' '+str(_need_fields))
            return result
        _need_fields -= _total
        for i in range(0,4):
            if not self.fields[i]:  # если данное поле еще не создано
                c_name = self.field_cells[i]
                _cell = self.GetAttrCellFixed(c_name)
                if self.IsPlaceFreeFor(_cell, 'farm_field1', even_in_fog):
                    map_index = self.get('locind')
                    field = BMgr.AddItemToMap(map_index, 'farm_field1', field_variant, _cell, True)
                    field.AddEffect('building_install_01', field.get('pos'))
                    field.PlaySoundG('SND_BUILDING_SET')
                    result.append(field)
                    self.fields[i] = field
                    field.set_parent_farm(self) # передадим полю ссылку на ферму
                    if self.status == 'normal':
                        #field.reinit_field_effectiveness_params(self)
                        #if not self.is_deactivated:
                        #    field.start_wheat_growth()
                        #else:
                        if self.is_deactivated:
                            field.is_deactivated = True
                    _need_fields -= 1
                    if not _need_fields:
                        break
        return result
# ============================================================
    def can_append_field(self):
        for i in range(0,4):
            if not self.fields[i]:  # если данное поле еще не создано
                c_name = self.field_cells[i]
                _cell = self.GetAttrCellFixed(c_name)
                _place_free = self.IsPlaceFreeFor(_cell, 'farm_field1', False)
                if _place_free:
                    return True
        return False
# ============================================================
    def _need_upgrade_button_extra(self, place):
        if place == 'after_next_level_name':
            _can_append_field = self.can_append_field()
            if not _can_append_field:
                return [True, (True, 'gray_fields')]
        return [False]
# ============================================================
    #@override
    def activate(self):
        base_building.BaseBuilding.activate(self)
        if self.status == 'normal':
            for field in self.fields:
                if field:
                    field.reinit_field_effectiveness_params(self)
                    field.activate()
                    field.start_wheat_growth()
# ============================================================
    def on_field_growth_complete(self, field):
        '''Данный метод вызывается полем'''
        self.grown_stack.append(field)  # добавим поле в стек
        self.SendMsg('field_grown')
# ============================================================
    def on_field_grown(self):
        '''Данный метод вызывается по событию о созревании одного из полей'''
        self.check_grown_stack()
# ============================================================
    def check_grown_stack(self):
        if not self.is_generated_res:   # если ресурса перед зданием нет
            _status = self.GetAttrStr('status')
            if _status == 'normal':
                if self.grown_stack and not self.is_unit_working:
                    field = self.grown_stack.pop(0) # стек все-таки
                    if field:
                        field.start_unit_gather()   #@ пока время берем то же самое
                        self.is_unit_working = True
                        self._start_resource_generation()
                        self.switch_animation()
# ============================================================
    def on_field_gather_complete(self, field):
        '''Вызывается полем, когда завершен сбор урожая'''
        #self.generate_resource()
        self.FSM.HandleEvent('generated_event')
        field.start_wheat_growth()  # тут же запустим на поле рост урожая
        self.is_unit_working = False
        self.switch_animation()
# ============================================================
    #@override
    def _switch_animation_extra(self, tag):
        if tag == 'end':
            #state = self. SM .GetCurrentState()
            #if state in self._normal__empty:
            if self.is_normal_state():
                self.Drawer.ShowImages('anim', self.is_generating_res)
            else:
                self.Drawer.ShowImages('anim', False)
            _fsm_state = self.FSM.GetStateID()
            if not self.worker_units:
                self.Drawer.ShowImages('sleep_unit', False)
            elif _fsm_state in ['upgrading', 'repairing', 'building']:
                self.Drawer.ShowImages('sleep_unit', False)
            else:
                self.Drawer.ShowImages('sleep_unit', not self.is_unit_working)
# ============================================================
    def get_current_lowest_effectiveness(self):
        _min_eff = 100
        for f in self.fields:
            if f:
                if _min_eff > f.current_effectiveness:
                    _min_eff = f.current_effectiveness
        return _min_eff
# ============================================================
    def restore_eff(self):
        _need_restore_res = self.GetRes('restore_res')
        _is_res_enough = UStats.IsResEnough(_need_restore_res)
        if not _is_res_enough:
            return False
        UStats.SubRes(_need_restore_res)
        for f in self.fields:
            if f: f.restore_eff()
        EMgr.SendGlobalMsg('effectiveness_changed', '')
        EMgr.send_event('building_restore_eff', self)
        self.PlaySound('SND_EFFICIENCY_UP')
        self.show_arrow(False)
        self.Drawer.ShowImages('restore_eff')
        self.Drawer.StartEffectOnce('restore_eff')
        if True:
            letter = []
            letter.append( ('text', 'MSG_RESTORED_EFF') )
            letter.append( 'space' )
            _loc_ind = self.get('locind')
            EMgr.AppendLetter(letter, self.get('uid'), _loc_ind)
        TMgr.tutHelper.refresh()
        return True
# ============================================================
    #@override
    def on_action_hit_me(self):
        for field in self.fields:
            if field:
                field.Drawer.Blink('', 1, 0xffff8888, 0.5)
# ============================================================
    #@override
    def on_action_hits_me_complete(self):
        base_building.BaseBuilding.on_action_hits_me_complete(self)
        for field in self.fields:
            if field:
                field.break_me()
# ============================================================
    def show_select(self, show):
        self.Drawer.ShowImages('select', show)
        for field in self.fields:
            if field:
                field.show_select(show)
# ============================================================
    #@override
    def start_bonus_effect_double_generation(self):
        self.Drawer.StartEffectOnce('bonus_double')
        for field in self.fields:
            if field:
                field.Drawer.StartEffectOnce('bonus_double')
# ============================================================
    def apply_bonus_restore_effectiveness(self, time):
        _can_restore_bonus = self.GetAttrBool('can_restore_bonus')
        if _can_restore_bonus:
            for f in self.fields:
                if f:
                    f._apply_bonus_restore_effectiveness_internal(time)
            self.Signals.Restart('bonus_restore_eff', time)
            self.set_bonus_restore_eff_k()
            self.start_bonus_effect_restore_effectiveness()
# ============================================================
    #@override
#    def start_bonus_effect_restore_effectiveness(self):
#        self.Drawer.StartEffectOnce('bonus_restore')
#        for field in self.fields:
#            if field:
#                field.Drawer.StartEffectOnce('bonus_restore')
# #######################################################
# #######################################################
# ============================================================
class StateRemoving(fsm_state.StateRemoving):
    def __init__(self, item = None):
        fsm_state.StateRemoving.__init__(self, item)
        self.ser_name = 'FarmStateRemoving'

    def on_enter(self, evID):
        fsm_state.StateRemoving.on_enter(self, evID)
        b = self.item
        _show_out_layers = ''
        for field in b.fields:
            if field:
                field.Drawer.ShowImages('boom')
                field.Drawer.StartEffectOnce('boom')
                field.Drawer.ShowOut(_show_out_layers, 1800)    # плавное гашение здания
                field.Drawer.ShowImages(['anim'], False)
                field.Drawer.StopEffect('anim')
                field.Signals.Restart('building_need_afterdeath', 1000)
                field.Signals.Restart('add_afterdeath_item', 200)

    def on_exit(self, next_state):
        fsm_state.StateRemoving.on_exit(self, next_state)
        #b = self.item
        #b.add_afterdeath_item()  # добавляем воронку под зданием
# ============================================================
class StateRemovingFinal(fsm_state.StateRemovingFinal):
    def __init__(self, item = None):
        fsm_state.StateRemovingFinal.__init__(self, item)
        self.ser_name = 'FarmStateRemovingFinal'

    #@override
    def on_exit(self, next_state):
        fsm_state.StateRemovingFinal.on_exit(self, next_state)
        b = self.item
        for field in b.fields:
            if field:
                BMgr.RemoveItemFromMap(field)
# #######################################################
