# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import building_helper
import base_building_tips

# ============================================================
# ============================================================
# ============================================================
class HelperAction(object):
    def __init__(self, name = None, params = None):
        self.ser_name = 'FarmHelperAction'
        self.name = name
        self.params = params
# ============================================================
# ============================================================
class FarmFieldTips(base_building_tips.BuildingTips):
    # ============================================================
    def __init__(self, parent = None):
        base_building_tips.BuildingTips.__init__(self, parent)
        self.ser_name = 'FarmFieldTips'
    # ============================================================
    #@override
    def _append_genbar(self, result):
        '''Бар производства на типсе'''
        _parent = self.parent
        _has_growth = _parent.Signals.Has('growth_complete')
        if not _parent.is_production_started:  # if not gathering and not growing
            return
        _progress = 0   #self.Signals.GetProgress('resource_generated')
        if _has_growth:         # need growth progress
            _progress = _parent.Signals.GetProgress('growth_complete') * _parent.growth_percent
        else:                   # need gather progress
            _has_gather = _parent.Signals.Has('gather_complete')
            if _has_gather:
                _progress = _parent.growth_percent + _parent.Signals.GetProgress('gather_complete') * (1-_parent.growth_percent)
            else:
                _progress = _parent.growth_percent # пока ожидаем освобождения юнита
        if _progress < 0:
            return
        result.append('newline')
        result.append( ('text', 'COLOR_LINES') )
        result.append( ('text', 'TIP_RES_GENERATION') )
        result.append('newline')
        _bar_images = {'left':'TIP_GENBAR_L', 'right':'TIP_GENBAR_R', 'full':'TIP_GENBAR_FULL', 'empty':'TIP_GENBAR_EMPTY'}
        self._append_bar(result, _progress, _bar_images)
# ============================================================
# ============================================================
# ============================================================
class FarmField(base_building.BaseBuilding):
    field_group = ['wheat' + str(i) for i in range(1,10)]
    unit_group = ['unit' + str(i) for i in range(1,10)]
    field_shaded_layers = ['floor','default','unit','resources','upgrading','build','anim']

    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'FarmField'
        self.tips = FarmFieldTips(self)
        self.SetIsHome(False)
        self.bhelper = building_helper.BuildingHelper() # помощник в смене групп изображений
        self.bhelper.initialize(self.Drawer)
        #for i in range(9):
        #    self.bhelper.skip_boost_animations['unit' + str(i+1)] = 1
        self.is_production_started = False  # is growth-gather started
        self.is_process_started = False     # is gathering started
        self.growth_percent = 0.5     # отношение времени созревания к суммарному времени сбора+созревания
        self.farm = None
        if self.is_shaded:
            self.shade_layers()
# ============================================================
    #@override
    def extra_init(self):
        base_building.BaseBuilding.extra_init(self)
        for group in self.field_group:   # чистим поля
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', 1000)
            self.Drawer.StopAnimation(group, 0)    # очистим поля
# ============================================================
    def deserialize_init(self):
        self.bhelper.initialize(self.Drawer)
        self.bhelper.signaler = self.Signals
        self.bhelper.parent = self
# ============================================================
    def set_parent_farm(self, farm):
        self.farm = farm
# ============================================================
    def shade_layers(self):
        self.Drawer.SetShaded(self.field_shaded_layers, True)
        self.Drawer.SetShaded(self.field_group, True)
        self.Drawer.SetShaded(self.unit_group, True)
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'mower_event':
            self.PlaySound('SND_FARM_MOWER')
# ============================================================
    #@override
    def switch_animation(self):
        base_building.BaseBuilding.switch_animation(self)
        drawer = self.Drawer
        if not drawer.IsPartiallyInFog():
            drawer.ShowImages(self.field_group, (self.status == 'normal'))
            for field in self.field_group:
                drawer.SetTreeMode(field, 62, 3, 0.3)
            if self.status != 'normal' or (self.farm and self.farm.status != 'normal'):
                for group in self.field_group:
                    drawer.StopAnimation(group, 0)    # очистим поля
            #if self.is_generating_res:
            if self.is_process_started:
                self.bhelper.do_subactions()    # чтобы не потерлись действия, если еще раз вызвана эта функция
# ============================================================
    def clean_field_group(self):
        #drawer.ShowImages(self.field_group, _groups_visible)
        for group in self.field_group:
            self.Drawer.StopAnimation(group, 0)    # очистим поля
# ============================================================
    def need_base_resource(self):
        return False
# ============================================================
    # @override
    def create_building_units(self, external_units_count):
        pass
# ============================================================
    # @override
    def build_road_on_upgrade(self, prev_status):
        pass
# ============================================================
    # @override
    def _start_resource_generation(self):
        pass
# ============================================================
    # @override
    def start_resource_generation(self):
        pass
# ============================================================
    #@override
    def on_building_create(self):
        base_building.BaseBuilding.on_building_create(self)
        if self.is_shaded:
            self.shade_layers()
        for group in self.field_group:
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', self.get_effective_gen_time())
            self.Drawer.StopAnimation(group, 0)    # очистим поля
# ============================================================
# ============================================================
# ============================================================
    def set_generation_res_mul(self, k):
        base_building.BaseBuilding.set_generation_res_mul(self, k)
        if self.is_generation_boost_enabled:
            self.Signals.SetTimeMul('growth_complete', k)
        elif k == 1:
            self.Signals.SetTimeMul('growth_complete', 1)
# ============================================================
    def init_grow(self):
        '''Инициализация процесса роста'''
        self.enable_generation_boost(True)
        self.bhelper.reset()
        self.is_production_started = True
        # запустим рост:
        _anim_time = 2 * self.GetAnimationTime('EP_B_PERS_FARM_FARMER')
        _full_time = self.get_effective_gen_time()
        _growth_time = _full_time - _anim_time*9
        self.growth_percent = _growth_time / _full_time
        if _growth_time < 0: _growth_time = 1
        _growth_times = []
        for i in range(0,9):
            _growth_times.append(tools.Rand(_growth_time*7/10, _growth_time))   # 70%-100%
        i = 0
        for group in self.field_group:
            self.Drawer.PlayAnimationOnce(group, _growth_times[i])
            i += 1
        self.Signals.Restart('growth_complete', _growth_time)
# ============================================================
    def init_gather(self):
        '''Инициализация процесса сбора урожая'''
        self.bhelper.reset()
        _anim_time = 2 * self.GetAnimationTime('EP_B_PERS_FARM_FARMER')
        # запустим сбор урожая:
        _gather_time = _anim_time*9
        #_time_per_bush = _gather_time / 9
        _time_per_bush = _anim_time
        _time = 0
        _actions = []
        _act = []
        _act.append(HelperAction( 'pause_boost' ))
        _act.append(HelperAction( 'reset_anim_time', {'images':'unit1'} ))
        _act.append(HelperAction( 'show', {'images':'unit1'} ))
        _act.append(HelperAction( 'show', {'images':'wheat1'} ))
        _mower_percent = 50
        _act.append(HelperAction( 'set_event', {'image':'unit1', 'percent':_mower_percent, 'event':'mower_event'} ))
        for k in range(2, 10):
            _act.append(HelperAction( 'show', {'images':'wheat'+str(k)} ))
        _actions.append( {'time':_time, 'actions':_act} )
        for i in range(2,10):
            _time += _time_per_bush
            _a = []
            _a.append( HelperAction( 'hide', {'images':'unit'+str(i-1)} ) )
            _a.append( HelperAction( 'stop_animation', {'image':'wheat'+str(i-1), 'time':0} ) )
            _a.append( HelperAction( 'hide', {'images':'wheat'+str(i-1)} ) )
            _unit = 'unit' + str(i)
            _a.append( HelperAction( 'reset_anim_time', {'images':_unit, 'no_repeat':True} ) )
            _a.append( HelperAction( 'show', {'images':_unit} ) )
            _a.append(HelperAction( 'show', {'images':'wheat'+str(i)} ))
            _a.append( HelperAction( 'set_event', {'image':_unit, 'percent':_mower_percent, 'event':'mower_event'} ) )
            for k in range(i+1, 10):
                _a.append(HelperAction( 'show', {'images':'wheat'+str(k)} ))
            _actions.append({'id':'gather_animations', 'time':_time, 'actions':_a})
        _time = _gather_time - 1
        _a = []
        for i in range(9):
            _ind = str(i+1)
            _a.append( HelperAction( 'hide', {'images':'unit'+_ind} ) )
            _a.append( HelperAction( 'stop_animation', {'image':'wheat'+_ind, 'time':0} ) )
            _a.append( HelperAction( 'hide', {'images':'wheat'+_ind} ) )
        # 20.02.2013
        _a.append( HelperAction( 'remove_actions', {'id':'gather_animations'} ) )
        _a.append( HelperAction( 'stop_animation', {'image':'wheat9', 'time':0} ) )
        _actions.append({'time':_time, 'actions':_a})
        self.bhelper.set_actions(self, self.Signals, _actions)
        self.Signals.Restart('gather_complete', _gather_time)   # само событие нужно только для измерения времени
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal in ['bhelper_signal', 'bhelper_signal_noboost']:
            #self.bhelper.do_next_action()
            if not self.bhelper.do_next_action():
                self.is_production_started = False
                self.is_process_started = False # если выполнили все действия
                self.update_effectiveness()
                self.farm.on_field_gather_complete(self)
                for group in self.field_group:
                    self.Drawer.StopAnimation(group, 0)    # очистим поля
        elif signal == 'growth_complete':
            # отправить сообщение ферме
            self.farm.on_field_growth_complete(self)
        elif signal == 'init_grow':
            self.init_grow()
        else:
            base_building.BaseBuilding.on_signal(self, signal)
# ============================================================
    def start_wheat_growth(self):
        _status = self.GetAttrStr('status')
        if _status == 'normal':
            #self.is_process_started = True
            self.Signals.Restart('init_grow', 1)
# ============================================================
    def start_unit_gather(self):
        _status = self.GetAttrStr('status')
        if _status == 'normal':
            self.is_process_started = True
            self.init_gather()
# ============================================================
    def on_farm_build_complete(self):
        if self.status == 'place':
            self.switch_level()
        if self.status in ['place', 'normal']:
            self.start_wheat_growth()
# ============================================================
#    def _get_tips_extra(self, place, result, by_click):
#        return True
# ============================================================
    def reinit_effectiveness_params(self, force = False):
        pass
# ============================================================
    def reinit_field_effectiveness_params(self, farm, force = False):
        '''Переинициализирует параметры эффективности здания'''
        _can_reinit = False
        if not self.has_effectivness_inited: _can_reinit = True
        if force: _can_reinit = True
        if _can_reinit:
            self.need_max_time = False
            self.has_effectivness_inited = True
            self.res_generation_time = farm.GetAttrInt('res_gen_time')
            _new_max_res_generation_time = farm.GetAttrInt('res_gen_time_max')
            _new_max_res_gen_time = farm.GetAttrInt('gen_max_time')
            if _new_max_res_generation_time == self.max_res_generation_time and\
                _new_max_res_gen_time == self.max_res_gen_time:
                    return  # если параметры не изменились, то значит значения сохраняем
            self.res_gen_first_times = farm.GetAttrInt('res_gen_first_times')
            self.max_res_gen_time = farm.GetAttrInt('res_gen_maxtime')
            _need_init_gen_first_time = False
            if self.current_effectiveness == 100:
                if self.res_gen_first_times > 0:  # если первый отсчет еще
                    if not force:   # только в первом случае, после восстановления эффективности этого делать не нужно
                        _need_init_gen_first_time = True
            if _need_init_gen_first_time:
                self.effectiveness_degradation_count = self.res_gen_first_times
            else:
                self.effectiveness_degradation_count = farm.GetAttrInt('res_gen_ef_count')   # через сколько генераций ресурсов эффективность падает на 10%
            if self.effectiveness_degradation_count == 0:
                self.effectiveness_degradation_count = 3    # дефолтное значение
            self.max_res_generation_time = farm.GetAttrInt('res_gen_time_max')
            if self.max_res_generation_time < self.res_generation_time:
                self.max_res_generation_time = self.res_generation_time + 1 # +1 чтобы избежать деления на ноль
            self.generation_count = 0               # сколько раз уже генерировали ресурс
            _prev_eff = self.current_effectiveness
            self.current_effectiveness = 100        # текущая эффективность (проценты)
            if self.current_effectiveness != _prev_eff:
                EMgr.SendGlobalMsg('effectiveness_changed', '')
# ============================================================
    def get_attr_int(self, attr_name):
        '''Чтобы можно было легко переопределить в полях'''
        if attr_name == 'res_gen_ef_count':
            return self.farm.get_attr_int(attr_name)
        else:
            return self.GetAttrInt(attr_name)
# ============================================================
    def break_me(self):
        self.show_arrow(False)
        #self.energy = 100   # на всякий случай восстановим энергию
        trace('field_break_me:')
        trace(' field_break_me:', self.get('name'), self.get('uid'))
        self.SwitchLevel('broken_1')   # переключаемся на сломанный вариант
        self.bhelper.reset()
        self.is_production_started = False
        self.is_process_started = False
        for group in self.field_group:
            self.Drawer.StopAnimation(group, 0)    # очистим поля
        self.update_next_res()
        self.Drawer.ShowImages(self.field_group, False)
        self.Drawer.ShowImages(self.unit_group, False)
        self.switch_animation()
        self.on_switch_level_complete()
        self.update_res_enough_state()
        self.Signals.Cancel('growth_complete')
        self.Signals.Cancel('init_grow')
        self.Signals.Cancel('gather_complete')
        #self.Signals.Cancel('growth_complete')
# ============================================================
    def on_minimal_effectiveness(self):
        _loc_ind = self.get('locind')
        EMgr.AppendLetter([('text','LETTER_EFFECTIVENESS_10')], self.get('uid'), _loc_ind)
        UI.show_top_text2('MSG_EFFECTIVENESS_FARM_FIELD', _loc_ind, self.get('uid'))
# ============================================================
    def restore_eff(self):
        if self.get_effective_gen_time(100) != 0:
            self.restore_eff_k = self.get_effective_gen_time() / self.get_effective_gen_time(100)
        else:
            self.restore_eff_k = 1
        self.current_effectiveness = 100
        trace('CALL: RestoreEff (farm field)', self.get('uid'), self.get('name'))
        trace('Effectiveness (1):', self.current_effectiveness, self.restore_eff_k)
        self.reinit_field_effectiveness_params(self.farm, True)
        trace('Effectiveness (2):', self.current_effectiveness)
        EMgr.send_event('building_restore_eff', self)
        self.show_arrow(False)
        self.Drawer.ShowImages('restore_eff')
        self.Drawer.StartEffectOnce('restore_eff')
        return True
# ============================================================
    def _apply_bonus_restore_effectiveness_internal(self, time):
        self.Signals.Restart('bonus_restore_eff', time)
        _effective_gen_time100 = self.get_effective_gen_time(100)
        if _effective_gen_time100 != 0:
            self.restore_eff_k = self.get_effective_gen_time() / _effective_gen_time100
        else:
            self.restore_eff_k = 1
        self.start_bonus_effect_restore_effectiveness()
# ============================================================
# ============================================================
    #@override
    def on_restore_eff_bonus_start(self, k):
        base_building.BaseBuilding.on_restore_eff_bonus_start(self, k)
        self.bhelper.on_restore_eff_bonus_start(k)
        #for group in self.bush_group:
        #    self.Drawer.SetAnimBoost(group, k*1000)
# ============================================================
    #@override
    def on_restore_eff_bonus_stop(self):
        base_building.BaseBuilding.on_restore_eff_bonus_stop(self)
        self.bhelper.on_restore_eff_bonus_stop()
        #for group in self.bush_group:
        #    self.Drawer.SetAnimBoost(group, 1000)
# ============================================================
