# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import building_helper
import base_building_tips

# ============================================================
# ============================================================
# ============================================================
class HelperAction(object):
    def __init__(self, name = None, params = None):
        self.ser_name = 'ForesterHelperAction'
        self.name = name
        self.params = params
# ============================================================
# ============================================================
class ForesterFieldTips(base_building_tips.BuildingTips):
    # ============================================================
    def __init__(self, parent = None):
        base_building_tips.BuildingTips.__init__(self, parent)
        self.ser_name = 'ForesterFieldTips'
    # ============================================================
    #@override
    def _append_genbar(self, result):
        '''Бар производства на типсе'''
        _parent = self.parent
        _has_growth = _parent.Signals.Has('growth_complete')
        if not _parent.is_process_started and not _has_growth:
            return
        _progress = 0#self.Signals.GetProgress('resource_generated')
        if _has_growth:
            _progress = _parent.Signals.GetProgress('growth_complete') * _parent.growth_percent
        else:
            _has_gather = _parent.Signals.Has('gather_complete')
            if _has_gather:
                _progress = _parent.growth_percent + _parent.Signals.GetProgress('gather_complete') * (1-_parent.growth_percent)
            else:
                _progress = _parent.growth_percent # пока ожидаем освобождения юнита
        if _progress < 0:
            return
        result.append('newline')
        result.append( ('text', 'COLOR_LINES') )
        result.append( ('text', 'TIP_RES_GENERATION') )
        result.append('newline')
        _bar_images = {'left':'TIP_GENBAR_L', 'right':'TIP_GENBAR_R', 'full':'TIP_GENBAR_FULL', 'empty':'TIP_GENBAR_EMPTY'}
        self._append_bar(result, _progress, _bar_images)
# ============================================================
# ============================================================
# ============================================================
class ForesterField(base_building.BaseBuilding):
    field_group = ['wheat' + str(i) for i in range(1,10)]
    unit_group = ['unit' + str(i) for i in range(1,10)]
    plant_group = ['plant' + str(i) for i in range(1,10)]
    field_shaded_layers = ['floor','default','unit','resources','upgrading','build','anim']

    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'ForesterField'
        self.tips = ForesterFieldTips(self)
        self.SetIsHome(False)
        self.bhelper = building_helper.BuildingHelper() # помощник в смене групп изображений
        self.bhelper.initialize(self.Drawer)
        self.is_process_started = False
        self.is_growing = False
        self.is_unit_plant = False
        self.growth_percent = 0.5     # отношение времени созревания к суммарному времени сбора+созревания
        self.forester = None
        self.trees_count = 0
        if self.is_shaded:
            self.shade_layers()
# ============================================================
    #@override
    def extra_init(self):
        base_building.BaseBuilding.extra_init(self)
        for group in self.field_group:   # чистим поля
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', 1000)
            self.Drawer.StopAnimation(group, 0)    # очистим поля
        self.Drawer.ShowImages(self.field_group, False)    # очистим поля
# ============================================================
    def deserialize_init(self):
        self.bhelper.initialize(self.Drawer)
        self.bhelper.signaler = self.Signals
        self.bhelper.parent = self
# ============================================================
    def set_parent_forester(self, forester):
        self.forester = forester
# ============================================================
    def shade_layers(self):
        self.Drawer.SetShaded(self.field_shaded_layers, True)
        self.Drawer.SetShaded(self.field_group, True)
        self.Drawer.SetShaded(self.unit_group, True)
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'mower_event':
            self.PlaySound('SND_CHOP_TREE')
        elif msg_type == 'plant_event':
            self.PlaySound('SND_FORESTER_PLANT')
# ============================================================
    #@override
    def switch_animation(self):
        _fsm_state = self.FSM.GetStateID()
        #state = self. SM .GetCurrentState()
        drawer = self.Drawer
        # далее финальная часть, чтобы в тумане не вылазили части объекта:
        if drawer.IsPartiallyInFog():
            drawer.ShowImages('', False)
            drawer.ShowImages('floor')
            drawer.SetSequence('floor', self.current_episode-1)
            return
        drawer.ShowImages('', False)
        drawer.ShowImages('floor')
        drawer.SetSequence('floor', self.current_episode-1)
        #if state in self._normal__empty:         # '' (empty) - это состояние после выхода из любого другого
        if self.is_normal_state():
            drawer.ShowImages('default')
        elif self.is_wait_worker_state(_fsm_state):
            drawer.ShowImages('default')
        elif _fsm_state == 'building':
            drawer.ShowImages('build')
            drawer.SetProgressedAnimation('build')
            self._switch_animation_extra('build')
        elif _fsm_state == 'repairing':
            drawer.ShowImages('default')  # у сломанного состояния нет доп. изображения
            drawer.ShowImages('repair')
        elif _fsm_state == 'upgrading':
            drawer.ShowImages('upgrading')
        elif _fsm_state == 'extinguishing':
            drawer.ShowImages('default')
            drawer.ShowImages('extinguishing')
        elif _fsm_state in self._default_view_states:
            drawer.ShowImages('default')
        self.show_select(not self.is_deactivated and self.is_selected)
        _is_active = self.is_active()
        if not _is_active:
            _has_remove_or_restore = self.has_remove_or_restore(_fsm_state)
            if _has_remove_or_restore:
                _is_active = True
        if not _is_active or self.is_builders_state():   # state in ['build', 'upgrade', 'awaiting_action']:
            self.show_select(False)
            drawer.HighlightImages(self.highlight_layers, False)
        drawer.ShowImages('arrow', self.arrow_shown)
        #self.anim_helper.show_last_anim()
        # ---
        if _fsm_state in ['preremoving','removing','removing_final']:
            if _fsm_state in ['removing','removing_final']:
                drawer.ShowImages('boom')
            drawer.ShowImages(['anim','pers_no_action','unit','worker_inside', 'resources'], False)
            drawer.StopEffect('anim')

        #base_building.BaseBuilding.switch_animation(self)

        if not drawer.IsPartiallyInFog():
            for field in self.field_group:
                drawer.SetTreeMode(field, 100, 3, 0.3)
            if self.status != 'normal' or (self.forester and self.forester.status != 'normal'):
                for group in self.field_group:
                    drawer.StopAnimation(group, 0)    # очистим поля
            _has_growth = self.Signals.Has('growth_complete')
            _can_go = _has_growth
            if self.is_process_started: _can_go = True
            if _can_go:
                for i in range(self.trees_count):
                    drawer.ShowImages(self.field_group[i], True)#(self.status == 'normal'))
                self.bhelper.do_subactions()    # чтобы не потерлись действия, если еще раз вызвана эта функция
            #for i in range(self.trees_count):
            for i in range(self.trees_count, 9):
                drawer.ShowImages(self.field_group[i], False)
# ============================================================
    def need_base_resource(self):
        return False
# ============================================================
    # @override
    def create_building_units(self, external_units_count):
        pass
# ============================================================
    # @override
    def build_road_on_upgrade(self, prev_status):
        pass
# ============================================================
    # @override
    def _start_resource_generation(self):
        pass
# ============================================================
    # @override
    def start_resource_generation(self):
        pass
# ============================================================
    #@override
    def on_building_create(self):
        base_building.BaseBuilding.on_building_create(self)
        if self.is_shaded:
            self.shade_layers()
        for group in self.field_group:
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', self.get_effective_gen_time())
            self.Drawer.StopAnimation(group, 0)    # очистим поля
        self.Drawer.ShowImages(self.field_group, False)
# ============================================================
#    def on_create(self):
#        base_building.BaseBuilding.on_create(self)
# ============================================================
    #def get_effective_gen_time(self):
    #    '''Возвращает время генерации ресурса с учетом эффективности здания'''
    #    _need_res = self.need_base_resource()
    #    if _need_res or self.subtype in self.has_no_effectiveness:
    #        return self.res_generation_time
    #    if self.need_max_time and self.max_res_gen_time:
    #        return self.max_res_gen_time    # если требуется максимальное время генерации
    #    coeff = self.get_effectiveness_coeff()
    #    gen_time = self.res_generation_time
    #    max_gen_time = self.max_res_generation_time
    #    tim = gen_time + ((1 - coeff)/0.9)*(max_gen_time - gen_time)
    #    return tim
# ============================================================
    def reinit_trees_count(self):
        if self.forester:
            self.trees_count = self.forester.GetAttrInt('trees_count')
            self.switch_animation()
# ============================================================
    def set_generation_res_mul(self, k):
        base_building.BaseBuilding.set_generation_res_mul(self, k)
        if self.is_generation_boost_enabled:
            self.Signals.SetTimeMul('growth_complete', k)
        elif k == 1:
            self.Signals.SetTimeMul('growth_complete', 1)
# ============================================================
    def init_grow(self):
        '''Инициализация процесса роста'''
        self.enable_generation_boost(True)
        self.bhelper.reset()
        self.is_growing = True
        self.is_unit_plant = True
        self.forester.switch_animation()
        _trees_count = self.trees_count
        _plant_time = 2 * self.GetAnimationTime('UNIT_WOODCUTTER_ACTION')
        _cut_time = 2 * self.GetAnimationTime('UNIT_WOODCUTTER_ACTION')
        _full_time = self.get_effective_gen_time()
        _growth_time = _full_time - (_cut_time+_plant_time)*_trees_count # вычитаем время рубки и посадки деревьев
        _plant_growth_time = _full_time - _cut_time*_trees_count
        self.growth_percent = _plant_growth_time / _full_time
        if _growth_time < 0: _growth_time = 1
        # запустим посадку деревьев:
        _time = 0
        _actions = []
        _act = []
        _plant1 = 'plant1'
        _act.append(HelperAction( 'pause_boost' ))
        _act.append(HelperAction( 'reset_anim_time', {'images':_plant1, 'no_repeat':True} ))
        _act.append(HelperAction( 'show', {'images':_plant1} ))
        _plant_percent = 50
        _act.append(HelperAction( 'set_event', {'image':_plant1, 'percent':_plant_percent, 'event':'plant_event'} ))
        _actions.append( {'time':_time, 'actions':_act} )
        _plant = 'plant'
        _time += _plant_time
        for i in range(2, _trees_count+1):
            _a = []
            _a.append( HelperAction( 'hide', {'images':_plant+str(i-1)} ) )
            _a.append( HelperAction( 'start_once_animation', {'image':'wheat'+str(i-1), 'time':_growth_time, 'no_repeat':True} ) )
            _plantI = _plant + str(i)
            _a.append( HelperAction( 'reset_anim_time', {'images':_plantI} ) )
            _a.append( HelperAction( 'show', {'images':_plantI} ) )
            _a.append( HelperAction( 'set_event', {'image':_plantI, 'percent':_plant_percent, 'event':'plant_event'} ) )
            _actions.append({'time':_time, 'actions':_a})
            _time += _plant_time
        #   спрячем сеятелей, запустим рост последнего дерева
        _a = []
        for i in range(1,10):
            _str_i = str(i)
            _a.append( HelperAction( 'hide', {'images':_plant+_str_i} ) )
        _a.append( HelperAction( 'start_once_animation', {'image':'wheat'+str(_trees_count), 'time':_growth_time, 'no_repeat':True} ) )
        _a.append( HelperAction('restart_signal', {'signal':'worker_plant_free', 'time':1, 'no_repeat':True}) )
        _a.append(HelperAction( 'resume_boost' ))
        _actions.append({'time':_time, 'actions':_a})
        self.bhelper.set_actions(self, self.Signals, _actions)
        self.Signals.Restart('growth_complete', _plant_growth_time)
# ============================================================
    def force_complete_growth(self):
        '''Принудительное завершение (НЕ прекращение) роста'''
        self.Signals.Cancel('growth_complete')
        #self.Signals.Cancel('worker_plant_free')
        self.bhelper.reset()
        self.is_growing = True
        self.is_unit_plant = False
        self.forester.switch_animation()
        _trees_count = self.trees_count
        _plant_time = 2 * self.GetAnimationTime('UNIT_WOODCUTTER_ACTION')
        _cut_time = 2 * self.GetAnimationTime('UNIT_WOODCUTTER_ACTION')
        _full_time = self.get_effective_gen_time()
        _growth_time = _full_time - (_cut_time+_plant_time)*_trees_count # вычитаем время рубки и посадки деревьев
        #_plant_growth_time = _full_time - _cut_time*_trees_count
        if _growth_time < 0: _growth_time = 1
        # запустим посадку деревьев:
        _actions = []
        _plant = 'plant'
        _a = []
        for i in range(1,10):
            _plantI = _plant+str(i)
            _a.append( HelperAction( 'reset_anim_time', {'images':_plantI} ) )
            _a.append( HelperAction( 'hide', {'images':_plantI} ) )
        for i in range(1, _trees_count+1):
            _wheatI = 'wheat'+str(i)
            _a.append( HelperAction( 'reset_anim_time', {'images':_wheatI} ) )
            _a.append( HelperAction( 'start_once_animation', {'image':_wheatI, 'time':_growth_time} ) )
            _a.append( HelperAction( 'stop_animation', {'image':_wheatI, 'time':100} ) )
            _a.append( HelperAction( 'show', {'images':_wheatI} ) )
        _actions.append({'time':1, 'actions':_a})
        self.bhelper.set_actions(self, self.Signals, _actions)
        _has_worker_plant_free = self.Signals.Has('worker_plant_free')
        if _has_worker_plant_free:
            self.Signals.Restart('worker_plant_free', 1)
        self.Signals.Restart('growth_complete', 1000)
# ============================================================
    def init_gather(self):
        '''Инициализация процесса сбора урожая'''
        self.bhelper.reset()
        _trees_count = self.trees_count
        _anim_time = 2 * self.GetAnimationTime('UNIT_WOODCUTTER_ACTION')
        # запустим сбор урожая:
        _gather_time = _anim_time*_trees_count
        _time_per_bush = _gather_time / _trees_count
        _time = 0
        _actions = []
        _act = []
        _act.append(HelperAction( 'pause_boost' ))
        _act.append(HelperAction( 'reset_anim_time', {'images':'unit1', 'no_repeat':True} ))
        _act.append(HelperAction( 'show', {'images':'unit1'} ))
        _act.append(HelperAction( 'show', {'images':'wheat1'} ))
        _mower_percent = 50
        _act.append(HelperAction( 'set_event', {'image':'unit1', 'percent':_mower_percent, 'event':'mower_event'} ))
        for k in range(2, _trees_count+1):
            _act.append(HelperAction( 'show', {'images':'wheat'+str(k)} ))
        _actions.append( {'time':_time, 'actions':_act} )
        for i in range(2,_trees_count+1):
            _time += _time_per_bush
            _a = []
            _a.append( HelperAction( 'hide', {'images':'unit'+str(i-1)} ) )
            _a.append( HelperAction( 'stop_animation', {'image':'wheat'+str(i-1), 'time':0} ) )
            _a.append( HelperAction( 'hide', {'images':'wheat'+str(i-1)} ) )
            _unit = 'unit' + str(i)
            _a.append( HelperAction( 'reset_anim_time', {'images':_unit, 'no_repeat':True} ) )
            _a.append( HelperAction( 'show', {'images':_unit} ) )
            _a.append(HelperAction( 'show', {'images':'wheat'+str(i)} ))
            _a.append( HelperAction( 'set_event', {'image':_unit, 'percent':_mower_percent, 'event':'mower_event'} ) )
            for k in range(i+1, _trees_count+1):
                _a.append(HelperAction( 'show', {'images':'wheat'+str(k)} ))
            _actions.append({'time':_time, 'actions':_a})
        _time = _gather_time - 1
        _a = []
        for i in range(1,10):
            _str_i = str(i)
            _a.append( HelperAction( 'hide', {'images':'unit'+_str_i} ) )
            _a.append( HelperAction( 'stop_animation', {'image':'wheat'+_str_i, 'time':0} ) )
            _a.append( HelperAction( 'hide', {'images':'wheat'+_str_i} ) )
        _a.append( HelperAction('restart_signal', {'signal':'gather_real_complete', 'time':1}) )
        _actions.append({'time':_time, 'actions':_a})
        self.bhelper.set_actions(self, self.Signals, _actions)
        self.Signals.Restart('gather_complete', _gather_time)   # само событие нужно только для измерения времени
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'bhelper_signal':
            self.bhelper.do_next_action()
            #if not self.bhelper.do_next_action():
                #self.is_process_started = False # если выполнили все действия
                #self.update_effectiveness()
                #self.forester.on_field_gather_complete(self)
        elif signal == 'gather_real_complete':
            self.is_process_started = False # если выполнили все действия
            self.update_effectiveness()
            self.forester.on_field_gather_complete(self)
            for group in self.field_group:
                self.Drawer.StopAnimation(group, 0)    # очистим поля
            self.reinit_trees_count()
        elif signal == 'growth_complete':
            # отправить сообщение ферме
            self.is_growing = False
            self.is_unit_plant = False
            self.forester.on_field_growth_complete(self)
        elif signal == 'worker_plant_free':
            #self.is_growing = False
            self.is_unit_plant = False
            self.forester.switch_animation()
        elif signal == 'init_grow':
            self.init_grow()
        else:
            base_building.BaseBuilding.on_signal(self, signal)
# ============================================================
    def start_wheat_growth(self, is_upgrade = False):
        _is_growth_started = self.is_growth_started()
        _status = self.GetAttrStr('status')
        if is_upgrade:
            if self.is_process_started:
                return
            if _is_growth_started:
                self.force_complete_growth()
        if _is_growth_started:
            return
        if self.is_process_started:
            return
        if _status == 'normal':
            self.reinit_trees_count()
            #self.is_process_started = True
            self.Signals.Restart('init_grow', 1)
# ============================================================
    def is_growth_started(self):
        '''Просто проверка, что сейчас запущен рост, если уже закончился, то вернет False'''
        if self.is_growing: return True
        _has_init_grow = self.Signals.Has('init_grow')
        if _has_init_grow: return True
        return False
# ============================================================
    def start_unit_gather(self):
        _status = self.GetAttrStr('status')
        if _status == 'normal':
            self.is_process_started = True
            self.init_gather()
# ============================================================
    def on_forester_build_complete(self):
        if self.status == 'place':
            self.switch_level()
        if self.status in ['normal']:
            self.start_wheat_growth()
# ============================================================
#    def _get_tips_extra(self, place, result, by_click):
#        return True
# ============================================================
    def reinit_effectiveness_params(self, force = False):
        pass
# ============================================================
    def reinit_field_effectiveness_params(self, forester, force = False):
        '''Переинициализирует параметры эффективности здания'''
        _can_reinit = False
        if not self.has_effectivness_inited: _can_reinit = True
        if force: _can_reinit = True
        if _can_reinit:
            self.need_max_time = False
            self.has_effectivness_inited = True
            self.res_generation_time = forester.GetAttrInt('res_gen_time')
            _new_max_res_generation_time = forester.GetAttrInt('res_gen_time_max')
            _new_max_res_gen_time = forester.GetAttrInt('gen_max_time')
            if _new_max_res_generation_time == self.max_res_generation_time and\
                _new_max_res_gen_time == self.max_res_gen_time:
                    return  # если параметры не изменились, то значит значения сохраняем
            self.res_gen_first_times = forester.GetAttrInt('res_gen_first_times')
            self.max_res_gen_time = forester.GetAttrInt('res_gen_maxtime')
            _need_init_gen_first_time = False
            if self.current_effectiveness == 100:
                if self.res_gen_first_times > 0:  # если первый отсчет еще
                    if not force:   # только в первом случае, после восстановления эффективности этого делать не нужно
                        _need_init_gen_first_time = True
            if _need_init_gen_first_time:
                self.effectiveness_degradation_count = self.res_gen_first_times
            else:
                self.effectiveness_degradation_count = forester.GetAttrInt('res_gen_ef_count')   # через сколько генераций ресурсов эффективность падает на 10%
            if self.effectiveness_degradation_count == 0:
                self.effectiveness_degradation_count = 3    # дефолтное значение
            self.max_res_generation_time = forester.GetAttrInt('res_gen_time_max')
            if self.max_res_generation_time < self.res_generation_time:
                self.max_res_generation_time = self.res_generation_time + 1 # +1 чтобы избежать деления на ноль
            self.generation_count = 0               # сколько раз уже генерировали ресурс
            _prev_eff = self.current_effectiveness
            self.current_effectiveness = 100        # текущая эффективность (проценты)
            if self.current_effectiveness != _prev_eff:
                EMgr.SendGlobalMsg('effectiveness_changed', '')
# ============================================================
    def break_me(self):
        self.show_arrow(False)
        #self.energy = 100   # на всякий случай восстановим энергию
        trace('field_break_me:')
        trace(' field_break_me:', self.get('name'), self.get('uid'))
        self.SwitchLevel('broken_1')   # переключаемся на сломанный вариант
        self.bhelper.reset()
        self.is_process_started = False
        for group in self.field_group:
            self.Drawer.StopAnimation(group, 0)    # очистим поля
        self.update_next_res()
        self.Drawer.ShowImages(self.field_group, False)
        self.Drawer.ShowImages(self.unit_group, False)
        self.switch_animation()
        self.on_switch_level_complete()
        self.update_res_enough_state()
        self.Signals.Cancel('growth_complete')
        self.Signals.Cancel('init_grow')
        self.Signals.Cancel('gather_complete')
        self.Signals.Cancel('gather_real_complete')
        self.Signals.Cancel('worker_plant_free')
        self.is_growing = False
        self.forester.switch_animation()
        #self.Signals.Cancel('growth_complete')
# ============================================================
    def on_minimal_effectiveness(self):
        _loc_ind = self.get('locind')
        EMgr.AppendLetter([('text','LETTER_EFFECTIVENESS_10')], self.get('uid'), _loc_ind)
        UI.show_top_text2('MSG_EFFECTIVENESS_FORESTER_FIELD', _loc_ind, self.get('uid'))
# ============================================================
    def restore_eff(self):
        if self.get_effective_gen_time(100) != 0:
            self.restore_eff_k = self.get_effective_gen_time() / self.get_effective_gen_time(100)
        else:
            self.restore_eff_k = 1
        self.current_effectiveness = 100
        self.reinit_field_effectiveness_params(self.forester, True)
        EMgr.send_event('building_restore_eff', self)
        self.show_arrow(False)
        self.Drawer.ShowImages('restore_eff')
        self.Drawer.StartEffectOnce('restore_eff')
        self.switch_animation()
        return True
# ============================================================
    def get_attr_int(self, attr_name):
        '''Чтобы можно было легко переопределить в полях'''
        if attr_name == 'res_gen_ef_count':
            return self.forester.get_attr_int(attr_name)
        else:
            return self.GetAttrInt(attr_name)
# ============================================================
# ============================================================
    #@override
    def on_restore_eff_bonus_start(self, k):
        base_building.BaseBuilding.on_restore_eff_bonus_start(self, k)
        self.bhelper.on_restore_eff_bonus_start(k)
        #for group in self.bush_group:
        #    self.Drawer.SetAnimBoost(group, k*1000)
# ============================================================
    #@override
    def on_restore_eff_bonus_stop(self):
        base_building.BaseBuilding.on_restore_eff_bonus_stop(self)
        self.bhelper.on_restore_eff_bonus_stop()
        #for group in self.bush_group:
        #    self.Drawer.SetAnimBoost(group, 1000)
# ============================================================
    def _apply_bonus_restore_effectiveness_internal(self, time):
        self.Signals.Restart('bonus_restore_eff', time)
        _effective_gen_time100 = self.get_effective_gen_time(100)
        if _effective_gen_time100 != 0:
            self.restore_eff_k = self.get_effective_gen_time() / _effective_gen_time100
        else:
            self.restore_eff_k = 1
        self.start_bonus_effect_restore_effectiveness()
# ============================================================
# ============================================================
