# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import building_helper


# ============================================================
# ============================================================
# ============================================================
class HelperAction(object):
    def __init__(self, name = None, params = None):
        self.ser_name = 'GardenHelperAction'
        self.name = name
        self.params = params
# ============================================================
# ============================================================
# ============================================================
class GardenBuilding(base_building.BaseBuilding):
    bush_group = ['bush' + str(i+1) for i in range(0,6)]
    bush_shaded_layers = ['floor','default','unit','resources','upgrading','build','anim',
                            'bush1', 'bush2', 'bush3', 'bush4', 'bush5', 'bush6']
# ============================================================
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'GardenBuilding'
        self.bhelper = building_helper.BuildingHelper() # помощник в смене групп изображений при сборке кустов
        self.bhelper.initialize(self.Drawer)    # not initialized
        #for i in range(6):
        #    self.bhelper.skip_boost_animations['gather' + str(i+1)] = 1
        self.to_building_subtype = '__store__'
        self.SetIsHome(False)
        if self.is_shaded:
            self.Drawer.SetShaded(self.bush_shaded_layers, True)
# ============================================================
    #@override
    def extra_init(self):
        base_building.BaseBuilding.extra_init(self)
        for group in self.bush_group:   # чистим кусты
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', 1000)
            self.Drawer.StopAnimation(group, 0)    # очистим кусты
# ============================================================
    def deserialize_init(self):
        self.bhelper.initialize(self.Drawer)
        self.bhelper.signaler = self.Signals
        self.bhelper.parent = self
# ============================================================
    # @override
    def _start_resource_generation(self):
        base_building.BaseBuilding._start_resource_generation(self)
        self.init_gather()
# ============================================================
    def stop_resource_generation(self):
        #if self.is_generating_res:
        self.bhelper.reset()
        for group in self.bush_group:
            self.Drawer.StopAnimation(group, 0)    # кусты сбросим
        base_building.BaseBuilding.stop_resource_generation(self)
# ============================================================
    #@override
    def generate_resource(self):
        for group in self.bush_group:
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', self.get_effective_gen_time())
            self.Drawer.StopAnimation(group, 0)    # очистим кусты
        base_building.BaseBuilding.generate_resource(self)
# ============================================================
    #@override
    def need_base_resource(self):
        return False
# ============================================================
    def on_minimal_effectiveness(self):
        _loc_ind = self.get('locind')
        EMgr.AppendLetter([('text','LETTER_EFFECTIVENESS_10')], self.get('uid'), _loc_ind)
        UI.show_top_text2('MSG_EFFECTIVENESS_GARDEN', _loc_ind, self.get('uid'))
# ============================================================
    #@override
    def on_building_create(self):
        base_building.BaseBuilding.on_building_create(self)
        if self.is_shaded:
            self.Drawer.SetShaded(self.bush_shaded_layers, True)
        if not self.is_generating_res:
            for group in self.bush_group:   # чистим кусты
                self.Drawer.PlayAnimationOnce(group, '', '', -1, '', 1000)
                self.Drawer.StopAnimation(group, 0)    # очистим кусты
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'gather_event':
            self.PlaySound('SND_GARDEN_GATHER')
# ============================================================
    #@override
    def switch_animation(self):
        drawer = self.Drawer
        base_building.BaseBuilding.switch_animation(self)
        _partially_fog = drawer.IsPartiallyInFog()
        if not _partially_fog:
            drawer.ShowImages(self.bush_group, (self.status == 'normal'))
            for bush in self.bush_group:
                drawer.SetTreeMode(bush, 32, 2, 0.33)
            if self.is_generating_res:
                self.bhelper.do_subactions()    # чтобы не потерлись действия, если еще раз вызвана эта функция
            else:
                if self.status == 'normal':
                    for group in self.bush_group:
                        drawer.StopAnimation(group, 0)    # очистим кусты
# ============================================================
    def init_gather(self):
        '''Инициализация процесса сбора ягод'''
        self.enable_generation_boost(True)
        self.bhelper.reset()
        _anim_time = 2 * self.GetAnimationTime('EP_B_PERS_GARDEN_GATHER')
        _gather_time = _anim_time*6
        _time_per_bush = _anim_time
        #_growth_time = self.get_effective_gen_time()/2
        _final_time = self.get_effective_gen_time() - 1
        _growth_time = self.get_effective_gen_time() - _gather_time
        #mbox('effective_gen_time='+str(self.get_effective_gen_time()))

        for group in self.bush_group:   # очистим кусты на всякий случай
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', _growth_time)
            self.Drawer.StopAnimation(group, 0)    # очистим кусты
        # запустим рост кустов:
        #_growth_times = []
        _times = []
        _min = _growth_time*700/1000
        _max = _growth_time
        for i in range(0,6): _times.append( _min + i*(_max-_min)/5.0 )   # сортированный диапазон
        for i in range(5,-1,-1):    # reverse
            #shuffle:
            j = int( tools.random() * (i+1) )
            if i != j:
                _times[i], _times[j] = _times[j], _times[i]
        for i in range(0,6):
            #_grow = tools.Rand(_growth_time*700/1000, _growth_time)
            _grow = _times[i]
            self.Drawer.PlayAnimationOnce(self.bush_group[i], '', '', -1, '', _grow)
        _time = _growth_time
        _actions = []
        _act = []
        _gather_percent = 35
        _act.append(HelperAction( 'pause_boost' ))
        _act.append(HelperAction( 'reset_anim_time', {'images':'gather1'} ))
        _act.append(HelperAction( 'show', {'images':'gather1'} ))
        _act.append(HelperAction( 'set_event', {'image':'gather1', 'percent':_gather_percent, 'event':'gather_event'} ))
        _actions.append( {'time':_time, 'actions':_act} )

        for i in range(2,7):
            _time += _time_per_bush
            _a = []
            _a.append( HelperAction( 'hide', {'images':'gather'+str(i-1)} ) )
            _a.append( HelperAction( 'stop_animation', {'image':'bush'+str(i-1), 'time':0} ) )
            _gather = 'gather'+str(i)
            _a.append( HelperAction( 'reset_anim_time', {'images':_gather} ))
            _a.append( HelperAction( 'show', {'images':_gather} ) )
            _a.append( HelperAction( 'set_event', {'image':_gather, 'percent':_gather_percent, 'event':'gather_event'} ) )
            _actions.append({'id':'gather_animations', 'time':_time, 'actions':_a})
        _time = _final_time
        #_time = self.get_effective_gen_time() - 1
        _a = []
        for i in range(6):
            _a.append( HelperAction( 'hide', {'images':'gather'+str(i+1)} ) )
        _a.append( HelperAction( 'remove_actions', {'id':'gather_animations'} ) )
        #_a.append( HelperAction( 'hide', {'images':'gather6'} ) )
        _a.append( HelperAction( 'stop_animation', {'image':'bush6', 'time':0} ) )
        _actions.append({'time':_time, 'actions':_a})
        self.bhelper.set_actions(self, self.Signals, _actions)
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'bhelper_signal':
            self.bhelper.do_next_action()
        elif signal == 'bhelper_signal_noboost':
            self.bhelper.do_next_action()
        else:
            base_building.BaseBuilding.on_signal(self, signal)
# ============================================================
    #@override
    def on_restore_eff_bonus_start(self, k):
        base_building.BaseBuilding.on_restore_eff_bonus_start(self, k)
        self.bhelper.on_restore_eff_bonus_start(k)
        for group in self.bush_group:
            self.Drawer.SetAnimBoost(group, k*1000)
# ============================================================
    #@override
    def on_restore_eff_bonus_stop(self):
        base_building.BaseBuilding.on_restore_eff_bonus_stop(self)
        self.bhelper.on_restore_eff_bonus_stop()
        for group in self.bush_group:
            self.Drawer.SetAnimBoost(group, 1000)
# ============================================================
    # @override
    def on_repair_complete(self):
        base_building.BaseBuilding.on_repair_complete(self)
        LBoard.Core.try_open_help(1,1, 1, self, '4684')
        LBoard.Core.try_open_help(1,2, 12, self, '4891')
# ============================================================
# #######################################################
# #######################################################
# #######################################################
