# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import building_helper

# ============================================================
class HelperAction(object):
    def __init__(self, name, params):
        self.name = name
        self.params = params
# ============================================================
# ============================================================
class SilvermineBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'SilvermineBuilding'
        self.bhelper = building_helper.BuildingHelper() # помощник в смене групп изображений при сборке кустов
        self.bhelper.initialize(self.Drawer)
        self.to_building_subtype = "mint"
        self.SetIsHome(False)
# ============================================================
    def deserialize_init(self):
        self.bhelper.initialize(self.Drawer)
        self.bhelper.signaler = self.Signals
        self.bhelper.parent = self
# ============================================================
    def need_base_resource(self):
        return False
# ============================================================
    def on_minimal_effectiveness(self):
        _loc_ind = self.get('locind')
        EMgr.AppendLetter([('text','LETTER_EFFECTIVENESS_10')], self.get('uid'), _loc_ind)
        UI.show_top_text2('MSG_EFFECTIVENESS_SILVERMINE', _loc_ind, self.get('uid'))
# ============================================================
    def on_res_gen_started(self):
        self.PlaySound('SND_SILVER_MINE_WORK')
        self.Drawer.SetAnimationEvent('unit_from_cave', 1, 'cart_from_cave')
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'cart_from_cave':
            self.PlaySound('SND_SILVERMINE_CART')
# ============================================================
    #@override
    def on_res_gen_started(self):
        self.init_work()
# ============================================================
    def stop_resource_generation(self):
        self.bhelper.reset()
        for group in self.used_groups:
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', 1000)
            self.Drawer.StopAnimation(group, 0)
        self.Drawer.ShowImages(self.used_groups, False)
        base_building.BaseBuilding.stop_resource_generation(self)
# ============================================================
    #@override
    def switch_animation(self):
        base_building.BaseBuilding.switch_animation(self)
        if not self.Drawer.IsPartiallyInFog():
            if self.is_generating_res:
                self.bhelper.do_subactions()    # чтобы не потерлись действия, если еще раз вызвана эта функция
# ============================================================
# ============================================================
# ============================================================
    used_groups = ['unit_to_cave', 'unit_from_cave']
    def init_work(self):
        '''Инициализация процесса'''
        for group in self.used_groups:
            self.Drawer.PlayAnimationOnce(group, '', '', -1, '', 1000)
            self.Drawer.StopAnimation(group, 0)
        self.Drawer.ShowImages(self.used_groups, False)
        # запустим рост кустов:
        _gen_time = self.get_effective_gen_time()
        _animin_time = self.GetAnimationTime('EP_B_PERS_SILVERMINE_IN')
        _animout_time = self.GetAnimationTime('EP_B_PERS_SILVERMINE_OUT')
        _out_time = _gen_time - _animout_time
        # ---
        _actions = []
        _a = []
        _a.append( HelperAction( 'show', {'images':'unit_to_cave'} ) )
        _a.append( HelperAction( 'start_once_animation', {'image':'unit_to_cave'} ) )
        _actions.append({'time':0, 'actions':_a})
        _a = []
        _a.append( HelperAction( 'stop_animation', {'image':'unit_to_cave', 'time':0} ) )
        _a.append( HelperAction( 'hide', {'images':'unit_to_cave'} ) )
        _actions.append({'time':_animin_time-1, 'actions':_a})
        _a = []
        _a.append( HelperAction( 'show', {'images':'unit_from_cave'} ) )
        _a.append( HelperAction( 'start_once_animation', {'image':'unit_from_cave'} ) )
        _actions.append({'time':_out_time, 'actions':_a})
        _a = []
        _a.append( HelperAction( 'hide', {'images':'unit_from_cave'} ) )
        _a.append( HelperAction( 'stop_animation', {'image':'unit_from_cave', 'time':0} ) )
        _a.append( HelperAction( 'hide', {'images':'unit_from_cave'} ) )
        _actions.append({'time':_gen_time-1, 'actions':_a})
        self.bhelper.set_actions(self, self.Signals, _actions)
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'bhelper_signal':
            self.bhelper.do_next_action()
        else:
            base_building.BaseBuilding.on_signal(self, signal)
# #######################################################
# #######################################################
