# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import toolz

class TowerBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'TowerBuilding'
        self.SetIsHome(False)
        self.attack_force = self.GetAttrInt('attack_force')
        self.attack_period = self.GetAttrInt('attack_period')   # in milliseconds
        self.in_attack = False
        #self.Drawer.ShowImages('no_strike', False)
# ============================================================
    #@override
    def extra_init(self):
        base_building.BaseBuilding.extra_init(self)
        self.Drawer.ShowImages('no_strike', False)
# ============================================================
    #@override
    def add_fog_circle(self):
        '''Данный метод нужен для того, чтобы его можно было переопределять'''
        _workers_count = len(self.worker_units)
        if self.status != 'broken':
            if _workers_count:      # вырезать туман можно только когда в башне есть юнит
                self.AddFogCircle()
                self.AddEffect('fog_off_big', self.get('position'))
# ============================================================
    def on_switch_level_complete(self):
        base_building.BaseBuilding.on_switch_level_complete(self)
        #if self.get('stype') == 'watchtower':   # только для башен
        self.prev_fog_r = self.fog_r
        self.fog_r = self.GetAttrInt('fog_r')
        if self.prev_fog_r > 0 and self.fog_r > self.prev_fog_r:
            #_workers_count = len(self.worker_units)
            #if _workers_count > 0:
            letter = [('text', 'MSG_BUILDING_TOWER_ZONE_UP')]
            _loc_ind = self.get('locind')
            EMgr.AppendLetter(letter, self.get('uid'), _loc_ind)
            self.PlaySoundG('SND_NEW_AREA') # немного не там, где происходит открытие, но должно сработать
            if LBoard.Core.episode == 1:
                if LBoard.Core.level <= 2:
                    UI.show_chest_text('CHEST_MSG_TERRITORY_NEW', self.get('uid'))
        self.in_attack = False
# ============================================================
    #@override
    def on_worker_in(self, unit):
        '''Чтобы здание могло отловить когда пришел юнит-работник в здание'''
        self.add_fog_circle()
        self.PlaySoundG('SND_NEW_AREA')
        self.PlaySound('SND_BUILDING_OCCUPY')
        self.update_next_res()
        self.switch_animation()
        letter = [ ('text', 'MSG_BUILDING_TOWER_WORKERIN') ]
        #BMgr.SendGlobalMsg('tutorial_msg', 'hack1')
        if LBoard.Core.episode == 1:
            if LBoard.Core.level <= 2:
                UI.show_chest_text('CHEST_MSG_TERRITORY_NEW', self.get('uid'))
        _loc_ind = self.get('locind')
        EMgr.AppendLetter(letter, self.get('uid'), _loc_ind)
        if self.is_territory:
            UStats.AddLevelValue('territory', 1)
        BMgr.SendGlobalMsg('soldier_in_tower')
        _workers = len(self.worker_units)
        if _workers == 1:
            self._on_first_unit_in()
        self.FSM.HandleEvent('unit_in_event')
# ============================================================
    def on_upgrade_complete(self, building_info):
        base_building.BaseBuilding.on_upgrade_complete(self, building_info)
        self.attack_force = self.GetAttrInt('attack_force')
        self.attack_period = self.GetAttrInt('attack_period')   # in milliseconds
        self.Signals.Restart('attack_barbarians', self.attack_period)
        #UStats.AddLevelValue('progress', 1) # обновляем прогресс
# ============================================================
    def on_repair_complete(self):
        base_building.BaseBuilding.on_repair_complete(self)
        self.attack_force = self.GetAttrInt('attack_force')
        self.attack_period = self.GetAttrInt('attack_period')   # in milliseconds
        self.Signals.Restart('attack_barbarians', self.attack_period)
        UStats.AddLevelValue('progress', 1) # обновляем прогресс
        LBoard.Core.try_open_help(1,1, 4, self, '4771')
# ============================================================
    def on_build_complete(self, building_info):
        base_building.BaseBuilding.on_build_complete(self, building_info)
        self.attack_force = self.GetAttrInt('attack_force')
        self.attack_period = self.GetAttrInt('attack_period')   # in milliseconds
        self.Signals.Restart('attack_barbarians', self.attack_period)
        UStats.AddLevelValue('progress', 1) # обновляем прогресс
# ============================================================
    def on_survived_activate(self):
        base_building.BaseBuilding.on_survived_activate(self)
        self.attack_force = self.GetAttrInt('attack_force')
        self.attack_period = self.GetAttrInt('attack_period')   # in milliseconds
        self.Signals.Restart('attack_barbarians', self.attack_period)
        UStats.AddLevelValue('progress', 1) # обновляем прогресс
# ============================================================
    def _switch_animation_extra(self, place):
        '''Метод для переопределения в подклассах
        place: точка вызова метода'''
        if place == 'begin':
            extra_case = False
            #_fsm_state = self.FSM_GetStateID()
            if self.is_builders_state():
                extra_case = True
            if self.status == 'normal' and not extra_case:
                self.Drawer.ShowImages('attack', self.in_attack)
                self.Drawer.ShowImages('no_strike', not self.in_attack)
            else:
                self.Drawer.ShowImages('attack', False)
                self.Drawer.ShowImages('no_strike', False)
# ============================================================
    def on_signal(self, signal):
        if signal == 'attack_barbarians':
            self.Signals.Restart('attack_barbarians', self.attack_period)
            if self.attack_force > 0:
                self.attack_barbarians()
        #elif signal == 'attack_complete':
        #    self.in_attack = False
        #    self.Drawer.ShowImages('attack', False)
        #    self.Drawer.ShowImages('no_strike', True)
        else:
            base_building.BaseBuilding.on_signal(self, signal)
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'attack_complete':
            self.in_attack = False
            self.Drawer.ShowImages('attack', False)
            self.Drawer.ShowImages('no_strike', True)
# ============================================================
    def attack_barbarians(self):
        _action_cell = self.FixAddCell(self.cell_pos, self.GetAttrCell('fog_cell'))
        barbarians = self.GetUnitsInRadius('barb', _action_cell, self.GetAttrInt('fog_r'))
        _b_count = len(barbarians)
        # запустить эффект атаки на здании
        if _b_count > 0:
            has_attacking = False
            for b in barbarians:
                if b.is_attacking:
                    has_attacking = True
                    break
            if has_attacking:
                _vname = self.get('variant_name')
                if _vname == 'level_2':
                    self.PlaySound('SND_TOWER_SHOT_RIFLE')
                elif _vname == 'level_3':
                    self.PlaySound('SND_TOWER_SHOT_CANNON')
                self.in_attack = True
                self.Drawer.ShowImages('no_strike', False)
                self.Drawer.StopAnimation('attack', 0)
                self.Drawer.PlayAnimationOnce('attack', 'attack_complete')
                for b in barbarians:
                    if b.is_attacking:
                        b.attack(self.attack_force)
# ============================================================
# #######################################################
# #######################################################
# #######################################################
