# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building

class Waterpump(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'Waterpump'
        #self.to_building_subtype = "__store__"
        self.SetIsHome(False)
        self.is_internal_free_units = True
        self.unit_boost = self.GetAttrInt('unit_boost')
        self.firefighter = None
        self.is_unit_inside = False
# ============================================================
    def need_base_resource(self):
        return False
# ============================================================
    def on_switch_level_complete(self):
        base_building.BaseBuilding.on_switch_level_complete(self)
        self.unit_boost = self.GetAttrInt('unit_boost')
# ============================================================
    def on_res_gen_started(self):
        pass
# ============================================================
    def on_minimal_effectiveness(self):
        pass
# ============================================================
    #@override
    def need_to_show_unit(self):
        return False
# ============================================================
    #@override
    def need_no_action(self):
        return False
# ============================================================
    #@override
    def need_pers_no_action(self):
        return False
# ============================================================
    def init_anim(self):
        self.Drawer.SetAnimationEvent('unit', 10, 'worker_event')
        settings = []
        settings.append( ['unit', ('random', 8, 10)] )
        settings.append( ['pers_no_action', 5] )
        self.anim_helper.set_animations('default', settings)
# ============================================================
    #@override
    def on_worker_in(self, unit):
        '''Чтобы здание могло отловить когда пришел юнит-работник в здание'''
        base_building.BaseBuilding.on_worker_in(self, unit)
        self.init_anim()
        self.anim_helper.start_group('default')
        _len = len(self._internal_free_units)
        if _len > 0:
            self.firefighter = self._internal_free_units[0]
# ============================================================
    def on_upgrade_complete(self, building_info):
        base_building.BaseBuilding.on_upgrade_complete(self, building_info)
        self.Drawer.SetAnimationEvent('unit', 10, 'worker_event')
# ============================================================
    #@override
    def on_awaiting_action(self):
        self.anim_helper.hide()
        self.anim_helper.stop()
# ============================================================
    #@override
    def has_workers(self):
        return len(self._internal_free_units) > 0
# ============================================================
    # @override
    def _start_resource_generation(self):
        #self.anim_helper.start_group('default')
        pass
# ============================================================
    # @override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'worker_event':
            self.PlaySound('SND_WATERPUMP_UNIT_SOUND')
# ============================================================
    #@override
    def on_functional(self):
        LBoard.Core.append_building(self)
        self.update_buildings_count()
# ============================================================
    #@override
    def on_non_functional(self):
        LBoard.Core.remove_building(self)
        self.update_buildings_count()
# ============================================================
    def on_repair_complete(self):
        base_building.BaseBuilding.on_repair_complete(self)
        LBoard.Core.try_open_help(2,1, 21, self, '3844')
# ============================================================
    def update_buildings_count(self):
        w_buildings = BMgr.get_buildings_raw(-1, 'waterpump')
        for b in w_buildings:
            _is_functional = b.is_functional()
            if _is_functional:
                UStats.SetLevelValue('HAS_FIREFIGHTERS', 1)
                BMgr.refresh_buildings_next_res()
                return
        BMgr.refresh_buildings_next_res()
        UStats.SetLevelValue('HAS_FIREFIGHTERS', 0)
# ============================================================
    #@override
    def get_worker(self):
        _has_workers = self.has_workers()
        if _has_workers and self.is_unit_inside:
            return self.worker_units[0]
            #return self._internal_free_units[0]
        else:
            return None
# ============================================================
    #@override
    def pop_firefighter(self):
        _has_workers = self.has_workers()
        if _has_workers and self.is_unit_inside:
            _internal_count = len(self._internal_free_units)
            if _internal_count:
                _firefighter = self._internal_free_units.pop(0)
                _firefighter.work_boost = self.unit_boost / 100
                self.is_unit_inside = False
                return _firefighter
        else:
            return None
# ============================================================
    #@override
    def _on_predestroy(self):
        if not self.is_unit_inside: # если юнит еще где-то бегает
            _free_count = len(self._internal_free_units)
            if _free_count > 0:
                _unit = self._internal_free_units.pop(0)
                if _unit.task:
                    _unit.task.cancel() # отменяем таск - пусть бежит в штаб
                else:       # уже бежит к домику
                    _unit.stop_walk()
                    _unit.parent_home = None # чтобы юнит бежал в штаб/домик
                    UMgr.add_free_unit(_unit)
# ============================================================