# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_building
import task_wc

class WoodcutterBuilding(base_building.BaseBuilding):
    def __init__(self, cdata):
        base_building.BaseBuilding.__init__(self, cdata)
        self.ser_name = 'WoodcutterBuilding'
        self.to_building_subtype = "sawmill"
        self.SetIsHome(False)
        self.is_internal_free_units = True
        self.has_no_more_trees = False
        self.is_unit_inside = False
        self.unit_boost = self.GetAttrInt('unit_boost')
        self.around_trees_count = 0
        self.trees_lasts = 0
# ============================================================
#    # @override
#    def dispose(self):
#        EMgr.remove_event_listener(self)
# ============================================================
    # @override
    def on_building_create(self):
        base_building.BaseBuilding.on_building_create(self)
        action_cell = self.GetAttrCellFixed('action_cell')
        r = self.GetAttrInt('action_r')
        _trees = BMgr.GetItemsInRadius(self, action_cell, r, 'tree')
        self.around_trees_count = len(_trees)
        self.trees_lasts = self.around_trees_count
# ============================================================
    # @override
    def need_base_resource(self):
        return False
# ============================================================
#    def on_event(self, name, obj, extra_dict):
#        '''Обработчик игровых событий'''
#        if name == 'tree_grown':            # если выросло новое дерево
#            self._send_woodcutter_later()
# ============================================================
    # @override
    def take_resource(self, resource):
        _single_value = resource.GetSingleValue()
        if _single_value > 0:
            self.SendMsg('resource_received')
# ============================================================
    def on_switch_level_complete(self):
        base_building.BaseBuilding.on_switch_level_complete(self)
        self.unit_boost = self.GetAttrInt('unit_boost')
# ============================================================
    # @override
    def _start_resource_generation(self):
        '''У нас ресурс не генерится, но нужно отправлять юнита'''
        self._send_woodcutter_later()
        self.on_res_gen_started()
# ============================================================
    def need_to_show_unit(self):
        '''Нужно ли показывать юнита в здании'''
        return False
# ============================================================
    def need_pers_no_action(self):
        '''Нужно ли показывать юнита в здании'''
        return self.is_unit_inside and not self.is_under_attack
# ============================================================
    #@override
    def has_workers(self):
        return len(self._internal_free_units) > 0
# ============================================================
    #@override
    def get_worker(self):
        _has_workers = self.has_workers()
        if _has_workers and self.is_unit_inside:
            return self.worker_units[0]
            #return self._internal_free_units[0]
        else:
            return None
# ============================================================
    def _send_woodcutter_later(self, time = 500):
        self.Signals.Restart('send_woodcutter', time)
# ============================================================
    def _send_woodcutter(self):
        '''Отправляет лесоруба за древесиной, попутно проверяя необходимые условия'''
        if not self.is_generated_res and self.is_under_attack == 0 and self.status != 'broken':
            _int_units_count = len(self._internal_free_units)
            if _int_units_count > 0:
                _unit = self._internal_free_units[0]
                result = self.get_nearest_tree()
                if result:
                    self.show_clock(False)  # песочные часы ожидания
                    _tree, _path = result
                    # путь тоже нужно передавать, чтобы не считать лишний раз
                    _unit.work_boost = self.unit_boost / 100
                    TMgr.add_task( task_wc.WoodCutterTask(_tree, _unit, _path) )
                    self.is_unit_inside = False
                else:
                    _has_trees = self.has_more_items()
                    if not _has_trees:
                        self.has_no_more_trees = True   #@ нужно отслеживать, вдруг появились новые деревья!
                                                        #@ тогда нужно сбросить переменную
                                                        #@ сделать show_lock(False)
                                                        #@ вызвать работника
                                                        #@ убрать замок
                        # показать замок
                        self.FSM.HandleEvent('thrown_event')
                        letter = [ ('text', 'MSG_BUILDING_NO_TREES') ]
                        _loc_ind = self.get('locind')
                        EMgr.AppendLetter(letter, self.get('uid'), _loc_ind)
                        UI.show_top_text2('MSG_BUILDING_NO_TREES', _loc_ind, self.get('uid'))
                    else:
                        self._send_woodcutter_later(2000)   # через две секунды попробуем еще раз - вдруг туман откроется
                self.switch_animation()
# ============================================================
    def has_more_items(self):
        '''Проверяем - есть ли еще деревья вокруг, учитываем даже в тумане'''
        action_cell = self.GetAttrCellFixed('action_cell')
        r = self.GetAttrInt('action_r')
        _trees = BMgr.GetItemsInRadius(self, action_cell, r, 'tree')
        _around_trees_count = len(_trees)
        return (_around_trees_count > 0)
# ============================================================
    #@override
    def update_effectiveness(self):
        pass
# ============================================================
    #@override
    def reinit_effectiveness_params(self, force = False):
        '''Хитрый хак'''
        _cur_eff = self.current_effectiveness
        base_building.BaseBuilding.reinit_effectiveness_params(self, force)
        self.current_effectiveness = _cur_eff
# ============================================================
    #@override
    def return_worker_home(self):
        #base_building.BaseBuilding.action_hit_me(self)
        _return_unit = None
        if self.is_unit_inside:
            _return_unit = base_building.BaseBuilding.return_worker_home(self) # returning_unit =
            if self._internal_free_units:
                __unit = self._internal_free_units.pop(0)
            #has_workers = len(self.worker_units)
            self.is_unit_inside = False
        self.switch_animation()
        return _return_unit
# ============================================================
    def get_nearest_tree(self):
        _action_cell = self.GetAttrCellFixed('action_cell')
        result = self.GetNearestTreeInRadius('tree', 'tree', _action_cell, self.GetAttrInt('action_r'))
        return result
# ============================================================
    _upgrade_complete__resource_was_taken = ['upgrade_complete', 'resource_was_taken']
    _preremoving__removing__removing_final = ['preremoving','removing','removing_final']
    #@override
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type in self._upgrade_complete__resource_was_taken:
            if self.status == 'normal':
                if self.worker_units:
                    self._send_woodcutter_later()
        elif msg_type == 'resource_received':       # когда юнит принес ресурс
            self.trees_lasts -= 1       # уменьшим число собранных деревьев
            self.current_effectiveness = 100 * (self.trees_lasts / self.around_trees_count)
            EMgr.SendGlobalMsg('effectiveness_changed')
            _fsm_state = self.FSM.GetStateID()
            #if _fsm_state not in ['preremoving','removing','removing_final']:  # потому что при уничтожении здания теряется ресурс производства
            self.generate_resource()
            self.is_unit_inside = True
            self.switch_animation()
            if _fsm_state in self._preremoving__removing__removing_final: # если вернулись в момент разрушения
                self.lost_resource()       # ресурс производства теряется
                #_has_workers = self.has_workers()
                #if _has_workers:
                _unit = self.get_worker()
                if _unit:
                    _unit.show()
                    _unit.stop_walk()
        #elif msg_type == 'tree_grown': #@ должно вызываться извне, когда вырастет новое дерево, желательно сразу проверить - в досягаемости ли оно для здания
        #    pass
# ============================================================
    #@override
    def _on_predestroy(self):
        trace('WoodcutterBuilding _on_predestroy')
        self.Signals.Cancel('send_woodcutter')
        if not self.is_unit_inside: # если юнит еще где-то бегает
            trace('WoodcutterBuilding  unit outside')
            _free_count = len(self._internal_free_units)
            if _free_count > 0:
                _unit = self._internal_free_units.pop(0)
                if _unit.task:
                    trace('WoodcutterBuilding cancel task')
                    _unit.task.cancel() # отменяем таск - пусть бежит в штаб
                else:       # уже бежит к домику
                    trace('WoodcutterBuilding free unit')
                    _unit.stop_walk()
                    _unit.parent_home = None # чтобы юнит бежал в штаб/домик
                    UMgr.add_free_unit(_unit)
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'send_woodcutter':
            self._send_woodcutter()
        else:
            if signal == 'remove_countdown':
                LBoard.Core.try_open_help(1,1, 6, self, '4685')
            base_building.BaseBuilding.on_signal(self, signal)
            if signal == 'show_zone':
                action_cell = self.GetAttrCellFixed('action_cell')
                r = self.GetAttrInt('action_r')
                LBoard.HighlightRadius(self, 'tree','tree', action_cell, r)
# ============================================================
    def on_worker_in(self, unit):
        base_building.BaseBuilding.on_worker_in(self, unit)
        self.is_unit_inside = True
# ============================================================
    #@override
#    def on_mouse_in(self, over_zone):
#        base_building.BaseBuilding.on_mouse_in(self, over_zone)
# ============================================================
    #@override
    def on_mouse_out(self):
        base_building.BaseBuilding.on_mouse_out(self)
        LBoard.HighlightRadius(False)
# ============================================================
    #@override
    def _get_tips_extra(self, place, result, by_click):
        if place == 'after_name':
            #if self.SM.GetState() not in self._normal__empty:
            if not self.is_normal_state():
                self.tips._append_pbar(result)   # добавим прогрессбар, если нужно
                return False
        return True
# #######################################################
# #######################################################
# #######################################################
