# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_walkg
import task_walkw
import task_classes
import toolz
import fsm_state
import fsm_methods

# ============================================================
# ============================================================
class OpenKeySubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'OpenKeySubtask'
        self.actor_action_animation = 'pickup'
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        item.set_unit_direction(self.actor)
        if item.key_required == 'art2_key':
            item.PlaySound('SND_KEY_UNLOCK')
        elif item.snd_unlock:
            item.PlaySound(item.snd_unlock) #'SND_KEY_UNLOCK')
        item.remove_key()
        self.actor.action_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        return True
# ============================================================
# ============================================================
# ============================================================

class DiffItem(DifferentItem):
    resource_names = ['food', 'bricks', 'boards', 'coins', 'gold']  #, 'gem_red', 'gem_green', 'gem_blue']
    res_images = {
        'food':'IMG_I_TIP_FOOD',
        'boards':'IMG_I_TIP_BOARDS',
        'bricks':'IMG_I_TIP_BRICKS',
        'coins':'IMG_I_TIP_COINS',
        'gold': 'IMG_I_TIP_GOLD',
        '_berries':'IMG_I_TIP_FOOD',
        '_wood':'IMG_I_TIP_WOOD',
        '_fish':'IMG_I_TIP_FISH',
        '_flour':'IMG_I_TIP_BAG',
        '_ore':'IMG_I_TIP_ORE',
        '_stones':'IMG_I_TIP_STONE',
        '_granite':'IMG_I_TIP_STONE',
        }
# ============================================================
    item_layers = ['floor','default','unit','resources','upgrading','build','anim','cover','anim_up',
                    'transition']
    bump_layers = ['default']
    hlight_layers = ['default','cover']
# ============================================================
    _color_header = ('text', 'COLOR_HEADER')
    _color_lines = ('text', 'COLOR_LINES')
    _color_red = ('text', 'COLOR_RED')
    _not_enough_text = ('text', 'NOT_ENOUGH')
# ============================================================
    def __init__(self, cdata):
        DifferentItem.__init__(self, cdata)
        self.ser_name = '__none_differentitem__'
        self.subtype = self.get('stype')
        #self.states = {}
        self.fsm_methods = fsm_methods.FsmMethods(self)
        self.action_time = 1
        self.action_progress = 0
        self.cell_pos = self.get('cell')        # initializing position member
        # --- взаимодействие с ключами:
        self.snd_unlock = self.GetAttrStr('snd_unlock')
        self.key_required = self.GetAttrStr('key_required')
        self.key_need = self.key_required != ''
        self.key_removes = self.GetAttrBool('key_removes')
        # -----------------------------
        self.res_action_cost = self.get_action_cost_res()
        #self.action_was_started = False
        if not self.key_need:
            self.action_hits_last = self.GetAttrInt('action_hits_count')
            self.action_hits_count = self.action_hits_last
        else:
            self.action_hits_last = 1
            self.action_hits_count = 1
        self.task = None    # затычка, чтобы не падало на объектах, у которых нет своего класса
        self.is_mouse_over = False
        self.completeGivesRes = self.GetRes('gives') # ресурсы, которые отдает объект после окончания работы с ним
        self.is_mouse_over_zone = ''
        self.has_check = False      # установлена ли галка участия в задании
        self.can_show_uncheck = False   # можно ли показывать иконку отмены (это затрагивает только визуальную часть)
        self.need_attention_effect = False
        self.need_born_effect = False
        self.arrow_shown = False
        self.activate_show_arrow = self.GetAttrBool('activate_show_arrow')
        self.clock_shown = False
        self.action_enable_complete = False
        self.activate_uid = self.GetAttrStr('activate_uid')
        self.activated_focus = self.GetAttrBool('activated_focus')
        self.generated_loot = []
        self.had_action = False     # было ли хоть одно действие над объектом?
        self.extra_event = self.GetAttrStr('extra_event')   # нужно в самом начале прочитать эти данные, чтобы вариант не перетер
        self.tut_text_id = self.GetAttrStr('tutorial_text')
        self.is_secret = self.GetAttrBool('is_secret')
        self.need_to_remove_offscreen = False   # нужно ли удалить юнита при выходе за пределы экрана (используется индейцем после окончания квеста)
        self.is_offscreen = True
        self.is_zone_hlighted = False   # находится ли курсор над объектом, отличается от self.is_mouse_over, предназначен для показа зоны подсветки
        self.show_action_zone = False
        self.has_pending_actions = False
        #self.secret_tips_shown = False
        self.no_autoloot = self.GetAttrBool('no_autoloot')
        self.snd_eff_id = self.GetAttrStr('snd_eff_id')
        self.snd_anim_id = self.GetAttrStr('snd_anim_id')
        self.is_shaded = self.GetAttrBool('shaded')
        if self.is_shaded:
            self.shade_me(True)
        self.is_deactivated = self.GetAttrBool('deactivated')
        self.non_selectable = self.GetAttrBool('non_selectable')
        if self.non_selectable:
            self.SetMouseActive(False)
        self.allow_clicks = True    # нужно сбрасывать при "автоподборе лута"
# ============================================================
    def shade_me(self, shaded):
        #self.Drawer.SetShaded(self.item_layers, shaded)
        self.Drawer.SetShaded('', shaded)
        #self.Drawer.SetShaded(['shadow', 'arrow', 'clock_wait',
        #                        'button', 'icon', 'icons', 'select',
        #                        'select_under', 'task', 'arrow_tut',
        #                        'check', 'sign_talk', 'sign_red', 'sign_e',
        #                        'zone', 'secret'], False)
# ============================================================
    def dispose(self):
        pass
# ============================================================
    def set_shaded(self, _shaded):
        self.is_shaded = _shaded
        self.shade_me(self.is_shaded)
# ============================================================
    def has_access(self):
        _has_access = False
        _item_actor = self.GetAttrStr('item_actor')
        if _item_actor == 'general':
            _has_access = self.HasGeneralAccess()
        else:
            _has_access = self.HasAccess()
        return _has_access
# ============================================================
    def has_access_by_units(self):
        _has_access = self.HasAccess()
        return _has_access
# ============================================================
    def on_msg(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'signal_msg':
            self.on_signal(msg_param1)
        elif msg_type == 'state_machine':
            self.on_state_machine(msg_param1, msg_param2, msg_dict)
        elif msg_type == 'click':
            if not self.allow_clicks: return
            _has_access = self.has_access()
            if _has_access:
                self.on_click(msg_param1)
            else:
                LBoard.CrossNoToLastClick()
                trace('--> DiffItem no access', self.get('uid'))
        elif msg_type != 'on_create':
            self.on_message(msg_type, msg_param1, msg_param2, msg_dict)
        elif msg_type == 'on_create':
            self.on_create()
        #elif msg_type == 'variant_change':
        #    self.on_variant_change(msg_param1)
# ============================================================
    def on_msg_now(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'mouse_in':
            self.on_mouse_in(msg_param1)
            #if self.is_secret:
                #if self.is_active():
                    #self.Drawer.ShowImages('secret')
                    #self.secret_tips_shown = True
        elif msg_type == 'mouse_out':
            #if self.is_secret:
                #self.Drawer.ShowImages('secret', False)
                #self.secret_tips_shown = False
            self.on_mouse_out()
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'on_random_eff_start':
            #if param1 == 'default':
            if self.snd_eff_id:
                self.PlaySound(self.snd_eff_id)
        elif msg_type == 'on_random_anim_start':
            if self.snd_anim_id:
                self.PlaySound(self.snd_anim_id)
# ============================================================
    def on_signal(self, signal):
        if signal == 'attention_signal':
            if self.is_active():
                self.show_attention_effect()
            self.add_attention_signal() # сразу перезапустим сигнал
        elif signal == 'offscreen':
            if self.is_offscreen: return    # чтобы не дублировать
            self.is_offscreen = True
            if self.need_to_remove_offscreen:   # если нужно удалить после выхода за экран
                BMgr.RemoveItemFromMap(self)    # удалим объект
                return
            if self.has_pending_actions:
                self.has_pending_actions = False    # строго до 'try_do_postpoined_action' !
                TMgr.try_do_postpoined_action(self)
            #elif not self.is_shown:
            #    if self.need_to_show:
            #        self.show_as_possible()
        elif signal == 'onscreen':
            self.is_offscreen = False
        elif signal == 'kill_self':
            BMgr.RemoveItemFromMap(self)    # удаляем себя
            return
        elif signal == 'got_access':
            # показ подсказок на объекты:
            self.show_tutorial_text()
            self.refresh_arrow_color()
            #self.switch_animation()
        elif signal == 'no_access':
            self.Drawer.SetSequence('arrow', 1)    # в любом случае красная будет
            self.switch_animation()
        elif signal == 'show_zone':
            # покажем зону объекта
            self.show_action_zone = True
            self.is_zone_hlighted = True
            self.Drawer.ShowImages('zone', True)
# ============================================================
    def refresh_arrow_color(self):
        _seq = 0
        _is_green_state = self.is_green_state()
        if not _is_green_state: _seq = 1
        self.Drawer.SetSequence('arrow', _seq)
        self.switch_animation()
# ============================================================
    def show_tutorial_text(self):
        '''Один раз показывает подсказку генерала'''
        if self.tut_text_id:
            UI.show_tut_text(self.tut_text_id, False, self.get('uid'))
            self.tut_text_id = ''
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == 'enter':
            self.on_state_enter(state_name, additional)
            return
        elif type == 'exit':
            self.on_state_exit(state_name, additional)
            return
        elif type == 'change':
            self.on_state_change(additional.prev_state, additional.next_state)
# ============================================================
    def on_state_enter(self, state_name, additional):
        self.switch_animation()
# ============================================================
    def on_state_exit(self, state_name, additional):
        pass
# ============================================================
    def on_state_change(self, cur_state, next_state):
        pass
# ============================================================
    def is_extra_check_allow(self):
        '''For overriding'''
        return True
# ============================================================
    def on_click(self, btn_name):
        '''Вызывается при клике по объекту'''
        _is_active = self.is_active()
        if _is_active:
            _is_enough_params = self.is_enough_params()
            if _is_enough_params:
                if not self.task:
                    _extra_check_allow = self.is_extra_check_allow()
                    if not _extra_check_allow:
                        trace('--> DiffItem extra check disallow', self.get('uid'))
                        LBoard.CrossNoToLastClick()
                        return
                    _item_actor = self.GetAttrStr('item_actor')
                    if _item_actor == 'general':
                        #if UMgr.general and not UMgr.general.task:   # чтобы на генерала вешалось максимум одно задание
                        _is_general_ok = UMgr.is_general_ok()
                        if not _is_general_ok:
                            LBoard.CrossNoToLastClick()
                            return
                        _can_go = True
                        if self.key_need: _can_go = self.has_key()
                        if _can_go:
                            self.task = task_walkg.WalkGeneralTask(self)
                            TMgr.add_task( self.task )
                            self.show_arrow(False)
                            trace('--> DiffItem general ok', self.get('uid'))
                        else:
                            trace('--> DiffItem no key', self.get('uid'))
                            LBoard.CrossNoToLastClick()
                        return
                    else:
                        _res = self.res_action_cost     # сколько стоит действие
                        if _res:
                            _is_res_enough = UStats.IsResEnough(_res)
                            if _is_res_enough:    # если ресурсов достаточно
                                self.task = task_walkw.WalkWorkerTask(self)
                                TMgr.add_task( self.task )
                                self.show_arrow(False)
                                trace('--> DiffItem ok', self.get('uid'))
                                return
                            else:
                                trace('--> DiffItem res not enough', self.get('uid'))
                                LBoard.CrossNoToLastClick()
                        else:
                            trace('--> DiffItem no res', self.get('uid'))
                            LBoard.CrossNoToLastClick()
                elif not self.task.is_action_started:
                    self.task.cancel()
                    self.task = None
                    self.show_arrow(False)
                    trace('--> DiffItem task cancelled', self.get('uid'))
                    return
                else:
                    trace('--> DiffItem task action started', self.get('uid'))
            else:
                _param_name = self._get_not_enough_param_name()
                UI.BlinkGem(3, _param_name)
        #else:
        #LBoard.MoveCameraToLastClick()
        trace('--> DiffItem inactive', self.get('uid'))
        LBoard.CrossNoToLastClick()
# ============================================================
    def on_mouse_in(self, over_zone):
        _is_active = self.is_active()
        self.colorize_zone()
        if not _is_active:
            self.is_mouse_over = False
            self.is_selected = False
            self.Drawer.ShowImages('select', False)
            return
        if _is_active:
            self.Signals.Restart('show_zone', 700)
            self.is_mouse_over = True
            self.is_mouse_over_zone = over_zone
            drawer = self.Drawer
            drawer.HighlightImages(self.hlight_layers, True)
            drawer.ShowImages('select', True)
            check_frame = 0
            if self.need_show_uncheck(): check_frame = 1
            drawer.SetFrame('check', check_frame)
# ============================================================
    def colorize_zone(self):
        _has_color = self.HasAttr('zone_color')
        if _has_color:
            self.Drawer.ColorizeImages('zone', self.GetAttrInt('zone_color'))
# ============================================================
    def on_mouse_out(self):
        self.is_zone_hlighted = False
        self.show_action_zone = False
        drawer = self.Drawer
        drawer.ShowImages('zone', False)
        self.Signals.Cancel('show_zone')
        self.RemoveMsg('signal_msg', 'show_zone')
        _is_active = self.is_active()
        if not _is_active:
            self.is_mouse_over = False
            self.is_selected = False
            return
        self.can_show_uncheck = True    # теперь крестик показывать можно
        self.is_mouse_over = False
        self.is_mouse_over_zone = ''
        drawer.HighlightImages(self.hlight_layers, False)
        drawer.ShowImages('select', False)
        check_frame = 0
        if self.need_show_uncheck(): check_frame = 1
        drawer.SetFrame('check', check_frame)
# ============================================================
    def on_variant_change(self, new_variant_name):
        '''Вызывается после смены варианта в этом же кванте'''
        self.res_action_cost = self.get_action_cost_res()
        if not self.key_need:
            self.action_hits_last = self.GetAttrInt('action_hits_count')
            self.action_hits_count = self.action_hits_last
        else:
            self.action_hits_last = 1
            self.action_hits_count = 1
            self.key_required = self.GetAttrStr('key_required')
            self.key_need = self.key_required != ''
        #if self.action_hits_last > 0:
        #    self.FSM.HandleEvent('regenerate_event')    # чтобы вернуить в активное состояние
        if self.is_shaded:
            self.shade_me(True)
        self.non_selectable = self.GetAttrBool('non_selectable')
        if self.non_selectable:
            self.make_non_selectable()
        self.switch_animation()
# ============================================================
    def make_non_selectable(self):
        self.non_selectable = True
        self.SetMouseActive(False)
        self.is_mouse_over = False
        self.is_selected = False
        self.Drawer.ShowImages('select', False)
        self.Drawer.HighlightImages(self.hlight_layers, False)
        self.on_mouse_out()
# ============================================================
    def remove_me_later(self):
        ''' Удаляет тестовый объект с поля, считаем его "взятым" '''
        #BMgr.RemoveItemFromMap(self)
        self.Signals.Add('kill_self', 100)
        self.Drawer.Hide(True)
        self.deactivate()
        self.SetMouseActive(False)
# ============================================================
    def on_create(self):
        self.init_fsm()
        # деревья переопределяют данный метод!
        self.arrow_shown = self.GetAttrBool('arrow_shown')
        self.refresh_arrow_color()
        #self.switch_animation()
        _has_access = self.IsPassable()
        if _has_access:     # если проходимость есть при старте
            self.show_tutorial_text()
        # #if self.GetAttrStr('debug') == '123':
        # #    mbox(str(_seq))
# #######################################################
# #######################################################
# #######################################################
    _floor__icons__shadow = ['floor','icons','shadow']
    def switch_animation(self):
        drawer = self.Drawer
        _ShowImages = drawer.ShowImages
        if drawer.IsPartiallyInFog():
            _ShowImages('', False)
            return
        _ShowImages('', False)
        #if self.is_active():
        #    drawer.ShowImages('secret', self.secret_tips_shown)
        _ShowImages(self._floor__icons__shadow)
        self._switch_animation_extra('begin')
        image_category = 'default'
        #dict = {}
        #if state_name in dict:
        #    image_category = dict[state_name]
        _ShowImages(image_category, True)
        _state = self.FSM.GetStateID()
        _ShowImages('select', self.is_mouse_over and not self.is_deactivated and _state == 'normal')
        _ShowImages('check', self.has_check)
        check_frame = 0
        if self.need_show_uncheck(): check_frame = 1
        drawer.SetFrame('check', check_frame)
        _ShowImages('arrow', self.arrow_shown)
        _ShowImages('active', not self.is_deactivated)
        _ShowImages('clock_wait', self.clock_shown)
        _ShowImages('born', self.need_born_effect)
        _ShowImages('attention', self.need_attention_effect)
        _ShowImages('extra_1', True)
        if self.is_active():    # sure?
            if self.show_action_zone:
                _ShowImages('zone', self.is_zone_hlighted)  # для b_woodcutter нужно
            else:
                _ShowImages('zone', False)
        else:
            _ShowImages('zone', False)
        self._switch_animation_extra('end')
# ============================================================
    def _switch_animation_extra(self, place):
        '''Метод для переопределения в подклассах
        place: точка вызова метода'''
        pass
# ============================================================
    def need_show_uncheck(self):
        return self.is_mouse_over and self.can_show_uncheck
# ============================================================
    def Invalidate(self):
        '''перерисовка объекта, вызывается из C++'''
        self.switch_animation()
# ============================================================
    def switch_variant(self, variant_name):
        self.SwitchVariant(variant_name)
        self.on_variant_change(variant_name)
        self.is_secret = self.GetAttrBool('is_secret')
# ============================================================
    def set_check(self, visible):
        '''Ставит/убирает галку над объектом'''
        if self.has_check == visible:
            return
        self.has_check = visible
        if self.get('stype') != 'tree':
            if visible: self.PlaySoundG('SND_CLICK_TASK')
        if self.has_check:
            self.can_show_uncheck = False   # чтобы крестик не показывался, пока курсор не выйдет за пределы объекта
        self.Drawer.SetFrame('check', 0)
        self.switch_animation()
# ============================================================
# воздействие юнита:
# ============================================================
    def is_action_complete(self):
        '''Действие [юнитом] над данным объектом уже было выполнено'''
        _state = self.FSM.GetStateID()
        return (_state == 'complete' or self.action_hits_last == 0)
        #return self.action_hits_last <= 0
# ============================================================
#    def get_hits_last(self):
#        '''Сколько ударов осталось сделать для того, чтобы разбить/отремонтировать/построить данный объект'''
#        return self.action_hits_last
# ============================================================
    def show_secret_dialog(self):
        _secret_desc = self.GetAttrStr('secret_desc')
        self.PlaySoundG('SND_SECRET_FOUND')
        Camera.MoveTo(self)
        dlg.Open('SecretMenu', 'dlg_secret', {'single_mode':'1', 'text':_secret_desc, 'showin_effect':'secret_fly_in', 'pause':str(400)})
# ============================================================
    def action_hit_me(self, count=1):
        '''По объекту сделан удар
        count: сколько ударов за раз'''
        if self.is_action_complete():     # на всякий случай, чтобы объект нельзя было разрушить дважды
            return
        #self.action_was_started = True  # чтобы уже нельзя было отменить
        self.action_hits_last -= count
        self.action_progress = 1 - self.action_hits_last / self.action_hits_count
        self.on_action_hit_me()        # чтобы можно было переопределить
        if self.action_hits_last <= 0:
            #self.action_was_started = False # чтобы объект работал, если у него сложная логика
            effect_digged_name = self.GetAttrStr('effect_complete_name')
            if effect_digged_name:
                self.AddEffect(effect_digged_name, self.get('position'))
            if self.is_secret:
                UStats.AddLevelValue('secrets', 1)
                self.show_secret_dialog()
                EMgr.send_event( 'secret_found', self )
            self.on_action_hits_me_complete()
            self.activate_on_action_complete()
            self.show_arrow(False)
            self.FSM.HandleEvent('complete_event', True)
# ============================================================
    def action_hit_anim_complete(self):
        '''For overriding'''
        pass    # after INTERACTING animation complete (by general)
# ============================================================
    def on_action_hit_me(self):
        pass
# ============================================================
    def on_action_hits_me_complete(self):
        pass        # need to override
# ============================================================
    def on_before_action_started(self):
        # отключим селекты перед началом действия
        self.is_selected = False
        self.Drawer.HighlightImages(self.hlight_layers, False)
        self.Drawer.ShowImages('select', False)
# ============================================================
    def start_comet(self):
        '''Запускает комету, если указана в атрибутах, иначе просто дает сигнал на завершение удаления'''
        _comet_effect_name = self.GetAttrStr('comet_name')
        if _comet_effect_name:
            LBoard.StartComet(self, _comet_effect_name, 'comet_fly_complete')
            self.PlaySoundG('SND_ITEM_FLIGHT')
        else:
            self.Signals.Restart('comet_fly_complete', 1)
# ============================================================
# ============================================================
    def get_subtask(self, parent_task, actor):
        return task_classes.Subtask(parent_task, actor)
# ============================================================
    def get_unit_subtype(self):
        _unit_subtype = self.GetAttrStr('actor_subtype')
        if not _unit_subtype:
            return 'simple'
        else:
            return _unit_subtype
# ============================================================
    def show_arrow(self, show = True):
        self.arrow_shown = show
        self.refresh_arrow_color()
        #self.switch_animation()
# ============================================================
    def show_effect(self, group):
        '''Показать/скрыть эффект'''
        #self.arrow_shown = show
        self.Drawer.ShowImages(group)
        self.Drawer.StartEffectOnce(group)
# ============================================================
    def is_active(self):
        '''Возможны ли действия над объектом в данный момент. Метод может быть переопределен'''
        _not_deactivated = not self.is_deactivated
        _is_selectable = not self.non_selectable
        #_action_not_started = not self.action_was_started   # and not self.action_started
        if _not_deactivated:
            if _is_selectable:
                _state = self.FSM.GetStateID()
                #if _action_not_started:
                if _state == 'normal' or _state == '':  # empty - hack/fix for just generated objects
                    return True
        return False
# ============================================================
    def deactivate(self):
        self.is_deactivated = True
        self.switch_animation()
# ============================================================
    def activate(self):
        if self.is_deactivated:
            if self.activate_show_arrow:
                self.show_arrow()
                self.activate_show_arrow = False
        self.is_deactivated = False
        self.refresh_arrow_color()
        #self.switch_animation()
# ============================================================
    def get_action_cell_name(self):
        '''Метод возвращает название ячейки к которой будет подбегать юнит'''
        return 'action_cell'
# ============================================================
    def _filter_available_cells(self, cells_names):
        '''Выбирает из списка имен ячеек-атрибутов те, которые доступны для подхода'''
        _cells_names_result = []
        for _cell_name in cells_names:
            _cell = self.GetAttrCellFixed(_cell_name)
            _is_free = self.IsCellFree(_cell)
            if _is_free:
                _is_accessible = self.IsCellAccessible(_cell)
                if _is_accessible:
                    _cells_names_result.append(_cell_name)
        return _cells_names_result
# ============================================================
    def is_green_state(self):
        '''Цвет типса/селекта'''
        _has_access = self.has_access()
        if self.is_deactivated:
            if self.non_selectable:
                return True
            elif _has_access:
                return True
            else:
                return False
        if not _has_access:
            return False
        if not self.is_enough_params():
            return False
        if not self.has_check:
            if self.key_need:
                if not self.has_key():
                    return False
        _item_actor = self.GetAttrStr('item_actor')
        if _item_actor == 'general':
            _is_general_ok = UMgr.is_general_ok()   # болен ли генерал
            if not _is_general_ok:
                return False
        if not self.has_check:
            _action_started = False
            if self.task:
                if self.task.is_action_started:
                    _action_started = True
            if not _action_started:
                if self.res_action_cost:
                    _is_res_enough = UStats.IsResEnough(self.res_action_cost)
                    if not _is_res_enough:   # если хватает ресурсов на действие над объектом
                        return False
        _is_active = self.is_active()
        if _is_active:
            return True
        return True
# ============================================================
    def Get_tip_image(self):
        '''Возвращает ID изображения, которое используется для бэка типса'''
        # чтобы обновить кадр селекта:
        _frame = 0
        if not self.is_green_state(): _frame = 1
        drawer = self.Drawer
        drawer.SetFrame('select', _frame)
        drawer.SetSequence('arrow', _frame)    # номер последовательности сейчас совпадает с номером кадра
        # ---
        if self.is_green_state():
            return 'IMG_I_TIP_BG'
        return 'IMG_I_TIP_BG_RED'
# ============================================================
    def get_item_name(self):
        return self.GetAttrStr('item_name')
# ============================================================
    def Get_tip_text(self, by_click = False):
        '''Возвращает LIST, содержащий текстовые ID'''
        result = []
        if not self._get_tips_extra('before_name', result, by_click):
            self._append_need_boat_text(result, True)
            return result
        item_name = self.get_item_name()
        _item_actor = self.GetAttrStr('item_actor')
        if by_click:
            if _item_actor == 'general':
                _need_doctor = self._append_sick_general(result, True)
                if _need_doctor:
                    return result
        if self.is_secret and not by_click:
            result = [('image', 'IMG_I_TIP_SECRET')]
            #if _item_actor == 'general':
            #    self._append_sick_general(result)
            return result
        if not item_name or item_name == 'NO_TIPS':
            return result
        if not by_click:
            result.append( self._color_header )
            result.append( ('text', item_name) )
            if not self.is_active():
                return result
            if not self.has_check:
                res = self.res_action_cost
                _is_res_empty = True
                if res: _is_res_empty = res.Empty()
                if not _is_res_empty:
                    result.append( 'newline' )
                    result.append( self._color_lines )
                    result.append( ('text', 'NEED_FOR_ACTION') )
                    result.append( 'space' )
                    res_numbers = self._append_res_values(res)
                    result.extend( res_numbers )
                    if not UStats.IsResEnough(res):
                        diff_res = UStats.GetNotEnoughRes(res)
                        if diff_res:
                            result.append( 'newline' )
                            result.append( self._not_enough_text )
                            result.append( 'space' )
                            res_numbers = self._append_res_values(diff_res)
                            result.extend( res_numbers )
                self._append_need_gems(result)
            if self.key_need:
                self._append_need_key(result)
            # теперь опишем что дает данный элемент
            self._get_gives(result)
            self._append_need_boat_text(result, False)
            if not self._get_tips_extra('end', result, by_click): return result
            if _item_actor == 'general':
                self._append_sick_general(result)
        else:       # by click
            if not self._get_tips_extra('before_click', result, by_click): return result
            _has_access = self.has_access()
            if not _has_access:
                result = []
                result.append( self._color_red )
                result.append( ('text', 'NO_PATH') )
                self._append_need_boat_text(result, True)
                return result
            if not self._get_tips_extra('before_res', result, by_click): return result
            _is_active = self.is_active()
            if not _is_active:
                return []
            if not self.has_check:
                res = self.res_action_cost
                _is_res_empty = True
                if res: _is_res_empty = res.Empty()
                _nokey = False
                if not _is_res_empty:
                    if not UStats.IsResEnough(res):
                        diff_res = UStats.GetNotEnoughRes(res)
                        if diff_res:
                            result.append( self._color_red )
                            result.append( self._not_enough_text )
                            result.append( 'space' )
                            res_numbers = self._append_res_values(diff_res)
                            result.extend( res_numbers )
                        #else:
                        #    result = [('digit', 1)]
                else:
                    if self.key_need:
                        _has_key = self.has_key()
                        if not _has_key:
                            self._append_need_key(result, True)
                            _nokey = True
                self._append_not_enough_gems(result, _nokey)
            if not self._get_tips_extra('end_click', result, by_click): return result
            if not result:
                return self.Get_tip_text(False) # если по клику нет типса, то вернем его как без клика
            self._append_need_boat_text(result, True)
        return result
# ============================================================
    def _get_tips_extra(self, place, result, by_click):
        '''Предназначается для переопределения в потомках
        place: метка позиции вставки типса
        result: список значений, в который нужно добавлять данные
        by_click: типс по клику или нет
        returns: False - для завершения типса, т.е. после него ни одной строки не прибавится'''
        return True
# ============================================================
    def _append_sick_general(self, result, onclick = False):
        _get_sick_general_text = UMgr.get_sick_general_text(onclick)
        if _get_sick_general_text:
            if not onclick:
                result.append('newline')
            result.extend( _get_sick_general_text )
            return True
        return False
# ============================================================
    def _append_need_gems(self, result):
        _need_power = self.GetAttrInt('need_power')
        _need_oratory = self.GetAttrInt('need_oratory')
        # проверка на недостаточность чего-либо:
        _cur_res = UStats.GetCurRes()
        _ne_oratory = (_need_oratory-1) - _cur_res.Get('gem_green')
        _ne_power = (_need_power-1) - _cur_res.Get('gem_blue')
        if _need_oratory > 1 and _ne_oratory > 0:
            result.append( self._color_lines )
            result.append('newline')
            result.append(('text', 'TO_SPEAK'))
            result.append( 'space' )
            result.append(('text', 'TIPS_ORATORY'))
            result.append( 'space' )
            result.append(('text', 'TIPS_LEV'))
            result.append( 'space' )
            result.append( ('digit', _need_oratory) )
        if _need_power > 1 and _ne_power > 0:
            result.append( self._color_lines )
            result.append('newline')
            result.append(('text', 'TO_SPEAK'))
            result.append( 'space' )
            result.append(('text', 'TIPS_POWER'))
            result.append( 'space' )
            result.append(('text', 'TIPS_LEV'))
            result.append( 'space' )
            result.append( ('digit', _need_power) )
# ============================================================
    def _append_need_key(self, result, red=False):
        _key_name = BMgr.GetTemplateAttr(self.key_required, 'item_name')
        _has_key = self.has_key()
        if not red:
            result.append('newline')
            result.append( self._color_lines )
        else:
            result.append( self._color_red )
        result.append(('text', ['NEED_KEY', 'USE_KEY'][_has_key == True]))
        result.append('newline')
        result.append(('text', _key_name))
# ============================================================
    def _append_not_enough_gems(self, result, nokey):
        _need_power = self.GetAttrInt('need_power') - 1
        _need_oratory = self.GetAttrInt('need_oratory') - 1
        # здесь нужно указать чего и сколько нужно (по клику)
        _cur_res = UStats.GetCurRes()
        _ne_oratory = _need_oratory - _cur_res.Get('gem_green')
        _ne_power = _need_power - _cur_res.Get('gem_blue')
        if _ne_oratory > 0:
            if nokey:
                result.append( 'newline' )
                nokey = False
            result.append( self._color_red )
            result.append(('text', 'ATTR_NOT_ENOUGH'))
            result.append( 'space' )
            result.append(('text', 'TIPS_ORATORY'))
            result.append( 'space' )
            result.append( ('digit', _ne_oratory) )
            result.append( 'space' )
            result.append(('text', 'TIPS_LEV'))
        if _ne_power > 0:
            if nokey:
                result.append( 'newline' )
                nokey = False
            result.append( self._color_red )
            result.append(('text', 'ATTR_NOT_ENOUGH'))
            result.append( 'space' )
            result.append(('text', 'TIPS_POWER'))
            result.append( 'space' )
            result.append( ('digit', _ne_power) )
            result.append( 'space' )
            result.append(('text', 'TIPS_LEV'))
# ============================================================
    def _get_gives(self, result):
        '''Возвращает текст для части типса GIVES'''
        _gives = self.completeGivesRes
        if _gives:
            if not _gives.Empty():
                #_gives = _gives.Clone()
                #_gives.FixFood()
                #if not _gives.MainResEmpty():
                result.append( 'newline' )
                result.append( self._color_lines )
                result.append( ('text', 'GIVES') )
                result.append( 'space' )
                res_numbers = self._append_res_values(_gives)
                result.extend( res_numbers )
# ============================================================
    def _append_res_values(self, resources):
        '''Для типса. Генерирует список со значениями переданного ресурса, с соответствующими изображениями'''
        result = []
        first_res = True
        for res_name in self.res_images:    #resource_names:    # выводим значения ресурсов...
            #trace('res: ', resources, res_name)
            res_val = resources.Get(res_name)
            if res_val > 0:     # на всякий случай
                if not first_res:
                    result.append( 'comma' )
                    result.append( 'space' )
                first_res = False
                result.append( ('digit', resources.Get(res_name)) )
                result.append( ('image', self.res_images[res_name]) )
        return result
# ============================================================
    def Get_tip_position(self, tip_size, camera_pos, window_rect, zoom):
        '''Возвращает координаты типса от левого-верхнего угла картинки типса'''
        return toolz.Tools.get_tip_position(self, tip_size, camera_pos, window_rect)
# ============================================================
#    def apply_image_subtype(self, img_group):
#        self.Drawer
# ============================================================
# ============================================================
    def get_unpacked_content(self, content_string):
        _preparts = content_string.split(';')
        _result = []
        for prepart in _preparts:
            if prepart:             # если не пустой текст
                parts = prepart.split(':')
                if parts[0]:        # если не пустой текст
                    _result.append(parts)
        return _result
# ============================================================
    def get_free_gift_cell(self):
        _cell_pos = self.get('cell')
        for i in range(1,6):
            _cell_name = 'gift' + str(i) + '_cell'
            _cell = self.GetAttrCell(_cell_name)
            _cell = self.FixAddCell(_cell_pos, _cell)
            _is_free = self.IsCellFree(_cell)
            if _is_free:
                return _cell
        mbox(to_str("ERROR: all cells are busy. UID:", str(self.uid), 'Name:', self.get('name'), 'Cell:', _cell_pos))
        return None
# ============================================================
    def get_loot(self):
        return self.generated_loot
# ============================================================
    def publish_contents(self, contents, cell = None):
        '''Выкладывает на поле объекты'''
        #mbox(to_str(contents))
        if not contents:
            return []
        content_template, content_variant = '', ''
        result = []
        for content in contents:
            part0 = content[0]      # tinypy bug workaround
            content_parts = len(content)
            if content_parts == 2:
                content_template, content_variant = content
            elif content_parts == 1:
                content_template, content_variant = part0, ''
            else:
                continue
            if not cell:
                _cell = self.get_free_gift_cell()
            else:
                _cell = cell
            # - create item and place item on map in cell_pos
            if _cell:   # если есть такая ячейка
                map_index = self.get('locind')
                _new_item = BMgr.AddItemToMap(map_index, content_template, content_variant, _cell)        # hack - BuildingsManager может добавлять на карту не только здания...
                if _new_item:
                    _pos = _new_item.GetAttrPoint('center_item_pos')
                    _new_item.Drawer.ResIn(_new_item.item_layers, _pos, 1000)
                    _new_item.play_content_sound(self)
                    result.append(_new_item)
        self.generated_loot = result[:]
        return result
# ============================================================
    def publish_contents_by_one(self, contents, cell = None):
        '''Выкладывает на поле объекты для показа по очереди (сам метод объекты не показывает, а только выкладывает)'''
        #mbox(to_str(contents))
        if not contents:
            return []
        content_template, content_variant = '', ''
        result = []
        for content in contents:
            part0 = content[0]      # tinypy bug workaround
            content_parts = len(content)
            if content_parts == 2:
                content_template, content_variant = content
            elif content_parts == 1:
                content_template, content_variant = part0, ''
            else:
                continue
            if not cell:
                _cell = self.get_free_gift_cell()
            else:
                _cell = cell
            # - create item and place item on map in cell_pos
            if _cell:   # если есть такая ячейка
                map_index = self.get('locind')
                _new_item = BMgr.AddItemToMap(map_index, content_template, content_variant, _cell)        # hack - BuildingsManager может добавлять на карту не только здания...
                if _new_item:
                    #_pos = _new_item.GetAttrPoint('center_item_pos')
                    #_new_item.Drawer.ResIn(_new_item.item_layers, _pos, 1000)
                    _new_item.Drawer.Hide(True)  # скроем объекты
                    result.append(_new_item)
        self.generated_loot = result[:]
        return result
# ============================================================
    def play_content_sound(self, generator):
        '''Звук выпадающего из чего-либо объекта, определяется у самого объекта
        generator: объект, из которого выпадает ресурс'''
        pass
# ============================================================
    def show_attention_effect(self):
        '''Запускает эффект, привлекающий внимание к объекту'''
        self.need_attention_effect = True
        self.Drawer.RestartEffect('attention')
        self.switch_animation()
# ============================================================
    def show_born_effect(self):
        '''Запускает эффект, привлекающий внимание к объекту'''
        self.need_born_effect = True
        self.Drawer.RestartEffect('born')
        self.switch_animation()
# ============================================================
    def add_attention_signal(self, _from = 3000, _till = 7000):
        _time = tools.Rand(_from, _till)
        if _time < 300:
            mbox('attention_signal too often')  # dbg
        self.Signals.Restart('attention_signal', _time)
# ============================================================
    def stop_attention_signal(self):
        self.Signals.Cancel('attention_signal')
# ============================================================
    def is_enough_params(self):
        '''Достаточно ли красноречия и силы'''
        _need_power = self.GetAttrInt('need_power') - 1
        _need_oratory = self.GetAttrInt('need_oratory') - 1
        _cur_res = UStats.GetCurRes()
        _have_oratory = _cur_res.Get('gem_green')
        _have_power = _cur_res.Get('gem_blue')
        if _have_oratory < _need_oratory or _have_power < _need_power:
            return False
        else:
            return True
# ============================================================
    def _get_not_enough_param_name(self):
        '''Название характеристики, которо не хватает'''
        _need_power = self.GetAttrInt('need_power') - 1
        _need_oratory = self.GetAttrInt('need_oratory') - 1
        _cur_res = UStats.GetCurRes()
        _have_oratory = _cur_res.Get('gem_green')
        _have_power = _cur_res.Get('gem_blue')
        if _have_oratory < _need_oratory:
            return 'oratory'
        elif _have_power < _need_power:
            return 'power'
        else:
            return ''
# ============================================================
    def get_action_text(self, for_task_slide = False):
        return ''
# ============================================================
    _need_boat_text = ('text', 'NEED_BOAT')
    _need_tele_text = ('text', 'NEED_TELEPORT')

    def _append_need_boat_text(self, result, _red):
        '''Добавляем строчку, что необходима лодка или телепорт, если это нужно'''
        if self.is_deactivated: # для деактивированных ничего не нужно
            return
        _need_text = False
        _has_access = self.has_access()
        if not _has_access: # если к объекту нет никакого доступа
            _need_tele = self.GetAttrBool('need_teleport')
            _need_boat = self.GetAttrBool('need_boat')
            if _need_boat:
                _need_text = True
            elif _need_tele:
                _need_text = True
            else:       # для объектов на родном берегу
                _has_normal_access = self.HasAccess()
                if _has_normal_access:      # если сейчас объект недоступен, но обычный доступ к нему есть, то значит он на противоположном берегу
                    _need_text = True
                    _last_transport = UMgr.get_cluster( self.get('locind') ).general.last_transport
                    _need_boat = (_last_transport == 'boat')
                    _need_tele = (_last_transport == 'teleport')
        if _need_text:
            if not _red:
                result.append( self._color_lines )
            else:
                result.append( self._color_red )
            result.append('newline')
            if _need_boat:
                result.append(self._need_boat_text)
            elif _need_tele:
                result.append(self._need_tele_text)
# ============================================================
    def append_letter_resource(self, res_gives, res_text_id):
        '''Добавляет сообщение о подъеме ресурса генералом'''
        pass
        #_rcount = res_gives.GetSingleValue()
        #letter = []
        #letter.append( ('text', 'MSG_TAKEN_RES') )
        #letter.append('space')
        #letter.append( ('digit', _rcount) )
        #letter.append('space')
        #letter.append( ('text', res_text_id) )
        #EMgr.AppendLetter(letter, '')
# ============================================================
    def play_hit_sound(self):
        pass
# ============================================================
    def play_complete_sound(self):
        pass
# ============================================================
    def scale_bump(self):
        _pos = self.GetAttrPoint('center_item_pos')
        self.Drawer.ScaleBump(self.bump_layers, _pos, 200, 1.2)
# ============================================================
    def send_extra_event(self):
        if self.extra_event:
            BMgr.SendGlobalMsg('extra_event', self.extra_event)
            self.extra_event = ''
# ============================================================
    def get_action_cost_res(self):
        has_resname = self.HasAttr('acost_res_name')
        acost_res_name = 'action_cost'
        if has_resname:
            acost_res_name = self.GetAttrStr('acost_res_name')
        return self.GetRes(acost_res_name)
# ============================================================
    def activate_on_action_complete(self):
        if self.activate_uid:
            _item = BMgr.get_item(self.activate_uid, self.get('locind'))
            if _item:
                _item.activate()
                if self.activated_focus:
                    LBoard.MoveCameraToUID(_item.get('uid'), True)
            self.activate_uid = ''  # чтобы повторно нельзя было активировать
# ============================================================
    def do_action_enable(self):
        if not self.action_enable_complete:
            self.action_enable_complete = True
            self.switch_variant('enabled')
# ============================================================
    def Force_highlight(self, highlight):
        self.Drawer.HighlightImages(self.hlight_layers, highlight)
        #self.switch_animation()
# ============================================================
    def Force_colorize(self, color, highlight):
        '''color - without alpha part, only RGB. Alpha always 0xFF'''
        if not highlight:
            self.Drawer.ColorizeImages(self.hlight_layers, 0xffffff, False)
        else:
            self.Drawer.ColorizeImages(self.hlight_layers, color, False)
# ============================================================
    def Get_workers_count(self):
        return 0    # затычка
# ============================================================
    def set_unit_direction(self, unit):
        if self.HasAttr('direction'):
            unit.set_direction_to(self.GetAttrInt('direction'))
# ============================================================
    def has_key(self):
        if not self.key_need:
            #trace('>>> NO KEY NEED', self.get('name'))
            return False
        else:
            _has_key = UStats.Inventory.HasQuestItem(self.key_required)
            #trace('>>> KEY NEED', self.key_required, self.get('name'))
            #trace('>>> KEY RESULT', _has_key)
            return _has_key
# ============================================================
    def remove_key(self):
        '''Удаляет ключ из инвентаря, если это необходимо'''
        if self.key_need:
            if self.key_removes:
                UStats.Inventory.RemoveQuestItem(self.key_required, 1)
# ============================================================
    def do_game_action(self, data, time_shift = 0):
        '''type, txt_param1, txt_param2, int_param1, int_param2, is_offscreen'''
        TMgr.do_game_action(self, data, time_shift)
# ============================================================
    def do_action(self, name):
        _action_done = self._do_action(name)  # for overriding
        if not _action_done:
            if name == '':
                pass
# ============================================================
    def _do_action(self, name):
        '''For overriding in child classes'''
        return False
# ============================================================
    # заглушка пока только для task_item
    def is_busy(self):
        if self.task != None:
            return True
        else:
            return False
# ============================================================
    def remove_faded(self, time):
        self.Signals.Restart('kill_self', time)
        self.Drawer.ShowOut('', time)
        self.deactivate()
        self.SetMouseActive(False)
# ============================================================
# ============================================================
    def init_fsm(self):
        FSM = self.FSM

        self.UseFSM(True)
        FSM.Init('normal')
        _states = [StateNormal, StateInteracting, StateComplete]
        #for state in _states: self.states[state.name] = state(self)
        self.fsm_methods.apply_states(_states)

        FSM.AddTransition('start_event', 'normal', 'interacting')  #\/
        FSM.AddTransition('complete_event', 'interacting', 'complete')  #\/
        FSM.AddTransition('regenerate_event', 'complete', 'normal')  #\/
        FSM.AddTransition('cancel_interaction_event', 'interacting', 'normal')  #\/

        FSM.AddTransition('regenerate_event', 'interacting', 'normal')  #\/
# ============================================================
# ============================================================
# ============================================================
class StateNormal(fsm_state.FSMState):
    name = 'normal'

    def __init__(self, item = None):
        fsm_state.FSMState.__init__(self, item)
        self.ser_name = 'StateDINormal'

    def on_enter(self, evID):
        self.item.action_hits_last = self.item.GetAttrInt('action_hits_count')
        self.item.action_hits_count = self.item.action_hits_last
        pass

    def on_handle(self, evID):
        pass
# ============================================================
class StateInteracting(fsm_state.FSMState):
    name = 'interacting'

    def __init__(self, item = None):
        fsm_state.FSMState.__init__(self, item)
        self.ser_name = 'StateDIInteracting'

    def on_enter(self, evID):
        pass

    def on_handle(self, evID):
        pass
# ============================================================
class StateComplete(fsm_state.FSMState):
    name = 'complete'

    def __init__(self, item = None):
        fsm_state.FSMState.__init__(self, item)
        self.ser_name = 'StateDIComplete'

    def on_enter(self, evID):
        pass

    def on_handle(self, evID):
        pass
# ============================================================
