# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

class BuildingHelper(object):
# ============================================================
    def __init__(self):
        self.ser_name = 'BuildingHelper'
        self.drawer = None
        self.parent = None
        self.signal_name = ''
        self.noboost_signal_name = ''
        self.actions = []           # текущие действия, [ [('action', params)], [('action', params), ('action', params)] ]
        self.removed_ids = []
        self.current_subactions = {'id':'__none__', 's_actions':[], 'uid':0}
        self.uid = 0
        self.prev_uid = 0
        self.signaler = None
        self.bonus_boost = 1
        self.is_bonus_restore_eff_started = False
        self.is_boost_allowed = True
        self.once_animations = {}
        self.skip_boost_animations = {}
# ============================================================
    #def on_update(self, ms):
    #    pass
# ============================================================
    def set_current_actions_uid(self):
        self.uid += 1
        self.current_subactions['uid'] = self.uid
# ============================================================
    def initialize(self, drawer, signal_name = 'bhelper_signal'):
        '''Задает связку с LayeredDrawer объекта'''
        self.drawer = drawer
        self.signal_name = signal_name
        self.noboost_signal_name = signal_name + '_noboost'
# ============================================================
    def reset(self):
        #self.on_restore_eff_bonus_stop()   # нельзя!
        #self.once_animations = {}
        self.actions = []
        self.removed_ids = []
        self.current_subactions = {'id':'__none__', 's_actions':[]}
        self.set_current_actions_uid()
        self.prev_uid = 0
        self.is_boost_allowed = True
        if self.signaler:
            self.signaler.Cancel(self.signal_name)
            self.signaler.Cancel(self.noboost_signal_name)
# ============================================================
    _start_once_animation__restart_signal__reset_anim_time = ['start_once_animation', 'restart_signal', 'reset_anim_time']
    def set_actions(self, parent, signaler, actions = []):
        '''Передает список действий-'сигналов' для LayeredDrawer'''
        self.parent = parent
        self.signaler = signaler
        # нужно распарсить actions, и создать несколько сигналов
        # сначала переведем все в списки:
        _actions = []
        for subactions in actions:
            _sub_action = {}
            if istype(subactions.actions, 'dict'):
                _sub_action = {'time':subactions.time, 'actions':[subactions.actions]}
                if 'noboost' in subactions: _sub_action.noboost = 1
                if 'id' in subactions: _sub_action.id = subactions.id
                _actions.append(_sub_action)
            elif istype(subactions.actions, 'list'):
                _sub_action = {'time':subactions.time, 'actions':subactions.actions}
                if 'noboost' in subactions: _sub_action.noboost = 1
                if 'id' in subactions: _sub_action.id = subactions.id
                _actions.append(_sub_action)
            else:
                mbox('ERROR. Wrong data for BuildingHelper')

        self.actions = []
        # теперь можно обрабатывать подготовленный список
        for subactions in _actions:
            if 'noboost' in subactions:
                signaler.Add( self.noboost_signal_name, subactions.time )
            else:
                signaler.Add( self.signal_name, subactions.time )
            _subactions = {'id':'__none__', 's_actions':[]}
            if 'id' in subactions:
                _subactions.id = subactions.id
            for action in subactions.actions:   # обработаем список действий на один сигнал
                _action = {}        # нам время хранить не нужно,
                _action.name = action.name      # только название действия
                _action.params = action.params  # и параметры
                if _action.name in self._start_once_animation__restart_signal__reset_anim_time:
                    _action.params['no_repeat'] = True  # чтобы эти действия не повторять
                _subactions.s_actions.append(_action)
            self.actions.append(_subactions)
        # ---
        #if self.is_bonus_restore_eff_started:
        #    if self.signaler:
        #        self.signaler.SetTimeMul(self.signal_name, self.bonus_boost)
        self.set_bonus_boost(self.bonus_boost)
# ============================================================
    def set_bonus_boost(self, boost, anim_names = []):
        _can_change = False
        if boost == 1: _can_change = True
        if self.is_boost_allowed:
            if self.is_bonus_restore_eff_started: _can_change = True
        if _can_change:
            #self.bonus_boost = boost
            if self.signaler:
                self.signaler.SetTimeMul(self.signal_name, boost)
            if anim_names:
                for anim in anim_names:
                    self.drawer.SetAnimBoost(anim, boost*1000)
# ============================================================
    def do_next_action(self):
        '''Выполняет следующее действие из списка действий'''
        if self.actions:
            self.current_subactions = self.actions.pop(0)    # берем первый список действий
            self.set_current_actions_uid()
            self.do_subactions()
            if not self.actions:    # если действия закончились
                self.signaler.Add( self.signal_name, 1 )
            return True
        else:
            return False    # no more actions
# ============================================================
    def has_more_actions(self):
        if self.actions:
            return True
        else:
            return False
# ============================================================
    def do_subactions(self):
        _is_repeat = False
        if self.prev_uid == self.current_subactions.uid:
            _is_repeat = True
        if self.current_subactions.id not in self.removed_ids:
            for action in self.current_subactions.s_actions:
                if _is_repeat:
                    if action.params:
                        if 'no_repeat' in action.params:
                            if action.params['no_repeat'] == True:
                                continue
                self.do_action(action)
            self.prev_uid = self.current_subactions.uid
# ============================================================
    def on_restore_eff_bonus_start(self, k):
        self.bonus_boost = k
        self.is_bonus_restore_eff_started = True
        #if self.signaler:
        #    self.signaler.SetTimeMul(self.signal_name, self.bonus_boost)

        #_animations = []
        #for key in self.once_animations:
        #    anim_name = self.once_animations[key]
        #    _animations.append(anim_name)
        #    #self.drawer.SetAnimBoost(anim_name, self.bonus_boost*1000)
        #self.set_bonus_boost(self.bonus_boost, _animations)
        self.set_bonus_boost(self.bonus_boost, [''])
        if self.parent:
            #if self.is_boost_allowed
            self.parent.enable_generation_boost(True)
# ============================================================
    def on_restore_eff_bonus_stop(self):
        self.bonus_boost = 1
        self.is_bonus_restore_eff_started = False
        #if self.signaler:
        #    self.signaler.SetTimeMul(self.signal_name, self.bonus_boost)    # как только закончился бонус - сбросим

        #_animations = []
        #for key in self.once_animations:
        #    anim_name = self.once_animations[key]
        #    _animations.append(anim_name)
        #    #self.drawer.SetAnimBoost(anim_name, 1000)
        #self.set_bonus_boost(self.bonus_boost, _animations)
        self.set_bonus_boost(self.bonus_boost, [''])
# ============================================================
    def do_action(self, action):
        '''Выполняет одно простое действие'''
        name = action.name
        params = action.params
        if name == 'show':
            self.drawer.ShowImages(params.images, True)
            images = []
            if istype(params.images, 'list'): images = [img for img in params.images]
            else: images = [params.images]
            if self.is_boost_allowed:
                for img in images:
                    if img not in self.skip_boost_animations:
                        self.once_animations[img] = 1
                        if self.is_bonus_restore_eff_started:
                            self.drawer.SetAnimBoost(img, self.bonus_boost*1000)
        elif name == 'hide':
            self.drawer.ShowImages(params.images, False)
        elif name == 'reset_anim_time':    # сбрасывает время анимации в C++
            self.drawer.ResetAnimTime(params.images)
        elif name == 'set_frame':
            self.drawer.SetFrame(params.image, params.frame)
        elif name == 'stop_animation':
            self.drawer.StopAnimation(params.image, params.time)
        elif name == 'start_once_animation':
            if 'time' in params:
                self.drawer.PlayAnimationOnce(params.image, params.time)
            else:
                self.drawer.PlayAnimationOnce(params.image, '')
            if params.image not in self.skip_boost_animations:
                self.once_animations[params.image] = 1
                if self.is_boost_allowed:
                    if self.is_bonus_restore_eff_started:
                        self.drawer.SetAnimBoost(params.image, self.bonus_boost*1000)
        elif name == 'set_event':
            self.drawer.SetAnimationEvent(params.image, params.percent, params.event)
        elif name == 'restart_signal':
            if self.signaler:
                self.signaler.Restart(params.signal, params.time)
        elif name == 'pause_boost':
            self.is_boost_allowed = False
            self.set_bonus_boost(1, [''])
            if self.parent:
                self.parent.enable_generation_boost(False)
        elif name == 'resume_boost':
            self.is_boost_allowed = True
            self.set_bonus_boost(self.bonus_boost, [''])
            if self.parent:
                self.parent.enable_generation_boost(True)
        elif name == 'remove_actions':
            _id = params.id
            self.removed_ids.append(_id)
            _to_remove = []
            for subactions in self.actions:
                if subactions.id == _id:
                    _to_remove.append(subactions)
            for s in _to_remove:
                self.actions.remove(s)
# ============================================================
# ============================================================
