# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes
import task_attack_barb


# ============================================================
# ============================================================
# ============================================================
class Nest(base_diff_item.DiffItem):
    '''Гнездо варваров'''

    barb_in_groups = ['barb_in'+str(i) for i in range(1,9)]
    barb_out_groups = ['barb_in'+str(i) for i in range(1,9)]

    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'Nest'
        self.action_r = self.GetAttrInt('action_r')
        self.barbarians = []        # варвары в подчинении гнезда
        self.time_from_last_return = 0
        self.gen_time_min = self.GetAttrInt('gen_time_min')
        self.gen_time_max = self.GetAttrInt('gen_time_max')
        self.gen_time_first = self.GetAttrInt('gen_time_first')
        self.cur_time = 0
        self.generate_time = 0
        self.task = None
        #self.generate_time = tools.Rand(self.gen_time_min, self.gen_time_max)
        self.generate_time = self.gen_time_first
        self.last_index = -1
        self.selected_building = None
        self.barbarian_out_anim = ''
        self.barbarian_in_anim = ''
        self.barbarian_embrio = None
        self.barbarian_template_name = self.GetAttrStr('barb_template_name', 'barb1')
        self.snd_appear = self.GetAttrStr('snd_appear', 'SND_BARB_APPEAR_VOICE')
        self.cells = [self.GetAttrCellFixed('barb_cell' + str(i)) for i in range(1,9)]
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        self.switch_animation()
# ============================================================
    def get_subtask(self, parent_task, actor):
        return None
# ============================================================
    # @override
    #def on_signal(self, signal):
    #    base_diff_item.DiffItem.on_signal(self, signal)
# ============================================================
    # @override
    def on_message(self, msg, p1, p2, extra):
        if msg == 'barb_out_complete':
            self.Drawer.StopAnimation(self.barbarian_out_anim, 100)
            self.barbarian_out_anim = ''
            self.switch_animation()
            self.start_barbarian()
        elif msg == 'barb_in_complete':
            self.Drawer.StopAnimation(self.barbarian_in_anim, 100)
            self.barbarian_in_anim = ''
            self.switch_animation()
# ============================================================
    # @override
    def switch_animation(self):
        drawer = self.Drawer
        drawer.ShowImages('', False)
        drawer.ShowImages(['floor','icons'])
        image_category = 'default'
        drawer.ShowImages(image_category, True)
        drawer.ShowImages('select', self.is_mouse_over and not self.is_deactivated)
        check_frame = 0
        if self.need_show_uncheck(): check_frame = 1
        drawer.SetFrame('check', check_frame)
        drawer.ShowImages('check', self.has_check)
        drawer.ShowImages('arrow', self.arrow_shown)
        drawer.ShowImages('clock_wait', self.clock_shown)
        # анимации входящего/выходящего дикаря:
        if self.barbarian_out_anim == '':
            drawer.ShowImages(self.barb_out_groups, False)
        else: drawer.ShowImages(self.barbarian_out_anim, True)
        if self.barbarian_in_anim == '':
            drawer.ShowImages(self.barb_in_groups, False)
        else: drawer.ShowImages(self.barbarian_in_anim, True)
# ============================================================
    #@override
    def scale_bump(self):
        _pos = self.GetAttrPoint('center_item_pos')
        self.Drawer.ScaleBump(['default','cover'], _pos, 200, 1.10)
# ============================================================
    def play_hit_sound(self):
        self.PlaySound('SND_GRAVE_HIT')
# ============================================================
    def on_rare_update(self, ms):
        _barb_count = len(self.barbarians)
        if _barb_count > 0:     # один варвар уже вышел
            return
        if self.is_deactivated:
            return
        _final_complete = UStats.GetLevelValue('final_quest_complete') > 0
        if _final_complete:    # после финального квеста уже не нападаем
            return
        _has_attack = UStats.GetLevelValue('barbarian_attack') > 0
        if _has_attack:    # не генерим дикарей, пока есть другие дикари
            return
        _has_enemy = UStats.GetLevelValue('enemy_go') > 0
        if _has_enemy:    # не генерим дикарей, пока есть враг на экране
            return
        if BMgr.IsSwitchingLevels():    # если уровни уже переключаются, то атаку не стартуем
            return
        _cur_map_index = BMgr.GetCurMapIndex()
        map_index = self.get('locind')
        if _cur_map_index != map_index:
            return
        _has_columns = BMgr.has_column(map_index)
        if _has_columns:
            return
        # - проверить доступность гнезда, если недоступно, то скипуем
        _has_access = self.has_access()
        if not _has_access:
            return
        #_last_gen_time = self.generate_time
        #_last_cur_time = self.cur_time
        self.cur_time += ms
        if self.cur_time >= self.generate_time:
            self.cur_time = 0
            self.generate_time = tools.Rand(self.gen_time_min, self.gen_time_max)
            # - проверить наличие зданий, доступных для атаки
            buildings = BMgr.get_nest_attack_buildings(self, 'action_cell', self.action_r)
            _b_count = len(buildings)
            if _b_count == 0:   # нет доступных зданий
                #self.generate_time = _last_gen_time
                #self.cur_time = _last_cur_time
                return
            else:
                self.generate_barbarian(buildings[0])
# ============================================================
    def generate_barbarian(self, building):
        UStats.AddLevelValue('barbarian_attack', 1) # таким хитрым образом будем проверять наличие атаки варваров
        index = tools.Rand(0,7)
        self.last_index = index
        _cell = self.GetAttrCellFixed('barb_cell' + str(index+1))
        self.selected_building = building
        self.barbarian_out_anim = 'barb_out' + str(index+1)
        #self.Drawer.ShowImages(self.barbarian_out_anim)
        self.Drawer.StopAnimation(self.barbarian_out_anim, 0)
        self.Drawer.PlayAnimationOnce(self.barbarian_out_anim, 'barb_out_complete')
        self.PlaySound(self.snd_appear)
        #self.start_barbarian()

        _cell = self.GetAttrCellFixed('barb_cell' + str(self.last_index+1))
        barbarian = UMgr.create_unit(self.get('locind'), self.barbarian_template_name)
        barbarian.home = self
        _dir = self.last_index - 2
        if _dir > 7: _dir -= 8
        if _dir < 0: _dir += 8
        barbarian.SetDirection(_dir)
        self.barbarians.append(barbarian)
        barbarian.is_in_home = False
        #barbarian.current_subtype = 'barbarian'
        barbarian.hide()
        barbarian.set('cell', _cell)   # обязательно поставим координату "выхода юнита" - от нее будет считать расстояние
        #barbarian.show()
        #TMgr.add_task( task_attack_barb.AttackBarbarianTask(self, barbarian, self.selected_building) )
        #self.selected_building = None   # чтобы освободить ссылку
        self.barbarian_embrio = barbarian
        _embrio_uid = self.barbarian_embrio.get('uid')
        _image_name = self.barbarian_embrio.GetAttrStr('image_name', 'IMG_I_CAVEMAN_BG')
        BMgr.SendGlobalMsg('barbarian_attack', _embrio_uid)
        EMgr.send_event('barbarian_attack_started', self.barbarian_embrio)
        if self.barbarian_template_name == 'raptor1':
            EMgr.send_event('need_raptor_text', self.barbarian_embrio)
        else:
            EMgr.send_event('need_barbarian_text', self.barbarian_embrio)
# ============================================================
    def start_barbarian(self):
        #_cell = self.GetAttrCellFixed('barb_cell' + str(self.last_index+1))
        #barbarian = UMgr.create_unit('barb1')
        #barbarian.home = self
        #_dir = self.last_index - 2
        #if _dir > 7: _dir -= 8
        #if _dir < 0: _dir += 8
        #barbarian.SetDirection(_dir)
        #self.barbarians.append(barbarian)
        #barbarian.is_in_home = False
        # #barbarian.current_subtype = 'barbarian'
        #barbarian.set("cell", _cell)   # обязательно поставим координату "выхода юнита" - от нее будет считать расстояние
        self.barbarian_embrio.show()
        _need_barb_arrow = LBoard.Core.need_barbarian_arrow()
        _barb_uid = self.barbarian_embrio.get('uid')
        if _need_barb_arrow:
            self.barbarian_embrio.is_first = True
            self.barbarian_embrio.switch_animation()
            LBoard.MoveCameraToUID(_barb_uid, True)
        TMgr.add_task( task_attack_barb.AttackBarbarianTask(self, self.barbarian_embrio, self.selected_building) )
        _image_name = self.barbarian_embrio.GetAttrStr('image_name', 'IMG_I_CAVEMAN_BG')
        self.barbarian_embrio = None    # больше не нужен
        self.selected_building = None   # чтобы освободить ссылку
        EMgr.SendGlobalMsg('barbarian_out')
        UI.set_face_bar_info(_image_name + ',0', _barb_uid)
        UI.show_face_bar(True)
# ============================================================
    def get_cells(self):
        return self.cells
# ============================================================
    def hide_barbarian(self, barb):
        barb.hide()
        self.barbarians = []
        BMgr.RemoveItemFromMap(barb)
        self.barbarian_in_anim = 'barb_in' + str(barb.cell_index)
        self.Drawer.StopAnimation(self.barbarian_in_anim, 0)
        self.Drawer.PlayAnimationOnce(self.barbarian_in_anim, 'barb_in_complete')
# ============================================================
# #######################################################
# #######################################################
# #######################################################
