# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes


class ChopBushSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'ChopBushSubtask'
        self.actor_action_animation = 'hit'
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        #item.stop_attention_signal()
        self.actor.stop_walk()
        self.actor.action_hit_item(item)     # генерал рубит куст
        return True
# ============================================================
class GatherBushSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'GatherBushSubtask'
        self.actor_action_animation = 'gather'
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        item.stop_attention_signal()        # не показываем эффект когда идет сбор ягод
        self.actor.stop_walk()
        self.actor.action_hit_item(item)
        return True
# ============================================================
# ============================================================
class Bush(base_diff_item.DiffItem):
    '''Куст'''
    def __init__(self, cdata):
        self.has_berries = False
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'Bush'
        self.has_berries = self.GetAttrBool('berries_at_start')
        self.can_chop = self.GetAttrBool('can_chop')
        if not self.has_berries:
            self.start_berries_generation()
        else:
            self.add_attention_signal(5000, 12000)
        self.task = None
        self.need_attention_effect = False
        self.shear_r = self.GetAttrInt('shear_r', 50)
        self.shear_shift = self.GetAttrInt('shear_shift', 5)
        self.shear_disabled = self.GetAttrBool('shear_disabled')
        if not self.shear_disabled:
            self.Drawer.SetTreeMode(['default','growth'], self.shear_r, self.shear_shift)
# ============================================================
    # @override
    def is_action_complete(self):
        _state = self.FSM.GetStateID()
        return not self.has_berries or (_state == 'complete')   # проверка на ягоды на случай, если на старте у куста нет ягод
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        #self.action_hits_last = self.GetAttrInt('action_hits_count')   # уже после смены варианта
        self.action_hits_last = 0   # больше бить не нужно
        # отдать ягоды и запустить генерацию снова
        if self.can_chop:
            self.remove_me_later()
            self.switch_animation()
            effect_cut_name = self.GetAttrStr('effect_cut_name')
            if effect_cut_name:
                self.AddEffect(effect_cut_name, self.get('position'))
            EMgr.send_event( 'do_action_complete', self )
            EMgr.send_event( 'chop_bush', self )
            self.send_extra_event()
            #BMgr.RemoveItemFromMap(self)
        else:
            self.task.subtask.actor.resource = self.give_resource()
            EMgr.send_event( 'do_action_complete', self )
            EMgr.send_event( 'gather_bush', self )
# ============================================================
    def on_message(self, msg_id, p1, p2, msg_dict):
        if msg_id == 'growth_complete':
            self.has_berries = True
            self.FSM.HandleEvent('regenerate_event')
            self.show_born_effect()
            self.add_attention_signal(5000, 12000)
            self.switch_animation()
# ============================================================
    #@override
    def get_action_text(self, for_task_slide = False):
        if self.can_chop:
            return 'DLG_TASK_CUT'
        else:
            return 'DLG_TASK_GATHER'
# ============================================================
    def get_subtask(self, parent_task, actor):
        if self.can_chop:
            return ChopBushSubtask(parent_task, actor)
        elif self.has_berries:
            return GatherBushSubtask(parent_task, actor)
        else:
            return None
# ============================================================
    def give_resource(self):
        '''Вызывается в момент, когда юнит забирает ресурс у куста. Скрывает картинку ягод и начинает новую цепочку производства'''
        self.has_berries = False
        self.stop_attention_signal()
        self.switch_animation()
        _gives = self.GetRes('gives')
        self.start_berries_generation()
        return _gives
# ============================================================
    #@override
    def start_berries_generation(self):
        self.action_hits_last = self.GetAttrInt('action_hits_count')
        _growth_time = self.GetAttrInt('generate_time')
        self.Drawer.PlayAnimationOnce('growth', 'growth_complete', '', -1, '', _growth_time)
# ============================================================
    #@override
    def switch_animation(self):
        drawer = self.Drawer
        drawer.ShowImages('', False)
        if not self.shear_disabled:
            drawer.SetTreeMode(['default','growth'], self.shear_r, self.shear_shift)
        drawer.ShowImages('default', self.has_berries)
        drawer.ShowImages('growth', not self.has_berries)
        _state = self.FSM.GetStateID()
        drawer.ShowImages('select', self.is_mouse_over and not self.is_deactivated and _state == 'normal')
        check_frame = 0
        if self.need_show_uncheck(): check_frame = 1
        drawer.SetFrame('check', check_frame)
        drawer.ShowImages('check', self.has_check)
        drawer.ShowImages('arrow', self.arrow_shown)
        drawer.ShowImages('clock_wait', self.clock_shown)
        drawer.ShowImages('attention', self.need_attention_effect)
        drawer.ShowImages('born', self.need_born_effect)
# ============================================================
    #@override
    def scale_bump(self):
        _pos = self.GetAttrPoint('center_item_pos')
        self.Drawer.ScaleBump(self.bump_layers, _pos, 200, 1.10)
# ============================================================
    #@override
    def is_active(self):
        if not self.has_berries:
            return False
        else:
            return base_diff_item.DiffItem.is_active(self)
# ============================================================
    #@override
    def get_item_name(self):
        if self.has_berries and not self.can_chop and not self.non_selectable:
            return 'BUSH_G_READY'
        else:
            return self.GetAttrStr("item_name")
# ============================================================
    def play_hit_sound(self):
        if self.can_chop:
            self.PlaySound('SND_CHOP_BUSH')
        else:
            self.PlaySound('SND_GATHER_BUSH')
# ============================================================
    def play_complete_sound(self):
        if self.can_chop:
            self.PlaySound('SND_CHOP_BUSH_DISAPPEAR')
# ============================================================
# #######################################################
# #######################################################
# #######################################################
