# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes


class OpenChestSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'OpenChestSubtask'
        self.actor_action_animation = 'hit'
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        self.actor.action_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        return True
# ============================================================
# ============================================================
# ============================================================
class Chest(base_diff_item.DiffItem):
    '''Горшок'''
    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'Chest'
        self.task = None
        self.is_opened = False
        _cur_variant_name = self.get("variant_name")
        if _cur_variant_name not in self.variants:
            self.is_opened = True
        self.content_items = []
        self.snd_complete = self.GetAttrStr('snd_complete', 'SND_CHEST_OPEN')
        self.snd_hit = self.GetAttrStr('snd_hit', 'SND_CHEST_HIT')
# ============================================================
    variants = {
        'closed_1' : 'opened_1',
        'closed_2' : 'opened_2'
    }
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        _cur_variant_name = self.get('variant_name')
        _content = self.get_unpacked_content( self.GetAttrStr('content') )  # строго до смены варианта
        #self.publish_contents(_content)
        self.content_items = self.publish_contents_by_one(_content)
        _time = 0
        for item in self.content_items:
            self.Signals.Add('show_item', _time)
            _time += 500
        self.switch_variant( self.variants[_cur_variant_name] ) # переключаемся на открытый вариант
        self.is_opened = True
        self.action_hits_last = self.GetAttrInt('action_hits_count')   # уже после смены варианта
        chest_info = {'type':self.get('type'), 'subtype':self.get('subtype')}
        EMgr.send_event( 'chest_opened', self, chest_info )
        self.deactivate()
        self.SetMouseActive(False)
        self.send_extra_event()
        self.switch_animation()
# ============================================================
    def on_signal(self, signal):
        if signal == 'show_item':
            if self.content_items:
                item = self.content_items.pop(0)
                _pos = item.GetAttrPoint('center_item_pos')
                item.Drawer.Hide(False)
                item.Drawer.ResIn(item.item_layers, _pos, 1000)
                item.play_content_sound(self)
        else:
            base_diff_item.DiffItem.on_signal(self, signal)
# ============================================================
    def get_subtask(self, parent_task, actor):
        if not self.is_opened:
            if self.key_need:
                _has_key = self.has_key()
                if _has_key:
                    return base_diff_item.OpenKeySubtask(parent_task, actor)
                else: return None
            return OpenChestSubtask(parent_task, actor)
        else: return None
# ============================================================
    #@override
    #def on_signal(self, signal):
    #    if signal == 'offscreen':
    #        if self.is_opened:
    #            BMgr.RemoveItemFromMap(self)    # удалим сундук нафиг
    #            return
    #    else:
    #        base_diff_item.DiffItem.on_signal(self, signal)
# ============================================================
    #@override
    def switch_animation(self):
        drawer = self.Drawer
        drawer.ShowImages('', False)
        drawer.ShowImages(['floor','icons'])
        image_category = 'default'
        drawer.ShowImages(image_category, True)
        _state = self.FSM.GetStateID()
        drawer.ShowImages('select', self.is_mouse_over and not self.is_deactivated and _state == 'normal')
        check_frame = 0
        if self.need_show_uncheck(): check_frame = 1
        drawer.SetFrame('check', check_frame)
        drawer.ShowImages('check', self.has_check)
        drawer.ShowImages('arrow', self.arrow_shown)
        drawer.ShowImages('clock_wait', self.clock_shown)
# ============================================================
    def play_hit_sound(self):
        self.PlaySound(self.snd_hit)
# ============================================================
    def play_complete_sound(self):
        self.PlaySound(self.snd_complete)
# ============================================================
# #######################################################
# #######################################################
# #######################################################
