# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes

# ============================================================
# ============================================================
class EnterInSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None, enter = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'EnterInSubtask'
        self.enter = enter
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        _teleport_result = self.enter.can_teleport()
        if _teleport_result != 'TELEPORT':
            if self.parent_task:
                self.parent_task.cancel()
            return False
        #s elf.a ctor.a ction_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        subindex = item.GetAttrInt('sublevel')
        if subindex >= 0:
            LBoard.CancelBuildMode()    # отключим режим строительства и диалог магазина
            item.start_transition()
            LBoard.OpenSublevel( subindex )
            self.actor.task.subtask.on_action_complete()
            TMgr.cancel_general_tasks()
            QMgr.stop_quests_generation()
            item.FSM.HandleEvent('complete_event', True)
            item.FSM.HandleEvent('regenerate_event')
        return True
# ============================================================
class EnterTransferUnitsSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None, enter = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'EnterTransferUnitsSubtask'
        self.enter = enter
    # ============================================================
    def start_action(self, item):
        self.actor.stop_walk()
        subindex = item.GetAttrInt('sublevel')
        if subindex >= 0:
            _dest_index = self.enter.GetAttrInt('sublevel')
            _target_uid = self.enter.GetAttrStr('target_uid')
            if _target_uid != '-1':
                BMgr.transfer_unit(self.actor, _dest_index)
                _target = BMgr.get_item(_target_uid, _dest_index)
                if _target:
                    _cell = _target.GetAttrCellFixed('action_cell')    # нужно получить клетку target_uid:_dest_index
                    self.actor.set('cell', _cell)
                    UMgr.add_free_unit(self.actor)
            else:
                BMgr.RemoveItemFromMap(self.actor)
            self.actor.task.subtask.on_action_complete()
        return True
# ============================================================
# ============================================================
# ============================================================
#@statemachine
class Enter(base_diff_item.DiffItem):
    '''Вход в подгружаемый уровень'''
    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'Enter'
        self.task = None
        #self.units_tasks = []
        self.button_shown = True
        # dirty hack for arrow color:
        _rnd_time = tools.Rand(450, 550)
        self.Signals.Restart('refresh_state', _rnd_time)
# ============================================================
    def on_create(self):
        base_diff_item.DiffItem.on_create(self)
        self.Get_tip_image()
# ============================================================
    def get_subtask(self, parent_task, actor):
        if actor.IsSubTypeOf('general'):
            _allow_teleport = self.can_teleport()
            if _allow_teleport == 'TELEPORT':
                return EnterInSubtask(parent_task, actor, self)
            else:
                return None
        else:
            return EnterTransferUnitsSubtask(parent_task, actor, self)
# ============================================================
    def on_signal(self, signal):
        if signal == 'refresh_state':
            _rnd_time = tools.Rand(450, 550)
            self.Signals.Restart('refresh_state', _rnd_time)
            # нужно обновить состояние кнопок и, возможно, типса
            btn_frame = self.Get_button_frame()
            if btn_frame >= 0:
                self.Drawer.SetFrame('button', btn_frame)
                self.Drawer.ShowImages('button')
            else:
                self.Drawer.ShowImages('button', False)
            # dirty hack for arrow color:
            self.Get_tip_image()
        elif signal == 'show_button':
            self.button_shown = True
            self.switch_animation()
        elif signal == 'got_access':
            LBoard.Core.refresh_nighbors_exits()
        elif signal == 'no_access':
            LBoard.Core.refresh_nighbors_exits()
        else:
            base_diff_item.DiffItem.on_signal(self, signal)
# ============================================================
    #@override
    def get_action_cell_name(self):
        '''Метод возвращает название ячейки к которой будет подбегать юнит'''
        _cell_names = []
        for i in range(1,9):
            _name = 'action_cell' + str(i)
            _has_attr = self.HasAttr(_name)
            if _has_attr:
                _cell_names.append(_name)
        _cell_names = self._filter_available_cells(_cell_names)
        _len = len(_cell_names)
        if _len > 0:
            _rnd = tools.Rand(0, _len-1)
            return _cell_names[_rnd]
        else:
            return 'action_cell'
# ============================================================
    def get_first_free_action_cell_name(self):
        '''Метод возвращает название ячейки к которой будет подбегать юнит'''
        _cell_names = []
        for i in range(1,9):
            _name = 'action_cell' + str(i)
            _has_attr = self.HasAttr(_name)
            if _has_attr:
                _cell_names.append(_name)
        _cell_names = self._filter_available_cells(_cell_names)
        _len = len(_cell_names)
        if _len > 0:
            return _cell_names[0]
        else:
            return 'action_cell'
# ============================================================
    #@override
    def switch_animation(self):
        drawer = self.Drawer
        drawer.ShowImages('', False)
        drawer.ShowImages(['floor','icons'])
        image_category = 'default'
        drawer.ShowImages(image_category, True)
        drawer.ShowImages('select', self.is_mouse_over and not self.is_deactivated)
        #check_frame = 0
        #_need_show_uncheck = self.need_show_uncheck()
        #if _need_show_uncheck: check_frame = 1
        #drawer.SetFrame('check', check_frame)
        #drawer.ShowImages('check', self.has_check)
        drawer.ShowImages('check', False)
        drawer.ShowImages('arrow', self.arrow_shown)
        btn_frame = self.Get_button_frame()
        drawer.SetFrame('button', btn_frame)
        drawer.ShowImages('button', btn_frame >= 0)
# ============================================================
# ============================================================
    def can_teleport(self):
        if self.is_deactivated:
            return 'DEACTIVATED'
        _partial_fog = self.Drawer.IsPartiallyInFog()
        if _partial_fog:
            return 'FOG'
        _has_access = self.has_access()
        if not _has_access:
            return 'ACCESS'
        _awiting_comet = UStats.GetLevelValue('GEM_COMET') > 0
        if _awiting_comet:
            return 'COMET'
        result = 'TELEPORT'
        _barb_attack_count = UStats.GetLevelValue('barbarian_attack')
        if _barb_attack_count > 0:
           result = 'ATTACK'
        _has_enemy = UStats.GetLevelValue('enemy_go')
        if _has_enemy > 0:
            result = 'ATTACK'       # просто чтобы что-то было
        _has_started_quest = QMgr.has_started_nonartefact_quests()
        if _has_started_quest:
            result = 'QUEST'
        if self.key_need:
            _has_key = self.has_key()
            if not _has_key:
                result = 'KEY'
        #if QMgr.IsSwitchingLevels():
        #    result = 'SWITCHING'
        return result
# ============================================================
    _ATTACK__QUEST__ACCESS__COMET__KEY__SWITCHING = ['ATTACK', 'QUEST', 'ACCESS', 'COMET', 'KEY', 'SWITCHING']
    def get_raw_button_frame(self):
        '''Возвращает номер кадра кнопки, без учета положения курсора мыши'''
        if self.is_deactivated:
            return -1
        _partial_fog = self.Drawer.IsPartiallyInFog()
        if _partial_fog:
            return -1
        if not self.button_shown:
            return -1
        frame = 0
        _teleport_state = self.can_teleport()
        if _teleport_state in self._ATTACK__QUEST__ACCESS__COMET__KEY__SWITCHING:
            frame = 2
        if self.has_check:  # если на объект назначено действие
            frame = 3
        # - проверка на то, что есть атакующие варвары
        # - проверка на запущенные квесты (кроме квеста на артефакт)
        # - еще должна быть проверка на уровень силы/ораторства...
            # - еще нужно, чтобы перед входом в пещеру - была проверка на запрещающие условия (но это уже не здесь)
        return frame
# ============================================================
    button_active_frames = [0,3]
    cancel_action_frame = 3
    blocked_frame = 2

    def is_cancel_button_mode(self):
        if self.is_mouse_over:
            frame = self.get_raw_button_frame()
            if frame in self.button_active_frames: # подсветка кнопки при наведении
                if frame == self.cancel_action_frame:     # если кнопка отмены
                    if self.can_show_uncheck:
                        return True
        return False
# ============================================================
    def Get_button_frame(self):
        frame = self.get_raw_button_frame()
        if frame >= 0:
            if self.is_mouse_over:
                if frame in self.button_active_frames: # подсветка кнопки при наведении
                    if frame == self.cancel_action_frame or self.has_check:     # для отмены подсвечиваем только над кнопкой
                        if self.can_show_uncheck:
                            if self.is_mouse_over_zone == 'button':
                                frame = 4
                    elif frame == 0:
                        frame = 1
        if frame < 0:
            self.Drawer.ShowImages('button', False)
        else:
            self.Drawer.ShowImages('button')
        return frame
# ============================================================
# ============================================================
    def check_button_only_selection(self):
        if self.is_cancel_button_mode():
            if self.is_mouse_over_zone != 'button':
                self.Drawer.HighlightImages(self.hlight_layers, False)
                self.Drawer.ShowImages('select', False)
            else:
                self.Drawer.HighlightImages(self.hlight_layers, False)
                self.Drawer.ShowImages('select', False)
# ============================================================
    def on_mouse_in(self, over_zone):
        _prev_zone = self.is_mouse_over_zone
        base_diff_item.DiffItem.on_mouse_in(self, over_zone)
        _is_active = self.is_active()
        _has_access = self.has_access()
        if _is_active:
            if _has_access:
                if over_zone != 'button' and _prev_zone == 'button':
                    self.can_show_uncheck = True
                self.check_button_only_selection()
                btn_frame = self.Get_button_frame()
                if btn_frame >= 0:
                    self.Drawer.SetFrame('button', btn_frame)
                self.is_selected = True
                self.Drawer.ShowImages('select')
# ============================================================
    def on_mouse_out(self):
        base_diff_item.DiffItem.on_mouse_out(self)
        self.can_show_uncheck = True
        btn_frame = self.Get_button_frame()
        if btn_frame >= 0:
            self.Drawer.SetFrame('button', btn_frame)
        self.Drawer.ShowImages('select', False)
# ============================================================
    def on_click(self, btn_name):
        if self.is_deactivated:
            return
        _partial_fog = self.Drawer.IsPartiallyInFog()
        if _partial_fog:
            return
        _allow_teleport = self.can_teleport()
        if _allow_teleport != 'TELEPORT':
            if not self.has_check:
                LBoard.CrossNoToLastClick()
                return
        if self.has_check:                  # если отображена галка
            #if not self.can_show_uncheck:   # но отменять нельзя
            #    return
            if self.is_mouse_over_zone != 'button':   # или клик не по кнопке
                return
        base_diff_item.DiffItem.on_click(self, btn_name)
# ============================================================
    #@override
    def Get_tip_image(self):
        '''Возвращает ID изображения, которое используется для бэка типса'''
        # чтобы обновить кадр селекта:
        btn_frame = self.Get_button_frame()
        _frame = 0
        _is_green_state = self.is_green_state()
        if not _is_green_state: _frame = 1
        if btn_frame == self.blocked_frame: # заблокировано
            _frame = 1
        self.Drawer.SetFrame('select', _frame)
        self.Drawer.SetSequence('arrow', _frame)    # номер последовательности сейчас совпадает с номером кадра
        # ---
        if _is_green_state:
            if btn_frame != self.blocked_frame:
                return 'IMG_I_TIP_BG'
        return 'IMG_I_TIP_BG_RED'
# ============================================================
    #@override
    def _get_tips_extra(self, place, result, by_click):
        '''Предназначается для переопределения в потомках
        place: метка позиции вставки типса
        result: список значений, в который нужно добавлять данные
        by_click: типс по клику или нет
        returns: False - для завершения типса, т.е. после него ни одной строки не прибавится'''
        if place == 'end':
            _teleport_state = self.can_teleport()
            if _teleport_state == 'QUEST':
                result.append('newline')
                result.append( ['text', 'ENTER_COMPLETE_QUEST_FIRST'] )
            elif _teleport_state == 'ATTACK':
                result.append('newline')
                result.append( ['text', 'ENTER_BARBARIAN_ATTACK'] )
            elif _teleport_state != 'TELEPORT':
                return False
        elif place == 'before_name':
            if self.is_deactivated:
                #result.append( ['text', 'COLOR_LINES'] )
                #result.append( ['text', 'NOT_ACCESSIBLE_YET'] )
                return False
            if by_click:
                _teleport_state = self.can_teleport()
                if _teleport_state == 'QUEST':
                    result.append( ['text', 'COLOR_RED'] )
                    result.append( ['text', 'ENTER_COMPLETE_QUEST_FIRST'] )
                    return False
                elif _teleport_state == 'ATTACK':
                    result.append( ['text', 'COLOR_RED'] )
                    result.append( ['text', 'ENTER_BARBARIAN_ATTACK'] )
                    return False
                elif _teleport_state == 'ACCESS':
                    result.append( ['text', 'COLOR_RED'] )
                    result.append( ['text', 'NO_PATH'] )
                    return False
                elif _teleport_state == 'DEACTIVATED':
                    result.append( ('text', 'COLOR_LINES') )
                    result.append( ['text', 'NOT_ACCESSIBLE_YET'] )
                    return False
                elif _teleport_state != 'TELEPORT':
                    return False
        elif place == 'before_res':
            if self.is_deactivated:
                result.append( ('text', 'COLOR_LINES') )
                result.append( ('text', 'NOT_ACCESSIBLE_YET') )
                return False
        return True
# ============================================================
    def start_transition(self):
        '''На пару секунд прячет кнопку, чтобы она не поменяла свой вид при переходе'''
        self.button_shown = False
        self.switch_animation()
        self.Signals.Restart('show_button', 1000)
# ============================================================
# #######################################################
# #######################################################
