# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes


class OpenGraveSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'OpenGraveSubtask'
        self.actor_action_animation = 'hit'
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        self.actor.action_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        return True
# ============================================================
# ============================================================
#@statemachine
class GraveInSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None, enter = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'GraveInSubtask'
        self.enter = enter
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        _teleport_result = self.enter.can_teleport()
        if _teleport_result != 'TELEPORT':
            if self.parent_task:
                self.parent_task.cancel()
            return False
        #s elf.a ctor.a ction_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        subindex = item.GetAttrInt('sublevel')
        if subindex >= 0:
            LBoard.CancelBuildMode()    # отключим режим строительства и диалог магазина
            item.start_transition()
            LBoard.OpenSublevel( subindex )
            self.actor.task.subtask.on_action_complete()
            TMgr.cancel_general_tasks()
            item.FSM.HandleEvent('complete_event', True)
            item.FSM.HandleEvent('regenerate_event')
        return True
# ============================================================
class GraveTransferUnitsSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None, enter = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'GraveTransferUnitsSubtask'
        self.enter = enter
    # ============================================================
    def start_action(self, item):
        self.actor.stop_walk()
        subindex = item.GetAttrInt('sublevel')
        if subindex >= 0:
            _target_uid = self.enter.GetAttrStr('target_uid')
            _dest_index = self.enter.GetAttrInt('sublevel')
            if _target_uid != '-1':
                BMgr.transfer_unit(self.actor, _dest_index)
                _target = BMgr.get_item(self.enter.GetAttrStr('target_uid'), _dest_index)
                if _target:
                    _cell = _target.get('cell')    # нужно получить клетку target_uid:_dest_index
                    self.actor.set('cell', _cell)
                    UMgr.add_free_unit(self.actor)
            else:
                BMgr.RemoveItemFromMap(self.actor)
            self.actor.task.subtask.on_action_complete()
            # - после этого вызвать у юнита go_home() или UMgr.add_free_unit
        return True
# ============================================================
# ============================================================
# ============================================================
class Grave(base_diff_item.DiffItem):
    '''Гробница-вход'''
    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'Grave'
        self.task = None
        self.is_opened = self.GetAttrBool('is_opened')
        if self.is_opened:
            self.need_attention_effect = False
        else:
            _rand_blink_time = tools.Rand(400, 600)
            self.Signals.Restart('blink_closed', _rand_blink_time)
        self.button_shown = True
        # dirty hack for arrow color:
        _rnd_time = tools.Rand(450, 550)
        self.Signals.Restart('refresh_state', _rnd_time)
        #@_cur_variant_name = self.get("variant_name")
        #@if _cur_variant_name not in self.variants:
        #@    self.is_opened = True
# ============================================================
    #@override
    def on_create(self):
        base_diff_item.DiffItem.on_create(self)
        self.Get_tip_image()
        self.add_attention_signal()
# ============================================================
    #@variants = {
    #@    'closed_1' : 'opened_1',
    #@    'closed_2' : 'opened_2'
    #@}
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        if not self.is_opened:
            _content = self.get_unpacked_content( self.GetAttrStr('content') )  # строго до смены варианта
            self.publish_contents(_content)
            #_cur_variant_name = self.get("variant_name")
            #self.switch_variant( self.variants[_cur_variant_name] ) # переключаемся на открытый вариант
            self.is_opened = True
            self.action_hits_last = 0   #self.GetAttrInt('action_hits_count')   # уже после смены варианта
            chest_info = {'type':self.get('type'), 'subtype':self.get('subtype')}
            EMgr.send_event( 'chest_opened', self, chest_info )
            self.PlaySound('SND_GRAVE_CRASH')
            self.send_extra_event()
            if self.is_opened:
                self.need_attention_effect = False
                #self.switch_animation()
        #self.deactivate()
        #self.SetMouseActive(False)
        self.switch_animation()
# ============================================================
    def action_hit_anim_complete(self):
        _state = self.FSM.GetStateID()
        if _state == 'complete':
            self.FSM.HandleEvent('regenerate_event')
# ============================================================
    def get_subtask(self, parent_task, actor):
        if actor.IsSubTypeOf('general'):
            if not self.is_opened:
                if self.key_need:
                    _has_key = self.has_key()
                    if _has_key:
                        return base_diff_item.OpenKeySubtask(parent_task, actor)
                    else: return None
                else:
                    return OpenGraveSubtask(parent_task, actor)
            else:
                _allow_teleport = self.can_teleport()
                if _allow_teleport == 'TELEPORT':
                    return GraveInSubtask(parent_task, actor, self)
                else:
                    return None
        else:
            return GraveTransferUnitsSubtask(parent_task, actor, self)
# ============================================================
    #@override
    def on_signal(self, signal):
        if signal == 'refresh_state':
            _rnd_time = tools.Rand(450, 550)
            self.Signals.Restart('refresh_state', _rnd_time)
            # нужно обновить состояние кнопок и, возможно, типса
            btn_frame = self.Get_button_frame()
            if btn_frame >= 0:
                self.Drawer.SetFrame('button', btn_frame)
                self.Drawer.ShowImages('button', True)
            else:
                self.Drawer.ShowImages('button', False)
            # dirty hack for arrow color:
            self.Get_tip_image()
        elif signal == 'show_button':
            self.button_shown = True
            self.switch_animation()
        elif signal == 'got_access':
            LBoard.Core.refresh_nighbors_exits()
        elif signal == 'no_access':
            LBoard.Core.refresh_nighbors_exits()
        elif signal == 'blink_closed':
            if not self.is_opened:
                if not self.is_deactivated:
                    _state = self.FSM.GetStateID()
                    if _state == 'normal':
                        if not self.has_check:
                            _layers = ['floor','default','cover']
                            self.Drawer.BlinkEx(_layers, 1000, 1)
                self.Signals.Restart('blink_closed', 2000)
        else:
            base_diff_item.DiffItem.on_signal(self, signal)
# ============================================================
    #@override
    def switch_animation(self):
        drawer = self.Drawer
        drawer.ShowImages('', False)
        drawer.ShowImages(['floor','icons'])
        image_category = 'default'
        drawer.ShowImages(image_category, True)
        _state = self.FSM.GetStateID()
        drawer.ShowImages('select', self.is_mouse_over and \
                                        not self.is_deactivated and \
                                        (_state == 'normal' or self.is_opened) )
        check_frame = 0
        _need_show_uncheck = self.need_show_uncheck()
        if _need_show_uncheck: check_frame = 1
        drawer.SetFrame('check', check_frame)
        if not self.is_opened:
            drawer.ShowImages('check', self.has_check)
        drawer.ShowImages('cover', not self.is_opened)
        drawer.ShowImages('arrow', self.arrow_shown)
        drawer.ShowImages('clock_wait', self.clock_shown)
        if self.is_opened:
            btn_frame = self.Get_button_frame()
            drawer.SetFrame('button', btn_frame)
            drawer.ShowImages('button', btn_frame >= 0)
        else: drawer.ShowImages('attention', self.need_attention_effect)
# ============================================================
# ============================================================
    def can_teleport(self):
        if self.is_deactivated:
            return 'DEACTIVATED'
        _partial_fog = self.Drawer.IsPartiallyInFog()
        if _partial_fog:
            return 'FOG'
        _has_access = self.has_access()
        if not _has_access:
            return 'ACCESS'
        _awiting_comet = UStats.GetLevelValue('GEM_COMET') > 0
        if _awiting_comet:
            return 'COMET'
        result = 'TELEPORT'
        _barb_attack_count = UStats.GetLevelValue('barbarian_attack')
        if _barb_attack_count > 0:
           result = 'ATTACK'
        _has_enemy = UStats.GetLevelValue('enemy_go')
        if _has_enemy > 0:
            result = 'ATTACK'       # просто чтобы что-то было
        _has_started_quest = QMgr.has_started_nonartefact_quests()
        if _has_started_quest:
            result = 'QUEST'
        if not self.is_opened:
            if self.key_need:
                _has_key = self.has_key()
                if not _has_key:
                    result = 'KEY'
        #if QMgr.IsSwitchingLevels():
        #    result = 'SWITCHING'
        return result
# ============================================================
    _ATTACK__QUEST__ACCESS__COMET__KEY__SWITCHING = ['ATTACK', 'QUEST', 'ACCESS', 'COMET', 'KEY', 'SWITCHING']
    def get_raw_button_frame(self):
        '''Возвращает номер кадра кнопки, без учета положения курсора мыши'''
        if self.is_deactivated:
            return -1
        _partial_fog = self.Drawer.IsPartiallyInFog()
        if _partial_fog:
            return -1
        if not self.is_opened:
            return -1
        if not self.button_shown:
            return -1
        frame = 0
        _teleport_state = self.can_teleport()
        if _teleport_state in self._ATTACK__QUEST__ACCESS__COMET__KEY__SWITCHING:
            frame = 2
        if self.has_check:  # если на объект назначено действие
            frame = 3
        # - еще должна быть проверка на уровень силы/ораторства...
        return frame
# ============================================================
    button_active_frames = [0,3]
    cancel_action_frame = 3
    blocked_frame = 2

    def is_cancel_button_mode(self):
        if self.is_mouse_over:
            frame = self.get_raw_button_frame()
            if frame in self.button_active_frames: # подсветка кнопки при наведении
                if frame == self.cancel_action_frame:     # если кнопка отмены
                    if self.can_show_uncheck:
                        return True
        return False
# ============================================================
    def Get_button_frame(self):
        frame = self.get_raw_button_frame()
        if frame >= 0:
            if self.is_mouse_over:
                if frame in self.button_active_frames: # подсветка кнопки при наведении
                    if frame == self.cancel_action_frame:# or self.has_check:     # для отмены подсвечиваем только над кнопкой
                        if self.can_show_uncheck:
                            if self.is_mouse_over_zone == 'button':
                                frame = 4
                    elif frame == 0:
                        frame = 1
        #trace('grave>>>', frame)
        if frame < 0:
            self.Drawer.ShowImages('button', False)
        else:
            self.Drawer.ShowImages('button')
        return frame
# ============================================================
# ============================================================
    def check_button_only_selection(self):
        if self.is_cancel_button_mode():
            if self.is_mouse_over_zone != 'button':
                self.Drawer.HighlightImages(self.hlight_layers, False)
                self.Drawer.ShowImages('select', False)
            else:
                self.Drawer.HighlightImages(self.hlight_layers, False)
                self.Drawer.ShowImages('select', False)
# ============================================================
    def on_mouse_in(self, over_zone):
        _prev_zone = self.is_mouse_over_zone
        base_diff_item.DiffItem.on_mouse_in(self, over_zone)
        _is_active = self.is_active()
        _has_access = self.has_access()
        if _is_active:
            if _has_access:
                if over_zone != 'button' and _prev_zone == 'button':
                    self.can_show_uncheck = True
            self.check_button_only_selection()
            btn_frame = self.Get_button_frame()
            if btn_frame >= 0:
                self.Drawer.SetFrame('button', btn_frame)
            self.is_selected = True
            self.Drawer.ShowImages('select')
# ============================================================
    def on_mouse_out(self):
        base_diff_item.DiffItem.on_mouse_out(self)
        self.can_show_uncheck = True
        btn_frame = self.Get_button_frame()
        if btn_frame >= 0:
            self.Drawer.SetFrame('button', btn_frame)
        self.Drawer.ShowImages('select', False)
# ============================================================
    def on_click(self, btn_name):
        if self.is_deactivated:
            return
        _partial_fog = self.Drawer.IsPartiallyInFog()
        if _partial_fog:
            return
        if self.is_opened:
            _allow_teleport = self.can_teleport()
            if _allow_teleport != 'TELEPORT' and not self.has_check:
                return
            if self.has_check:                  # если отображена галка
                #if not self.can_show_uncheck:   # но отменять нельзя
                #    return
                if self.is_mouse_over_zone != 'button':   # или клик не по кнопке
                    return
        base_diff_item.DiffItem.on_click(self, btn_name)
# ============================================================
    #@override
    def Get_tip_image(self):
        '''Возвращает ID изображения, которое используется для бэка типса'''
        # чтобы обновить кадр селекта:
        btn_frame = self.Get_button_frame()
        _frame = 0
        if not self.is_green_state(): _frame = 1
        if btn_frame == self.blocked_frame: # заблокировано
            _frame = 1
        self.Drawer.SetFrame('select', _frame)
        self.Drawer.SetSequence('arrow', _frame)    # номер последовательности сейчас совпадает с номером кадра
        # ---
        _is_green_state = self.is_green_state()
        if _is_green_state and btn_frame != self.blocked_frame:
            return 'IMG_I_TIP_BG'
        return 'IMG_I_TIP_BG_RED'
# ============================================================
    #@override
    def _get_tips_extra(self, place, result, by_click):
        '''Предназначается для переопределения в потомках
        place: метка позиции вставки типса
        result: список значений, в который нужно добавлять данные
        by_click: типс по клику или нет
        returns: False - для завершения типса, т.е. после него ни одной строки не прибавится'''
        if place == 'end':
            if self.is_opened:
                _teleport_state = self.can_teleport()
                if _teleport_state == 'QUEST':
                    result.append('newline')
                    result.append( ['text', 'ENTER_COMPLETE_QUEST_FIRST'] )
                elif _teleport_state == 'ATTACK':
                    result.append('newline')
                    result.append( ['text', 'ENTER_BARBARIAN_ATTACK'] )
                elif _teleport_state != 'TELEPORT':
                    return False
        elif place == 'before_name':
            #if self.is_deactivated:
            #    result.append( ['text', 'COLOR_LINES'] )
            #    result.append( ['text', 'NOT_ACCESSIBLE_YET'] )
            #    return False
            if by_click:
                if self.is_opened:
                    #_has_access = self.HasAccess()
                    #if not _has_access:
                    #    return True # если нет доступа, то не обрабатываем
                    _teleport_state = self.can_teleport()
                    if _teleport_state == 'QUEST':
                        result.append( ['text', 'COLOR_RED'] )
                        result.append( ['text', 'ENTER_COMPLETE_QUEST_FIRST'] )
                        return False
                    elif _teleport_state == 'ATTACK':
                        result.append( ['text', 'COLOR_RED'] )
                        result.append( ['text', 'ENTER_BARBARIAN_ATTACK'] )
                        return False
                    elif _teleport_state == 'ACCESS':
                        result.append( ['text', 'COLOR_RED'] )
                        result.append( ['text', 'NO_PATH'] )
                        return False
                    elif _teleport_state == 'DEACTIVATED':
                        result.append( ('text', 'COLOR_LINES') )
                        result.append( ['text', 'NOT_ACCESSIBLE_YET'] )
                        return False
                    elif _teleport_state != 'TELEPORT':
                        return False
        elif place == 'before_res':
            if self.is_deactivated:
                result.append( ('text', 'COLOR_LINES') )
                result.append( ('text', 'NOT_ACCESSIBLE_YET') )
                return False
        return True
# ============================================================
    def _append_need_gems(self, result):
        if not self.is_opened:
            base_diff_item.DiffItem._append_need_gems(self, result)
# ============================================================
    def _append_not_enough_gems(self, result, nokey):
        if not self.is_opened:
            base_diff_item.DiffItem._append_not_enough_gems(self, result, nokey)
# ============================================================
    def start_transition(self):
        '''На пару секунд прячет кнопку, чтобы она не поменяла свой вид при переходе'''
        self.button_shown = False
        self.switch_animation()
        self.Signals.Restart('show_button', 1000)
# ============================================================
    #@override
    def scale_bump(self):
        _pos = self.GetAttrPoint('center_item_pos')
        self.Drawer.ScaleBump(['default','cover'], _pos, 200, 1.10)
# ============================================================
    def play_hit_sound(self):
        self.PlaySound('SND_GRAVE_HIT')
# ============================================================
# #######################################################
# #######################################################
# #######################################################
