# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes


class BreakPotSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'BreakPotSubtask'
        self.actor_action_animation = 'hit'

    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        self.actor.action_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        return True


class SkeletonSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'SkeletonSubtask'
        self.actor_action_animation = 'pickup'

    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        self.actor.action_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        return True


class ImmediateSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'ImmediateSubtask'

    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        item.action_hit_me()
        self.on_action_complete()   # complete immediately
        return True
# ============================================================
# ============================================================
class Pot(base_diff_item.DiffItem):
    '''Горшок'''
    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'Pot'
        self.task = None
        self.contains_bomb = False
        # для кусто-подобных:
        #self.shear_r = self.GetAttrInt('shear_r', 50)
        #self.shear_shift = self.GetAttrInt('shear_shift', 5)
        self.shear_disabled = self.GetAttrBool('shear_disabled')
        # <<<<<<<
        self.like_tree_mode = self.GetAttrBool('like_tree')
        if self.like_tree_mode:
            if not self.shear_disabled:
                _shear_r = self.GetAttrInt('shear_r', 100)
                self.Drawer.SetTreeMode('default', _shear_r)
        self.like_bush_mode = self.GetAttrBool('like_bush')
        if self.like_bush_mode:
            if not self.shear_disabled:
                _shear_r = self.GetAttrInt('shear_r', 50)
                self.Drawer.SetTreeMode(['default','growth'], _shear_r)
        self.like_stalagmite = self.GetAttrBool('like_stalagmite')
        self.snd_hit = self.GetAttrStr('hit_sound')
        self.snd_complete = self.GetAttrStr('broken_sound')
        _name = self.get('name')
        if _name.startswith('secret_crack_'):   # hack
            self.add_attention_signal()
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        content_template_name, variant_name = self.get_content_ex()
        if not content_template_name:
            content_template_name = self.get_content()      # имя шаблона объекта внутри горшка
            variant_name = self.GetAttrStr('contains_variant_name')
        self.switch_variant( 'broken_' + str(tools.Rand(1,3)) )    # broken_1, broken_2, broken_3
        self.action_hits_last = self.GetAttrInt('action_hits_count')   # уже после смены варианта
        pot_info = {'type':self.get('type'), 'subtype':self.get('subtype')}
        EMgr.send_event( 'pot_broken', self, pot_info )
        _extra_complete_event = self.GetAttrStr('extra_complete_event')
        if _extra_complete_event:
            EMgr.send_event( _extra_complete_event, self )
        self.publish_content(content_template_name, variant_name)
        if self.snd_complete:
            self.PlaySound(self.snd_complete)
        elif self.like_tree_mode:
            self.PlaySound('SND_CHOP_TREE_FALL')
        elif self.like_bush_mode:
            self.PlaySound('SND_CHOP_BUSH_DISAPPEAR')
        elif self.like_stalagmite:
            self.PlaySound('SND_STALACTITE_CRASH')

        if self.like_tree_mode:
            if not self.shear_disabled:
                _shear_r = self.GetAttrInt('shear_r', 100)
                self.Drawer.SetTreeMode('default', _shear_r)
        if self.like_bush_mode:
            if not self.shear_disabled:
                _shear_r = self.GetAttrInt('shear_r', 50)
                self.Drawer.SetTreeMode(['default','growth'], _shear_r)

        if self.contains_bomb:
            # play sound
            self.PlaySound(self.GetAttrStr('bomb_sound'))
            # show effect
            self.Drawer.StartEffectOnce('explosion')
            self.add_afterdeath_item()  # оставим воронку от взрыва
            # attack general
            UMgr.get_general().attacked_by_explosion()

        if not self.like_tree_mode:
            if not self.like_bush_mode:
                if not self.like_stalagmite:
                    if self.subtype == 'burrel':
                        EMgr.send_event( 'broken_burrel', self )
                        UStats.AddLevelValue('broken_barrels', 1)
                        if not self.snd_complete: self.PlaySound('SND_BARREL_CRASH')
                    else:
                        EMgr.send_event( 'broken_pot', self )
                        UStats.AddLevelValue('broken_pots', 1)
                        if not self.snd_complete: self.PlaySound('SND_POT_CRASH')
        self.send_extra_event()
        BMgr.remove_item_uid(self)
# ============================================================
    def add_afterdeath_item(self):
        '''Добавляет воронку под здание'''
        _cell = self.GetAttrCellFixed('afterdeath_cell')
        _crater_template_name = self.GetAttrStr('crater_template', 'crator2')
        # добавить воронку под здание
        _variants = ['var1', 'var2', 'var3']
        _variant_name = _variants[tools.Rand(1,3)-1]
        _new_item = BMgr.AddItemToMap(self.get('locind'), _crater_template_name, _variant_name, _cell)
        BMgr.remove_item_uid(_new_item)
# ============================================================
    def get_content_ex(self):
        '''Контент из "content"'''
        _content = self.get_unpacked_content( self.GetAttrStr('content') )  # строго до смены варианта
        if _content:
            _item = _content[0]
            _part0 = _item[0]
            _item_parts = len(_item)
            if _item_parts == 1:
                return _part0, ''
            else:
                return _item
        return (None, None)
# ============================================================
    def get_content(self):
        '''Возвращает имя шаблона содержимого горшка, если в нем что-то есть'''
        content_template_name = self.GetAttrStr('contains_template')
        if not content_template_name:
            if self.GetAttrBool('contains_gold_small'):
                return 'gold_small'
            elif self.GetAttrBool('contains_gold_big'):
                return 'gold_big'
        return content_template_name
# ============================================================
    def publish_content(self, content_template_name, variant_name):
        '''Выкладывает на поле объект'''
        if not content_template_name:
            self.generated_loot = []
            return
        cell_pos = self.GetAttrCellFixed('content_cell')
        # - create item and place item on map in cell_pos
        map_index = self.get('locind')
        _new_item = BMgr.AddItemToMap(map_index, content_template_name, variant_name, cell_pos)        # hack - BuildingsManager может добавлять на карту не только здания...
        if _new_item:
            _pos = _new_item.GetAttrPoint('center_item_pos')
            _new_item.set_shaded(self.is_shaded)
            _new_item.Drawer.ResIn(_new_item.item_layers, _pos, 1000)
            _new_item.play_content_sound(self)
            self.generated_loot = [_new_item]
# ============================================================
    def get_subtask(self, parent_task, actor):
        _is_pickup = self.GetAttrBool('is_pickup')
        _is_immediate = self.GetAttrBool('is_immediate')
        if _is_pickup:
            return SkeletonSubtask(parent_task, actor)
        elif _is_immediate:
            return ImmediateSubtask(parent_task, actor)
        else:
            return BreakPotSubtask(parent_task, actor)
# ============================================================
    def play_hit_sound(self):
        if self.snd_hit:
            self.PlaySound(self.snd_hit)
        elif self.like_tree_mode:
            self.PlaySound('SND_CHOP_TREE')
        elif self.like_bush_mode:
            self.PlaySound('SND_CHOP_BUSH')
        elif self.like_stalagmite:
            self.PlaySound('SND_STALACTITE_HIT')
        elif self.subtype == 'burrel':
            self.PlaySound('SND_BARREL_HIT')
        else:
            self.PlaySound('SND_POT_HIT')
# ============================================================
    #@override
    def get_action_text(self, for_task_slide = False):
        return 'DLG_TASK_BREAK_POT'
# ============================================================
    def scale_bump(self):
        if not self.like_tree_mode:
            base_diff_item.DiffItem.scale_bump(self)
        else:
            _pos = self.GetAttrPoint('center_item_pos')
            self.Drawer.ScaleBump(self.bump_layers, _pos, 200, 1.1)
# ============================================================
    #@override
    def get_action_cell_name(self):
        '''Метод возвращает название ячейки к которой будет подбегать юнит'''
        _cell_names = []
        for i in range(1,9):
            _name = 'action_cell' + str(i)
            _has_attr = self.HasAttr(_name)
            if _has_attr:
                _cell_names.append(_name)
        _cell_names = self._filter_available_cells(_cell_names)
        _len = len(_cell_names)
        if _len > 0:
            _rnd = tools.Rand(0, _len-1)
            return _cell_names[_rnd]
        else:
            return 'action_cell'
# ============================================================
# #######################################################
# #######################################################
# #######################################################
