# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes
import toolz


class TakeResSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'TakeResSubtask'
        self.actor_action_animation = 'pickup'
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        self.actor.action_hit_item(item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
        return True
        #self.actor.stop_walk()
        #item.remove_me()
        #self.on_action_complete()
        #return True
# ============================================================
# ============================================================
class GeneralResource(base_diff_item.DiffItem):
    '''Ресурс, который может подобрать генерал'''

    _gems = ['gem_red', 'gem_green', 'gem_blue']

    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'GeneralResource'
        self.task = None
        self.actor = None
        self.is_gem = False
        _name = self.GetSingleResName('gives')
        if _name in self._gems:
            self.is_gem = True
        self.show_born_effect()
# ============================================================
    eff_names = {'gem_red':'magic_red', 'gem_green':'magic_green', 'gem_blue':'magic_blue'}
    def remove_me(self):
        ''' Удаляет ресурс с поля, считаем его "взятым" '''
        self.PlaySound('SND_ITEM_PICKUP')
        _gives_res = self.GetRes('gives')
        _res_name = _gives_res.GetSingleName()
        if _res_name in self._gems:
            UStats.AddLevelValue('GEM_COMET', 1)
            self.PlaySoundG('SND_LEVELUP')
            if self.actor:
                self.actor.need_show_magic()
                _eff_name = self.eff_names[_res_name]
                self.AddEffect(_eff_name, self.actor.get('pos'))
        if _res_name in self.res_images:
            res_count = _gives_res.Get(_res_name)
            if _res_name in self.res_images:
                _is_loc_active = self.get('is_loc_active')
                if _is_loc_active:
                    LBoard.FlyResCount(self, self.res_images[_res_name], res_count)
        #UStats.AddRes(_gives_res)
        self.start_comet()
        #if _res_name not in self._gems:
        #    EMgr.send_event('coins_taken', self, {'uid':self.get('uid')})   # в UI UID'ы проверяются, поэтому не так важно что все ресурсы генерят это событие
        BMgr.RemoveItemFromMap(self, True)
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        self.send_extra_event()
        self.remove_me()
# ============================================================
    def is_active(self):
        if self._is_gem_maximum():
            return False
        return base_diff_item.DiffItem.is_active(self)
# ============================================================
    def get_subtask(self, parent_task, actor):
        self.actor = actor  # сохраним для случая с камнями - нужно иметь доступ к генералу
        if self._is_gem_maximum():
            return None
        return TakeResSubtask(parent_task, actor)
# ============================================================
    _res_name_to_text = {'gem_green' : 'TO_ORATORY',
                         'gem_blue'  : 'TO_POWER',
                         'gem_red'   : 'TO_SPEED' }
    #@override
    def _get_gives(self, result):
        '''Возвращает текст для части типса GIVES'''
        _gives = self.completeGivesRes
        if _gives:
            if not _gives.Empty():
                if _gives.GetSingleName() in self._res_name_to_text:
                    result.append( 'newline' )
                    result.append( ('text', 'COLOR_LINES') )
                    result.append( ('text', 'PLUS') )
                    result.append( ('digit', _gives.GetSingleValue()) )
                    result.append( 'space' )
                    result.append( ('text', self._res_name_to_text[_gives.GetSingleName()]) )
                    res_numbers = self._append_res_values(_gives)
                    result.extend( res_numbers )
                else:
                    base_diff_item.DiffItem._get_gives(self, result)
# ============================================================
    def _is_gem_maximum(self):
        if self.is_gem:
            _name = self.GetSingleResName('gives')
            if _name == 'gem_red':  # скорость
                _speed = UStats.GetLevelValue('GENERAL_STATIC_SPEED')
                if _speed >= 8:
                    return True
            else:
                _total_res = UStats.GetCurRes()
                _total_gem_count = _total_res.Get(_name)
                if _total_gem_count >= 8:   # maximum
                    return True
        return False
# ============================================================
    def _get_tips_extra(self, place, result, by_click):
        if place == 'before_name':
            if self._is_gem_maximum():
                result.append( ('text', 'COLOR_LINES') )
                result.append( ('text', 'NOT_NEED_YET') )
                return False
        return True
# ============================================================
    # @override
    def on_create(self):
        base_diff_item.DiffItem.on_create(self)
        self.add_attention_signal()
# ============================================================
    id_map = {'silver':'MSG_SILVER', 'food':'MSG_FOOD', 'boards':'MSG_WOOD'}
    #@override
    def on_signal(self, signal):
        if signal == 'comet_fly_complete':
            _gives_res = self.GetRes('gives')
            _rname = _gives_res.GetSingleName()
            if _rname in self.id_map:
                _id = self.id_map[_rname]
                self.append_letter_resource(_gives_res, _id)
            UStats.AddRes(_gives_res)
            if _rname in self._gems:
                UStats.SubtractLevelValue('GEM_COMET', 1)
                _chest_shown = False
                if LBoard.Core.episode <= 1:
                    if LBoard.Core.level <= 1:
                        _total_gems_count = self._get_total_gems()
                        if _total_gems_count <= 1:
                            _count = _gives_res.Get(_rname)
                            _chest_shown = True
                            dlg.Open('ChestMenu', 'dlg_chest', {'gem':_rname, 'count':str(_count), 'showin_effect':'chest_fly_in', 'single_mode':'1'})
                if not _chest_shown:
                    UStats.AppendGem(_rname)  # т.к. сундук не вызывали - сами добавим камень в общее кол-во
                    tools.PlaySound('SND_DROP_GEM_NEXT')
                UI.on_global_event('general_params', '')
                EMgr.send_event( 'gem_taken', self )
                LBoard.Core.try_open_help(1,1, 3, self)
            BMgr.RemoveItemFromArray(self)
        else:
            base_diff_item.DiffItem.on_signal(self, signal)
# ============================================================
    def _get_total_gems(self):
        _cur_res = UStats.GetCurRes()
        _count = 0
        for _name in self._gems:
            _count += _cur_res.Get(_name)
        return _count
# ============================================================
    def play_content_sound(self, generator):
        _item_name = self.get('name')
        if _item_name == 'gem1':
            self.PlaySound('SND_DROP_GEM')
            return
        _name = self.GetSingleResName('gives')
        if _name == 'food':
            generator.PlaySound('SND_DROP_MEAT')
        elif _name == 'coins':
            generator.PlaySound('SND_DROP_SILVER')
        elif _name in self._gems:
            generator.PlaySound('SND_DROP_GEM')
# #######################################################
# #######################################################
# #######################################################
