# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_walkg
import task_classes


class TaskItemDlgTask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'TaskItemDlgTask'
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.item = item
        _uid = self.parent_task.uid
        #_dlg_settings = item.get_dlg_settings(item.quest.text, _uid)
        _dlg_settings = item.get_dlg_settings(_uid)
        #TMgr.cancel_general_tasks()
        Camera.MoveTo(item)
        _enough_res = UStats.GetResCount(self.item.res_need_name) >= self.item.res_need_count
        if _enough_res:
            dlg.Open('QuestMenu', 'dlg_quest', _dlg_settings)
            return True
        else:
            #self.on_action_complete()
            return False
# ============================================================
    def on_callback(self, result, extra_dict):
        '''Обработка данных от диалога (диалог у нас в другой VM)'''
        if result == 'agree':
            trace('TaskDialog "agree" pressed')
            self.parent_task.complete_task()    # таск по взятию квеста выполнен - теперь нужно выполнять сам квест и все управление переходит на него
            EMgr.SendGlobalMsg('npc_interacted')
            # вычесть ресурсы
            UStats.SubResValue(self.item.res_need_name, self.item.res_need_count)
            # выполнить действие:
            self.item.do_result_action()
            UStats.AddLevelValue('progress', 1) # обновляем прогресс
        elif result == 'postpone':
            trace('TaskDialog "postpone" pressed')
            self.parent_task.cancel()
        # отведем генерала от юнита:
        UMgr.step_aside_general(self.item.get('locind'))
# ============================================================
# ============================================================
# ============================================================
class TaskDiffItem(base_diff_item.DiffItem):
    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'TaskDiffItem'
        self.is_indian = True       # чтобы отличать индейцев от не индейцев
        self.res_need_name = self.GetAttrStr('res_need_name')
        self.res_need_count = self.GetAttrInt('res_need_count')
# ============================================================
    def deserialize_init(self):
        self.init_listeners()
# ============================================================
    def init_listeners(self):
        EMgr.add_concrete_event_listener('resources_changed', self.on_resources_changed)
        EMgr.add_concrete_event_listener('barbarian_attack_started', self.on_barbarian_event)
        EMgr.add_concrete_event_listener('barbarian_attack_completed', self.on_barbarian_event)
        EMgr.add_concrete_event_listener('barbarian_attack_canceled', self.on_barbarian_event)
        EMgr.add_concrete_event_listener('general_sick_event', self.on_general_sickness_event)
        EMgr.add_concrete_event_listener('general_recovered', self.on_general_sickness_event)
# ============================================================
    # @override
    def dispose(self):
        EMgr.remove_concrete_event_listener('resources_changed', self.on_resources_changed)
        EMgr.remove_concrete_event_listener('barbarian_attack_started', self.on_barbarian_event)
        EMgr.remove_concrete_event_listener('barbarian_attack_completed', self.on_barbarian_event)
        EMgr.remove_concrete_event_listener('barbarian_attack_canceled', self.on_barbarian_event)
        EMgr.remove_concrete_event_listener('general_sick_event', self.on_general_sickness_event)
        EMgr.remove_concrete_event_listener('general_recovered', self.on_general_sickness_event)
# ============================================================
# ============================================================
    def get_item(self, uid_str):
        parts = uid_str.split(':')
        _len = len(parts)
        _uid = parts[0]
        _map_index = self.get('locind')
        if _len == 2:
            _mi = parts[1]
            if _mi >= 0:
                _map_index = _mi
        return BMgr.get_item(_uid, _map_index)
# ============================================================
    def do_result_action(self):
        _do_name = self.GetAttrStr('do_name')
        _do_param = self.GetAttrStr('do_param')
        _do_param2 = self.GetAttrStr('do_param2')
        if _do_name == 'activate_object':
            uids = _do_param.split(',')
            for uid_str in uids:
                item = self.get_item(uid_str)
                if item: item.activate()
        elif _do_name == 'generate_object':
            _map_index = self.get('locind')
            templ = ''
            v = ''
            if ':' in _do_param:
                templ,v = _do_param.split(':')
            else:
                templ = _do_param
            _gift_cell = self.get_free_gift_cell()
            _new_item = BMgr.AddItemToMap(_map_index, templ, v, _gift_cell)
            if _new_item:
                _pos = _new_item.GetAttrPoint('center_item_pos')
                _new_item.Drawer.ResIn(_new_item.item_layers, _pos, 1000)
        elif _do_name == 'do_action':
            uids = _do_param.split(',')
            for uid_str in uids:
                item = self.get_item(uid_str)
                if item:
                    if _do_param2 == '': _do_param2 = 'default'
                    item.do_action(_do_param2)
        elif _do_name == 'switch_variant':
            uids = _do_param.split(',')
            for uid_str in uids:
                item = self.get_item(uid_str)
                if item:
                    _type = item.get('type')
                    if _type == 'building':
                        if _do_param2 == '':
                            item.switch_level()
                        else:
                            item.switch_level(_do_param2)
                    else:
                        item.switch_variant(_do_param2)
        self.PlaySound( self.GetAttrStr('snd_complete', 'SND_NONE') )
        self.switch_variant(self.GetAttrStr('next_variant'))
        self.is_deactivated = self.GetAttrBool('deactivated')
        if True:
            letter = []
            letter.append( ('text', 'MSG_ALTAR_ACTIVATED') )
            letter.append( 'space' )
            _loc_ind = self.get('locind')
            EMgr.AppendLetter(letter, self.get('uid'), _loc_ind)
        LBoard.Core.try_open_help(1,1, 7, self, '4773')
# ============================================================
    #@override
    def on_signal(self, signal):
        base_diff_item.DiffItem.on_signal(self, signal)
# ============================================================
    def on_message(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        pass
# ============================================================
    def on_click(self, btn_name):
        '''Вызывается при клике по объекту'''
        #@b если есть варвары на поле, то клики отменяем, квесты не сдаем
        _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
        if _has_barbarians:        # если есть варвары, то не даем кликать по индейцам, чтобы ни взять, ни сдать квест
            #LBoard.CrossNoToLastClick()
            return
        _is_active = self.is_active()
        if _is_active:
            _is_enough_params = self.is_enough_params()
            if _is_enough_params:
                if not self.task:
                    #@bookmark
                    _is_inited = self.is_res_inited()
                    if not _is_inited: return
                    _item_actor = self.GetAttrStr('item_actor')
                    _is_general_ok = UMgr.is_general_ok()
                    if not _is_general_ok:
                        LBoard.CrossNoToLastClick()
                        return
                    _enough_res = UStats.GetResCount(self.res_need_name) >= self.res_need_count
                    if _enough_res:
                        self.task = task_walkg.WalkGeneralTask(self)
                        TMgr.add_task( self.task )
                        if self.has_check:      # если галка установилась
                            Camera.MoveTo(self) # при установке галки - камеру на индейца
                            BMgr.SendGlobalMsg('unit_clicked', self.get('uid'))
                        return
                else:
                    self.task.cancel()
                    self.task = None
                    BMgr.SendGlobalMsg('unit_clicked', self.get('uid'))
                    return
            else:
                _param_name = self.get_not_enough_param()
                UI.BlinkGem(3, _param_name)     # подмигнем параметром
                return
        LBoard.CrossNoToLastClick()
# ============================================================
# ============================================================
    def on_create(self):
        self.init_listeners()
        self.switch_animation()
# ============================================================
    def __str__(self):
        return base_diff_item.DiffItem.__str__(self)
# ============================================================
# ============================================================
    def on_resources_changed(self, event):
        self.switch_animation()     # чтобы знак над индейцем обновить
# ============================================================
    def on_barbarian_event(self, event):
        self.switch_animation()     # чтобы знак над индейцем обновить
# ============================================================
    def on_general_sickness_event(self, event):
        self.switch_animation()
# ============================================================
# ============================================================
# ============================================================
    def switch_animation(self):
        if self.get('uid') == '3579': trace('===> ===>')
        _is_partial_fog = self.Drawer.IsPartiallyInFog()
        if _is_partial_fog:
            self.Drawer.ShowImages('', False)
            return
        self.Drawer.ShowImages('', False)
        #new_animation_name = 'default'
        #if state_name in self.indian_animation_dict:
        #    new_animation_name = self.indian_animation_dict[state_name]
        #state_name = self.SM.GetState()
        #self.Drawer.ShowImages([new_animation_name,'shadow'], True)
        self.Drawer.ShowImages(['default', 'shadow'], True)
        self.Drawer.ShowImages('select', self.is_mouse_over and not self.is_deactivated)
        self.Drawer.ShowImages('check', self.has_check)
        #_has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
        #if not _has_barbarians:
        _has_access = self.has_access()
        if _has_access:
            _is_active = self.is_active()
            if _is_active:
                self.Drawer.ShowImages('sign', self.has_sign())
        self.Drawer.ShowImages('sign_red', self.need_sign_red())
# ============================================================
    # @override
    def is_active(self):
        '''Возможны ли действия над объектом в данный момент. ПЕРЕОПРЕДЕЛЯЕМ'''
        if not self.non_selectable:
            if not self.is_deactivated:
                return True
        return False
# ============================================================
    def need_sign_red(self):
        '''Нужно ли показать красное облако над индейцем'''
        if not self.non_selectable:
            if not self.is_deactivated:
                #@bookmark
                _is_inited = self.is_res_inited()
                if not _is_inited:
                    return False
                _is_enough_params = self.is_enough_params()
                if not _is_enough_params:
                    return True
                _is_general_ok = UMgr.is_general_ok()   # болен ли генерал
                if not _is_general_ok:
                    return True
                _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
                if _has_barbarians:
                    return True
                _not_enough_res = UStats.GetResCount(self.res_need_name) < self.res_need_count
                return _not_enough_res
        return False
# ============================================================
    def has_sign(self):
        if not self.non_selectable:
            if not self.is_deactivated:
                #@bookmark
                _is_inited = self.is_res_inited()
                if not _is_inited: return False
                _is_enough_params = self.is_enough_params()
                if not _is_enough_params:
                    return False
                _is_general_ok = UMgr.is_general_ok()   # болен ли генерал
                if not _is_general_ok:
                    return False
                _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
                if _has_barbarians: return False
                _enough_res = UStats.GetResCount(self.res_need_name) >= self.res_need_count
                return _enough_res
        return False
# ============================================================
    def is_enough_params(self):
        '''Достаточно ли красноречия и силы'''
        _need_power = self.GetAttrInt('need_power') - 1
        _need_oratory = self.GetAttrInt('need_oratory') - 1
        _cur_res = UStats.GetCurRes()
        _have_oratory = _cur_res.Get('gem_green')
        _have_power = _cur_res.Get('gem_blue')
        if _have_oratory < _need_oratory or _have_power < _need_power:
            return False
        else:
            return True
# ============================================================
    def get_not_enough_param(self):
        '''Возвращает имя параметра, которого недостаточно'''
        _need_power = self.GetAttrInt('need_power') - 1
        _need_oratory = self.GetAttrInt('need_oratory') - 1
        _cur_res = UStats.GetCurRes()
        _have_oratory = _cur_res.Get('gem_green')
        _have_power = _cur_res.Get('gem_blue')
        if _have_oratory < _need_oratory:
            return 'oratory'
        elif _have_power < _need_power:
            return 'power'
        else:
            return ''
# ============================================================
#    def is_action_complete(self):
#        if not self.quest:
#            return True
#        else:
#            return False
# ============================================================
    def get_subtask(self, parent_task, actor):
        return TaskItemDlgTask(parent_task, actor)
# ============================================================
# ============================================================
    #@override
    def get_dlg_settings(self, _uid = 0):
        _text = self.GetAttrStr('dlg_text')
        _tasks = self.get_tasks_text()
        _dlg_settings = {'imageid': self.GetAttrStr('dlg_avatar'),
                        'nameid': self.GetAttrStr('dlg_name'),
                        'textid': _text,
                        'tasks_text': _tasks,
                        'task_uid':str(_uid),
                        'single_mode':str(True),
                        'showin_effect': 'task_fly_in',
                        'pause':str(400),
                        }
        return _dlg_settings
# ============================================================
    def get_tasks_text(self):
        _list = []
        _count = 0
        _text_item_prefix = ''
        _text_action = self.GetAttrStr('dlg_text_action')
        if not _text_action: _text_action = 'DLG_TASK_GIVE_RES'
        _count = self.res_need_count
        _list.append(['text', 'TASK_STAR'])
        _list.append(['text', 'COLOR_TASKS_TEXT'])
        _list.append(['text', _text_action]) # _text_action
        if _count > 0:
            _list.append(['text', 'COLOR_TASKS_NUMBER'])
            _list.append('space')
            _list.append(['digit', _count])
            _list.append(['text', 'COLOR_TASKS_TEXT'])
        _list.append('space')
        _task_prefix_item_name = self.GetAttrStr('dlg_task_text_part')
        _textid_prefixed = self._get_textid_with_count(_task_prefix_item_name, _count)
        _list.append(['text', _textid_prefixed])
        return _list
# ============================================================
    def _get_textid_with_count(self, _prefix, _count):
        if _count <= 1:
            return _prefix
        elif _count >= 2 and _count < 5:
            return _prefix+'_2'
        elif _count >= 5:
            return _prefix+'_5'
# ============================================================
# ============================================================
    #@override
    def is_green_state(self):
        '''Цвет типса/селекта'''
        _has_access = self.has_access()
        if self.is_deactivated:
            if _has_access: return True
            else:           return False
        if not _has_access: return False
        _is_enough_params = self.is_enough_params()
        if not _is_enough_params:
            return False
        _is_general_ok = UMgr.is_general_ok()   # болен ли генерал
        if not _is_general_ok:
            return False
        #_has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
        #if _has_barbarians: return False
        _is_active = self.is_active()
        if _is_active:
            _need_sign_red = self.need_sign_red()
            if _need_sign_red:
                return False
            if self.has_sign():
                return True    # если не хватает только ресурсов
            else:
                return False
        return base_diff_item.DiffItem.is_green_state(self)
# ============================================================
    def Get_tip_text(self, by_click = False):
        '''Возвращает LIST, содержащий текстовые ID'''
        # нужно писать что нужно индейцу, например: красноречие, сила, ресурсы
        result = []
        item_name = self.GetAttrStr('item_name')
        if not item_name:
            return result
        if not by_click:
            result.append( self._color_header )
            result.append( ('text', item_name) )
            if self.is_deactivated:
                return result

            res_need = self._get_res_need()
            res_need_ne = self._get_res_need(True)
            # дальше хитрое шаманство недостающих параметров и ресурсов:
            result_params = []
            self.get_params_not_enough_text(result_params, [self._color_lines])
            result_ne_res = []
            parts = []
            parts.append(res_need)
            parts.append(result_params)
            needs_combined = self._combine_not_enough_texts(parts)
            if needs_combined:
                result.append('newline')
                result.append( self._color_lines )
                result.append( ('text', 'NEED_FOR_ACTION') )
                result.extend(needs_combined)

            result_params = []
            self.get_params_not_enough_text(result_params,\
                                            [self._color_red], True )
            parts = []
            parts.append(res_need_ne)
            parts.append(result_params)
            not_enough_combined = self._combine_not_enough_texts(parts)
            if not_enough_combined:
                result.extend( ['newline', self._color_red, self._not_enough_text] )
                result.extend(not_enough_combined)
            _has_access = self.has_access()
            if not self.is_deactivated or not _has_access:
                if self._append_sick_general(result):
                    return result
        else:   # by click:
            _has_access = self.has_access()
            _is_active = self.is_active()
            if _has_access:
                if not self.is_deactivated:
                    if self._append_sick_general(result, True):
                        return result
            #if not self.non_selectable:
            #    #if not self.is_deactivated:
            #    if _has_access:
            #        result.append( self._color_lines )
            #        result.append( ('text', 'NOT_READY_TO_TALK') )
            #        return result
            if True:
                result = []
                if not _has_access:
                    result.append( self._color_red )
                    result.append( ('text', 'NO_PATH') )
                    return result
            _no_time_to_talk = False
            _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
            if _has_barbarians: _no_time_to_talk = True
            if _no_time_to_talk:
                result.append( self._color_lines )
                result.append( ('text', 'YOU_HAVE_NO_TIME_TO_TALK') )
                return result

            res_need_ne = self._get_res_need(True)
            result_params = []
            self.get_params_not_enough_text(result_params,\
                                            [self._color_red], True )
            parts = []
            parts.append( res_need_ne )
            parts.append( result_params )
            combined = self._combine_not_enough_texts(parts)
            result = []
            if not combined:
                return self.Get_tip_text(False)
            else:
                result.extend( [self._color_red, self._not_enough_text] )
                result.extend( combined )
        return result
# ============================================================
    _tips_lev = ('text', 'TIPS_LEV')
    _tips_oratory = ('text', 'TIPS_ORATORY')
    _tips_power = ('text', 'TIPS_POWER')
    def get_params_not_enough_text(self, result, extra_text, how_much = False):
        _need_power = self.GetAttrInt('need_power')
        _need_oratory = self.GetAttrInt('need_oratory')
        _need_oratory_d = 0
        _need_power_d = 0
        _cmp_value = 0
        if True:
            _need_power_d = _need_power - 1
            _need_oratory_d = _need_oratory - 1
            _cur_res = UStats.GetCurRes()
            _need_oratory_d = _need_oratory_d - _cur_res.Get('gem_green')
            _need_power_d = _need_power_d - _cur_res.Get('gem_blue')
        if _need_oratory_d > _cmp_value:
            if extra_text: result.extend(extra_text)
            result.append(self._tips_oratory)
            result.append( 'space' )
            result.append(self._tips_lev)
            result.append( 'space' )
            result.append( ('digit', [_need_oratory, _need_oratory_d][how_much == True]) )
        if _need_power_d > _cmp_value:
            if extra_text: result.extend(extra_text)
            result.append(self._tips_power)
            result.append( 'space' )
            result.append(self._tips_lev)
            result.append( 'space' )
            result.append( ('digit', [_need_power, _need_power_d][how_much == True]) )
# ============================================================
    def _get_res_not_enough_text(self, result, show_red = False):
        res = self.res_action_cost
        _is_res_enough = UStats.IsResEnough(res)
        if not _is_res_enough:
            diff_res = UStats.GetNotEnoughRes(res)
            if diff_res:
                if show_red:     result.append( self._color_red )
                result.append( self._not_enough_text )
                result.append( 'space' )
                res_numbers = self._append_res_values(diff_res)
                result.extend( res_numbers )
                return True
        return False
# ============================================================
    _tips_FACES_NOT_ENOUGH_TO_SPEAK_TI = ('text', 'FACES_NOT_ENOGH_TO_SPEAK_TI')
    _tips_FACES_READY_TO_SPEAK_TI = ('text', 'FACES_READY_TO_SPEAK_TI')
    #@override
    def get_ui_subtext(self):
        '''Текст, который должен быть написан в типсе возле панели с
        лицом персонажа второй строкой'''
        if not self.non_selectable and not self.is_deactivated:
            parts = []
            ne_result = [self._tips_FACES_NOT_ENOUGH_TO_SPEAK_TI]
            result = []
            _is_enough_params = self.is_enough_params()
            if not _is_enough_params:
                self.get_params_not_enough_text(result, [self._color_red], True)
                #return [result, False]
                parts.append(result)

            _enough_res = UStats.GetResCount(self.res_need_name) >= self.res_need_count
            result = []
            if not _enough_res:
                parts.append(self._get_res_need(True))

            result = self._combine_not_enough_texts(parts)
            if result:
                ne_result.extend(result)
                return (ne_result, False)

            _is_active = self.is_active()
            if _is_active:
                _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
                return ([self._tips_FACES_READY_TO_SPEAK_TI], not _has_barbarians)
        _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
        return [[], not _has_barbarians]
# ============================================================
    def _combine_not_enough_texts(self, parts):
        _len = 0
        filtered_parts = []
        for part in parts:
            if part:    # учитываем только не пустые записи
                #_size = len(part)
                #if _size > 0:
                _len += 1
                filtered_parts.append(part)
        result = []
        if _len == 1:
            result.append('space')  # пробел-разделитель после двоеточия
            result.extend( filtered_parts[0] )
        else:
            for part in filtered_parts:
                result.append('newline')  # перевод строки-разделитель после двоеточия и после предыдущего пункта
                result.extend( part )
        return result
# ============================================================
    def _get_res_need(self, get_not_enough = False):
        result = []
        _is_inited = self.is_res_inited()
        if not _is_inited: return result
        if not get_not_enough:
            result.append( ('digit', self.res_need_count) )
            result.append( ('image', self.res_images[self.res_need_name]) )
        else:
            _enough_res = UStats.GetResCount(self.res_need_name) >= self.res_need_count
            if not _enough_res:
                _has_res = UStats.GetResCount(self.res_need_name)
                if _has_res < self.res_need_count:
                    result.append( ('digit', self.res_need_count - _has_res) )
                    result.append( ('image', self.res_images[self.res_need_name]) )
        return result
# ============================================================
    def is_res_inited(self):
        if self.res_need_name == '':
            return False
        if self.res_need_count <= 0:
            return False
        return True
# ============================================================
# ============================================================
