# -*- coding: utf-8 -*-

import sys
if not 'tinypy' in sys.version:
    from extra import *

import task_classes
import base_diff_item

class TeleportSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'TeleportSubtask'
        self.actor_action_animation = 'pickup'
# ============================================================
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        self.actor.action_hit_item(item)
        item.PlaySound( item.GetAttrStr('snd_teleport') )
        return True
# ============================================================
# ============================================================
class Teleport(base_diff_item.DiffItem):
    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'Teleport'
        self.linked_uid = self.GetAttrStr('linked_uid')
        self.target = None
# ============================================================
    # @override
#    def switch_animation(self):
#        pass
# ============================================================
    def on_create(self):
        base_diff_item.DiffItem.on_create(self)
        _linked_uid_len = len(self.linked_uid)
        if _linked_uid_len:
            self.target = BMgr.get_item(self.linked_uid, self.get('locind'))
            if self.target:
                self.target.target = self
# ============================================================
#    def CheckGeneralPathAfterTeleport(self):
#        # --- Checking access to teleports ---
#        _mi = self.get('locind')
#        _g = UMgr.get_cluster(_mi).general
#        _g.ClearGeneralAccess()
#        _tele_cell = self.target.GetAttrCellFixed('action_cell')
#        _g.CheckGeneralAccess(_tele_cell)
#        _tele_cell = self.GetAttrCellFixed('action_cell')
#        _g.CheckGeneralAccess(_tele_cell)
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        self.send_extra_event()
        _g = UMgr.get_general()
        self.task.complete_task()
        TMgr.cancel_general_tasks() # потому что дальше ничего выполнять нельзя
        if self.target:
            _g.on_teleport_in(self.target) # removes Teleport inside
            #@! добавить событие
            BMgr.SendGlobalMsg('teleportation_started', self.get('uid'))
            self.Drawer.StartEffectOnce('teleport')
            #@! здесь бы паузу добавить в момент перелета генерала...
            self.target.Drawer.StartEffectOnce('teleport')
        self.FSM.HandleEvent('complete_event', True)
        self.FSM.HandleEvent('regenerate_event')
# ============================================================
    def get_subtask(self, parent_task, actor):
        _is_target_teleport_accessible = self.is_target_teleport_accessible()
        if _is_target_teleport_accessible:
            return TeleportSubtask(parent_task, actor)
        return None
# ============================================================
    def is_target_teleport_accessible(self):
        if self.target:
            if not self.target.is_deactivated:
                if not self.target.non_selectable:
                    _tele_cell = self.target.GetAttrCellFixed('action_cell')
                    _is_free = self.IsTeleportCellPassable(_tele_cell, True)
                    if _is_free:
                        return True
        return False
# ============================================================
    def is_active(self):
        _is_active = base_diff_item.DiffItem.is_active(self)
        if _is_active:
            _is_target_teleport_accessible = self.is_target_teleport_accessible()
            if _is_target_teleport_accessible:
                return True
        return False
# ============================================================
    def scale_bump(self):
        pass
# ============================================================
    def _get_tips_extra(self, place, result, by_click):
            if place == 'before_click':
                _is_deactivated = self.is_deactivated
                if not _is_deactivated:
                    if self.target:
                        if self.target.is_deactivated:
                            _is_deactivated = True
                if _is_deactivated:
                    result.append( ('text', 'COLOR_LINES') )
                    result.append( ('text', 'NOT_TELEPORT') )
                    return False
            return True
# ============================================================
# #######################################################
# #######################################################
# #######################################################
