# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_diff_item
import task_classes
# ============================================================
# ============================================================
class ChopSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'ChopSubtask'
        self.actor_action_animation = 'action_hit'
# ============================================================
    # @override
    def start_action(self, item):
        item.FSM.HandleEvent('start_event', True)
        self.actor.stop_walk()
        self.actor.action_hit_item(item)
        return True
# ============================================================
# ============================================================
# ============================================================
class Tree(base_diff_item.DiffItem):
    '''Дерево'''
    def __init__(self, cdata):
        base_diff_item.DiffItem.__init__(self, cdata)
        self.ser_name = 'Tree'
        self.shear_disabled = self.GetAttrBool('shear_disabled')
        if not self.shear_disabled:
            self.Drawer.SetTreeMode('default', 100)
        self.need_fall_effect = False
# ============================================================
    # @override
    def get_subtask(self, parent_task, actor):
        return ChopSubtask(parent_task, actor)
# ============================================================
    # @override
    def on_action_hits_me_complete(self):
        self.completeGivesRes = self.GetRes('gives')
        self.switch_variant( self.GetAttrStr('next_variant') )
        self.action_hits_last = 0
        tree_info = {'type':self.get('type'), 'subtype':self.get('subtype')}
        EMgr.send_event( 'tree_chop', self, tree_info )
        EMgr.send_event( 'do_action_complete', self, tree_info )
        #self.AddEffect('tree_off', self.get('position'))
        self.need_fall_effect = True
        self.Drawer.ShowImages('effect')
        self.Drawer.RestartEffect('effect')
# ============================================================
    #@override
    def on_create(self):
        self.switch_animation()
# ============================================================
    def switch_animation(self):
        drawer = self.Drawer
        _ShowImages = drawer.ShowImages
        if drawer.IsPartiallyInFog():
            _ShowImages('', False)
            return
        _ShowImages('', False)
        _ShowImages('default', True)
        _ShowImages('arrow', self.arrow_shown)
        _ShowImages('effect', self.need_fall_effect)
# #######################################################
# #######################################################
    def Is_tree_normal(self):
        '''Вызывается из C++ для проверка какие деревья
        еще не срублены и на которые нет заданий'''
        return (not self.is_action_complete() and not self.task)
# #######################################################
# #######################################################
    def Get_tip_text(self, by_click = False):
        '''Возвращает LIST, содержащий текстовые ID'''
        result = []
        _is_active = self.is_active()
        if not _is_active:
            return result
        item_name = self.GetAttrStr('item_name')
        if not item_name:
            return result
        result.append( ('text', 'COLOR_HEADER') )
        result.append( ('text', item_name) )
        res = self.GetRes('gives')
        if res:
            res_empty = res.Empty()
            if not res_empty:
                result.append( 'newline' )
                result.append( ('text', 'COLOR_LINES') )
                result.append( ('text', 'NEED_FOR_ACTION') )
                result.append( 'space' )
                #res_numbers = 0
                trace(res)
                res_numbers = res.GetSingleValue()
                result.append( ('digit', res_numbers) )
        return result
# ============================================================
    #@override
    def Force_highlight(self, highlight):
        if self.get('variant_name') != 'stump':     # если это не пенек
            self.Drawer.HighlightImages('default', highlight)
        else:
            self.Drawer.HighlightImages('default', False)
# ============================================================
    #@override
    def Force_colorize(self, color, highlight):
        '''color - without alpha part, only RGB. Alpha always 0xFF'''
        _is_stump = self.get('variant_name') == 'stump'
        if not highlight or _is_stump:     # если это пенек
            self.Drawer.ColorizeImages(self.hlight_layers, 0xffffff, False)
        else:
            self.Drawer.ColorizeImages(self.hlight_layers, color, False)
# ============================================================
    def play_hit_sound(self):
        self.PlaySound('SND_CHOP_TREE')
# ============================================================
    def play_complete_sound(self):
        self.PlaySound('SND_CHOP_TREE_FALL')
# ============================================================
    #@override
    def scale_bump(self):
        _pos = self.GetAttrPoint('center_item_pos')
        self.Drawer.ScaleBump(self.bump_layers, _pos, 200, 1.1)
# ============================================================

