# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

# ============================================================
# ============================================================
class DependantBuildingMgr(object):
    def __init__(self):
        self.ser_name = 'DependantBuildingMgr'
        self.generating = {}    # 'res_name': [buildings]
        self.processing = {}    # 'res_name': [buildings]
# ============================================================
# ============================================================
    def add_building(self, building):
        _need_base_res = building.need_base_resource()
        if _need_base_res:
            _processing_res_name = self._get_processing_res_name(building)
            if not _processing_res_name:
                return
            else:
                if _processing_res_name not in self.processing:
                    self.processing[_processing_res_name] = [building]
                    self._refresh_flags(_processing_res_name, False)
                elif building not in self.processing[_processing_res_name]:
                    self.processing[_processing_res_name].append(building)
                    self._refresh_flags(_processing_res_name, False)
        else:
            _generate_res_name = self._get_generating_res_name(building)
            if not _generate_res_name:
                return
            else:
                if _generate_res_name not in self.generating:
                    self.generating[_generate_res_name] = [building]
                elif building not in self.generating[_generate_res_name]:
                    self.generating[_generate_res_name].append(building)
# ============================================================
    def remove_building(self, building):
        _need_base_res = building.need_base_resource()
        if _need_base_res:
            _processing_res_name = self._get_processing_res_name(building)
            if not _processing_res_name:
                return
            elif _processing_res_name in self.processing:
                self.processing[_processing_res_name].remove(building)
                _count = len(self.processing[_processing_res_name])
                if _count == 0: self._refresh_flags(_processing_res_name, True)
        else:
            _generate_res_name = self._get_generating_res_name(building)
            if not _generate_res_name:
                return
            elif _generate_res_name in self.generating:
                self.generating[_generate_res_name].remove(building)
# ============================================================
    def _refresh_flags(self, res_name, show_flag):
        if res_name in self.generating:
            _buildings = self.generating[res_name]
            for b in _buildings:
                b.need_flag_update(show_flag)
# ============================================================
    def need_build_processor_for(self, building):
        _need_base_res = building.need_base_resource()
        if _need_base_res:
            return False
        else:
            _generate_res_name = self._get_generating_res_name(building)
            if not _generate_res_name:
                return False
            if _generate_res_name not in self.processing:
                return True
            _processors = self.processing[_generate_res_name]
            _count = len(_processors)
            if _count == 0:
                return True
            return False
# ============================================================
    def _get_generating_res_name(self, building):
        return building.get_generate_resource_name()
# ============================================================
    def _get_processing_res_name(self, building):
        return building.get_base_resource_name()
# ============================================================
