# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import dialog
import dlg_exit
import dlg_buildings
import dlg_quest
import dlg_letter
import dlg_chest
#import dlg_map
import dlg_book

import dlg_loadlevel
import dlg_help
import dlg_secret
import dlg_trade
import dlg_main

import dlg_new_profile
import dlg_edit_profile
import dlg_profiles
import dlg_options
import dlg_top_scores
import dlg_gamemenu
import dlg_mode
import dlg_asking
import dlg_msg
import dlg_comics

import dlg_key
import dlg_prizes
import dlg_timeoff
import dlg_stats
import dlg_start
import dlg_upgrade
import dlg_share
import dlg_share_ty
import dlg_rate
import dlg_like
import dlg_internet

import pmgr

dlg_mgr_prevent_gc = None

dlg_name_to_class = {
    '': dialog.Dialog,
    'dlg_exit': dlg_exit.ExitDlg,
    'dlg_buildings': dlg_buildings.BuildingsDlg,
    'dlg_quest': dlg_quest.QuestDlg,
    'dlg_quest_ok': dlg_quest.QuestDlg,
    'dlg_general_ok': dlg_quest.QuestDlg,
    'dlg_letter': dlg_letter.LetterDlg,
    'dlg_chest': dlg_chest.ChestDlg,
    #'dlg_map': dlg_map.MapDlg,
    'dlg_book': dlg_book.BookDlg,
    'dlg_prizes': dlg_prizes.PrizesDlg,
    'dlg_key': dlg_key.KeyDlg,
    'dlg_stats': dlg_stats.StatsDlg,
    'dlg_timeoff': dlg_timeoff.TimeOffDlg,
    'dlg_loadlevel': dlg_loadlevel.LoadLevelDlg,
    'dlg_help': dlg_help.HelpDlg,
    'dlg_secret': dlg_secret.SecretDlg,
    'dlg_trade': dlg_trade.TradeDlg,
    'dlg_main': dlg_main.MainDlg,

    'dlg_new_profile': dlg_new_profile.NewProfileDlg,
    'dlg_edit_profile': dlg_edit_profile.EditProfileDlg,
    'dlg_profiles': dlg_profiles.ProfilesDlg,
    'dlg_options': dlg_options.OptionsDlg,
    'dlg_top_scores': dlg_top_scores.TopScoresDlg,
    'dlg_gamemenu': dlg_gamemenu.GameMenuDlg,
    'dlg_mode': dlg_mode.ModeDlg,
    'dlg_asking': dlg_asking.AskingDlg,
    'dlg_msg': dlg_msg.MsgDlg,
    'dlg_comics': dlg_comics.ComicsDlg,
    'dlg_start': dlg_start.StartDlg,
    'dlg_upgrade': dlg_upgrade.UpgradeMenuDlg,
    'dlg_share': dlg_share.ShareMenuDlg,
    'dlg_share_ty': dlg_share_ty.ShareTyMenuDlg,
    'dlg_rate': dlg_rate.RateMenuDlg,
    'dlg_like': dlg_like.LikeMenuDlg,
    'dlg_internet': dlg_internet.InternetMenuDlg,
    }

# SAMPLE of dialog open:    dlg.Open("QuestMenu", "dlg_quest", _dlg_settings)

class DlgMgr(DialogsMgr_class):
    non_collected_dialogs = []
    back_dialog = None      # полноэкранный диалог, отображаемый в данный момент
    next_dialog = None      # следующий диалог, после фэйда
    dialogs_stack = []      # стэк НЕполноэкранных диалогов
    awaiting_dialogs_stack = []
# ============================================================
    def __init__(self, cdata, pmgr_cdata):
        DialogsMgr_class.__init__(self, cdata)
        global dlg_mgr_prevent_gc
        dlg_mgr_prevent_gc = self
        self.PMgr = pmgr.ProfileMgr(pmgr_cdata)
        #self.PMgr.DLG = self
        self.ticks = 0
        self.single_dialogs_allowed = True
        trace('==== DlgMgr constructor (V3) ====')
# ============================================================
    def on_msg(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'on_create':
            self.on_create()
        elif msg_type == 'state_machine':
            self.on_state_machine(msg_param1, msg_param2, msg_dict)
        elif msg_type == 'signal_msg':
            self.on_signal(msg_param1)
        else:
            self.on_message(msg_type, msg_param1, msg_param2, msg_dict)
# ============================================================
    def on_message(self, msg_type, msg_param1, msg_param2, msg_dict):
        if msg_type == 'level_loading_complete':
            trace('DialogsMgr: "level_loading_complete" msg received')
            _load_dlg = self.get_dialog('LoadLevelMenu')
            if _load_dlg:
                trace('DialogsMgr: "level_loading_complete" signal added')
                _load_dlg.Signals.Add('level_loading_complete', 100)
            #if self.dialogs_stack:
            #    self.dialogs_stack[-1].Signals.Add('level_loading_complete',100)
            else:
                trace('LoadLevelMenu-dialog was not found. Unknown fatal error.')
# ============================================================
    def on_signal(self, signal):
        pass
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == 'enter':
            pass
        elif type == 'exit':
            pass
        elif type == 'change':
            self.on_state_change(additional.prev_state, additional.next_state)
        pass
# ============================================================
    def on_state_change(self, cur_state, next_state):
        pass
# ============================================================
    def on_create(self):
        '''Вызывается по окончанию загрузки'''
        #self.Open_dialog("BookMenu", "dlg_book")
        self.Open_dialog('MainMenu', 'dlg_main')
        #self.PMgr.init_user()
# ============================================================
    def on_update(self, ms):
        self.ticks += 1
        if not (self.ticks%50): # каждые полсекунды
            self.check_awaiting_dialogs()
        #if not (self.ticks%25):
        #    trace('>>> stuck control <<<')
# ============================================================
# ============================================================
# ============================================================
    # создает TinyPy враппер поверх C++ диалога:
    def Create_dlg_wrapper(self, py_name, cdata, params):
        #dlg = None
        if py_name not in dlg_name_to_class:
            py_name = ''
        dlg = dlg_name_to_class[py_name](cdata, params)
        dlg.on_create_now() # инициализация диалога
        if dlg not in self.non_collected_dialogs:
            self.non_collected_dialogs.append(dlg)       # чтобы GC не удалил наш диалог
        return dlg
# ============================================================
    def remove_dialog(self, dlg):
        self.non_collected_dialogs.remove(dlg)
# ============================================================
# ============================================================
    def can_show_single_dialog(self, cloud_important = False):
        '''Можно ли показать одиночный диалог, который не может отображаться поверх других диалогов'''
        if not self.dialogs_stack:
            if self.single_dialogs_allowed:
                _is_general_cloud_shown = False
                if cloud_important: _is_general_cloud_shown = self.IsGeneralCloudShown()
                if not _is_general_cloud_shown:
                    return True
        return False
# ============================================================
    def Open_dialog(self, xml_name, py_name, params = {}):
        '''Открывает диалог, предварительно проверяя
        возможно ли открыть его в данный момент'''
        if 'single_mode' in params:
            if params['single_mode'] == '1':
                _cloud_important = False
                if 'general_cloud' in params: _cloud_important = int(params['general_cloud'])
                _can_show_dlg_now = self.can_show_single_dialog(_cloud_important)
                if not _can_show_dlg_now: # если диалог не может показываться поверх других
                    a_dlg = {'xml_name':xml_name, 'py_name':py_name, 'params':params}
                    self.awaiting_dialogs_stack.append(a_dlg)    # нужно добавить данные диалога в список ожидающих
                    return
        # во всех остальных случаях просто открываем диалог:
        trace('Opening dialog:', xml_name, py_name, params)
        dlg = self.open_dialog_now(xml_name, py_name, params)
        trace('Opened dialog:', xml_name)
        return dlg
# ============================================================
    def open_dialog_now(self, xml_name, py_name, params):
        '''Открывает диалог немедленно без каких-либо доп. условий'''
        trace('open_dialog_now:', xml_name, py_name, params)
        dlg = self.CreateDlg(xml_name, py_name, params)
        dlg.xml_name = xml_name
        self.AddDialog(dlg, (dlg.size == 'fullscreen' or xml_name == 'StartMenu')) # добавляет диалог на экран
        self.dialogs_stack.append(dlg)
        self.DisableGameTips(True)  # отключим типсы
        self.DisableCameraMove()
        return dlg
# ============================================================
    def open_game(self, params = {}):
        pass
# ============================================================
    def restart_game(self, params = {}):
        pass
# ============================================================
    def Close_shop_dlg(self):
        '''Чтобы можно было закрыть диалог магазина из C++'''
        _shop = self.get_dialog('BuildingsMenu')
        if _shop:
            _shop.close_me()
# ============================================================
    def Close_dialog(self, dlg):
        trace('Dialog closed:', dlg.xml_name)
        self.CloseDialog(dlg)   # можно вызывать только в этом классе!
        self.dialogs_stack.remove(dlg)
        if dlg == self.back_dialog:
            self.back_dialog = None
        _dlg_count = len(self.dialogs_stack)
        if _dlg_count == 0:
            self.DisableGameTips(False)
        self.non_collected_dialogs.remove(dlg)
        self.check_awaiting_dialogs()
# ============================================================
    def Allow_single_dialogs(self, allow):
        '''Разрешает/запрещает показ одиночных диалогов'''
        self.single_dialogs_allowed = allow
        trace('Allow_single_dialogs', str(allow))
        trace('Allow_single_dialogs:', str(self.single_dialogs_allowed))
        if allow:
            self.check_awaiting_dialogs()   # проверим - нет ли ожидающих диалогов
# ============================================================
    def check_awaiting_dialogs(self):
        self.ticks = 0
        _can_show_single_dialog = self.can_show_single_dialog()
        if _can_show_single_dialog:  # на экране нет диалогов
            if self.awaiting_dialogs_stack:
                dlg_info = None
                index = 0
                dialogs_count = len(self.awaiting_dialogs_stack)
                while index < dialogs_count:
                    dlg_info = self.awaiting_dialogs_stack[index]
                    _cloud_important = False
                    if 'general_cloud' in dlg_info.params:
                        _cloud_important = int(dlg_info.params['general_cloud'])
                    if _cloud_important: # если облако генерала нужно учитывать
                        _is_general_cloud_shown = self.IsGeneralCloudShown()
                        if _is_general_cloud_shown: # если облако генерала показано
                            index += 1
                            continue
                    break
                if index < dialogs_count:
                    dlg_info = self.awaiting_dialogs_stack.pop(index)   # берем подходящий диалог
                    trace(dlg_info)
                    trace('Opening awaiting dialog:', dlg_info.xml_name, dlg_info.py_name, dlg_info.params)
                    self.open_dialog_now(dlg_info.xml_name, dlg_info.py_name, dlg_info.params)
                    trace('Opened awaiting dialog:', dlg_info.xml_name)
                    trace('Opened awaiting dialog params:', dlg_info.params)
        pass
# ============================================================
    def clear_awaitinig_dialogs(self):
        '''Чистит список ожидающих диалогов - нужно при любом выходе из игры'''
        trace('DialogsMgr: clear_awaitinig_dialogs')
        self.awaiting_dialogs_stack = []
# ============================================================
    def get_dialog(self, xml_name):
        _dialogs = []
        _dialogs.append(self.back_dialog)
        for _d in self.dialogs_stack:
            _dialogs.append(_d)
        for _d in _dialogs:
            if _d:
                if _d.xml_name == xml_name:
                    return _d
        return None
# ============================================================
    def prepare_serialization(self):
        # сохранить информацию об отложенных диалогах...
        for dlg in self.dialogs_stack:
            dlg.pre_serialize()
        return self.awaiting_dialogs_stack  # данные об отложенных диалогах
# ============================================================
    def after_serialization(self):
        DLG.StopGame()                  # теперь спокойно стопорим игру
        DLG.clear_awaitinig_dialogs()
# ============================================================
    def deserialization(self, awaiting_info):
        self.awaiting_dialogs_stack = awaiting_info
# ============================================================
    def Show_try_to_close_dlg(self):
        self.Open_dialog('ExitMenu', 'dlg_exit')
# ============================================================
    def Get_debug_dlg_info(self):
        r = []
        for dlg in self.dialogs_stack:
            if dlg:
                r.append(['text', 'COLOR_YELLOW'])
                r.append(['debug', dlg.xml_name])
                r.append('space')
                r.append(['digit', dlg.is_start_paused])
                r.append('newline')
        return r
# ============================================================
# ============================================================
# ============================================================
gcDLG = None
# ============================================================
def create_dlg_mgr(cdata, pmgr_cdata):
    gcDLG = DlgMgr(cdata, pmgr_cdata)
    return gcDLG
# ============================================================