# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import toolz

#@statemachine
class GameHelper(object):
    PAUSE_TIME = 10000
    achievements = {    # [index, (silver count, gold count), (silver score, gold score)]
        '_boards_reached': [0, (100, 300), (1000, 10000)],
        'chop_bush': [1, (30, 50), (1000, 5000)],
        'broken_burrel': [2, (10, 30), (500, 1000)],
        'tree_chop': [3, (100, 200), (10000, 50000)],
        'secret_room_event': [4, (1, 5), (100000, 500000)],
        'building_extinguished': [5, (5, 10), (100000, 500000)],
        'building_restore_eff': [6, (5, 10), (100000, 500000)],
        'building_removed': [7, (5, 10), (100000, 500000)],
        'bonus_used': [8, (3, 5), (100000, 500000)],
        'warehouse_builded': [9, (3, 5), (100000, 500000)],
        'barbarian_attack_stopped': [10, (3, 5), (100000, 500000)],
        'building_upgraded': [11, (15, 30), (100000, 500000)],

        'resources_changed': [-1, (0, 0), (0, 0)],           # for event check only

        #'broken_pot': [2, (10, 30), (1500, 3000)],
        #'gather_bush': [0, (5, 25), (500, 1000)],   # 1 V
        #'building_repaired': [6, (2, 4), (50, 100)],    # 7 V
        #'building_built': [6, (20, 35), (1000, 5000)],       # 8 V
        #'building_upgraded': [7, (5, 25), (3000, 10000)],    # 9 V
        #'_gold_reached': [9, (50, 70), (50, 100)],      # 10 V
        #'ind_task_complete': [10, (2, 4), (50, 100)],   # 11  V # алтари и тотемы не учитывать...
        #'trade_complete': [8, (5, 10), (1000, 5000)],      # 12 V
        #'secret_found': [9, (30, 107), (50000, 100000)],        # 13 V
        #'_survived_saved': [10, (7, 12), (10000, 50000)],     # 11 V
        #'_worker_added': [11, (22, 25), (100000, 500000)],     # 15 V
    }
# ============================================================
    def __init__(self):
        self.ser_name = 'GameHelper'
        self.need_deserialize_init_extra = True
        self.helper_item_uid = ''        # UID последнего сгенерированного объекта
        self.helper_bricks_item_uid = ''        # UID последнего сгенерированного объекта
        self.units_timer_started = False
        self.units_timer_time = 0
        self.units_msg_shown_timer = 0
        self.ticks = 0
        self.levels_settings = {}
        self.repeat_time = Globals.GInt('REPEAT_UNITS_MSG_TIME')
        self.gold_enough = False
        self.pause_time = self.PAUSE_TIME     # время, на которое паузится подбрасывание древесины
        self.bricks_pause_time = self.PAUSE_TIME     # время, на которое паузится подбрасывание кирпичей
        self.sounds_counters = {}
        self.safe_sounds_counters = {}
        self.achievements_info = {}
        self.is_paused = False
        for key in self.achievements:
            self.achievements_info[key] = [0, 0]  # count, current_value (0 - none, 2 - silver, 3 - gold)
        #_prizes_str = LBoard.GetUserInfo('prizes')
        #_prizes = _prizes_str.split('#')
        #_p_index = 0
        #for p in _prizes:
        for key in self.achievements:
            _info = self.achievements[key]
            _index = _info[0]
            #if _index == _p_index:
            p = LBoard.GetUserSetting('prize '+str(_index))     # текущая награда, сохраненная в профиле
            self.achievements_info[key][1] = int(p)  # награда, которая уже есть
        #    _p_index += 1
# ============================================================
    def deserialize_init_extra(self):
        #for snd_id in self.safe_sounds_counters:
        #    tools.PlaySoundCycled(snd_id)
        # Note: необходимо, чтобы при загрузке звуки не запускались до того, как
        # игрок закроет стартовый диалог
        self.is_paused = False
# ============================================================
    def on_create(self):
        self.levels_settings[0] = BMgr.GetLevelSettings(0)
        trace('Level settings(0):', self.levels_settings[0])
        #_secrets = LBoard.GetUserSetting('total_secrets_found ' +str(LBoard.episode) + ' ' + str(LBoard.level))
        #self.achievements_info['secret_found'][0] = _secrets   # чтобы очки лишний раз не накидывать
        trace('achievements_info:')
        for key in self.achievements_info:
            trace(key, '-', self.achievements_info[key])
# ============================================================
    def on_update(self, ms):
        '''Вызывается каждый второй quant! См. BaseManager.cpp::Update -> mgr_t.on_update'''
        if self.ticks > 1000000: self.ticks = 0
        self.ticks += 1
        if self.helper_item_uid == '':
            if self.pause_time > 0: self.pause_time -= ms   # для паузы генерации досок
        if self.helper_bricks_item_uid == '':
            if self.bricks_pause_time > 0: self.bricks_pause_time -= ms   # для паузы генерации досок
        map_index = BMgr.GetCurMapIndex()
        if not (self.ticks % 15):   #! проверяем каждые 0.333 секунды (10)
            if self.pause_time <= 0:
                if self.helper_item_uid == '':
                    self.try_generate_boards(map_index)
        if not (self.ticks % 15):   #! проверяем каждые 0.333 секунды (10)
            if self.bricks_pause_time <= 0:
                if self.helper_bricks_item_uid == '':
                    self.try_generate_bricks(map_index)
        # проверяем кол-во юнитов:
        if not (self.ticks % 10):       #! проверяем каждые 0.2 секунды (4)
            self.check_units_count(ms*10, map_index)
        if not (self.ticks % 50):   # раз в секунду
            _is_main_level = LBoard.Core.is_main_level()
            if _is_main_level:
                if not self.gold_enough:
                    if map_index in self.levels_settings:
                    #if self.levels_settings[map_index] != None:
                        res = UStats.GetCurRes()
                        _gold = res.Get('gold')
                        _need_gold = int(self.levels_settings[map_index].need_gold)
                        if _gold >= _need_gold and _need_gold > 0:
                            self.gold_enough = True
                            letter = [ ('text', 'MSG_GOLD_GATHERED') ]
                            EMgr.AppendLetter(letter, '', BMgr.GetCurMapIndex())
# ============================================================
    def check_units_count(self, ms, map_index):
        '''Проверяет - достаточно ли юнитов, если нет, то выводит подсказку о домиках'''
        if BMgr.CanBeBuilded('house1', map_index):     # если домики уже можно строить
            if not UMgr.is_free_real_units_enough(map_index):
                self.units_timer_started = True
            else:
                self.units_timer_started = False
                self.units_timer_time = 0
                self.units_msg_shown_timer = 0
            if self.units_timer_started:
                self.units_timer_time += ms
            if self.units_timer_time >= 17000:    # 17 сек (полсекунды, чтобы не ноль)
                if self.units_msg_shown_timer > 0 and self.units_msg_shown_timer < self.repeat_time:
                    self.units_msg_shown_timer += ms
                elif self.units_msg_shown_timer <= 0:
                    # показать подсказку
                    if map_index in self.levels_settings:
                        has_house_in_building = BMgr.Has_buildings_in_build('house1', map_index)
                        if not has_house_in_building:
                            _lev_settings = self.levels_settings[map_index]
                            if 'level_type' in _lev_settings:
                                if _lev_settings.level_type != 'underlevel':
                                    UI.show_top_text('HELPER_UNITS', '', True)
                            else:
                                UI.show_top_text('HELPER_UNITS', '', True)
                    self.units_msg_shown_timer += ms
                else:
                    self.units_msg_shown_timer = 0
                    self.units_timer_time = 0
# ============================================================
    def on_event(self, event, extra):
        #perf_begin('game_helper::on_event')
        if event == 'unit_res_taken':
            if self.helper_item_uid == extra.uid:
                self.helper_item_uid = ''
            elif self.helper_bricks_item_uid == extra.uid:
                self.helper_bricks_item_uid = ''
        # -------
        elif event in self.achievements:# or event == 'resources_changed':
            #if tools.IsDebug(): #@temporary
            #    return
            _count_and_medal = None
            _ids = []
            if event == 'resources_changed':
                #for 'boards'
                _count_and_medal = self.achievements_info['_boards_reached']
                _cur_res = UStats.GetCurRes()
                _start_boards = int(LBoard.Core.get_setting('boards'))
                _val = _cur_res.Get('boards') - _start_boards
                _cur_val = _count_and_medal[0]
                if _val > _cur_val:
                    _ids.append( ['_boards_reached', _val] )
                #for 'gold'
                #_count_and_medal = self.achievements_info['_gold_reached']
                #_cur_res = UStats.GetCurRes()
                #_val = _cur_res.Get('gold')
                #_cur_val = _count_and_medal[0]
                #if _val > _cur_val:
                #    _ids.append( ['_gold_reached', _val] )
            else:
                _count_and_medal = self.achievements_info[event]
                _val = 0
                if event == '_worker_added' or event == '_survived_saved':
                    if event == '_survived_saved':
                        _val = UStats.GetLevelValue('survived')
                    elif event == '_worker_added':
                        _max = 0
                        for key in UMgr.clusters:
                            _tmp = UMgr.Get_free_real_units_count(key)
                            if _tmp > _max: _max = _tmp
                        _val = _max
                else:
                    _val = _count_and_medal[0] + 1
                if event == 'secret_found':
                    _val = LBoard.GetUserSetting('total_secrets_found ' +str(LBoard.episode) + ' ' + str(LBoard.level))
                    _val += UStats.GetLevelValue('secrets')
                elif event == 'secret_room_event':
                    UStats.AddLevelValue('secret_rooms', 1)
                    _val = LBoard.GetUserSetting('total_secret_rooms ' +str(LBoard.episode) + ' ' + str(LBoard.level))
                    _val += UStats.GetLevelValue('secret_rooms')
                _ids.append( [event, _val] )
            # проверки на достижение ачивки
            for c_m in _ids:
                _id = c_m[0]
                _new_count = c_m[1]
                _count_and_medal = self.achievements_info[_id]
                _prev_count = _count_and_medal[0]
                _info = self.achievements[_id]
                _need = _info[1][1]
                if _new_count >= _need:
                    if _prev_count < _need:
                        # add new medal and score
                        self.add_medal_and_score(_id, 3, 1) # gold
                else:
                    _need = _info[1][0]
                    if _new_count >= _need:
                        if _prev_count < _need:
                            # add new medal and score
                            self.add_medal_and_score(_id, 2, 0)     # silver
                _count_and_medal[0] = _new_count
        #perf_end('game_helper::on_event')
# ============================================================
    def add_medal_and_score(self, _id, medal_type, score_index):
        _info = self.achievements[_id]
        _cur_info = self.achievements_info[_id]
        _index = int(_info[0])
        _medal = _cur_info[1]
        #mbox(_id + '1 - ' + str(_index) + ' ' + str(medal_type) + ' ' + str(_medal))
        if medal_type > _medal:
            _cur_info[1] = medal_type
            # добавить данные в PMgr
            LBoard.SetUserSetting('prize '+str(_index), medal_type)
            dlg.Open('PrizesMenu', 'dlg_prizes', {'medal':str(_index), 'medal_type':str(medal_type), 'showin_effect':'prizes_fly_in', 'single_mode':'1'})
        _score = _info[2][score_index]
        UStats.SetLevelValue('EXTRA_SCORE', _score)
# ============================================================
    def add_score(self, _score):
        pass
# ============================================================
    def on_global_event(self, event, param):
        if event == 'new_location_loaded':
            self.on_new_location_loaded(int(param))
        elif event == 'location_switched':
            self.on_location_switched(int(param))

        if LBoard.episode == 2:
            if LBoard.level == 5:
                if event == 'time_event':
                    _is_gone = UStats.GetLevelValue('time_50_gone') + \
                                    UStats.GetLevelValue('time_75_gone') + \
                                    UStats.GetLevelValue('time_30s')
                    if _is_gone > 0:
                        UI.set_competitor_half(True)
                elif event == 'enemy_gone':
                    UI.set_competitor_half(True)
# ============================================================
    def on_location_switched(self, subindex):
        pass
# ============================================================
    def on_new_location_loaded(self, subindex):
        self.levels_settings[subindex] = BMgr.GetLevelSettings(subindex)
        trace('Level settings('+str(subindex)+'):', self.levels_settings[subindex])
# ============================================================
    wood_help_shown = False     # была ли подсказка на первые подброшенные дрова?
    def try_generate_boards(self, map_index):
        self.helper_item_uid = ''
        _woodcutters = BMgr.get_buildings_simple_allmaps(['woodcutter', 'forester'])
        for b in _woodcutters:
            _has_access = b.HasAccess() # если есть доступные/незаброшенные лесорубы
            if _has_access:   # если здание доступно
                if not b.lock_shown:
                    return
        self.pause_time = self.PAUSE_TIME
        _total = UStats.GetCurRes()
        _total_boards = _total.Get('boards')    # сколько всего древесины
        if _total_boards < 15:
            self.generate_boards(map_index)  # генерим ресурс
# ============================================================
    def get_maximum_index_item(self, _generators):
        pass
# ============================================================
    def generate_boards(self, map_index):
        self.pause_time = self.PAUSE_TIME
        _generators = BMgr.get_items_by_subtype(map_index, 'boards_helper', False, True)
        _gen_count = len(_generators)
        if _gen_count > 0:
            _max_index = -1
            _generator = None
            for _g in _generators:
                _index = _g.GetAttrInt('index')
                if _index > _max_index:
                    _max_index = _index
                    _generator = _g
            #index = tools.Rand(0, _gen_count-1)
            #_generator = _generators[index]
            _items = _generator.generate_res()
            if _items:
                _item = _items[0]
                self.helper_item_uid = _item.get('uid')
                if not self.wood_help_shown:
                    if LBoard.episode <= 1:
                        UI.show_top_text('WOOD_HELPER_GENERATED', _generator.get('uid'))
                    elif LBoard.episode == 2:
                        UI.show_top_text('WOOD_HELPER_GENERATED_2', _generator.get('uid'))
                    self.wood_help_shown = True
                    Camera.MoveTo(_item)
                    _item.show_arrow()
# ============================================================
    def _has_upgrading_towers(self, map_index):
        for tower in BMgr.get_cluster(map_index).all_buildings:
            _stype = tower.get('stype')
            if _stype == 'watchtower':
                if tower.status != 'map_place':
                    _is_active_raw = tower.is_active_raw()
                    if _is_active_raw:
                        #if tower.status == 'broken':
                        if tower.clock_shown:
                            return True
                        _is_builders_state = tower.is_builders_state()
                        if _is_builders_state:
                            return True
        return False
# ============================================================
    bricks_help_shown = False     # была ли подсказка на первые подброшенные кирпичи?
    _granitemine_list = ['granitemine']
    _quarry_list = ['quarry']
    def try_generate_bricks(self, map_index):
        # закомментил, потому как на начало уровня строить действительно еще нельзя, но подкидывание уже нужно
        #if not BMgr.CanBeBuilded('granitemine1'):    # если нельзя его строить, то
        #    return                                 # сразу выходим
        #if self.helper_bricks_item_uid and BMgr.get_item(self.helper_bricks_item_uid):    # если сгенерированный элемент еще на поле
        #    return
        # ---
        self.helper_bricks_item_uid = ''
        _items = BMgr.get_buildings_simple(map_index, self._granitemine_list)
        for b in _items:
            _has_access = b.HasAccess()
            if _has_access:   # если здание доступно
                return
        _items = BMgr.get_items_by_subtype(map_index, 'granite')
        _has_granites = False
        for b in _items:
            _has_access = b.HasAccess()
            if _has_access:   # если ресурс доступен
                _has_granites = True
                break
        if _has_granites:   # если есть ресурсы, то нужно проверить наличие quarry
            _items = BMgr.get_buildings_simple(map_index, self._quarry_list)
            for b in _items:
                _has_access = b.HasAccess()
                if _has_access:   # если здание доступно
                    if not b.lock_shown:    # и не заброшено
                        return
        self.bricks_pause_time = self.PAUSE_TIME
        _total = UStats.GetCurRes()
        _total_bricks = _total.Get('bricks')    # сколько всего древесины
        if _total_bricks < 40:
            self.generate_bricks(map_index)  # генерим ресурс
# ============================================================
    def generate_bricks(self, map_index):
        self.bricks_pause_time = self.PAUSE_TIME * 2    # дольше проверка должна быть
        _generators = BMgr.get_items_by_subtype(map_index, 'bricks_helper', False, True)
        _gen_count = len(_generators)
        if _gen_count > 0:
            _has_up_towers = self._has_upgrading_towers(map_index)   # если башня ремонтируется или апгрейдится, то камни не подбрасываем
            if _has_up_towers:
                return
            _max_index = -1
            _generator = None
            for _g in _generators:
                _index = _g.GetAttrInt('index')
                if _index > _max_index:
                    _max_index = _index
                    _generator = _g
            #index = tools.Rand(0, _gen_count-1)
            #_generator = _generators[index]
            _items = _generator.generate_res()
            if _items:
                _item = _items[0]
                self.helper_bricks_item_uid = _item.get('uid')
                if not self.bricks_help_shown:
                    if LBoard.episode <= 1:
                        UI.show_top_text('BRICKS_HELPER_GENERATED', _generator.get('uid'))
                    elif LBoard.episode == 2:
                        UI.show_top_text('BRICKS_HELPER_GENERATED_2', _generator.get('uid'))
                    self.bricks_help_shown = True
                    Camera.MoveTo(_item)
                    _item.show_arrow()
# ============================================================
    def PlaySoundCounted(self, snd_id):
        if snd_id not in self.sounds_counters:
            self.sounds_counters[snd_id] = 0
        if self.sounds_counters[snd_id] == 0:
            tools.PlaySoundCycled(snd_id)
        self.sounds_counters[snd_id] += 1
        #mbox('on ' + str(self.sounds_counters[snd_id]))
# ============================================================
    def StopSoundCounted(self, snd_id):
        if snd_id not in self.sounds_counters:
            return
        self.sounds_counters[snd_id] -= 1
        _count = self.sounds_counters[snd_id]
        if _count <= 0:
            tools.StopSoundCycled(snd_id)
        #mbox('off ' + str(self.sounds_counters[snd_id]))
# ============================================================
    def PlaySoundCountedSafe(self, c_uid, snd_id):
        if snd_id not in self.safe_sounds_counters:
            callers = {}
            counter = 0
            muted = False
            self.safe_sounds_counters[snd_id] = (callers, counter, muted)
        counter_info = self.safe_sounds_counters[snd_id]
        if c_uid in counter_info[0]:
            return
        counter_info[0][c_uid] = 0         # in callers
        if counter_info[1] == 0:            # counter
            tools.PlaySoundCycled(snd_id)
        counter_info[1] += 1                # counter
        #mbox('on ' + str(self.sounds_counters[snd_id]))
# ============================================================
    def StopSoundCountedSafe(self, c_uid, snd_id, need_fade = True):
        if snd_id not in self.safe_sounds_counters:
            return
        counter_info = self.safe_sounds_counters[snd_id]
        if c_uid not in counter_info[0]:    # in callers
            return
        counter_info[1] -= 1                # counter
        del counter_info[0][c_uid]          # in callers
        _count = counter_info[1]            # counter
        if _count <= 0:
            if need_fade:
                tools.StopSoundCycled(snd_id)
            else:
                tools.StopSoundCycledFast(snd_id)
            counter_info[0] = {}            # in callers
# ============================================================
    def mute_safe_sounds(self, extra_id, mute):
        for snd_id in self.safe_sounds_counters:
            counter_info = self.safe_sounds_counters[snd_id]
            if extra_id in counter_info[0]:
                if counter_info[2] != mute:
                    counter_info[2] = mute
                    if counter_info[1] > 0:
                        if mute:
                            tools.StopSoundCycledFast(snd_id)
                        else:
                            tools.PlaySoundCycled(snd_id)
                #mute/unmute
# ============================================================
    def inform_pause_state(self, is_enabled):
        if is_enabled == self.is_paused:
            return      # skip same calls
        if is_enabled:
            for snd_id in self.safe_sounds_counters:
                counter_info = self.safe_sounds_counters[snd_id]
                if counter_info[2] != True:
                    if counter_info[1] > 0:
                        tools.StopSoundCycledFast(snd_id)
        else:
            for snd_id in self.safe_sounds_counters:
                counter_info = self.safe_sounds_counters[snd_id]
                if counter_info[2] != True:
                    if counter_info[1] > 0:
                        tools.PlaySoundCycled(snd_id)
        self.is_paused = is_enabled
# ============================================================
