# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_build
import task_build_place
import smgr_bonus
import dependant_mgr
# ============================================================
# ============================================================
class BActionsHistory(object):
    def __init__(self):
        self.ser_name = 'BActionsHistory'
        self.sublevels_actions = {}

    def add_action(self, map_index, uid, action):
        if map_index not in self.sublevels_actions:
            self.sublevels_actions[map_index] = {}
        _actions = self.sublevels_actions[map_index]
        if uid not in _actions:
            _actions[uid] = {}
        _uid_actions = _actions[uid]
        _uid_actions[action] = True
# ============================================================
    def has_action(self, map_index, uid, action):
        if map_index not in self.sublevels_actions:
            return False
        _actions = self.sublevels_actions[map_index]
        if uid not in _actions:
            return False
        _uid_actions = _actions[uid]
        if action not in _uid_actions:
            return False
        return True
# ============================================================
class BuildingsContainer(object):
    def __init__(self):
        self.ser_name = 'BuildingsContainer'
        self.arrays = {}

    def append(self, building):
        if not building: return
        stype = building.get('stype')
        if stype not in self.arrays:
            self.arrays[stype] = []
        self.arrays[stype].append(building)

    def remove(self, building):
        if not building: return
        stype = building.get('stype')
        if stype not in self.arrays:
            return
        self.arrays[stype].remove(building)

    def get(self, stype):
        if not stype: return []
        if stype not in self.arrays:
            return []
        return self.arrays[stype]

    def reset(self):
        self.arrays = {}
# ============================================================
# ============================================================
class LevelCluster(object):
    def __init__(self, index = 0):
        self.ser_name = 'LevelCluster'
        self.cur_index = index
        self.all_buildings = []
        self.FBC = BuildingsContainer()
        self.all_units = []
        self.all_items = []
        self.uids_dict = {}
        self.nests = []
# ============================================================
    def reset(self):
        del self.all_buildings[:]
        del self.all_units[:]
        del self.all_items[:]
        self.uids_dict = {}
        del self.nests[:]
        self.FBC.reset()
# ============================================================
# ============================================================
# ============================================================
# ============================================================
class BuildingsMgr(BuildingsMgr_class):
    #all_buildings = []
    #FBC = None
    #all_units = []
    #all_items = []
    #uids_dict = {}
    #nests = []
    _broken__map_place = ['broken', 'map_place']
    _bad_states = {'removing':0, 'removing_final':0, 'map_place':0, 'thrown':0}
# ============================================================
    def __init__(self, cdata):
        BuildingsMgr_class.__init__(self, cdata)
        self.ser_name = 'BuildingsMgr'
        self.AddSelfToGlobal('BMgr')
        self.bonus_mgr = smgr_bonus.BonusMgr()
        #self.bomb_mgr = BombMgr()
        self.all_levels_units = []
        self.all_levels_buildings = []
        self.all_levels_items = []
        self.clusters = {0:LevelCluster(0)}
        self.cur_cluster = self.clusters[0]
        self.cur_map_index = 0
        self.dependant_buildings = dependant_mgr.DependantBuildingMgr()
        self.actions_history = BActionsHistory()
        self.columns = {}
# ============================================================
    def deserialize_init(self):
        self.init_listeners()
# ============================================================
# ============================================================
    def reset(self):
        for key in self.clusters:
            self.clusters[key].reset()
        del self.all_levels_buildings[:]
        del self.all_levels_units[:]
        del self.all_levels_items[:]
        self.clusters = {0:LevelCluster(0)}
        self.cur_cluster = self.clusters[0]
        self.bonus_mgr.reset()
# ============================================================
    def get_cluster(self, map_index):
        if map_index in self.clusters:
            return self.clusters[map_index]
        else:
            return None
# ============================================================
    def switch_cluster(self, new_location_index):
        if new_location_index in self.clusters:
            self.cur_cluster = self.clusters[new_location_index]
            self.cur_map_index = new_location_index
# ============================================================
    def Reset_cache(self):
        for b in self.cur_cluster.all_buildings:
            b.reset_cache()
# ============================================================
    def transfer_unit(self, unit, dest_map_index):
        _cur_mi = unit.get('locind')
        _src = self.get_cluster(_cur_mi)
        _dest = self.get_cluster(dest_map_index)
        _uid = unit.get('uid')
        UMgr.transfer_unit(unit, dest_map_index)
        _src.all_units.remove(unit)
        if _uid in _src.uids_dict:
            del _src.uids_dict[_uid]
        # ---
        trace('Transfering unit from', _cur_mi, 'to', dest_map_index, _uid, unit.get('stype'))
        BMgr.TransferUnit(unit, dest_map_index)
        _dest.all_units.append(unit)
        _dest.uids_dict[unit.get('uid')] = unit # здесь UID будет уже другой
# ============================================================
    def on_msg(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'global_event':
            self.on_global_event(msg_param1, msg_param2)
            self.bonus_mgr.on_global_event(msg_param1, msg_param2)
        elif msg_type == 'on_create':
            self.on_create()
            self.bonus_mgr.on_create()
        elif msg_type == 'state_machine':
            self.on_state_machine(msg_param1, msg_param2, msg_dict)
        elif msg_type == 'signal_msg':
            if msg_param1.startswith(self.bonus_mgr.prefix):
                self.bonus_mgr.on_signal(msg_param1[self.bonus_mgr.prefix_len:])
            else:
                self.on_signal(msg_param1)
        elif msg_type == 'refresh_next_res': # аналогично реагируем и при изменении ресуров в on_event
            # обновить переменные всех зданий
            #for b in self.all_levels_buildings:
            #    b.SendMsg('refresh_next_res')
            self.refresh_buildings_next_res(True)
# ============================================================
    def refresh_buildings_next_res(self, partial = False):
        #for b in self.all_levels_buildings:
        for b in self.cur_cluster.all_buildings:    # обновляем только текущий подуровень - другие обновим при переходе
            if not b.is_offscreen:
                b.SendMsg('refresh_next_res')
            elif partial:
                b.SendMsg('refresh_next_res_partial')
# ============================================================
    def refresh_items_arrows(self):
        #for b in self.all_levels_buildings:
        for item in self.all_levels_items:
            if item.arrow_shown:
                item.refresh_arrow_color()
# ============================================================
    def on_signal(self, signal):
        if signal == 'exit_menu':
            dlg.Open('ExitMenu', 'dlg_exit')
        elif signal == 'complete_init':
            self.CompleteInitialize()   #@!!! если время где-то затормозится, то пиши пропало! (Теперь вызывается только для главного уровня...)
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == 'enter':
            return
        elif type == 'exit':
            return
        elif type == 'change':
            self.on_state_change(additional.prev_state, additional.next_state)
# ============================================================
    def on_state_change(self, cur_state, next_state):
        #trace(cur_state.append(next_state))
        return
# ============================================================
    def Add_build_item(self, map_index, template_name, cell):
        result = TMgr.add_task( task_build.BuildingTask('build_building', map_index, template_name, 'place', cell ) )
        if result:
            UStats.ApplyReserveRes()
# ============================================================
    def Build_fixed_item(self, map_place_element):
        result = TMgr.add_task( task_build_place.BuildingPlaceTask('build_building', map_place_element ) )
        if result:
            UStats.ApplyReserveRes()
# ============================================================
    def cancel_build_mode(self):
        UStats.UndoReserveRes()
# ============================================================
    def on_create(self):
        trace('BuildingsManager on_create')
        self.init_listeners()
        #self.nests = []
        #for item in self.cur_cluster.all_items: # выгребаем все гнезда варваров
        #    _stype = item.get('stype')
        #    if _stype == 'nest':
        #        self.cur_cluster.nests.append(item)
        _block_remove = LBoard.Core.get_setting('block_remove')
        _block_restore_eff = LBoard.Core.get_setting('block_restore_eff')
        if _block_remove in ['1', 'true']:
            UStats.SetLevelValue('BLOCK_REMOVE', 1)
        if _block_restore_eff in ['1', 'true']:
            UStats.SetLevelValue('BLOCK_RESTORE_EFF', 1)
        _free_water = LBoard.Core.get_setting('free_water')
        if _free_water in ['1', 'true', 't']:
            UStats.SetLevelValue('FREE_WATER', 1)
        LBoard.Core.refresh_nighbors_exits()
# ============================================================
    def _generate_level_bombs(self, _bombs_param, map_index):
        '''count1:uid,uid,uid;count2:uid,uid'''
        _param_len = len(_bombs_param)
        if _param_len == 0:
            return
        if ':' not in _bombs_param:
            return
        groups = _bombs_param.split(';')
        count = 0
        uids_str = None
        parts = None
        for group in groups:
            group = group.strip()
            parts = group.split(':')
            count = int(parts[0].strip())    # trim whitespace
            uids_str = parts[1]
            uids = uids_str.split(',')
            uids_count = len(uids)
            for i in range(uids_count):
                uids[i] = uids[i].strip()    # trim whitespace
            self._generate_group_bombs(count, map_index, uids)
# ============================================================
    def _generate_group_bombs(self, count, map_index, uids):
        #trace('_generate_group_bombs:', count, map_index, uids)
        if count == 0:
            return
        _explodes = self.get_items_by_uids_list(uids, map_index)
        _len = len(_explodes)
        for i in range(_len-1,-1,-1):    # reverse
            #shuffle:
            j = int( tools.random() * (i+1) )
            if i != j:
                _explodes[i], _explodes[j] = _explodes[j], _explodes[i]
        count = min(_len, count)
        for i in range(count):
            _explodes[i].contains_bomb = True
# ============================================================
    def init_listeners(self):
        EMgr.add_concrete_event_listener('game_win', self.on_game_win)
        EMgr.add_concrete_event_listener('resources_changed', self.on_resources_changed)
        EMgr.add_concrete_event_listener('gems_changed', self.on_gems_changed)
        EMgr.add_concrete_event_listener('time_is_up', self.on_time_is_up)

        EMgr.add_concrete_event_listener('location_switched', self.on_location_switched)
        #EMgr.add_concrete_event_listener('new_location_loading', self.on_new_location_loading)
        #EMgr.add_concrete_event_listener('new_location_loaded', self.on_new_location_loaded)
        EMgr.add_concrete_event_listener('location_loaded', self.on_location_loaded)
# ============================================================
#    def on_event(self, name, obj, extra):
#        '''Обработчик игровых событий, позже перенести в LevelBoard'''
#        pass
# ============================================================
    def on_game_win(self, event):
        '''Обработчик игровых событий, позже перенести в LevelBoard'''
        # отменить режим строительства если включен
        dlg.Open('StatsMenu', 'dlg_stats', {'single_mode':'1', 'final':''})

    def on_resources_changed(self, event):
        # обновить переменные всех зданий
        #for b in self.all_levels_buildings:
        #    b.SendMsg('refresh_next_res')
        self.refresh_buildings_next_res()

    def on_gems_changed(self, event):
        self.refresh_items_arrows()

    def on_time_is_up(self, event):
        self.disable_time_altars()

# ============================================================
    def on_global_event(self, event, param):
        #if event == 'location_switched':
        #    self.on_location_switched(int(event.param))
        if event == 'new_location_loading':
            self.on_new_location_loading(int(param))
        #elif event == 'new_location_loaded':
        #    self.on_new_location_loaded(int(event.param))
# ============================================================
    def on_location_switched(self, event):
        subindex = int(event.extra.param)
        if subindex != self.cur_map_index:
            self.switch_cluster(subindex)
            self.refresh_buildings_next_res(True)
# ============================================================
    def on_new_location_loading(self, subindex):
        self.clusters[subindex] = LevelCluster(subindex)
# ============================================================
#    def on_new_location_loaded(self, event):
#        subindex = int(event.extra.param)
#        _cluster = self.clusters[subindex]
#        for item in _cluster.all_items: # выгребаем все гнезда варваров
#            _stype = item.get('stype')
#            if _stype == 'nest':
#                _cluster.nests.append(item)
#        self.refresh_buildings_next_res(True)
#        # ---
#        _bombs_params_str = LBoard.Core.get_setting('bombs_params', subindex)
#        trace('Bombs generation ('+str(subindex)+'):', _bombs_params_str)
#        self._generate_level_bombs(_bombs_params_str)
# ============================================================
    def on_location_loaded(self, event):
        subindex = int(event.extra.param)
        _cluster = self.clusters[subindex]
        for item in _cluster.all_items: # выгребаем все гнезда варваров
            _stype = item.get('stype')
            if _stype == 'nest':
                _cluster.nests.append(item)
        self.refresh_buildings_next_res(True)
        # ---
        _bombs_params_str = LBoard.Core.get_setting('bombs_params', subindex)
        trace('Bombs generation ('+str(subindex)+'):', _bombs_params_str)
        self._generate_level_bombs(_bombs_params_str, subindex)
# ============================================================
    def on_update(self, ms):                # данный метод должен вызываться из C++ каждый квант
        pass
# ============================================================
    def on_rare_update(self, ms):
        for nest in self.cur_cluster.nests:
            nest.on_rare_update(ms)
# #######################################################
# #######################################################
# #######################################################
    def find_nearest_unit_building(self, target, cell_shift_name = '', with_units = True, vacant_only = False):
        #perf_begin('find_nearest_unit_building')
        buildings = self.GetHomeBuildings(target.get('locind'), with_units, vacant_only) # true - with free units
        pathes = []
        cells = [b.start_cell_pos for b in buildings]
        #for b in buildings:
        #    cells.append(b.start_cell_pos)
        result_index = BMgr.FindNearestCell(target, cell_shift_name, cells)
        if result_index >= 0:  # найдем здание по ячейке
            return buildings[result_index]
        trace('find_nearest_unit_building: no pathes')
        trace(result_index)
        #perf_end('find_nearest_unit_building')
        return None
# ============================================================
    _removing_fsm_states = ['preremoving', 'removing', 'removing_final', '_dead_']
    def get_buildings(self, map_index, subtype, for_task_type = '', skip_status = []):
        '''Возвращает список зданий указанного типа, причем здания могут принимать ресурсы (и есть другие доп. условия).
            Также может возвращать спец.типы: __store__ - хранилища ресурсов, __home__ - дома с вакантными местами, __main__ - главное здание'''
        #perf_begin('get_buildings')
        _is_broken_in = 'broken' in skip_status
        result = []
        cluster = self.get_cluster(map_index)
        if subtype == '__store__':
            for building in cluster.all_buildings:
                _is_store = building.is_store()
                if _is_store:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        #_is_broken = False
                        _fsm_state = building.FSM_GetStateID()
                        if _fsm_state in self._removing_fsm_states:
                            continue
                        if building.status in skip_status:
                            #_is_broken = True
                            continue
                        elif _is_broken_in:
                            if building.is_under_attack:
                                #_is_broken = True
                                continue
                        _stype = building.get('stype')
                        if _stype not in self._hq_list:
                            if not building.worker_units:
                                #_is_broken = True       # потому что внутри еще нет юнита
                                continue
                        #if not _is_broken:
                        result.append(building)
            trace('STORE BUILDINGS: >>>')
            for b in result:    trace(b.get('name'))
            trace('STORE BUILDINGS <<<')
        elif subtype == '__home__':
            for building in cluster.all_buildings:
                _is_home = building.is_home()
                if _is_home:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        _is_broken = False
                        if building.status in skip_status: _is_broken = True
                        elif _is_broken_in:
                            if building.is_under_attack: _is_broken = True
                        if not _is_broken:
                            _vacant_places_count = building.Get_vacant_places_count()
                            if _vacant_places_count:  # если есть свободные места
                                result.append(building)
        elif subtype == '__main__': # возвращает главное здание, но в списке
            for building in cluster.all_buildings:
                _stype = building.get('subtype')
                if _stype in self._hq_list:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        # не бывает поломанным
                        #_is_broken = False
                        #if building.status in skip_status: _is_broken = True
                        #elif _is_broken_in:
                        #    if building.is_under_attack: _is_broken = True
                        #if not _is_broken:
                            result.append(building)
        elif subtype == '__generating__':   # здания, которые могут производить ресурсы
            # вообще данная возможно здесь - это больше ХАК, потому как условия отбора здесь сильно отличаются от других вариантов
            for building in cluster.all_buildings:
                _need_base_resource = building.need_base_resource()
                if _need_base_resource:
                    _is_active_raw = building.is_active_raw()
                    if _is_active_raw:
                        if building.status not in skip_status:
                            #if building.status == 'normal': # если здание не сломано и не в режиме постройки
                                #_state = building. SM .GetCurrentState()
                                #if _state in ['', 'normal']: # и не апгрейдится
                                #_is_participating = building.is_task_participating(for_task_type)
                                #if not _is_participating:    # не генерирует ресурс и не является участником такого же задания
                                result.append(building)
        else:
            _buildings = []
            if subtype == '':
                _buildings = cluster.all_buildings
            else:
                _buildings = cluster.FBC.get(subtype)
            for building in _buildings:
                _is_active_raw = building.is_active_raw()
                if _is_active_raw:
                    #if building.status not in skip_status:
                        if building.status == 'normal': # если здание не сломано и не в режиме постройки
                            _is_normal_state = building.is_normal_state()
                            if _is_normal_state: # и не апгрейдится
                                _is_participating = building.is_task_participating(for_task_type)
                                if not _is_participating:    # не генерирует ресурс и не является участником такого же задания
                                    result.append(building)
        #perf_end('get_buildings')
        return result
# ============================================================
    inactive_fsm_state = {'removing':0, 'removing_final':0, 'broken':0, 'map_place':0, 'thrown':0}
    def get_buildings_raw(self, map_index, subtypes):
        '''Возвращает список зданий указанного типа, c меньшим числом доп. условий
            Также может возвращать спец.типы: __store__ - хранилища ресурсов, __home__ - дома с вакантными местами, __main__ - главное здание'''
        if not subtypes:
            return []
        #perf_begin('get_buildings_raw')
        _is_string = istype(subtypes, 'string')
        if _is_string: subtypes = [subtypes]
        _dict_stypes = {}
        for stype in subtypes: _dict_stypes[stype] = 0  # оптимизация для поиска
        result = []
        first = subtypes[0]
        clusters = []
        if map_index < 0:   # пробежать все кластеры
            for k in self.clusters: clusters.append(self.clusters[k])
        else:
            clusters.append(self.get_cluster(map_index))
        for cluster in clusters:
            if first == '__store__':
                for building in cluster.all_buildings:
                    _is_store = building.is_store()
                    if _is_store:
                        _is_active_raw = building.is_active_raw()
                        if _is_active_raw:
                            result.append(building)
            elif first == '__home__':
                for building in cluster.all_buildings:
                    _is_home = building.is_home()
                    if _is_home:
                        _is_active_raw = building.is_active_raw()
                        if _is_active_raw:
                            result.append(building)
            elif first == '__main__': # возвращает главное здание, но в списке
                for building in cluster.all_buildings:
                    if building.get('subtype') in self._hq_list:
                        _is_active_raw = building.is_active_raw()
                        if _is_active_raw:
                            result.append(building)
            else:
                for building in cluster.all_buildings:
                    _stype = building.get('subtype')
                    if _stype in _dict_stypes:
                        if building.status not in self._broken__map_place: # если здание не сломано и это не положение на карте
                            _fsm_state = building.FSM_GetStateID()
                            if _fsm_state not in self.inactive_fsm_state:
                                _is_active_raw = building.is_active_raw()
                                if _is_active_raw:
                                    result.append(building)
        #perf_end('get_buildings_raw')
        return result
# ============================================================
    def get_hospitals_simple_allmaps(self):
        _exclude_levels = LBoard.Core.get_hospitals_skip_levels()
        cur_mapindex = self.GetCurMapIndex()
        skip_only = LBoard.Core.get_hospitals_skip_only_values(cur_mapindex)
        _skip_mapindex = skip_only[0]   # если нужно исключить один из подуровней
        _only_mapindex = skip_only[1]   # если проверять нужно только один подуровень
        if _only_mapindex  >= 0:
            return self.get_buildings_simple(_only_mapindex, 'hospital')
        result = []
        for index in self.clusters:
            if index != _skip_mapindex:
                _r = self.get_buildings_simple(index, 'hospital')
                result.extend(_r)
        return result
# ============================================================
    def get_buildings_simple_allmaps(self, subtypes):
        '''Аналогично get_buildings_simple, но для всех подуровней'''
        result = []
        for index in self.clusters:
            _r = self.get_buildings_simple(index, subtypes)
            result.extend(_r)
        return result
# ============================================================
    _hq_list = ['hq', 'hq_small']
    def get_buildings_simple(self, map_index, subtypes):
        '''Возвращает список зданий указанного типа, вообще без доп. условий
            Также может возвращать спец.типы: __store__ - хранилища ресурсов, __home__ - дома, __main__ - главное здание'''
        if not subtypes:
            return []
        #perf_begin('get_buildings_simple')
        #---
        _is_string = istype(subtypes, 'string')
        if _is_string: subtypes = [subtypes]
        #---
        _dict_stypes = {}
        for stype in subtypes: _dict_stypes[stype] = 0  # оптимизация для поиска
        result = []
        first = subtypes[0]
        cluster = self.get_cluster(map_index)
        if first == '__store__':
            for building in cluster.all_buildings:
                _is_store = building.is_store()
                if _is_store:
                    result.append(building)
        elif first == '__home__':
            for building in cluster.all_buildings:
                _is_home = building.is_home()
                if _is_home:
                    result.append(building)
        elif first == '__main__': # возвращает главное здание, но в списке
            for building in cluster.all_buildings:
                _stype = building.get('subtype')
                if _stype in self._hq_list:
                    result.append(building)
        else:
            for building in cluster.all_buildings:
                _stype = building.get('subtype')
                if _stype in _dict_stypes:
                    result.append(building)
        #perf_end('get_buildings_simple')
        return result
# ============================================================
    def get_items_by_subtype(self, map_index, subtype, skip_activity = False, only_accessible = False, by_units = False):
        '''Возвращает DiffItem'ы по их subtype'''
        #perf_begin('get_items_by_subtype')
        result = []
        cluster = self.get_cluster(map_index)
        if only_accessible:
            #for item in cluster.all_items:
            def func_accessible(item, params):
                if item:
                    our_stype = item.IsSubTypeOf(params.subtype)
                    if our_stype:
                        _acceptable = True
                        if not params.skip_activity:
                            _is_active = item.is_active()
                            if not _is_active:
                                _acceptable = False
                        if _acceptable:
                            _has_access = False
                            if not params.by_units:
                                _has_access = item.has_access()
                            else:
                                _has_access = item.has_access_by_units()
                            if _has_access:
                                #result.append(item)
                                return True
                return False
            result = filter(func_accessible, cluster.all_items, {'subtype':subtype, 'skip_activity':skip_activity, 'by_units':by_units})
        else:
            #for item in cluster.all_items:
            def func_all(item, params):
                if item:
                    our_stype = item.IsSubTypeOf(params.subtype)
                    if our_stype:
                        _acceptable = True
                        if not params.skip_activity:
                            _is_active = item.is_active()
                            if not _is_active:
                                _acceptable = False
                        if _acceptable:
                            #result.append(item)
                            return True
                return False
            result = filter(func_all, cluster.all_items, {'subtype':subtype, 'skip_activity':skip_activity})
        #perf_end('get_items_by_subtype')
        return result
# ============================================================
    def Has_building(self, name, param_name = 'stype', active_fsm_only = False, subindex = -1, skip_mapindex = -1):
        '''Есть ли здание указанного типа, в любом состоянии'''
        #perf_begin('Has_building')
        buildings = []
        _clusters = []
        if subindex >= 0:
            _clusters.append(self.clusters[subindex])
        else:   # subindex == -1
            for key in self.clusters:
                if skip_mapindex >= 0:          # skip specific index
                    if key == skip_mapindex:    # skip specific index
                        continue
                cluster = self.clusters[key]
                _clusters.append(cluster)
        _cur_mapindex = self.GetCurMapIndex()
        for cluster in _clusters:
            for b in cluster.all_buildings:
                _name = b.get(param_name)
                if _name == name:
                    _has_access = b.HasAccess()
                    if _has_access:
                        if active_fsm_only:
                            _fsm_state = b.FSM_GetStateID()
                            if _fsm_state not in self._bad_states:
                                if _fsm_state == 'broken':
                                    if b.was_functional:
                                        if not b.is_deactivated:
                                            _b_name = b.get('name')
                                            if _b_name == 'hospital1':
                                                _b_mi = b.get('locind')
                                                if _cur_mapindex == _b_mi:
                                                    return True
                                                else:
                                                    continue
                                            return True
                                else:
                                    #perf_end('Has_building')
                                    return True
                        else:
                            #perf_end('Has_building')
                            return True
        #perf_end('Has_building')
        return False
# ============================================================
    def find_nearest(self, buildings, buildings_shift_cell_name, target, target_shift_cell_name):
        #perf_begin('find_nearest')
        pathes = []
        cells = []
        for b in buildings:
            _cell = None
            _has_attr = b.HasAttr(buildings_shift_cell_name)
            if _has_attr:
                _cell = b.GetAttrCellFixed(buildings_shift_cell_name)
            else:
                _cell = b.start_cell_pos # если у здания запрашиваемой клетки, то стартовая подменяет ее
            cells.append(_cell)
        result_index = BMgr.FindNearestCell(target, target_shift_cell_name, cells)
        if result_index >= 0:  # найдем здание по ячейке
            return buildings[result_index]
        trace('find_nearest: no pathes')
        trace(' find_nearest: target(from) =', target.__str__())
        trace(' find_nearest: buildings_shift_cell_name =', buildings_shift_cell_name)
        trace(' find_nearest: target_shift_cell_name =', target_shift_cell_name)
        _str = ''
        for b in buildings: _str += b.__str__() + ', '
        trace(' find_nearest: buildings = [', _str, ']')
        #perf_end('find_nearest')
        return None
# ============================================================
    def remove_building(self, building):
        uid = building.get('uid')
        map_index = building.get('locind')
        TMgr.process_removing_item(uid, map_index)
        self.dependant_buildings.remove_building(building)
        cluster = self.get_cluster(map_index)
        if uid in cluster.uids_dict:
            del cluster.uids_dict[uid]
        if building in cluster.all_buildings:
            cluster.all_buildings.remove(building)
        cluster.FBC.remove(building)
        self.all_levels_buildings.remove(building)
        building.dispose()
        TMgr.tutHelper.refresh()
# ============================================================
    def Remove_item(self, item):
        uid = item.get('uid')
        map_index = item.get('locind')
        TMgr.process_removing_item(uid, map_index)
        cluster = self.get_cluster(map_index)
        units_cluster = UMgr.get_cluster(map_index)
        if uid in cluster.uids_dict:
            del cluster.uids_dict[uid]
        if item in cluster.all_buildings:
            cluster.all_buildings.remove(item)
        cluster.FBC.remove(item)
        #if item in cluster.all_units:
        cluster.all_units.remove(item)
        #if item in units_cluster.total_workers:
        units_cluster.total_workers.remove(item)
        #if item in cluster.all_items:
        cluster.all_items.remove(item)
        cluster.FBC.remove(item)
        self.all_levels_buildings.remove(item)
        self.all_levels_units.remove(item)
        self.all_levels_items.remove(item)
        item.dispose()
        TMgr.tutHelper.refresh()
# ============================================================
    def remove_item_uid(self, item):
        uid = item.get('uid')
        map_index = item.get('locind')
        self.remove_uid(uid, map_index)
# ============================================================
    def remove_uid(self, uid, map_index):
        cluster = self.get_cluster(map_index)
        if uid in cluster.uids_dict:
            del cluster.uids_dict[uid]
# ============================================================
    def has_item_uid(self, item):
        '''Для проверки случаев, когда объект на карте, но только визуально - не для взаимодействия'''
        uid = item.get('uid')
        map_index = item.get('locind')
        cluster = self.get_cluster(map_index)
        if uid in cluster.uids_dict:
            return True
        return False
# ============================================================
    def get_item(self, uid, map_index):
        cluster = self.get_cluster(map_index)
        if not cluster:
            trace('BMgr.map_index:', map_index, cluster)
        if uid in cluster.uids_dict:
            return cluster.uids_dict[uid]
        return None
# ============================================================
    def get_items(self, uids_string, map_index):
        '''Возвращает список элементов по строке UIDов (из редактора)'''
        _empty = False
        _len_uids = len('uids:')
        if not uids_string: _empty = True
        if len(uids_string) <= _len_uids: _empty = True
        if _empty:
            return []
        naked_uids = uids_string[_len_uids:]
        uids = naked_uids.split(';')
        return self.get_items_by_uids_list(uids, map_index)
# ============================================================
    def get_items_by_uids_list(self, uids, map_index):
        '''Возвращает список элементов по строке UIDов (из редактора)'''
        result = []
        for uid in uids:
            result.append(self.get_item(uid, map_index))
        return result
# ============================================================
    def get_UIDs_list(self, uids_string):
        '''Возвращает список UIDов (из редактора)'''
        #perf_begin('get_UIDs_list')
        _uids_len = len('uids:')
        _uids_string_len = len(uids_string)
        if not uids_string or _uids_string_len <= _uids_len:
            #perf_end('get_UIDs_list')
            return []
        naked_uids = uids_string[_uids_len:]
        uids = naked_uids.split(';')
        #perf_end('get_UIDs_list')
        return uids
# ============================================================
    house_templates = {'hq1':0, 'hq_small1':0, 'house1':0}
    def Is_build_action_possible(self, map_index, template_name, need_worker, building = None):
        '''Проверяет - достаточно ли юнитов для строительства'''
        #trace('Is_build_action_possible:', template_name)
        #perf_begin('Is_build_action_possible')
        if need_worker:
            if template_name in self.house_templates:
                #perf_end('Is_build_action_possible')
                return True
            else:
                _status = ''
                if building:
                    _status = building.status
                if template_name == 'watchtower1' or _status == 'burning':
                        _free_units = UMgr.get_free_real_units_count(map_index)
                        if _free_units > 1:
                            #perf_end('Is_build_action_possible')
                            return True
                else:
                    _real_units_enough = UMgr.is_free_real_units_enough(map_index)
                    if not _real_units_enough:
                        #perf_end('Is_build_action_possible')
                        return False
        #perf_end('Is_build_action_possible')
        return True
# ============================================================
    def Has_accessible_building(self, map_index, template_name, status=''): # @ split
        #perf_begin('Has_accessible_building')
        #map_index = self.GetCurMapIndex()   # вроде только с текущей локации вызывается
        _all_buildings = self.get_cluster(map_index).all_buildings
        for b in _all_buildings:
            _name = b.get('name')
            if _name == template_name:
                _is_accessible = b.HasAccess()
                if _is_accessible:
                    if b.status != 'map_place':
                        if not status or status == b.status:
                            #perf_end('Has_accessible_building')
                            return True
        #perf_end('Has_accessible_building')
        return False
# ============================================================
    _inactive_statuses = ['broken', 'burning']
    def Has_active_building(self, template_name, map_index): # @ combine - либо для одной локации, либо для всех (при map_index = -1)
        '''Есть ли доступное, активное, несломанное, либо начавшее ремонт, здание'''
        #perf_begin('Has_active_building')
        _clusters = []
        if map_index >= 0:
            cluster = self.get_cluster(map_index)
            _clusters.append(cluster)
        else:
            for key in self.clusters: _clusters.append(self.clusters[key])
        for cluster in _clusters:
            for b in cluster.all_buildings:
                _name = b.get('name')
                if _name == template_name:
                    _is_accessible = b.HasAccess()
                    if _is_accessible:
                        if not b.is_deactivated:
                            _is_builders_state = b.is_builders_state()
                            _b_status = b.status
                            if _b_status != 'map_place':
                                if _b_status not in self._inactive_statuses:
                                    #perf_end('Has_active_building')
                                    return True
                                elif b.lock_shown:
                                    #perf_end('Has_active_building')
                                    return True
                                elif _is_builders_state:
                                    #perf_end('Has_active_building')
                                    return True
        #perf_end('Has_active_building')
        return False
# ============================================================
    def Has_buildings_in_build(self, template_name, map_index):
        '''Есть ли доступное, активное, несломанное, либо начавшее ремонт, здание'''
        #perf_begin('Has_buildings_in_build')
        cluster = self.get_cluster(map_index)
        for b in cluster.all_buildings:
            _name = b.get('name')
            if _name == template_name:
                _is_accessible = b.HasAccess()
                if _is_accessible:
                    if not b.is_deactivated:
                        _is_builders_state = b.is_builders_state()
                        if _is_builders_state:
                            #perf_end('Has_buildings_in_build')
                            return True
        #perf_end('Has_buildings_in_build')
        return False
# ============================================================
    non_attack_statuses = {'broken':0, 'map_place':0, 'place':0, 'burning':0}
    non_attack_fsm_states = {'preremoving':0, 'removing':0, 'removing_final':0, 'wait_unit':0,
                                'thrown':0, 'burning':0, 'extinguishing':0, 'firefighter_wait':0}
    def get_nest_attack_buildings(self, item, cell_shift_name, radius):
        '''Возвращает список зданий доступных для атаки варварами в указанном радиусе'''
        result = []
        buildings = self.GetUserBuildingsInRadius(item, cell_shift_name, radius)
        for b in buildings:
            _stype = b.get('stype')
            _need_no_worker = b.GetAttrBool('need_no_worker')   # эти здания вроде как все не нужно атаковать (но ниже они перечислены)
            if _need_no_worker:
                continue
            if _stype not in b.not_for_attack:
                if b.status not in self.non_attack_statuses:
                    _is_builders_state = b.is_builders_state()
                    if not _is_builders_state:
                        _fsm_state = b.FSM_GetStateID()
                        if _fsm_state not in self.non_attack_fsm_states:
                            if b.energy_hit > 0:
                                #if not b.is_under_attack:   # совместная атака?
                                result.append(b)
        return result
# ============================================================
    def is_shop_red(self): # @ combine
        '''Проверяем - есть ли что строить в магазине (вызывается только для активной локации)'''
        #perf_begin('is_shop_red')
        map_index = self.cur_map_index
        _has_bridge = False
        _can_anything_to_build = self.CanAnythingToBuild()
        if not _can_anything_to_build:
            #perf_end('is_shop_red')
            return True
        templates = self.GetBuildableList()
        #tmp_t = []
        for item in templates:
            item.cannot = ''
            if item.is_visible:
                if item.subtype == 'bridge':
                    if _has_bridge: continue
                    else: _has_bridge = True
                _can_build = item.can_build
                _is_prior = self.CanBuildPriority(item.name, map_index)
                if _can_build:
                    if not item.is_res_enough:
                        _can_build = False
                        item.cannot = 'NOT_ENOUGH_RES'
                    if not _is_prior:
                        _can_build = False
                        item.cannot = 'OTHER_PRIORITY'
                    if item.depends_from:
                        if _can_build:
                            for dep in item.depends_from:
                                _has = self.HasActiveBuilding(dep)
                                _can_build = False
                                item.cannot = 'NO_DEPENDANT'
                                if _has:
                                    item.cannot = ''
                                    _can_build = True
                                    break
                    if _can_build:
                        _has_builders = self.HasEnoughBuilders(item.name)
                        _has_mplaces = self.HasActiveMapPlaces(item.name, item.subtype)
                        if not _has_builders:
                            _can_build = False
                            item.cannot = 'NO_BUILDERS'
                        elif not _has_mplaces:
                            _can_build = False
                            _has_deactive_mplaces = self.HasOnlyDeactivatedMapPlaces(item.name, item.subtype)
                            item.cannot = ('NO_MAP_PLACE', 'NO_ACTIVE_MAP_PLACE')[_has_deactive_mplaces == True]
                        if item.cannot == '':
                            if item.name == 'quarry1':
                                _has_granite = self.HasGranite()
                                if not _has_granite:
                                    _can_build = False
                                    item.cannot = 'NO_GRANITE'
                else:
                    if not _is_prior:
                        _can_build = False
                        item.cannot = 'OTHER_PRIORITY'
                    else:
                        item.cannot = 'CANNOT_BUILD'    # from C++
                if _can_build:
                    #perf_end('is_shop_red')
                    return False
            #tmp_t.append(item.cannot)
        #perf_end('is_shop_red')
        return True
# ============================================================
    def make_thrown_buildings(self, stype, variant_names):
        buildings = []
        for key in self.clusters:
            cluster = self.clusters[key]
            for b in cluster.all_buildings:
                _stype = b.get('stype')
                if _stype == stype:
                    _vname = b.get('variant_name')
                    if _vname in variant_names:
                        b.FSM.HandleEvent('thrown_event')       # забросим здание
# ============================================================
    def has_higher_level_buildings(self, stype, variant_names):
        buildings = []
        for key in self.clusters:
            cluster = self.clusters[key]
            for b in cluster.all_buildings:
                _stype = b.get('stype')
                if _stype == stype:
                    _vname = b.get('variant_name')
                    if _vname in variant_names:
                        return True
        return False
# ============================================================
    def has_territory_places(self, map_index, subtypes):
        _buildings = self.get_buildings_simple(map_index, subtypes)
        for b in _buildings:
            if b.status in self._broken__map_place:
                return True
        return False
# ============================================================
    def disable_time_altars(self):
        for _item in self.all_levels_items:
            _name = _item.get('name')
            if _name == 'bonus_altar_time':
                _item.deactivate()
                _item.make_non_selectable()
# ============================================================
    _exits = ['enter', 'grave_in']
    def get_submap_exit(self, cluster_map_index, to_map_index):
        cluster = self.get_cluster(cluster_map_index)
        def filter_exits(item, params):
            if item.IsSubTypeOf(BMgr._exits):
                if not item.is_deactivated:
                    _sublevel = item.GetAttrInt('sublevel')
                    if _sublevel == params.to:
                        _has_access = item.HasAccess()
                        if _has_access:
                            _cell_name = item.get_action_cell_name()
                            _cell = item.GetAttrCellFixed(_cell_name)
                            _is_cell_passable = item.IsCellPassable(_cell)
                            if _is_cell_passable:
                                return True
            return False
        _exits = filter(filter_exits, cluster.all_items, {'to':to_map_index})
        return _exits
# ============================================================
    def get_explode_items(self):
        def filter_explode(item):
            _can_explode = item.GetAttrBool('can_explode')
            if _can_explode:
                return True
            return False
        _explode_items = filter(filter_explode, self.all_levels_items)
        return _explode_items
# ============================================================
    def add_column(self, column):
        _mindex = column.get('locind')
        if _mindex not in self.columns:
            self.columns[_mindex] = []
        _list = self.columns[_mindex]
        if column not in _list:
            _list.append(column)
            TMgr.cancel_barbarian_tasks(_mindex)
# ============================================================
    def remove_column(self, column):
        _mindex = column.get('locind')
        if _mindex not in self.columns:
            return
        _list = self.columns[_mindex]
        if column in _list:
            _list.remove(column)
# ============================================================
    def has_column(self, map_index):
        if map_index in self.columns:
            _list = self.columns[map_index]
            _len = len(_list)
            return (_len > 0)
        return False
# ============================================================

