# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *
# ============================================================
# ============================================================
class Event(object):
    def __init__(self, name):
        self.ser_name = 'Event'
        self.name = name
# ============================================================
# ============================================================
class SignalItem(object):
    def __init__(self, creator = None, signal_id = None, time = None):
        self.ser_name = 'SignalItem'
        self.creator = creator
        self.signal_id = signal_id
        self.time = time
# ============================================================
    def is_ready(self):
        return (self.time <= 0)
# ============================================================
    def decrement(self, ms):
        self.time -= ms
# ============================================================
# ============================================================
class EventsMgr(EventsMgr_class):
    def __init__(self, cdata):
        EventsMgr_class.__init__(self, cdata)
        self.ser_name = 'EventsMgr'
        self.AddSelfToGlobal('EMgr')
        self.events = []
        self.signals = []       # сигналы, через промежутки времени
        self.listeners = []
        self.update_ticks = 0
        self.concrete_events = {}
        self.global_events = {}
# ============================================================
# ============================================================
    def Reset_cache(self):
        pass
# ============================================================
    def on_msg(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'global_event':
            self.on_global_event(msg_param1, msg_param2)
        elif msg_type == 'state_machine':
            self.on_state_machine(msg_param1, msg_param2, msg_dict)
        elif msg_type == 'signal_msg':
            self.on_signal(msg_param1)
        elif msg_type == 'on_create':
            self.on_create()
# ============================================================
    def on_signal(self, signal):
        pass
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == 'enter':
            return
        elif type == 'exit':
            return
        elif type == 'change':
            self.on_state_change(additional.prev_state, additional.next_state)
# ============================================================
    def on_state_change(self, cur_state, next_state):
        pass
# ============================================================
    def on_create(self):
        trace('EventsManager on_create')
# ============================================================
    def on_global_event(self, event, param):
        self.dispatch_global_event(event, param)    # заготовка для перевода глобальных событий в данный объект, а не напрямую из C++
        LBoard.Core.on_global_event(event, param)   # чтобы настройки уровня инициализить
        if event == 'inventory_add_qitem':  # затычка для сбора инвентори на разных локациях
            for listener in self.listeners:
                listener.on_event(event, None, {})
        elif event == 'unit_talk':  # затычка для сбора инвентори на разных локациях
            self.events.append( (event,None,{'param':param}) )
        elif event == 'location_switched':
            self.events.append( (event,None,{'param':param}) )
        elif event == 'new_location_loading':
            self.events.append( (event,None,{'param':param}) )
        elif event == 'new_location_loaded':
            self.events.append( (event,None,{'param':param}) )
        elif event == 'location_loaded':
            self.events.append( (event,None,{'param':param}) )
            #for listener in self.listeners:
            #    listener.on_event(event, None, {'param':param})
# ============================================================
    def on_update(self, ms):                # данный метод должен вызываться из C++ каждый квант
        # ###
        self.update_ticks += 1
        # >>>
        _events = self.events[:]
        del self.events[:]  # чистим список event'ов
        for event in _events:     # сгенерим эвенты
            _ev = event
            name, obj, extra = event
            self.dispatch_concrete_event(_ev)
            for listener in self.listeners:
                listener.on_event(name, obj, extra)
        if not (self.update_ticks%2):   # сигналы обновляем чуть реже
            _ms = ms*2
            for signal in self.signals: # уменьшаем время сигналов
                signal.decrement(_ms)
            _remove_signals = []
            _copy_signals = self.signals[:] # чтобы в сигналы можно было добавлять/удалять безопасно
            for signal in _copy_signals:
                _signal_ready = signal.is_ready()
                if _signal_ready:   # обрабатываем готовые к сработке сигналы
                    _remove_signals.append(signal)
                    _creator = signal.creator
                    _creator.on_external_signal(signal.signal_id)
            for s in _remove_signals:   # удаляем отработавшие сигналы
                self.signals.remove(s)
# ============================================================
    def dispatch_concrete_event(self, event):
        name = event[0]
        if name in self.concrete_events:
            ev = Event(name)
            ev.obj = event[1]
            ev.extra = event[2]
            _listeners = self.concrete_events[name]
            for _listener_func in _listeners:
                _listener_func(ev)
# ============================================================
    def dispatch_global_event(self, event, param):
        name = event
        if name in self.global_events:
            ev = Event(name)
            ev.param = param
            _listeners = self.global_events[name]
            for _listener_func in _listeners:
                _listener_func(ev)
# ============================================================
    def on_rare_update(self, ms):
        pass
# #######################################################
    def Send_event(self, event_name):
        '''Отправка события из C++'''
        self.send_event(event_name, None)
# ============================================================
    def send_event(self, event_name, event_object, extra_dict = {}):
        _has_stype = False
        if event_object:
            if 'subtype' in event_object:
                _has_stype = True
                #trace(event_object)
                trace('EMgr.send_event:', event_name, event_object.subtype)
        if not _has_stype:
            trace('EMgr.send_event:', event_name, event_object)
        self.events.append( (event_name, event_object, extra_dict) )
        #if len(self.events) > self.dbg_max_events_count:
        #    self.dbg_max_events_count = len(self.events)
        #    trace('EMGR.MAX_EVENTS_COUNT = ', self.dbg_max_events_count)
# ============================================================
    def add_signal(self, creator, signal_id, time):
        '''Добавляет сигнал для указанного объекта.
           Объект должен поддерживать метод on_external_signal(s_id)'''
        _signal = SignalItem(creator, signal_id, time)
        self.signals.append(_signal)
        #if len(self.signals) > self.dbg_max_signals_count:
        #    self.dbg_max_signals_count = len(self.events)
        #    trace('EMGR.MAX_SIGNALS_COUNT = ', self.dbg_max_signals_count)
# ============================================================
    def remove_signal(self, creator, signal_id):
        _remove_signals = []
        for signal in self.signals:
            if signal.creator == creator:
                if signal.signal_id == signal_id:
                    _remove_signals.append(signal)
        for signal in _remove_signals:
            self.signals.remove(signal)
# ============================================================
    def add_event_listener(self, listener):
        if listener not in self.listeners:
            self.listeners.append(listener)
# ============================================================
    def remove_event_listener(self, listener):
        self.listeners.remove(listener)
# ============================================================
    def add_concrete_event_listener(self, event_name, listener_func):
        if event_name not in self.concrete_events:
            self.concrete_events[event_name] = [listener_func]
        else:
            _list = self.concrete_events[event_name]
            if listener_func not in _list:
                _list.append(listener_func)
# ============================================================
    def remove_concrete_event_listener(self, event_name, listener_func):
        if event_name in self.concrete_events:
            self.concrete_events[event_name].remove(listener_func)
# ============================================================
    def add_global_event_listener(self, event_name, listener_func):
        if event_name not in self.global_events:
            self.global_events[event_name] = [listener_func]
        else:
            _list = self.global_events[event_name]
            if listener_func not in _list:
                _list.append(listener_func)
# ============================================================
    def remove_global_event_listener(self, event_name, listener_func):
        if event_name in self.global_events:
            self.global_events[event_name].remove(listener_func)
# ============================================================
