# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

class UnitsCluster(object):
    def __init__(self):
        self.ser_name = 'UnitsCluster'
        self.all_units = []
        self.free_units = []
        self.total_workers = []
        self.general = None
        self.units_building_workers = 0    # кол-во юнитов, которые работают в том или
                                            # ином здании, т.е. заняты почти навсегда
                                            # (исключение - лесоруб, но его сюда тоже добавляем)

    def add_building_worker(self):
        self.units_building_workers += 1
    def remove_building_worker(self):
        self.units_building_workers -= 1
# ============================================================
# ============================================================
class UnitsMgr(UnitsMgr_class):
# ============================================================
    def __init__(self, cdata):
        UnitsMgr_class.__init__(self, cdata)
        self.ser_name = 'UnitsMgr'
        self.AddSelfToGlobal('UMgr')
        _cluster = UnitsCluster()
        self.clusters = {0:_cluster}
        self.cur_cluster = _cluster
        self.cur_map_index = 0

        self.cache_free_real_units = {}
# ============================================================
    def get_cluster(self, map_index):
        if map_index in self.clusters:
            return self.clusters[map_index]
        else:
            return None
# ============================================================
    def switch_cluster(self, new_location_index):
        if new_location_index in self.clusters:
            self.cur_cluster = self.clusters[new_location_index]
            self.cur_map_index = new_location_index
# ============================================================
    def get_general(self):
        return self.cur_cluster.general
# ============================================================
    def Reset_cache(self):
        self.cache_free_real_units = {}
# ============================================================
    def transfer_unit(self, unit, dest_map_index):
        _cur_mi = unit.get('locind')
        _src = self.get_cluster(_cur_mi)
        _dest = self.get_cluster(dest_map_index)
        _src.all_units.remove(unit)
        _src.free_units.remove(unit)
        if unit in _src.total_workers:
            _src.total_workers.remove(unit)
            _dest.total_workers.append(unit)
        _dest.all_units.append(unit)
# ============================================================
    def on_msg(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'on_create':
            self.on_create()
        elif msg_type == 'state_machine':
            self.on_state_machine(msg_param1, msg_param2, msg_dict)
        elif msg_type == 'signal_msg':
            self.on_signal(msg_param1)
        elif msg_type == 'global_event':
            self.on_global_event(msg_param1, msg_param2)
# ============================================================
    def on_global_event(self, event, param):
        if event == 'talk_quest_started':    #$@ код для активации разговорчивых юнитов писать здесь
            for unit in BMgr.all_levels_units:
                unit.try_show_talk_sign()   # покажем значки разговора
        elif event == 'talk_quest_completed':
            for unit in BMgr.all_levels_units:
                unit.try_show_talk_sign(False)  # спрячем значки разговора
        elif event == 'give_obj_quest_started' or event == 'give_obj_quest_completed':
            for unit in BMgr.all_levels_units:
                unit.switch_animation()     # обновим анимации всех юнитов (включая значки)
        elif event == 'location_switched':
            self.on_location_switched(int(param))
        elif event == 'new_location_loading':
            self.on_new_location_loading(int(param))
# ============================================================
    def on_location_switched(self, subindex):
        if subindex != self.cur_map_index:
            self.switch_cluster(subindex)
# ============================================================
    def on_new_location_loading(self, subindex):
        self.clusters[subindex] = UnitsCluster()
# ============================================================
    def on_signal(self, signal):
        pass
# ============================================================
    def on_state_machine(self, type, state_name, additional):
        if type == "enter":
            return
        elif type == "exit":
            return
        elif type == "change":
            self.on_state_change(additional.prev_state, additional.next_state)
# ============================================================
    def on_state_change(self, cur_state, next_state):
        #trace(cur_state.append(next_state))
        return
# ============================================================
    def on_create(self):
        trace("UnitsManager on_create")
# ============================================================
    def on_update(self, ms):                # данный метод должен вызываться из C++ каждый квант
        pass
# ============================================================
    def on_rare_update(self, ms):
        for unit in BMgr.all_levels_units:
            if unit.is_indian:
                _has_access = unit.has_access()
                if _has_access:
                    if not unit.unit_is_talking:
                        _is_in_sick_states = unit.is_in_sick_states()
                        if not _is_in_sick_states:
                            unit.switch_animation()     # обновим знаки и прочее
            else:
                unit.refresh_run_animation()
# #######################################################
# #######################################################
# #######################################################

    #@ ! посмотреть где вызывается и добавить unit_subtype
    #def send_unit_to(self, target, cell_shift_name = ""):
    #    nearest_unit = self.get_free_nearest_unit(target, cell_shift_name)
    #    if nearest_unit == None:
    #        return None
    #    nearest_unit.run_to_target(target, None, cell_shift_name)
    #    return nearest_unit
# ============================================================
    def get_free_nearest_unit(self, target, cell_shift_name = '', unit_subtype = '', _from_units = None):
        if cell_shift_name == None:
            return None, None
        if not _from_units:
            units = self.get_free_units(target.get('locind'), unit_subtype)
        else:
            units = _from_units
        if not units:
            return None, None
        else:
            pathes = []
            # найдем ближайшего юнита:
            cells = []
            for unit in units:  # собираем ячейки всех свободных и подходящих юнитов
                _u_cell = unit.get('cell')
                cells.append(_u_cell)
            nearest_building = None
            nearest_building = BMgr.find_nearest_unit_building(target, cell_shift_name, False)
            result_index = UMgr.FindNearestCell(target, cell_shift_name, cells)
            nearest_unit = None
            if result_index >= 0:  # найдем юнита по ячейке
                nearest_unit = units[result_index]
            if nearest_unit:
                if not nearest_unit.is_in_home:
                    return nearest_unit, None   # тут баг, потому что это не ближайшая позиция, но пока оставим так, чтобы не забыть
                if nearest_building:
                    if nearest_unit.home_building == nearest_building:
                        return nearest_unit, None
                    else:
                        return nearest_unit, nearest_building
            return None, None
# ============================================================
    def teleport_unit(self, unit, building):
        if not building: return
        if not unit: return
        _cell = building.start_cell_pos
        #if cell_name:
        #    _cell = building.GetAttrCellFixed(cell_name)
        #else:
        #    _cell = building.get('cell')
        unit.TeleportToCell(_cell)
# ============================================================
    def teleport_unit_to_cell(self, unit, cell):
        unit.TeleportToCell(cell)
# ============================================================
    def create_unit(self, map_index, template_name, variant_name = ''):
        '''Создает нового юнита'''
        new_unit = self.CreateUnit(map_index, template_name, variant_name) # C++ вызывает затем level_board.create_unit(...)
        # перенесено в level_board:
        #self.all_units.append(new_unit)
        #if template_name in ['tunit1', 'survived1']:
        #    self._total_workers.append(new_unit)
        return new_unit
# ============================================================
    def add_free_unit_reface(self, unit):
        return self.add_free_unit(unit, True, True)
# ============================================================
    def add_free_unit(self, unit, _do_send_event = True, _force_reface = False):
        '''Добавляет юнита к списку свободных, генерирует соответствующее событие для TasksMgr'''
        _uname = unit.get('name')
        map_index = unit.get('locind')
        #if _uname != 'tunit1':  # чтобы все юниты подменялись на обычных
        _need_recreate_unit = False
        if _force_reface:
            if _uname != 'tunit1':
                _need_recreate_unit = True
        if _uname == 'survived1': # чтобы выжившие подменялись на обычных работников
            _need_recreate_unit = True
        if _need_recreate_unit:
            _new_unit = self.create_unit(map_index, 'tunit1')
            _cell = unit.get('cell')
            _new_unit.set('cell', _cell)
            #self.get_cluster(map_index).all_units.remove(unit)
            #BMgr.all_units.remove(unit) # - следующая строка это проделает
            BMgr.RemoveItemFromMap(unit)
            unit = _new_unit
            unit.CheckIsInFog(True)     # проверка на то, что юнит не в тумане и прочее
            UMgr.add_free_unit(unit, _do_send_event)     # новые юниты свободны: у нас свободное общество!
            TMgr.SendGlobalMsg('some_survived_saved')   # при спасении одного выжившего
            return unit  # потому что предыдущий вызов рекурсивен и добавит юнита
        _free_units = self.get_cluster(map_index).free_units
        if unit in _free_units:
            trace('Unit is already free', unit.get('name'), unit.get('uid'))
            return unit
        if unit not in _free_units:
            _free_units.append(unit)
            if _do_send_event:
                TMgr.send_event('free_unit', unit)
        return unit # на случай, если произошла подмена выжившего работником
# ============================================================
    def remove_free_unit(self, unit):
        '''Убирает юнита из списка свободных'''
        _free_units = self.get_cluster(unit.get('locind')).free_units
        if unit not in _free_units:
            trace('UMgr: unit is not free!', unit.get('name'), unit.get('uid'))
            return False
        _free_units.remove(unit)
        return True
# ============================================================
    def add_building_worker(self, unit):
        map_index = unit.get('locind')
        self.get_cluster(map_index).add_building_worker()
# ============================================================
    def remove_building_worker(self, unit):
        map_index = unit.get('locind')
        self.get_cluster(map_index).remove_building_worker()
# ============================================================
    def get_free_real_units_count(self, map_index):
        if not self.cache_free_real_units:
            self.cache_free_real_units = {}
        if map_index not in self.cache_free_real_units:
            self.cache_free_real_units[map_index] = self._get_free_real_units_count_int(map_index)
        _free_count = self.cache_free_real_units[map_index]
        return _free_count
# ============================================================
    def _get_free_real_units_count_int(self, map_index):
        _total_workers = 0
        units_cluster = self.get_cluster(map_index)
        for worker in units_cluster.total_workers:
            if not worker.home_building:
                if not worker.is_survived:
                    _total_workers += 1
            else:
                _b = worker.home_building
                if not _b.is_deactivated:
                    _total_workers += 1
                else:
                    _fsm_state = _b.FSM.GetStateID()
                    if _fsm_state == 'removing':
                        _total_workers += 1
                    elif _fsm_state == 'removing_final':
                        _total_workers += 1
        _free_count = _total_workers - units_cluster.units_building_workers
        return _free_count
# ============================================================
    def get_units_at_home_count(self, map_index):
        '''Кол-во юнитов в зданиях'''
        _count = 0
        _buildings = BMgr.get_buildings_simple(map_index, '__home__')
        for b in _buildings:
            if b.is_deactivated:
                _is_smallhouse = b.IsSubTypeOf('house')
                if _is_smallhouse:
                    continue
            _count += b.Get_units_count()
        return _count
# ============================================================
    #def Get_total_workers_count(self, map_index):   # @ map
    #    _total_workers = 0
    #    for worker in self.get_cluster(map_index).total_workers:
    #        if not worker.home_building:
    #            if not worker.is_survived:
    #                _total_workers += 1
    #        elif not worker.home_building.is_deactivated:
    #            _total_workers += 1
    #    return _total_workers
# ============================================================
    def is_free_real_units_enough(self, map_index):
        _free_count = self.get_free_real_units_count(map_index)
        _min_count = Globals.GInt('MIN_UNITS_COUNT')
        return (_free_count > _min_count)
# ============================================================
    worker_types = {'simple':0, 'builder':0, 'dt_wcutter':0, 'soldier':0, 'quarryman':0, 'breaker':0}
    def get_free_units(self, map_index, unit_subtype = '', only_at_home = False):
        '''Возвращает список свободных юнитов, список уникален по позициям юнитов, т.е. юниты, находящиеся
            в одной точке входят в список единожды'''
        #perf_begin('get_free_units')
        units = []
        cells = []
        _cells_dict = {}
        _free_units = self.get_cluster(map_index).free_units
        if not _free_units:
            trace('get_free_units: no free units')
            #perf_end('get_free_units')
            return units
        # выбираем юнитов, уникальных в разных ячейках:
        for unit in _free_units:
            # если подходит любой юнит (что редкость), либо юнит в доме - он любой, либо юнит подходящего типа
            _add = False
            if not unit_subtype:
                _add = True
            elif unit.is_in_home:
                if unit.current_subtype in self.worker_types:
                    _add = True
            if not only_at_home:
                if not _add:
                    if unit_subtype == unit.current_subtype:
                        _add = True
            _has_res = unit.has_resource()
            if _has_res:                    # затычка на всякий случай
                _add = False
            if _add:
                unit_cell = unit.get('cell')    # позиция юнита
                cell_present = False
                #_cell_hash = str(unit_cell.row) + ' ' + str(unit_cell.col)
                _cell_hash = unit_cell.row * 100000 + unit_cell.col
                if _cell_hash in _cells_dict:
                    #cell_present = True
                    break
                #for c in cells:     # ищем - нет ли уже такой ячейки:
                #    if unit_cell == c:
                #        cell_present = True
                #        break
                if not cell_present:
                    if unit.home_building:
                        if not unit.home_building.is_deactivated:    # проверяем только в активированных зданиях
                            _cells_dict[_cell_hash] = 0
                            cells.append(unit_cell)
                            units.append(unit)
        #perf_end('get_free_units')
        return units
# ============================================================
    def get_free_units_at_home(self, map_index, unit_subtype):
        '''Возвращает список свободных юнитов, список уникален по позициям юнитов, т.е. юниты, находящиеся
            в одной точке входят в список единожды'''
        #perf_begin('get_free_units_at_home')
        units = []
        cells = []
        _cells_dict = {}
        _free_units = self.get_cluster(map_index).free_units
        if not _free_units:
            trace('get_free_units: no free units')
            #perf_end('get_free_units_at_home')
            return units
        # выбираем юнитов, уникальных в разных ячейках:
        #_dbg = []
        for unit in _free_units:
            #_dbg.append(1)
            if unit.is_in_home:
                #_dbg.append('2'+unit.current_subtype)
                if unit.current_subtype == unit_subtype:
                    _has_res = unit.has_resource()
                    if _has_res:        # затычка на всякий случай
                        trace('===> get_free_units_at_home: unit has resource, not free')
                        #_dbg.append(3)
                        continue
                    #_is_store = unit.home_building.is_store()
                    #if _is_store:   # только из хранилища можно нести ресурсы
                    # по факту - мы вызываем юнита только из главного здания...
                    #_dbg.append(4)
                    unit_cell = unit.get('cell')    # позиция юнита
                    cell_present = False
                    #_cell_hash = str(unit_cell.row) + ' ' + str(unit_cell.col)
                    _cell_hash = unit_cell.row * 100000 + unit_cell.col
                    if _cell_hash in _cells_dict:
                        #cell_present = True
                        continue
                    #for c in cells:     # ищем - нет ли уже такой ячейки:
                    #    if unit_cell == c:
                    #        cell_present = True
                    #        break
                    if not cell_present:
                        #_dbg.append(5)
                        _cells_dict[_cell_hash] = 0
                        cells.append(unit_cell)
                        units.append(unit)
        if not units:
            trace('===> get_free_units_at_home: no units...', _free_units)
            for unit in _free_units:
                trace('======> unit:', unit.get('name'), unit.get('uid'), unit.is_in_home, unit.current_subtype)
            #trace('===> get_free_units_at_home: DBG:', _dbg)
        #perf_end('get_free_units_at_home')
        return units
# ============================================================
    def get_units_for_faces(self):
        '''Возвращает список юнитов, лица которых нужно показать'''
        result = []
        map_index = BMgr.GetCurMapIndex()
        _cluster = BMgr.get_cluster(map_index)
        _face_items = _cluster.all_units
        for unit in _face_items:         # выборка из всех юнитов
            _is_hided = unit.Drawer.IsHided()
            if _is_hided:
                continue
            _has_access = unit.has_access()
            if _has_access:                 # все, к которым есть доступ
                _has_sign = unit.has_sign()
                if _has_sign:
                    _is_active = unit.is_active()
                    #_is_enough_params = unit.is_enough_params()
                    if _is_active:
                        if unit.is_shown:              # только активные юниты которым достаточно доп. параметров (обычно это камни)
                            _is_talking = unit.GetAttrBool('is_talking')
                            _is_ready_to_talk = False
                            _stype = unit.get('stype')
                            if unit.is_survived:
                                if _is_talking:
                                    _is_ready_to_talk = True
                            if _is_ready_to_talk:        # если это выживший юнит
                                result.append(unit)
                            elif unit.unique_name:                      # если это юнит, с которым нужно поговорить
                                result.append(unit)
                            elif unit.quest:                            # если это юнит с квестом (индеец обычно)
                                result.append(unit)
                            elif _stype == 'trader':
                                result.append(unit)
                            elif _stype == 'task_item':
                                result.append(unit)
        _task_items = BMgr.get_items_by_subtype(map_index, 'task_item', False, True)
        #for ti in _task_items:
        #    #if ti.
        #    result.append(ti)
        result.extend(_task_items)
        return result
# ============================================================
    # @Obsolete
    def has_indians_with_quests(self):
        '''Возвращает список индейцев, к которым можно подойти за квестом'''
        result = []
        map_index = BMgr.GetCurMapIndex()
        for unit in BMgr.get_cluster(map_index).all_units:         # выборка из всех юнитов
            _has_access = unit.has_access()
            if _has_access:                 # все, к которым есть доступ
                if unit.is_indian:
                    if unit.quest:          # с квестом
                        _has_sign = unit.has_sign()
                        if _has_sign:
                            _is_active = unit.is_active()
                            #_is_enough_params = unit.is_enough_params()
                            if _is_active:
                                if unit.is_shown:              # только активные юниты которым достаточно доп. параметров (обычно это камни)
                                    _is_enough_params = unit.is_enough_params()
                                    if _is_enough_params:
                                        _is_start_possible = unit.quest.is_start_possible()
                                        if _is_start_possible:
                                            result.append(unit)
                                    #_is_talking = unit.GetAttrBool('is_talking')
                                    #if unit.is_survived and _is_talking:        # если это выживший юнит
                                    #    result.append(unit)
                                    #elif unit.unique_name:                      # если это юнит, с которым нужно поговорить
                                    #    result.append(unit)
                                    #elif unit.quest:                            # если это юнит с квестом (индеец обычно)
                                    #    result.append(unit)
        return result
# ============================================================
    def has_ready_units(self):
        '''Возвращает список юнитов, к которым можно подойти (за исключением говорящих индейцев)'''
        indians = []
        survived = []
        precomplete = []
        map_index = BMgr.GetCurMapIndex()
        for unit in BMgr.get_cluster(map_index).all_units:         # выборка из всех юнитов
            _has_access = unit.has_access()
            if _has_access:            # все, к которым есть доступ
                if unit.is_indian:
                    if unit.quest:          # с квестом
                        _has_sign = unit.has_sign()
                        if _has_sign:
                            _is_active = unit.is_active()
                            #_is_enough_params = unit.is_enough_params()
                            if _is_active:
                                if unit.is_shown:              # только активные юниты которым достаточно доп. параметров (обычно это камни)
                                    _is_enough_params = unit.is_enough_params()
                                    _is_precomplete = unit.quest.is_quest_precomplete()
                                    if _is_precomplete:
                                        precomplete.append(unit)
                                    elif _is_enough_params:
                                        _is_start_possible = unit.quest.is_start_possible()
                                        if _is_start_possible:
                                            indians.append(unit)
                else:       # not indian
                    _is_talking = unit.GetAttrBool('is_talking')
                    if unit.is_survived:
                        if _is_talking:
                            _has_sign = unit.has_sign()
                            if _has_sign:
                                _is_active = unit.is_active()
                                #_is_enough_params = unit.is_enough_params()
                                if _is_active:
                                    if unit.is_shown:              # только активные юниты которым достаточно доп. параметров (обычно это камни)
                                        survived.append(unit)
        return (indians, precomplete, survived)
# ============================================================
    def step_aside_general(self, map_index):
        UMgr.get_cluster(map_index).general.step_aside()
# ============================================================
    def Is_units_count_critical(self, map_index):
        _is_units_enough = self.is_free_real_units_enough(map_index)
        if not _is_units_enough:
            return True
        else:
            return False
# ============================================================
    def Get_total_units_text(self, map_index):
        '''Возвращает текст для индикатора кол-ва свободных юнитов'''
        result = []
        _free_count = self.get_free_real_units_count(map_index)
        if not self.is_free_real_units_enough(map_index):
            result.append( ("text", 'COLOR_RED') )
        result.append( ('digit', _free_count) )
        return result
# ============================================================
    def Get_free_real_units_count(self, map_index):    # просто чтобы видно было, что извне вызывается
        return self.get_free_real_units_count(map_index)
# ============================================================
    def has_doctor(self):
        _has_doctors = UStats.GetLevelValue('HAS_DOCTORS')
        if not _has_doctors:    # early check
            return False
        #h_buildings = BMgr.get_buildings_raw(-1, 'hospital')
        h_buildings = LBoard.Core.get_doctors(BMgr.GetCurMapIndex())    # не должно быть проверки на MapIndex, но должны учитываться исключения! Можно это учитывать внутри метода!
        for b in h_buildings:
            #_is_functional = b.is_now_functional()
            #if _is_functional:
            if not b.doctor:
                continue
            else:
                _fsm_state = b.FSM.GetStateID()
                if _fsm_state == 'broken':
                    continue    # indicates building broken by raptors or barbarians
            return True
        return False
# ============================================================
    def get_doctor(self):
        _has_doctors = UStats.GetLevelValue('HAS_DOCTORS')
        if not _has_doctors:
            return None
        #h_buildings = BMgr.get_buildings_raw(-1, 'hospital')
        h_buildings = LBoard.Core.get_doctors(BMgr.GetCurMapIndex())
        for b in h_buildings:
            #_is_functional = b.is_now_functional()
            #if _is_functional:
            if b.status == 'broken':
                continue
            if not b.doctor:
                continue
            return b.doctor
        return None
# ============================================================
    def get_doctor_building(self):
        _has_doctors = UStats.GetLevelValue('HAS_DOCTORS')
        if not _has_doctors:
            return None
        #h_buildings = BMgr.get_buildings_raw(-1, 'hospital')
        h_buildings = LBoard.Core.get_doctors(BMgr.GetCurMapIndex())
        for b in h_buildings:
            #_is_functional = b.is_now_functional()
            #if _is_functional:
            if not b.doctor:
                continue
            return b
        return None
# ============================================================
    def has_firefighter(self):
        _has_firefighter = UStats.GetLevelValue('HAS_FIREFIGHTERS')
        if not _has_firefighter:
            return False
        #w_buildings = BMgr.get_buildings_raw(-1, 'waterpump')
        w_buildings = LBoard.Core.get_firefighters(BMgr.GetCurMapIndex())
        for b in w_buildings:
            #_is_functional = b.is_now_functional()
            #if _is_functional:
            if not b.firefighter:
                continue
            return True
        return False
# ============================================================
    def get_nearest_firefighter(self, target):
        _has_firefighters = UStats.GetLevelValue('HAS_FIREFIGHTERS')
        if not _has_firefighters:
            return None
        units = []
        cells = []
        # filtering units from buildings:
        #w_buildings = BMgr.get_buildings_raw(-1, 'waterpump')
        _mi = BMgr.GetCurMapIndex()
        w_buildings = LBoard.Core.get_firefighters(_mi)
        for b in w_buildings:
            #_is_now_functional = b.is_now_functional()
            #if _is_now_functional:
            if not b.firefighter:
                continue
            units.append(b.firefighter)
            cells.append(b.firefighter.get('cell'))
        # filtering free units:
        _free_units = []
        _free_units_cells = []
        _units_count = len(units)
        if _units_count <= 0:
            return None
        for i in range(_units_count):
            u = units[i]
            if u.task == None:
                _free_units.append(u)
                _free_units_cells.append(cells[i])
        # searching nearest unit:
        if _free_units:
            if _free_units[0].get('locind') != _mi: # from another sublevel
                return _free_units[0]
        elif units:
            if units[0].get('locind') != _mi: # from other sublevel
                return units[0]   # можно добавить shuffle :)
                #for i in range(5,-1,-1):    # reverse
                # #shuffle:
                #j = int( tools.random() * (i+1) )
                #if i != j: _times[i], _times[j] = _times[j], _times[i]
        if _free_units_cells:
            result_index = UMgr.FindNearestCell(target, target.get_firefight_cell_name(), _free_units_cells)
            if result_index >= 0:
                return _free_units[result_index]
        elif cells:
            result_index = UMgr.FindNearestCell(target, target.get_firefight_cell_name(), cells)
            if result_index >= 0:
                return units[result_index]
        return None
# ============================================================
    def get_sick_general_text(self, onclick = False, for_sick = False):
        _is_general_ok = self.is_general_ok()
        _has_doctor = True
        _doctor_building = self.get_doctor_building()
        _is_doctor_building_broken = False
        if _doctor_building:
            _fsm_state = _doctor_building.FSM.GetStateID()
            if _fsm_state == 'broken':
                _doctor_building = None
                _is_doctor_building_broken = True
        if not _doctor_building:
            _has_doctor = False

        if for_sick:
            result = [('text', 'COLOR_RED')]
            if not onclick:
                if not _has_doctor:
                    result.append( ('text', 'NEED_HOSPITAL') )
                else:
                    return None
            else:
                if not _has_doctor:
                    if _is_doctor_building_broken:
                        result.append( ('text', 'REPAIR_HOSPITAL') )
                    else:
                        result.append( ('text', 'BUILD_HOSPITAL') )
                else:
                    return None
            return result
        elif not _is_general_ok:
            result = [('text', 'COLOR_RED')]
            if not onclick:
                if not _has_doctor:
                    result.append( ('text', 'NEED_HOSPITAL') )
                else:
                    result.append( ('text', 'NEED_CURING_GENERAL') )
            else:
                if not _has_doctor:
                    if _is_doctor_building_broken:
                        result.append( ('text', 'REPAIR_HOSPITAL') )
                    else:
                        result.append( ('text', 'BUILD_HOSPITAL') )
                else:
                    result.append( ('text', 'CLICK_TO_CURE') )
            return result
        return None
# ============================================================
    def is_general_ok(self):
        _g = self.get_general()
        _is_in_sick_states = _g.is_in_sick_states()
        if _is_in_sick_states:
            return False
        return True
# ============================================================