# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *
# ============================================================
# === LeaderUser                                           ===
# ============================================================
class LeaderUser(object):
    '''Данные для таблицы рекордов'''
    def __init__(self, _uid = -1):
        self.uid = _uid  # число, нужно для того, чтобы связать данную структуру с C-struct, потому что мы не можем хранить Unicode имя игрока
        self.name = []  # SexyString - список wchar_t в виде int
        self.score = 0
        self.levels_complete = 0
        self.is_fake = True # по дефолту
# ============================================================
    def to_list(self, result):
        result.append('LeaderUser')     # marker
        result.append(len(self.name))
        for v in self.name:
            result.append(str(v))
        result.append(str(self.score))
        result.append(str(self.levels_complete))
        result.append(str(self.is_fake))
# ============================================================
    def from_list(self, _l):
        marker = _l.pop(0)
        if marker != 'LeaderUser': mbox('Error in saved data (LeaderUser)')
        name_len = int(_l.pop(0))
        self.name = []
        for i in range(0, name_len):
            self.name.append( int(_l.pop(0)) )
        self.score = int(_l.pop(0))
        self.levels_complete = int(_l.pop(0))
        self.is_fake = int(_l.pop(0))
# ============================================================
# === LevelStat                                            ===
# ============================================================
class LevelStat(object):
    COMPLETED_NO = 0
    COMPLETED_SILVER = 1
    COMPLETED_GOLD = 2
    def __init__(self):
        self.clear()
# ============================================================
    def clear(self):
        self.values = {}    # все настройки уровня в dict
        for val in ['completion_type', 'score', 'cur_time', 'secrets', 'secret_rooms']:
            self.values[val] = 0
# ============================================================
    def append_values(self, ext_values):
        for v in ext_values:
            self.values[v] = ext_values[v]
# ============================================================
    def get_completion_type(self):
        '''Возвращает значение пройденности уровня'''
        return self.values['completion_type']
# ============================================================
    def to_list(self, result):
        result.append('LevelStat')
        # ##
        count = 0
        for v in self.values:
            count += 1
        result.append(str(count))
        for v in self.values:
            result.append( v )
            result.append( str(self.values[v]) )
# ============================================================
    def from_list(self, _l):
        self.values = {}
        marker = _l.pop(0)
        if marker != 'LevelStat': mbox('Error in saved data (LevelStat)')
        # ##
        count = int(_l.pop(0))
        for i in range(0,count):
            name = _l.pop(0)
            val = int(_l.pop(0))
            self.values[name] = int(val)
# ============================================================
# === UserProgress                                         ===
# ============================================================
class UserProgress(object):
    values_names = ['total_score', 'last_completed_level', 'prev_experts',
                        'comics_shown', 'help_button_clicked', 'secret_rooms']
    def __init__(self):
        self.clear()
# ============================================================
    def clear(self):
        self.values = {}
        for val in self.values_names:
            self.values[val] = 0
        self.shown_comicses = [False, False, False]    # 2 комикса?
        self.levels_stat = []   # <LevelStat>   11 уровней
        for i in range(0,ProfileMgr.LEVELS_COUNT):   # создаем структуры для уровней
            self.levels_stat.append(LevelStat())
        self.prizes = [0 for i in range(12)]
# ============================================================
    def append_value(self, key, value):
        self.values[key] = value
# ============================================================
    def get_level_index(self, episode, level):
        index = level + (episode-1)*5
        #if episode > 1: index -= 1      # потому что тоже с единицы начинается отсчет, а уровни уже подряд идут
        if index < 0: index = 0
        return index
# ============================================================
    def get_level(self, episode, level):
        index = self.get_level_index(episode, level)
        trace('pmgr', index, len(self.levels_stat))
        return self.levels_stat[index]
# ============================================================
    def get_expert_levels_count(self):
        expert_levels_count = 0
        for i in range(1, ProfileMgr.LEVELS_COUNT):
            _level = self.levels_stat[i]
            if _level.values.completion_type == LevelStat.COMPLETED_GOLD:
                expert_levels_count += 1
        return expert_levels_count
# ============================================================
    def get_prize(self, index):
        return self.prizes[index]
# ============================================================
    def to_list(self, result):
        result.append('UserProgress')
        # ##
        count = 0
        for v in self.values:
            count += 1
        result.append(str(count))
        for v in self.values:
            result.append( v )
            result.append( str(self.values[v]) )
        # ##
        count = len(self.shown_comicses)
        result.append(str(count))
        for _i in self.shown_comicses:
            result.append(str(_i))
        # ##
        count = len(self.levels_stat)
        result.append(str(count))
        for lstat in self.levels_stat:
            lstat.to_list(result)
        count = len(self.prizes)
        result.append(str(count))
        for p in self.prizes:
            result.append( str(p) )
# ============================================================
    def from_list(self, _l):
        marker = _l.pop(0)
        if marker != 'UserProgress': mbox('Error in saved data (UserProgress)')
        # ##
        count = int(_l.pop(0))
        for i in range(0,count):
            name = _l.pop(0)
            val = int(_l.pop(0))
            self.values[name] = int(val)
        # ##
        count = int(_l.pop(0))
        self.shown_comicses = []
        for i in range(0,count):
            self.shown_comicses.append(int(_l.pop(0)))
        # ##
        count = int(_l.pop(0))
        self.levels_stat = []
        for i in range(0,count):
            lstat = LevelStat()
            lstat.from_list(_l)
            self.levels_stat.append(lstat)
        count = int(_l.pop(0))
        self.prizes = []
        for i in range(0,count):
            p = int(_l.pop(0))
            self.prizes.append( p )
# ============================================================
# === UserStat                                             ===
# ============================================================
class UserStat(object):
    def __init__(self):
        self.clear()
# ============================================================
    def clear(self):
        self.mode = 'normal'
        self.normal_mode = UserProgress()
        self.relax_mode = UserProgress()
        self.hard_mode = UserProgress()
        self.easy_mode = UserProgress()
        self.start_comics_shown = False
        self.center_comics_shown = False
        self.finish_comics_shown = False
        self.allow_hard = False # разрешен ли для игрока HARD режим?
        self.help_shown = 0     # сколько хелп-экранов показано игроку
# ============================================================
    def get_cur_progress(self):
        if self.mode == 'normal':
            return self.normal_mode
        elif self.mode == 'relax':
            return self.relax_mode
        elif self.mode == 'hard':
            return self.hard_mode
        elif self.mode == 'easy':
            return self.easy_mode
        else:
            return self.normal_mode # для безопасности
# ============================================================
    def to_list(self, result):
        result.append('UserStat')
        # ##
        result.append(self.mode)
        # ##
        self.normal_mode.to_list(result)
        self.relax_mode.to_list(result)
        self.hard_mode.to_list(result)
        self.easy_mode.to_list(result)
        # ---
        result.append(str(self.start_comics_shown))
        result.append(str(self.center_comics_shown))
        result.append(str(self.finish_comics_shown))
        result.append(str(self.allow_hard))
        result.append(str(self.help_shown))
# ============================================================
    def from_list(self, _l):
        marker = _l.pop(0)
        if marker != 'UserStat': mbox('Error in saved data (UserStat)')
        # ##
        self.mode = _l.pop(0)
        # ##
        self.normal_mode.from_list(_l)
        self.relax_mode.from_list(_l)
        self.hard_mode.from_list(_l)
        self.easy_mode.from_list(_l)
        # ---
        self.start_comics_shown = int(_l.pop(0))
        self.center_comics_shown = int(_l.pop(0))
        self.finish_comics_shown = int(_l.pop(0))
        self.allow_hard = int(_l.pop(0))
        self.help_shown = int(_l.pop(0))
# ============================================================
# ============================================================
# === ProfileMgr                                           ===
# ============================================================
# ============================================================
class ProfileMgr(Profiles_class):
    LEADERS_PER_MODE_COUNT = 7
    LEADERS_TOTAL_COUNT = 4 * ProfileMgr.LEADERS_PER_MODE_COUNT
    LEVELS_COUNT = 11+1
# ============================================================
    def __init__(self, cdata):
        Profiles_class.__init__(self, cdata)
        # Текущий пользователь и его данные
        self.cur_user_stat = UserStat()
        self.cur_user_name = []
        # Статистика Leaderboard'а
        self.leaderboard = []
        for i in range(0,self.LEADERS_TOTAL_COUNT):
            self.leaderboard.append(LeaderUser(-1))
        self.last_level_stats = LevelStat()
        self.last_level_index = -1
        self.load_leaderboard()
        # tmp:
        self.cur_user_name = [] #[0x5f, 0x74, 0x65, 0x73, 0x74, 0x5f] # '_test_'
        #if self.AddUser(self.cur_user_name) == True:    # если не ошибка, то пользователь уже существует
        #self.AddUser(self.cur_user_name)    # добавим пользователя, если его нет
        #self.AddUser([0x5f, 0x64, 0x65, 0x73, 0x74, 0x5f])
# ============================================================
    def init_user(self):
        self.set_current_user(self.cur_user_name)   # загрузим данные пользователя
# ============================================================
    def dispose(self):
        self.save_leaderboard()
# ============================================================
    def get_cur_user_progress(self):
        '''Возвращает текущий прогресс текущего игрока в выбранном режиме игры'''
        return self.cur_user_stat.get_cur_progress()
# ============================================================
    def Set_user_setting(self, setting, value):
        _parts = setting.split(' ')
        _s = _parts[0]
        if _s == 'help_shown':
            self.cur_user_stat.help_shown = value
            self.save_current_user()
        elif _s == 'prize':
            _index = int(_parts[1])
            _prize = value
            self.get_cur_user_progress().prizes[_index] = _prize
            self.save_current_user()
# ============================================================
    def Get_user_setting(self, setting):
        _parts = setting.split(' ')
        _s = _parts[0]
        _cur_progress = self.get_cur_user_progress()
        if _s == 'has_expert_levels':  # затычка для определения есть ли экспертные прохождения
            for level in _cur_progress.levels_stat:
                if level.get_completion_type() == LevelStat.COMPLETED_GOLD:
                    return 1
            return 0
        elif _s == 'total_game_score':
            _total_score = 0
            for level in _cur_progress.levels_stat:
                if level.get_completion_type() != LevelStat.COMPLETED_NO:   # если уровень пройден (на всякий случай проверяем)
                    _total_score += level.values.score
            return _total_score
        elif _s == 'total_secrets_found':
            _total_secrets = 0
            _index = 0
            _skip_index = -1
            if len(_parts) > 1:
                _episode = int(_parts[1])
                _level = int(_parts[2])
                _skip_index = _level + (_episode-1)*5 # пропускаем текущий уровень?
            for level in _cur_progress.levels_stat:
                if _index != _skip_index:
                    if level.get_completion_type() != LevelStat.COMPLETED_NO:   # если уровень пройден
                        _total_secrets += level.values.secrets
                _index += 1
            return _total_secrets
        elif _s == 'total_secret_rooms':
            _total_secret_rooms = 0
            _index = 0
            _skip_index = -1
            if len(_parts) > 1:
                _episode = int(_parts[1])
                _level = int(_parts[2])
                _skip_index = _level + (_episode-1)*5 # пропускаем текущий уровень?
            for level in _cur_progress.levels_stat:
                if _index != _skip_index:
                    if level.get_completion_type() != LevelStat.COMPLETED_NO:   # если уровень пройден
                        if 'secret_rooms' not in level.values:
                            level.values['secret_rooms'] = 0
                        _total_secret_rooms += level.values.secret_rooms
                _index += 1
            return _total_secret_rooms
        elif _s == 'help_shown':
            return self.cur_user_stat.help_shown
        elif _s == 'prize':
            _index = int(_parts[1])
            _prize = _cur_progress.prizes[_index]
            return _prize
        #elif _s == 'secrets':
        #    return _cur_progress.

        #elif setting == 'allow_hard':
        #    return self.cur_user_stat.allow_hard
        if _s in _cur_progress.values:
            return self.get_cur_user_progress().values[_s]
        else:
            return 0
# ============================================================
    def Get_user_info(self, info_tag):
        _cur_progress = self.get_cur_user_progress()
        if info_tag == 'levels_completeness':     # если нужно получить как пройдены уровни
            _need_delimeter = False
            result = ''
            for level in _cur_progress.levels_stat:
                if _need_delimeter:
                    result += '#'
                result += str(level.get_completion_type())
                _need_delimeter = True
            return result
        elif info_tag == 'game_mode':   # запрос режима игры
            return self.cur_user_stat.mode
        elif info_tag == 'game_mode_folder':   # запрос режима игры
            _mode = self.cur_user_stat.mode
            folder_names = {'normal':'levels',
                            'relax': 'levels',
                            'hard': 'levels_hard',
                            'easy': 'levels_easy'}
            if _mode in folder_names:
                return folder_names[_mode]
            else:
                return 'levels'
        elif info_tag == 'prizes':
            _need_delimeter = False
            result = ''
            for p in _cur_progress.prizes:
                if _need_delimeter:
                    result += '#'
                result += str(p)
                _need_delimeter = True
            return result
        return ''
# ============================================================
    def set_user_setting(self, key, value):
        self.get_cur_user_progress().append_value(key, value)
# ============================================================
    def names_equal(self, name1, name2):
        '''Проверяем имен SexyString формате - одно ли и то же это имя'''
        _len1 = len(name1)
        _len2 = len(name2)
        if _len1 != _len2:
            return False
        else:
            for i in range(0,_len1):
                sym1 = name1[i]
                sym2 = name2[i]
                if sym1 != sym2:
                    return False
            return True
# ============================================================
    def lb_begin(self):
        dict_lb_begin = {'normal':0,
                    'relax':self.LEADERS_PER_MODE_COUNT,
                    'hard':self.LEADERS_PER_MODE_COUNT * 2,
                    'easy':self.LEADERS_PER_MODE_COUNT * 3
                    }
        _mode = self.cur_user_stat.mode
        if _mode in dict_lb_begin:
            return dict_lb_begin[_mode]
        else:
            return 0
# ============================================================
    def lb_end(self):
        '''Выходит за границу элементов!'''
        dict_lb_end = { 'normal':self.LEADERS_PER_MODE_COUNT,
                    'relax':self.LEADERS_PER_MODE_COUNT * 2,
                    'hard':self.LEADERS_PER_MODE_COUNT * 3,
                    'easy':self.LEADERS_TOTAL_COUNT
                    }
        _mode = self.cur_user_stat.mode
        if _mode in dict_lb_end:
            return dict_lb_end[_mode]
        else:
            return self.LEADERS_PER_MODE_COUNT
# ============================================================
    #def on_update(self, ms):
    #    pass
# ============================================================
#    def save_unit_info(self, uid):
#        pass
# ============================================================
#    def load_unit_info(self, uid):
#        pass
# ============================================================
    def save_leaderboard(self):
        #us = UserStat()
        #_l = []
        #us.to_list(_l)
        #_data = ''
        #for item in _l:
        #    _data += str(item) + '#'
        #self.SaveLeaderboard(_data)
        #return
        _list = []
        _list.append('LEADERBOARD_02')
        _list.append( str(len(self.leaderboard)) )
        for user in self.leaderboard:
            user.to_list(_list)
        data = ''
        for item in _list:
            data += str(item) + '#'
        self.SaveLeaderboard(data)
# ============================================================
    def load_leaderboard(self):
        data = self.LoadLeaderboard()
        if len(data) == 0:  # если данных нет
            self.generate_initial_leaderboard()
            return
        _list = data.split('#')
        marker = _list.pop(0)
        if marker != 'LEADERBOARD_02': mbox('Error in saved data (LEADERBOARD_02), marker='+str(marker))
        count = int(_list.pop(0))
        self.leaderboard = []
        for i in range(0,count):
            luser = LeaderUser()
            luser.from_list(_list)
            self.leaderboard.append(luser)
# ============================================================
    def get_leaderboard(self):
        '''Возвращает список рекордов для текущего выбраного режима игры'''
        self.sort_leaderboard()
        begin = self.lb_begin()
        end = self.lb_end()
        result = []
        for i in range(begin, end):
            result.append( self.leaderboard[i] ) # LeaderUser
        return result
# ============================================================
    def get_user(self, name):
        '''Возвращает LeaderUser по имени игрока'''
        for i in range(self.lb_begin(), self.lb_end()):
            luser = self.leaderboard[i] # LeaderUser
            if self.names_equal(name, luser.name):
                return luser
        return None
# ============================================================
# ============================================================
    def add_user(self, name):
        DLG.PMgr.AddUser(name)
        if not self.cur_user_name:
            self.set_current_user(name)
            self.save_current_user()
# ============================================================
    def delete_user(self, username):
        _are_names_equal = self.names_equal(username, self.cur_user_name)
        if _are_names_equal:
            self.cur_user_name = []
            self.SetCurrentUser([])
        self.DeleteUser(username)
# ============================================================
    def rename_user(self, old_username, new_username):
        _renaming_cur_user = self.names_equal(old_username, self.cur_user_name)
        if _renaming_cur_user:
            self.save_current_user()    # на всякий случай сохраним все, что не было сохранено
        renamed = self.RenameUser(old_username, new_username)
        if renamed:
            # переименовать игрока в таблице рекордов
            luser = self.get_user(old_username)  # LeaderUser
            if luser:
                luser.name = new_username
                self.save_leaderboard() # и пересохранить ее
            if _renaming_cur_user:
                self.cur_user_name = new_username
                self.SetCurrentUser(new_username)   # чтобы в настройках сохранилось
                self.save_current_user()
# ============================================================
    def save_current_user(self):
        _len = len(self.cur_user_name)
        if _len:
            _list = []
            self.cur_user_stat.to_list(_list)
            data = ''
            for item in _list:
                data += str(item) + '#'
            self.SaveUserStat(self.cur_user_name, data)
            # заодно сохраним очки в таблицу рекордов:
            self.add_to_leaderboard(self.cur_user_name, self.get_game_fullscore())
            self.save_leaderboard()
# ============================================================
    def reload_current_user(self):
        self.cur_user_stat.clear()
        if self.cur_user_name:  # не пустое имя
            data = self.LoadUserStat(self.cur_user_name)
            _data_len = len(data)
            if _data_len > 0:
                _l = data.split('#')
                self.cur_user_stat.from_list(_l)
        else:
            DLG.Open_dialog('NeedProfileMenu', 'dlg_new_profile', {'tag':'first_start'})
# ============================================================
    def set_current_user(self, name):
        self.cur_user_name = name
        #data = self.LoadUserStat(name)
        #_data_len = len(data)
        #if _data_len > 0:
        #    _l = data.split('#')
        #    self.cur_user_stat.from_list(_l)
        self.reload_current_user()
        self.SetCurrentUser(name)   # запишет имя в настройки - текущий пользователь
# ============================================================
# ============================================================
    def add_to_leaderboard(self, name, score):
        begin = self.lb_begin()
        end = self.lb_end()
        minLBScore = 1000000000
        trace('begin, end', begin, end)
        for i in range(begin, end):
            _score = self.leaderboard[i].score
            if _score < minLBScore:
                minLBScore = _score
        if score < minLBScore:	# если не входит в таблицу
            return
        # найти пользователя с таким же именем
        user_index = -1
        for i in range(begin, end):
            if not self.leaderboard[i].is_fake:
                _same_names = self.names_equal(name, self.leaderboard[i].name)
                if _same_names:
                    user_index = i
                    break
        if user_index < 0: # если в таблице юзера нет
            lbitem = self.leaderboard[end-1]    # последняя запись
            lbitem.name = name
            lbitem.score = score
            lbitem.is_fake = False
        else:
            lbitem = self.leaderboard[user_index]
            lbitem.score = score
        self.sort_leaderboard()
# ============================================================
    def sort_leaderboard(self):
        begin = self.lb_begin()
        end = self.lb_end() - 1
        #j = end
        k = end
        lb = begin      # границы неотсортированной части массива
        ub = end        # границы неотсортированной части массива

        condition = True
        while condition:
            # проход снизу вверх
            j = ub
            while j > lb:    # for( j = ub; j > 0; j-- )
                if int(self.leaderboard[j-1].score) < int(self.leaderboard[j].score):
                    tmp_user = self.leaderboard[j-1]
                    self.leaderboard[j-1] = self.leaderboard[j]
                    self.leaderboard[j] = tmp_user
                    k = j   # крайняя левая позиция, в которой была сортировка
                j -= 1
            lb = k + 1
            if lb < ub:
                # проход сверху вниз
                j = lb
                while j <= ub:
                    if int(self.leaderboard[j-1].score) < int(self.leaderboard[j].score):
                        tmp_user = self.leaderboard[j-1]
                        self.leaderboard[j-1] = self.leaderboard[j]
                        self.leaderboard[j] = tmp_user
                        k = j
                    j += 1
                ub = k - 1
            condition = ( lb < ub )
# ============================================================
    def add_leader_internal(self, index, name, score, levels):
        luser = self.leaderboard[index]
        luser.name = name
        luser.score = score
        luser.levels_complete = levels
# ============================================================
    def generate_initial_leaderboard(self):
        _names = self.GetPredefinedNames()
        _names_count = len(_names)
        #_scores = []
        #if self.cur_user_stat.mode == 'normal':
        _scores = [26975000, 25841300, 25096100, 24197200, 22859700, 20213500, 0]
        for i in range(0,_names_count):
            self.add_leader_internal(i, _names[i], _scores[i], 11)
        #elif self.cur_user_stat.mode == 'relax':
        _scores = [13237700, 12985300, 12674100, 11813300, 10929500, 10513800, 0]
        for i in range(0, _names_count):
            self.add_leader_internal(self.LEADERS_PER_MODE_COUNT + i, _names[i], _scores[i], 11)
        #elif self.cur_user_stat.mode == 'hard':
        _scores = [20910300, 19187200, 18653100, 17992500, 17536700, 16829300, 0]
        for i in range(0, _names_count):
            self.add_leader_internal(2*self.LEADERS_PER_MODE_COUNT + i, _names[i], _scores[i], 11)
        #elif self.cur_user_stat.mode == 'easy':
        _scores = [108135500, 106759200, 105981400, 104382600, 103867300, 103129800, 0]
        for i in range(0, _names_count):
            self.add_leader_internal(3*self.LEADERS_PER_MODE_COUNT + i, _names[i], _scores[i], 11)
        self.sort_leaderboard() # на всякий случай
# ============================================================
    def get_game_fullscore(self):
        full_score = 0
        for i in range(1,self.LEVELS_COUNT):
            lstat = self.cur_user_stat.get_cur_progress().levels_stat[i]
            # проблема - сохраняются числа, а не текст!
            if lstat.values.completion_type != LevelStat.COMPLETED_NO:
                full_score += lstat.values.score
        return full_score
# ============================================================
    def cheat_all_complete(self):
        progress = self.cur_user_stat.get_cur_progress()
        for lstat in progress.levels_stat:
            lstat.values['completion_type'] = LevelStat.COMPLETED_GOLD
# ============================================================
    def cheat_complete_next(self):
        progress = self.cur_user_stat.get_cur_progress()
        for lstat in progress.levels_stat:
            values = lstat.values
            if 'completion_type' not in values or values.completion_type == LevelStat.COMPLETED_NO:
                lstat.values['completion_type'] = LevelStat.COMPLETED_GOLD
                return
# ============================================================
    def cheat_complete_next_nonexpert(self):
        progress = self.cur_user_stat.get_cur_progress()
        for lstat in progress.levels_stat:
            values = lstat.values
            if values.completion_type == LevelStat.COMPLETED_NO:
                lstat.values['completion_type'] = LevelStat.COMPLETED_SILVER
                return
# ============================================================
