# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *


class QuestTaskData(object):
    '''Данные для задания квеста - это НЕ само задание!'''
    def __init__(self, map_index = 0, type = '', image = '', frame = 0, task_template = '', params = None, text = ''):
        self.ser_name = 'QuestTaskData'
        self.type = type    # task_action
        self.map_index = map_index
        self.image = image
        self.frame = frame
        self.task_template = task_template
        self.params = params
        self.items_need_count = 1
        self.task_prefix_item_name = text
        self.click_uids = ''
        # ----
        self.task_items = []            # элементы, над которыми должно быть выполнено действие
        # ----
        self.resName = ''               # какой ресурс нужен индейцу
        self.resCount = 0               # сколько ресурса нужно индейцу
        # ----
        self.quest_item_name = '__none__'   # not set
        self.quest_item_count = 0
        # ----
        self.quest_npc_unique_names = []
        # ----
        self.event_name = ''
        # ----
        if params != None:
            self.parse_params() # распарсим всё что нужно
# ============================================================
    def parse_params(self):
        t = self.type
        if t == 'build_building':
            uids_prefix = 'uids:'
            if len(self.params) > len(uids_prefix) and self.params[:len(uids_prefix)] == uids_prefix: # если набор объектов
                self.task_items = BMgr.get_items(self.params, self.map_index)
                self.items_need_count = len(self.task_items)    # кол-во должно быть равно кол-ву элементов
            else:
                _parts = self.params.split(';')
                count = len(_parts)
                if count == 2:  # кол-во = 1 by default
                    _clone = _parts     # tinypy bug workaround
                    (type, subtype) = _clone
                    self.template = {'type':type, 'subtype':subtype}
                    self.items_need_count = 1
                elif count == 3:    # кол-во задано третьим параметром
                    _clone = _parts     # tinypy bug workaround (in elif direct assign will fail)
                    (type, subtype, quantity) = _clone
                    self.template = {'type':type, 'subtype':subtype}
                    self.items_need_count = int(quantity)
                else:
                    error = to_str("FATAL ERROR: no ';' symbol in ", self.params)
                    mbox(error)
        elif t == 'give_resources':
            # если задание просто на передачу ресурсов
            self.template = {}
            self.task_items = []
            _res_parts = self.params.split(';')
            _res_parts_len = len(_res_parts)
            if _res_parts_len == 2:
                self.resName = _res_parts[0]
                self.resCount = int(_res_parts[1])
        elif t == 'give_object':
            # если задание на передачу объекта инвентори
            self.template = {}
            self.task_items = []
            parts = self.params.split(';')
            self.quest_item_name = parts[0]
            if len(parts) > 1:
                self.quest_item_count = int(parts[1])
            else:
                self.quest_item_count = 1
        elif t == 'talk':
            self.quest_npc_unique_names = []
            _tmp = self.params.split(';')
            for name in _tmp:
                if name:
                    self.quest_npc_unique_names.append(name)
        elif t == 'visit_sublevel':
            pass
        elif t == 'universal_event':
            self.event_name = self.params
        else:
            self.template = {}
            self.task_items = BMgr.get_items(self.params, self.map_index)
            self.items_need_count = len(self.task_items)    # кол-во должно быть равно кол-ву элементов
        #else:
        #    self.template = {}
        #    self.task_items = []
# ============================================================
    def generate_needed_res(self):
        return UStats.CreateRes(self.resName, self.resCount)
# ============================================================
    def clean_task_items(self):
        '''Для избавления от ссылок после старта квеста'''
        self.task_items = []
# ============================================================
# ============================================================
#@statemachine
class Quest(object):
    '''Базовый класс для квестов, абстрактный класс!'''
    def __init__(self, name = '', data_dict = None, map_index = 0):
        self.ser_name = 'Quest'
        self.uid = -1
        self.type = 'Quest'
        self.name = name
        self.map_index = map_index
        self.params = data_dict
        self.is_final = self.get_dattr(data_dict, 'is_final')   # если это финальный квест
        self.is_complete = False    # может изменяться в is_start_possible
        self.is_precomplete = False # если задания выполнены и нужно сдать квест
        self.is_started = False
        self.is_tutorial = self.get_dattr(data_dict, 'is_tutorial')
        self.need_visible_items = []    # элементы, которые должны быть видны для начала квеста
        self.need_exist_UIDs = []    # UID'ы элементов, которые должны существовать для начала квеста
        self.task_start_activate_items = [] # элементы, которые необходимо "активировать" при старте квеста
        self.task_start_camera_move = self.get_dattr(data_dict, 'task_start_camera_move')   # перемещать ли камеру на 'task_start_activate_items'
        self.task_complete_activate_items = []
        self.npc = None                     # NPC, у которого нужно брать квест
        self.quest_complete_activate = self.get_dattr(data_dict, 'quest_complete_activate')
        self.qcomplete_show_npc = self.get_dattr(data_dict, 'qcomplete_show_npc')
        self.quest_complete_show_npc = None
        self.text = self.get_dattr(data_dict, 'text')
        self.text_complete = ''
        self.text_complete = self.get_dattr(data_dict, 'text_complete')
        self.extra_info = ''
        self.extra_info = self.get_dattr(data_dict, 'extra_info')
        self.parent_quest_name = self.get_dattr(data_dict, 'parent_quest')  # имя родительского квеста
        self.after_parent_quest_completed = self.get_dattr(data_dict, 'parent_quest_completed') # можно запускать только после окончания родительского квеста
        self.quest_tasks = []       # здесь будут те таски, которые будут добавляться в UI
        self.quest_tasks_datas = []       # это НЕ те таски, которые добавляются в UI !!! Здесь список 'QuestTaskData'
        self.extra_auto_activate = self.get_dattr(data_dict, 'auto_activate')
        self.extra_wait_time = 0    # для доп. 2 секунд перед стартом квеста с need_visible_items
        # заполняем данные о заданиях для квеста (одно или два):
        if data_dict != None:
            for i in range(1,3):
                task_action = 'task'+str(i)+'_action'
                if task_action in data_dict:
                    if data_dict[task_action]:
                        if data_dict[task_action] != '__none__':
                            _str_i = str(i)
                            task_image = 'task'+_str_i+'_image'
                            task_image_frame = 'task'+_str_i+'_img_frame'
                            task_template = 'task'+_str_i+'_template'
                            task_params = 'task'+_str_i+'_params'
                            task_text = 'task'+_str_i+'_text_part'
                            task_uids = 'task'+_str_i+'_uids'
                            trace( 'quest task >>', task_action, task_image, task_image_frame, task_params )
                            #trace(data_dict)
                            qt = None
                            if task_template in data_dict:
                                qt = QuestTaskData(map_index, data_dict[task_action], data_dict[task_image],\
                                                    data_dict[task_image_frame], data_dict[task_template],\
                                                    data_dict[task_params], data_dict[task_text])
                            else:
                                qt = QuestTaskData(map_index, data_dict[task_action], data_dict[task_image],\
                                                    data_dict[task_image_frame], '',\
                                                    data_dict[task_params], data_dict[task_text])
                            _click_uids = self.get_dattr(data_dict, task_uids)
                            qt.click_uids = _click_uids
                            self.quest_tasks_datas.append(qt)
            #self.type = "__none__"
            self.need_visible_items = BMgr.get_items(self.get_dattr(data_dict, 'need_visible_items'), map_index)
            self.quest_complete_show_npc = BMgr.get_items(self.qcomplete_show_npc, map_index)
            self.need_exist_UIDs = BMgr.get_UIDs_list(self.get_dattr(data_dict, 'need_exist_items'))
            self.task_start_activate_items = BMgr.get_items(self.get_dattr(data_dict, 'task_start_activate'), map_index)
            self.task_complete_activate_items = BMgr.get_items(self.get_dattr(data_dict, 'task_complete_activate'), map_index)
            if data_dict['npc'] and data_dict['npc'] != 'uids:':
                _npc = BMgr.get_items(data_dict['npc'], map_index)
                trace('>Quest NPC>', _npc)
                if _npc[0]:
                    self.npc = _npc[0]
                    self.npc.assign_quest(self)
            self.qstarted_remove_priority = self.get_dattr(data_dict, 'qstarted_remove_priority')
# ============================================================
    def get_dattr(self, d, name):
        if d == None: return ''
        if name in d:
            return d[name]
        return ''
# ============================================================
    def get_first_qtd(self):
        return self.quest_tasks_datas[0]
# ============================================================
    def on_update(self, ms):
        pass
# ============================================================
    def is_only_res_not_enough_to_start(self):
        '''Затычка для того, чтобы проверить, что квесту не хватает только ресурсов'''
        return False
# ============================================================
    def is_already_completed(self):
        '''Если квест выполнен еще до старта (пока только для quest_tut_upgrade)'''
        return False
# ============================================================
    #@override
    def is_final_to_start(self):
        '''Можно ли запустить финальный квест (проверяет родительский и активацию сообщением)'''
        if self.is_final:
            # а дальше пошла копипаста...
            if self.parent_quest_name:
                _parent_q = QMgr.get_quest(self.parent_quest_name)
                if _parent_q:
                    if self.after_parent_quest_completed:
                        if _parent_q.is_complete:
                            return True
                    elif _parent_q.is_started:
                        return True
            elif self.extra_auto_activate:    # если еще не активировали сообщением
                    return False
            return True
        else:
            return False
# ============================================================
    def is_final_ready(self):
        '''Готов ли финальный квест к выполнению (выполнены ли все главные задания)'''
        if self.is_final:
            _tasks_complete = UStats.GetLevelValue('main_tasks_complete') > 0
            if _tasks_complete:
                return True
        return False
# ============================================================
    def is_parent_awaiting(self):
        '''Ожидаем запуска или окончания родительского квеста'''
        if self.parent_quest_name:
            _parent_q = QMgr.get_quest(self.parent_quest_name)
            if _parent_q:
                if self.after_parent_quest_completed:
                    return not _parent_q.is_complete
                else:
                    return not _parent_q.is_started
        return False
# ============================================================
    #def on_quest_event(self, name, sender, extra_dict):
    #    pass
# ============================================================
    def on_extra_event(self, event):
        if event == self.extra_auto_activate:
            self.extra_auto_activate = ''   # так мы активируем квест для индейцев... Плохо, но вот.
            if self.npc:
                self.npc.SendMsg('quest_activated')
# ============================================================
    def are_needed_items_exist(self):
        '''Существуют ли необходимые для начала квеста элементы'''
        for uid in self.need_exist_UIDs:
            _item = BMgr.get_item(uid, self.map_index)
            if not _item:
                return False
        return True
# ============================================================
    def _is_giveobject_start_possible(self):
        if self.is_started:
            return True
        types = [qtd.type for qtd in self.quest_tasks_datas]    # типы завершаемых заданий
        if 'give_object' in types:  # хак для бага со стороны левелдизайна - чтобы только на квесты по сбору предметов распространялось
            _count = UStats.Inventory.GetQuestItemsCount()
            if _count >= 4: # максимальное число мест в сумке   (чтобы больше нельзя было дать объектов)
                return False
        #    _give_obj_quest_started = UStats.GetLevelValue('give_obj_quest_started') > 0
        #    if _give_obj_quest_started:
        #        return False
        return True
# ============================================================
    def is_start_possible(self, skip_ne_cases = False):
        '''Проверяет выполнимость стартовых условий'''
        if not skip_ne_cases:
            if self.is_final:
                _main_tasks_complete = UStats.GetLevelValue('main_tasks_complete') > 0
                if not _main_tasks_complete:    # 525: incomplete
                    return False            # финальный квест нельзя запустить пока не выполнены задания уровня
        if self.need_visible_items: # добавил проверку
            for _item in self.need_visible_items:
                _is_passable = _item.IsPassable()
                if not _is_passable:
                    return False
        if not self.is_tutorial:
            _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
            if _has_barbarians:
                return False    # нетуториальные при варварах запускать нельзя
        _talk_quest_started = UStats.GetLevelValue('talk_quest_was_started') > 0
        if _talk_quest_started:
            return False        # 525 talk quest started
        _is_giveobject_start_possible = self._is_giveobject_start_possible()
        if not _is_giveobject_start_possible:
            return False
        #@ _need_inventory_items = UStats.Inventory.NeedAnyQuestItem()
        #if _need_inventory_items:
        #    return False
        if self.parent_quest_name:
            _parent_q = QMgr.get_quest(self.parent_quest_name)
            if _parent_q:
                if self.after_parent_quest_completed:
                    #@if _need_inventory_items:   # если сейчас требуются какие-то объекты, то нельзя стартовать
                    #@    return False
                    return _parent_q.is_complete
                else:
                    return _parent_q.is_started
            else:
                mbox('Parent quest was not found: ' + self.parent_quest_name)
        else:
            # учитываем ситуацию, когда собирают объекты в инвентори, но переходя в подуровен:
            #@if _need_inventory_items:
            #@    return False
            if self.extra_auto_activate:    # если еще не активировали сообщением
                return False
            return True
# ============================================================
    def start(self):
        '''Запускает квест, генерирует необходимые задания (таски),
        активирует все, что нужно и т.д.'''
        if self.qstarted_remove_priority:
            if ';' in self.qstarted_remove_priority:
                _parts = self.qstarted_remove_priority.split(';')
                BMgr.AddRepaired(_parts[0], _parts[1])
            else:
                BMgr.AddRepaired(self.qstarted_remove_priority, 'level_1')
        # рассылаем сообщения о запуске квеста:
        #for item in BMgr.all_buildings:
        #    item.SendMsg('quest_start', self.name)
        for item in BMgr.all_levels_units:
            _stype = item.get('subtype')
            if _stype not in ['worker', 'woodcutter', 'qman']:  # им данные о квестах не нужны
                if self.type != 'QuestGiveRes':    # этот квест с другими не связан никак
                    item.SendMsg('quest_start', self.name, str(self.map_index))
        #for item in BMgr.all_items:
        #    _stype = item.get('subtype')
        #    if _stype not in ['tree', 'bonfire', 'stone']:             # деревьев много и им данные о квестах не нужны
        #        item.SendMsg('quest_start', self.name)
        # активируем необходимые для выполнения квеста объекты поля:
        _qtd = self.get_first_qtd()
        _task_items_count = len(_qtd.task_items)
        for _item in self.task_start_activate_items:    # помигаем объектами и покажем стрелку
            _item.activate()
            _layers = ['floor','default','unit','resources','upgrading','build','anim']
            _item.Drawer.Blink(_layers, 2)
            _qtd = self.get_first_qtd()
            if _qtd.type != 'build_building': # нужно исключить из квеста на строительство
                _has_uid = BMgr.has_item_uid(_item) # чтобы не показывать стрелку над "уничтоженными" объектами
                if _has_uid:
                    _item.show_arrow()  # покажем стрелку
        if not _task_items_count:
            if self.task_start_camera_move:
                if self.task_start_activate_items:
                    Camera.MoveTo(self.task_start_activate_items[0])
        if self.npc:
            self.npc.deactivate()  # NPC больше не должен быть кликабельным (только после окончания выполнения квеста)
        if _task_items_count:
            _first_item = _qtd.task_items[0]
            Camera.MoveTo(_first_item)    # переводим камеру на первый элемент
        if self.extra_info == 'show_b_arrow':   # если нужно показать стрелку при старте квеста
            BMgr.ShowBuildButton(True)
        elif self.extra_info == 'hack_level1_shop':
            BMgr.ShowBuildButton(True)
            BMgr.EnableShopBuildings(False)
            BMgr.SendGlobalMsg('show_build_button', '')
        if _qtd.type in ['repair', 'repair_boat', 'upgrade']:
            _items = _qtd.task_items
            _item = _items[0]
            _name = _item.get('name')
            BMgr.AddException(_name, _qtd.type == 'repair')
        elif _qtd.task_template:
            _template = _qtd.task_template
            _parts = _template.split(':')
            _parts_len = len(_parts)
            if _parts_len == 1:
                BMgr.AddException(_template)
            elif _parts_len == 2:
                BMgr.AddException(_parts[0])
        #_start_items_count = len(self.task_start_activate_items)
        #if _start_items_count:
        #    _first_item = self.task_start_activate_items[0]
        #    Camera.MoveTo(_first_item)    # переводим камеру на первый элемент
        # обычно после этого вызывается еще переопределнная часть метода
# ============================================================
    def after_start(self):
        self.task_start_activate_items = [] # избавляемся от ссылок
        self.need_visible_items = []    # избавляемся от ссылок
        for qtd in self.quest_tasks_datas:
            qtd.clean_task_items()      #@! надеюсь после это уже не потребуется, иначе FAIL
# ============================================================
    def add_game_tasks(self):
        #'''Когда NPC (или кто-то еще) дал задание - он вызывает данный метод и
        #игроку даются игровые задания'''
        pass
# ============================================================
    def complete_quest_tasks(self):
        for qt in self.quest_tasks:
            qt.events_remain = 0
            qt.set_complete()
# ============================================================
    def complete_quest_one_task(self, qtd_index = 0):
        if self.quest_tasks:
            _qtd_count = len(self.quest_tasks)
            if qtd_index < _qtd_count:
                qt = self.quest_tasks[0]
                qt.decrease_remain_count()
# ============================================================
    def complete(self):
        '''Завершает квест и считает его выполненным'''
        trace('QUEST complete method start', self.name)
        self.on_complete_begin()
        BMgr.AddException(None)
        if not self.is_tutorial:
            UStats.AddLevelValue('progress', 1) # обновляем прогресс
        if self.extra_info == 'hack_level1_shop':
            BMgr.EnableShopBuildings(True)
        if self.is_tutorial:
            if self.text_complete:
                UI.show_tut_text(self.text_complete)
        for qtd in self.quest_tasks:          # удаляем с интерфейса задания квеста
            UI.SendMessage(qtd.slot, 'task_remove')
        _was_camera_move = False
        for _item in self.task_complete_activate_items:
            _item.activate()
            if not _was_camera_move:    # and _item.get("subtype") == 'no_fog':
                _was_camera_move = True
                Camera.MoveTo(_item)
            _layers = ['floor','default','unit','resources','upgrading','build','anim']
            _item.Drawer.Blink(_layers, 2)
        self.task_complete_activate_items = []  # избавимся от ссылок
        if self.quest_complete_activate:    # активируем квест, если есть чего активировать
            QMgr.activate_quest(self.quest_complete_activate, self.map_index)
        if self.quest_complete_show_npc:
            for npc in self.quest_complete_show_npc:
                npc.show_as_possible()
        self.quest_tasks = []
        # индеец должен отдать подарок, если он указан
        if self.npc:
            self.npc.SendMsg('your_quest_complete', self.name)
            _is_time_up = UStats.GetLevelValue('time_is_up') > 0
            _gives = self.params['gives']
            if _gives:  # если есть что подарить
                _give_gift = True
                if self.is_final:
                    if _is_time_up: # если время уже вышло, то артефакт не даем
                        _give_gift = False
                if _give_gift:
                    self.npc.publish_gift(_gives)
            self.npc.deactivate()      # на всякий случай
            #self.npc.activate_next_quest()   # чтобы больше не был связан с этим квестом
            self.npc.try_activate_next_quest()   # чтобы больше не был связан с этим квестом
            self.npc = None
            if self.is_final:
                if _is_time_up: # если время уже вышло, то просто покажем диалог статистики
                    EMgr.send_event('game_win', None)   # нужно закончить игру
        if self.is_final:
            # заполнить шкалу прогресса
            UStats.SetLevelValue('progress', 100)   # заведомо большое число
        self.is_complete = True
        self.on_complete_end()
        trace('QUEST COMPLETE method end', self.name)
# ============================================================
    def on_complete_begin(self):
        pass
# ============================================================
    def on_complete_end(self):
        pass
# ============================================================
    def on_after_complete(self):
        '''Данный метод вызывается извне, чтобы можно было начислить очки после выполнения квеста и т.д.'''
        pass
# ============================================================
    def is_quest_precomplete(self):
        '''Квест выполнен - его нужно сдать'''
        return self.is_precomplete and not self.is_complete
# ============================================================
    def autostart(self, sender):
        '''Используется для автоматического запуска туториальных квестов (учитывая, что
        они по порядку должны следовать'''
        pass
# ============================================================
    def __str__(self):
        extra = to_str('QUEST: ', self.uid, 'is_complete', self.is_complete, 'is_started', self.is_started, 'is_tut', self.is_tutorial)
        return to_str('{', extra, '}', str(self))
# #######################################################
# #######################################################
    def get_tasks_text(self, complete = False):
        '''Генерация текста для заданий (в диалоге)'''
        _list = []
        _need_newline = False
        for _qtd in self.quest_tasks_datas:
            if _need_newline:
                _list.append('newline')
            _task_text_list = self.get_task_text_list(_qtd, complete)
            _list.extend(_task_text_list)
            _need_newline = True
        return tools.ListToStr(_list)
# ============================================================
    def get_tasks_text_for_panel(self, qtd):
        '''Генерация текста для заданий (на панельках)'''
        _list = self.get_task_text_list(qtd, False, False) # это для подсказок над панельками заданий
        return tools.ListToStr(_list)
# ============================================================
    def get_taskslide_text(self, _qtd):
        '''Генерация текста одного задания (в диалоге)'''
        _text_action = ''
        _t = _qtd.type
        _str_map = {'build_building':'TASK_BUILD', 'give_resources':'DLG_TASK_GIVE_RES',
                    'give_object':'TASK_BRING', 'repair':'TASK_REPAIR',
                    'repair_boat':'TASK_REPAIR', 'building_upgrade':'TASK_UPGRADE',
                    'general_break':'DLG_TASK_GENERAL_BREAK', 'break':'DLG_TASK_BREAK',
                    'talk':'DLG_TASK_TALK', 'do_action':'DLG_TASK_DO_ACTION',
                    'visit_sublevel':'TASK_SUBLEVEL', 'universal_event':'TASK_UNIVERSAL',
                    'cure_unit':'DLG_TASK_HEAL'}
        if _t in _str_map:
            _text_action = self._get_text_action(_str_map[_t])
        # ###
        _text = self.get_qtd_action_text(_qtd, True)
        if _text: _text_action = _text
        return _text_action
# ============================================================
    _repair__repair_boat = ['repair', 'repair_boat']
    def get_task_text_list(self, _qtd, complete = False, for_window = True):
        '''Генерация текста одного задания (в диалоге)'''
        _list = []
        _need_digits = True  # нужно ли дописывать цифры и прочее после _text_action
        _count = 0
        _text_action = self._get_text_action('')
        _text_item_prefix = ''
        _t = _qtd.type
        if _t == 'build_building':
            _count = _qtd.items_need_count
            _text_action = self._get_text_action('DLG_TASK_BUILD')
        elif _t == 'give_resources':
            _count = _qtd.resCount
            _text_action = self._get_text_action('DLG_TASK_GIVE_RES')
        elif _t == 'give_object':
            _count = _qtd.quest_item_count
            _text_action = self._get_text_action('DLG_TASK_BRING')
        elif _t in self._repair__repair_boat:
            _count = _qtd.items_need_count
            _text_action = self._get_text_action('DLG_TASK_REPAIR')
        elif _t == 'building_upgrade':
            _count = _qtd.items_need_count
            _text_action = self._get_text_action('DLG_TASK_UPGRADE')
            _text = self.get_qtd_action_text(_qtd)
            if _text:
                _need_digits = False
                _text_action = _text
        elif _t == 'general_break':
            _count = _qtd.items_need_count
            _text_action = self._get_text_action('DLG_TASK_GENERAL_BREAK')
        elif _t == 'break':
            _count = _qtd.items_need_count
            _text_action = self._get_text_action('DLG_TASK_BREAK')
        elif _t == 'talk':
            _count = len(_qtd.quest_npc_unique_names)
            _text_action = self._get_text_action('DLG_TASK_TALK')  # для бага 88 править здесь
        elif _t == 'do_action':
            _count = _qtd.items_need_count
            if 'text_action' not in _qtd:
                _text_action = self._get_text_action('DLG_TASK_DO_ACTION')          # ###
                _text = self.get_qtd_action_text(_qtd)
                if _text:
                    _text_action = _text
                _qtd.text_action = _text_action
            else:
                _text_action = _qtd.text_action
        elif _t == 'visit_sublevel':
            _count = 0
            _text_action = self._get_text_action('DLG_TASK_SUBLEVEL')
        elif _t == 'universal_event':
            _text_action = self._get_text_action('')
        elif _t == 'cure_unit':
            if _qtd.items_need_count > 1:
                _count = _qtd.items_need_count
            else:
                _count = 0
            _text_action = self._get_text_action('DLG_TASK_HEAL')
        #elif _t == '':
        #    pass
        if for_window:
            if not complete:
                _list.append(['text', 'TASK_STAR'])
            else:
                _list.append(['text', 'TASK_CHECK'])
        if for_window: _list.append(['text', 'COLOR_TASKS_TEXT'])
        else: _list.append(['text', 'COLOR_HEADER'])
        _list.append(['text', _text_action])
        if _need_digits:
            if _count > 0:
                if for_window:
                    _list.append(['text', 'COLOR_TASKS_NUMBER'])
                else:
                    _list.append(['text', 'COLOR_HEADER'])
                _list.append('space')
                _list.append(['digit', _count])
                if for_window:
                    _list.append(['text', 'COLOR_TASKS_TEXT'])
                else:
                    _list.append(['text', 'COLOR_HEADER'])
            _list.append('space')
            _textid_prefixed = self._get_textid_with_count(_qtd.task_prefix_item_name, _count)
            _list.append(['text', _textid_prefixed])
        return _list
# ============================================================
    def _get_text_action(self, default_text):
        _text_action = self.get_dattr(self.params, 'text_action')
        if _text_action == '':
            _text_action = default_text
        return _text_action
# ============================================================
    def _get_textid_with_count(self, _prefix, _count):
        if _count <= 1:
            return _prefix
        elif _count >= 2 and _count < 5:
            return _prefix+'_2'
        elif _count >= 5:
            return _prefix+'_5'
# ============================================================
    def get_qtd_action_text(self, qtd, for_task_slide = False):
        _item = None
        #mbox(to_str(qtd))
        _items_count = len(qtd.task_items)
        if _items_count:
            _item = qtd.task_items[0]
        if _item:
            _task_text = _item.get_action_text(for_task_slide)
        else:
            _task_text = ''
        return _task_text
# ============================================================
    def on_dlg_callback(self, result, extra_dict):
        '''Обработка данных от диалога (диалог у нас в другой VM)'''
        if result == 'ok':      # других и быть не должно
            _needed_items_exist = self.are_needed_items_exist()
            if not _needed_items_exist:
                QMgr.complete_quest_early(self)
# ============================================================
    def get_tips_extra_text(self, ui_tips = False):
        '''Возвращает текст для доп. строки типса у индейца'''
        return ('','')
# ============================================================
    def get_tips_click_text(self):
        '''Возвращает текст для типса у индейца при клике'''
        return []
# #######################################################
# #######################################################
# #######################################################
