# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import quest
import game_task


class TutBreakTask(game_task.GameTask):
    '''Задание квеста (у игрока задание отображается в нижнем правом углу)'''
    def __init__(self, qtdata = None, items = None):
        game_task.GameTask.__init__(self, qtdata)
        self.ser_name = 'TutBreakTask'
        if qtdata != None:
            self.events_count = qtdata.items_need_count     #^ кол-во событий (game_event), необходимое для выполнения задания
            self.events_remain = self.events_count          #^ сколько событий осталось дождаться (при 0 - задание выполнено)
        self.game_event = 'pot_broken'     #^ событие, которое должно быть сгенерировано игрой, чтобы данное задание проверило его выполнение
        self.items = items
        self.is_prizes_count = False
# ============================================================
    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        if name == self.game_event:     # наше событие завершения
            if obj in self.items:
                self.events_remain -= 1
                self.update_count()
                if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
                    self.set_complete()                     # значит уже все сделали
# ============================================================
# ============================================================
# ============================================================
class QuestTutBreak(quest.Quest):
    def __init__(self, name = '', data_dict = None, map_index = 0):
        quest.Quest.__init__(self, name, data_dict, map_index)
        self.ser_name = 'QuestTutBreak'
# ============================================================
    def is_start_possible(self, skip_ne_cases = False):
        '''Проверяет выполнимость стартовых условий'''
        #if not QMgr.has_started_quests():      # если нет запущенных квестов   # > необязательно проверять - вызывающий код сам проверяет...
        _needed_items_exist = self.are_needed_items_exist()
        if not _needed_items_exist:   # чтобы быстрее его же закончить
            return True
        if QMgr.cur_cluster.time_without_quest >= self.params.time: # если прошло уже достаточно времени
            # нужно проверить, что данный квест еще не выполнен:
            qtd = self.quest_tasks_datas[0]   # в туториале только одно задание
            pots = qtd.task_items
            pots_count = len(pots)
            _incomplete_pots = 0
            for pot in pots:    # считаем сколько несломанных горшков
                if pot:
                    _is_pot_complete = pot.had_action   # достаточно этого условия
                    if not _is_pot_complete:
                        _incomplete_pots += 1
            qtd.items_need_count = _incomplete_pots # выставим кол-во несломанных горшков
            _visible_items = self.need_visible_items
            _visible_count = len(_visible_items)
            if pots_count < 1 or _incomplete_pots == 0:
                #self.is_complete = True     # горшки уже разбиты
                self.complete()
                return False
            elif _visible_items and _visible_count:
                pot = _visible_items[0]
                if pot:
                    if pot.IsInWindow() and not pot.Drawer.IsPartiallyInFog():    # если горшок виден на экране полностью
                        return True         # значит можно запускать квест
                    #if self.need_visible_items:
                    #    for _item in self.need_visible_items:
                    #        if not _item.IsPassable():
                    #            return False
        return False
# ============================================================
    def start(self):
        '''Запускает квест, генерирует необходимые задания (таски),
        активирует все, что нужно и т.д.'''
        quest.Quest.start(self)
        self.add_game_tasks()
        self.is_started = True  #! строго в конце метода
        pass
# ============================================================
    def add_game_tasks(self):
        #'''Когда NPC (или кто-то еще) дал задание - он вызывает данный метод (ну или start - позже будет ясно) и
        #игроку даются игровые задания'''
        if not self.is_started:
            qtd = self.quest_tasks_datas[0]     # QuestTaskData
            _task = TutBreakTask(qtd, qtd.task_items)
            self.quest_tasks.append(_task)
            _uid = '-1'
            _items_count = len(qtd.task_items)
            if _items_count == 1:
                _uid = qtd.task_items[0].get('uid')
            _task_text_pre = qtd.task_items[0].get_action_text()
            if _task_text_pre: _task_text = _task_text_pre
            else: _task_text = self.get_taskslide_text(qtd)
            _task_tips = tools.ListToStr(self.get_tasks_text_for_panel(qtd))
            _task.slot = UI.AddGameTask(_task.task_image, _task.task_image_frame, _task.task_template, {'uid':_uid, 'tips':_task_tips})
            UI.SendMessage(_task.slot, 'set_task_text', _task_text)
            UI.SendMessage(_task.slot, 'set_task_count', str(_task.events_remain))
            UI.SendMessage(_task.slot, 'set_task_tips_text', _task_tips )
            UI.SendMessage(_task.slot, 'set_uids', qtd.click_uids )
        pass
# ============================================================
    def on_update(self, ms):
        can_complete = True
        for task in self.quest_tasks:
            if not task.is_complete:
                can_complete = False
                break
        if can_complete:
            self.complete()
        pass
# ============================================================
# #######################################################
# #######################################################
