# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import quest
import game_task


class TutBuildingTask(game_task.GameTask):
    '''Задание квеста (у игрока задание отображается в нижнем правом углу)'''
    def __init__(self, event_name = '', qtdata = None, template = ''):
        game_task.GameTask.__init__(self, qtdata)
        self.ser_name = 'TutBuildingTask'
        if qtdata != None:
            self.events_count = qtdata.items_need_count     #^ кол-во событий (game_event), необходимое для выполнения задания
            self.events_remain = self.events_count          #^ сколько событий осталось дождаться (при 0 - задание выполнено)
        self.game_event = event_name     #^ событие, которое должно быть сгенерировано игрой, чтобы данное задание проверило его выполнение
        self.template = template
        if qtdata != None:
            self.items = qtdata.task_items
        self.is_prizes_count = False
# ============================================================
    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        if name == self.game_event:     # наше событие завершения
            if self.items:
                if obj in self.items:
                    self.events_remain -= 1
                    self.update_count()
                    if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
                        self.set_complete()                     # значит уже все сделали
            elif extra_dict.type == self.template.type:
                if extra_dict.subtype == self.template.subtype: # наш объект
                    self.events_remain -= 1
                    self.update_count()
                    if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
                        self.set_complete()                     # значит уже все сделали
# ============================================================
# ============================================================
# ============================================================
class QuestTutBuild(quest.Quest):
    def __init__(self, name = '', data_dict = None, map_index = 0):
        quest.Quest.__init__(self, name, data_dict, map_index)
        self.ser_name = 'QuestTutBuild'
# ============================================================
    def is_start_possible(self, skip_ne_cases = False):
        '''Проверяет выполнимость стартовых условий'''
        if not QMgr.has_started_quests():   # если нет запущенных квестов
            if QMgr.cur_cluster.time_without_quest >= self.params.time: # если прошло уже достаточно времени
                # нужно проверить, что данный квест еще не выполнен:
                if self.need_visible_items:
                    for _item in self.need_visible_items:
                        _is_passable = _item.IsPassable()
                        if not _is_passable:
                            return False
                qtd = self.get_first_qtd()   # в туториале только одно задание
                #need_quantity = qtd.items_need_count
                total_quantity = 0
                if not qtd.task_items:
                    pass
                else:       # для случая с ремонтом
                    # if qtd.task_items[0].GetAttrStr('status') != 'broken':
                    trace('quest_name:', self.name)
                    b = qtd.task_items[0]
                    trace('quest building:', b.get('name'), qtd.task_items)
                    if b.status != 'broken':        # умудрились отремонтировать до квеста
                        #self.is_complete = True
                        self.complete()
                        return False
                    for _qtd in self.quest_tasks_datas:
                        for _item in _qtd.task_items:
                            if _item:
                                _has_access = _item.has_access()
                                if not _has_access: # если нет проходимости к объекту
                                    return False
                trace('quest_tut_build just test:', qtd.items_need_count, total_quantity)
                qtd.items_need_count -= total_quantity    # вычитаем кол-во зданий указанного типа, которые уже построены
                return True
        return False
# ============================================================
    def start(self):
        '''Запускает квест, генерирует необходимые задания (таски),
        активирует все, что нужно и т.д.'''
        quest.Quest.start(self)
        self.add_game_tasks()
        self.is_started = True  # строго в конце метода
        # добавить template_name в текущее здание для стрелки
        _qtd = self.get_first_qtd()
        if _qtd.type == 'build_building':
            BMgr.ShowBuildButton(True, True)  # т.к. это постройка - покажем стрелку на кнопку постройки
            trace(self.name)
            BMgr.AddBuildTemplate(_qtd.task_template)
# ============================================================
    #@override
    def on_complete_begin(self):
        # здесь нужно очистить template_name для стрелки
        _qtd = self.get_first_qtd()
        if _qtd.type == 'build_building':
            BMgr.RemoveBuildTemplate(_qtd.task_template)
# ============================================================
    def add_game_tasks(self):
        #'''Когда NPC (или кто-то еще) дал задание - он вызывает данный метод и
        #игроку даются игровые задания'''
        if not self.is_started:
            qtd = self.get_first_qtd()     # QuestTaskData
            #_task = None
            _event = ''
            if qtd.type == 'build_building':
                _event = 'build_building_complete'
            elif qtd.type == 'repair':
                _event = 'build_repair_complete'
            elif qtd.type == 'repair_boat':
                _event = 'build_repair_complete'
                #_event = 'build_repair_boat_complete'
            _task = TutBuildingTask(_event, qtd, qtd.template)
            self.quest_tasks.append(_task)
            _uid = '-1'
            _items_count = len(_task.items)
            if _items_count == 1:
                _uid = _task.items[0].get('uid')
            _task_tips = tools.ListToStr(self.get_tasks_text_for_panel(qtd))
            _task.slot = UI.AddGameTask(_task.task_image, _task.task_image_frame, _task.task_template, {'uid':_uid, 'tips':_task_tips})
            _slide_text = self.get_taskslide_text(qtd)
            UI.SendMessage(_task.slot, 'set_task_text', _slide_text)
            UI.SendMessage(_task.slot, 'set_task_tips_text', _task_tips )
            UI.SendMessage(_task.slot, 'set_task_count', str(_task.events_remain))
            UI.SendMessage(_task.slot, 'set_uids', qtd.click_uids )
        pass
# ============================================================
    def on_update(self, ms):
        can_complete = True
        for task in self.quest_tasks:
            if not task.is_complete:
                can_complete = False
                break
        if can_complete:
            self.complete()
        pass
# ============================================================
# #######################################################
# #######################################################
