# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import quest
import game_task


class TutUniversalTask(game_task.GameTask):
    '''Задание квеста (у игрока задание отображается в нижнем правом углу)'''
    def __init__(self, qtdata = None):
        game_task.GameTask.__init__(self, qtdata)
        self.ser_name = 'TutUniversalTask'
        self.events_count = 1
        self.events_remain = 1
        if qtdata != None:
            self.game_event = qtdata.event_name     #^ событие, которое должно быть сгенерировано игрой, чтобы данное задание проверило его выполнение
        self.is_prizes_count = False
# ============================================================
    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        if name == self.game_event:     # наше событие завершения
            if self.events_remain > 0:
                self.events_remain -= 1
                self.update_count()
                if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
                    self.set_complete()                     # значит уже все сделали
# ============================================================
# ============================================================
class QuestTutUniversal(quest.Quest):
    def __init__(self, name = '', data_dict = None, map_index = 0):
        quest.Quest.__init__(self, name, data_dict, map_index)
        self.ser_name = 'QuestTutUniversal'
# ============================================================
    def is_start_possible(self, skip_ne_cases = False):
        '''Проверяет выполнимость стартовых условий'''
        if not QMgr.has_started_quests():   # если нет запущенных квестов
            if QMgr.cur_cluster.time_without_quest >= self.params.time: # если прошло уже достаточно времени
                if self.need_visible_items:
                    for _item in self.need_visible_items:
                        _is_passable = _item.IsPassable()
                        if not _is_passable:
                            return False
                return True
        return False
# ============================================================
    def start(self):
        '''Запускает квест, генерирует необходимые задания (таски),
        активирует все, что нужно и т.д.'''
        quest.Quest.start(self)
        self.add_game_tasks()
        self.is_started = True  # строго в конце метода
# ============================================================
    def add_game_tasks(self):
        #'''Когда NPC (или кто-то еще) дал задание - он вызывает данный метод и
        #игроку даются игровые задания'''
        if not self.is_started:
            qtd = self.quest_tasks_datas[0]     # QuestTaskData
            _task = TutUniversalTask(qtd)
            self.quest_tasks.append(_task)
            _task_tips = tools.ListToStr(self.get_tasks_text_for_panel(qtd))
            _info = {'tips':_task_tips}
            _uids_count = len(self.task_start_activate_items)
            if _uids_count > 0:
                _info['uid'] = self.task_start_activate_items[0].get('uid')
            _task.slot = UI.AddGameTask(_task.task_image, _task.task_image_frame, _task.task_template, _info)
            _task_text_result = self.get_taskslide_text(qtd)
            UI.SendMessage(_task.slot, 'set_task_text', _task_text_result)
            UI.SendMessage(_task.slot, 'set_task_tips_text', _task_tips )
            UI.SendMessage(_task.slot, 'set_task_count', str(_task.events_remain))
            UI.SendMessage(_task.slot, 'set_uids', qtd.click_uids )
# ============================================================
    def on_update(self, ms):
        can_complete = True
        for task in self.quest_tasks:
            if not task.is_complete:
                can_complete = False
                break
        if can_complete:
            self.complete()
# ============================================================
    #def autostart(self, sender):
    #    '''Используется для автоматического запуска туториальных квестов (учитывая, что
    #    они по порядку должны следовать'''
    #    pass

    #def __str__(self):
    #    extra = to_str("quest: ", self.uid, "is_complete", self.is_complete, "is_started", self.is_started, "is_tut", self.is_tutorial)
    #    return to_str("{", extra, "}", str(self))
# #######################################################
# #######################################################
# #######################################################
# #######################################################
# #######################################################
