# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import quest
import game_task


class TutUpgradingTask(game_task.GameTask):
    '''Задание квеста (у игрока задание отображается в нижнем правом углу)'''
    def __init__(self, event_name = '', qtdata = None, template = ''):
        game_task.GameTask.__init__(self, qtdata)
        self.ser_name = 'TutUpgradingTask'
        if qtdata != None:
            self.events_count = qtdata.items_need_count     #^ кол-во событий (game_event), необходимое для выполнения задания
            self.events_remain = self.events_count          #^ сколько событий осталось дождаться (при 0 - задание выполнено)
        self.game_event = event_name     #^ событие, которое должно быть сгенерировано игрой, чтобы данное задание проверило его выполнение
        self.template = template
        if qtdata != None:
            self.items = qtdata.task_items
        self.is_prizes_count = False
# ============================================================
    def on_event(self, name, obj, extra_dict):
        '''Обработчик игровых событий'''
        if name == self.game_event:     # наше событие завершения
            if self.items:
                if obj in self.items:
                    self.events_remain -= 1
                    self.update_count()
                    if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
                        self.set_complete()                     # значит уже все сделали
            elif extra_dict.type == self.template.type:
                if extra_dict.subtype == self.template.subtype: # наш объект
                    self.events_remain -= 1
                    self.update_count()
                    if self.events_remain <= 0:      # учитываем, что зданий построено достаточное кол-во
                        self.set_complete()                     # значит уже все сделали
# ============================================================
# ============================================================
class QuestTutUpgrade(quest.Quest):
    def __init__(self, name = '', data_dict = None, map_index = 0):
        quest.Quest.__init__(self, name, data_dict, map_index)
        self.ser_name = 'QuestTutUpgrade'
        #self.type = 'Quest'
# ============================================================
    def is_start_possible(self, skip_ne_cases = False):
        '''Проверяет выполнимость стартовых условий'''
        if not QMgr.has_started_quests():   # если нет запущенных квестов
            if QMgr.cur_cluster.time_without_quest >= self.params.time: # если прошло уже достаточно времени
                for _qtd in self.quest_tasks_datas:
                    for _item in _qtd.task_items:
                        if _item:
                            if not _item.has_access(): # если объект недоступен, то:
                                return False
                            _workers = len(_item.worker_units)
                            _need_no_worker = _item.GetAttrBool('need_no_worker')
                            if not _need_no_worker:
                                if _workers == 0:
                                    return False
                return True
        return False
# ============================================================
    def start(self):
        '''Запускает квест, генерирует необходимые задания (таски),
        активирует все, что нужно и т.д.'''
        quest.Quest.start(self)
        self.add_game_tasks()
        self.is_started = True  # строго в конце метода
        # ===
        _is_already_completed = self.is_already_completed()
        if _is_already_completed:
            self.complete_quest_tasks()
# ============================================================
    def is_already_completed(self):
        qtd = self.quest_tasks_datas[0]     # QuestTaskData
        _items = qtd.task_items
        _items_count = len(_items)
        _uid = '-1'
        if _items_count == 1:
            _uid = _items[0].get('uid')
        _has_already_completed = BMgr.actions_history.has_action(self.map_index, _uid, 'upgrade_complete')
        return _has_already_completed
# ============================================================
    def add_game_tasks(self):
        #'''Когда NPC (или кто-то еще) дал задание - он вызывает данный метод и
        #игроку даются игровые задания'''
        if not self.is_started:
            qtd = self.quest_tasks_datas[0]     # QuestTaskData
            _task = None
            _task = TutUpgradingTask('build_upgrade_complete', qtd, qtd.template)
            self.quest_tasks.append(_task)
            _uid = '-1'
            _items_count = len(_task.items)
            if _items_count == 1:
                _uid = _task.items[0].get('uid')
            _task_tips = tools.ListToStr(self.get_tasks_text_for_panel(qtd))
            _task.slot = UI.AddGameTask(_task.task_image, _task.task_image_frame, _task.task_template, {'uid':_uid, 'tips':_task_tips})
            _task_text_result = self.get_taskslide_text(qtd)
            UI.SendMessage(_task.slot, 'set_task_text', _task_text_result)
            UI.SendMessage(_task.slot, 'set_task_tips_text', _task_tips )
            UI.SendMessage(_task.slot, 'set_task_count', str(_task.events_remain))
            UI.SendMessage(_task.slot, 'set_uids', qtd.click_uids )
        pass
# ============================================================
    def on_update(self, ms):
        can_complete = True
        for task in self.quest_tasks:
            if not task.is_complete:
                can_complete = False
                break
        if can_complete:
            self.complete()
        pass
# ============================================================
    #def autostart(self, sender):
    #    '''Используется для автоматического запуска туториальных квестов (учитывая, что
    #    они по порядку должны следовать'''
    #    pass

    #def __str__(self):
    #    extra = to_str("quest: ", self.uid, "is_complete", self.is_complete, "is_started", self.is_started, "is_tut", self.is_tutorial)
    #    return to_str("{", extra, "}", str(self))
# #######################################################
# #######################################################
# #######################################################
# #######################################################
# #######################################################
