# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# ============================================================
# ============================================================
class BarbHitSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'BarbHitSubtask'
        self.actor_action_animation = 'hit'
# ============================================================
    def start_action(self, item):
        #item.stop_attention_signal()
        self.actor.stop_walk()
        self.actor.action_hit_item(item)
        return True
# #######################################################
# #######################################################
# #######################################################
class AttackBarbarianTask(task_classes.Task):
    def __init__(self, home = None, barbarian = None, target = None):
        task_classes.Task.__init__(self, 'walk_barbarian')
        self.ser_name = 'AttackBarbarianTask'
        self.home = home
        self.priority = 1000
        self.is_event_task = False   # EVENT ONLY
        self._event_triggers.append('free_barbarian') # триггеры, которые отрабатывает данный таск
        self.barbarian = barbarian
        self.item = target
        # нужно перенести начало атаки на фактическое начало атаки
        #if self.item:
        #    self.item.set_under_attack()
        self.subtask = None
# ============================================================
    def do_pre_action(self):
        return True
# ============================================================
    def try_do_action(self):
        trace('trying AttackBarbarianTask. uid:', self.uid)
        if self.barbarian:  # and not self.barbarian.task:       # is free
            if not self.item:
                trace('no building for attack')
                return False
            cell_name = self.item.get_attack_cell_name()
            #if self.barbarian.run_to_target(self.item, None, cell_name):    # отправим юнита к горшку
            result = self.barbarian.TryRunTo(self.item, cell_name, 0)
            if result:
                _subtask = BarbHitSubtask(self, self.barbarian)
                self.apply_start_task(self.barbarian, _subtask)
                trace("task 'AttackBarbarian' started")
                return True
        else:
            if self.barbarian:
                trace('Barbarian already has a task:', self.barbarian.task)
                trace('Task UID:', self.uid, 'Prev task:', self.barbarian.task.uid)
        return False
# ============================================================
    def _complete_task(self):
        self.is_complete = True
        trace("task 'AttackBarbarian' completed. uid:", self.uid)
        if self.barbarian:
            self.barbarian.SetMouseActive(False)
            self.barbarian.apply_stop_task()
            self.barbarian.is_going_home = True
            self.barbarian.is_attacking = False
            self.barbarian.run_home()
            UStats.SubtractLevelValue('barbarian_attack', 1)    # завершение атаки
            EMgr.send_event('barbarian_attack_completed', self.barbarian)
            self.barbarian = None
            UI.show_face_bar(False)
        self.item.FSM.HandleEvent('break_event')
        self.item = None
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-исполнителем при достижении цели'''
        if self.subtask != None:
            if not self.is_action_started:
                #self.item.on_attack_started(self)       # начали атаку здания
                # #if not self.subtask.start_action(self.item):
                # #    self.complete_task()
                # #else:
                #self.item.set_under_attack()
                if not self.subtask.start_action(self.item):
                    self.complete_task()
                else:
                    self.item.FSM.HandleEvent('attack_event')
                    _image_name = self.barbarian.GetAttrStr('image_name', 'IMG_I_CAVEMAN_BG')
                    UI.set_face_bar_info(_image_name + ',1', '__skip__')
                    self.subtask.actor.current_action_animation = self.subtask.actor_action_animation
                    self.is_action_started = True
                    self.on_subtask_started()
        else:
            self.complete_task()
# ============================================================
    #@override
    def on_subtask_started(self):
        EMgr.SendGlobalMsg('action_started')
# ============================================================
    def on_update(self, ms):
        pass
# ============================================================
    def cancel(self, force=False):
        '''Вызывается, если у варвара стало мало энергии'''
        if self.is_complete:
            return
        # делать нужно на основе _complete_task!!!
        self.is_complete = True
        trace("task 'AttackBarbarian' canceled. uid:", self.uid)
        # нельзя напрямую отменять действие генерала, если сейчас данный таск не обрабатывается!
        if self.is_started and self.barbarian:
            self.barbarian.SetMouseActive(False)
            self.barbarian.cancel_action_hits()
            self.barbarian.task = None
            self.barbarian.apply_stop_task()
            self.barbarian.is_going_home = True
            self.barbarian.is_attacking = False
            self.barbarian.run_home()
            UStats.SubtractLevelValue('barbarian_attack', 1)    # отмена атаки
            EMgr.send_event('barbarian_attack_canceled', self.barbarian)
            self.barbarian = None
            UI.show_face_bar(False)
        self.item.FSM.HandleEvent('cancel_event')
        self.item = None
        #if self.is_started:
            #TMgr.send_event("free_barbarian", None)
# ============================================================
    def on_callback(self, result, extra_dict):
        if self.subtask != None:
            self.subtask.on_callback(result, extra_dict)
# ============================================================
    def process_removing_item(self, r_uid, r_mi):
        # атакуемое здание не должно удаляться
        if self.item:   # заглушка на всякий случай
            _uid = self.item.get('uid')
            _mi = self.item.get('locind')
            if _uid == r_uid and _mi == r_mi:
                trace('_PRI_')
                self.cancel()
# ============================================================

