# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# #######################################################
# #######################################################
class BuildingTask(task_classes.Task):
    '''Класс-задание для постройки здания со свободным положением'''
    def __init__(self, type = '', map_index = 0, template_name = '', variant_name = '', cell_pos = None, builded_item = None):
        task_classes.Task.__init__(self, type)
        self.ser_name = 'BuildingTask'

        self.priority = 50
        if template_name != '':
            if not UMgr.is_free_real_units_enough(map_index):    # если мало юнитов, то:
                if template_name == 'house1':
                    self.priority = 100
        self.is_building_predefined = (builded_item != None)
        self.map_index = map_index
        self.building = builded_item        # by default None
        self.is_building_started = False
        self.template_name = template_name
        self.variant_name = variant_name
        self.cell_pos = cell_pos
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append('free_unit') # триггеры, которые отрабатывает данный таск
        self._units = []
# ============================================================
    def do_pre_action(self):
        '''Проверка необходимых условий начала таска, а так же минимальная инициализации при достаточности условий'''
        if self.type == 'build_building':    #если строительство здания
            if not self.building:
                self.building = BMgr.AddItemToMap(self.map_index, self.template_name, self.variant_name, self.cell_pos)
                if self.building == None:       # если здание нельзя добавить в указанную позицию
                    return False
        b = self.building
        self.resources_need = b.GetNeedsRes('level_1')
        b.FSM.HandleEvent('build_wait_event')
        b.start_task_participation(self)
        return True
# ============================================================
    def try_do_action(self):
        if self.try_call_unit():
            self.building.show_clock()  # песочные часы ожидания
            trace("BuildingTask: started", self.building.get('name'))   #unit.get('name'),
            #mbox(to_str("task:", self.uid, self._units))
            return True
        else:
            return False
# ============================================================
    def try_call_unit(self):
        '''Вызывает юнита на строительство'''
        if self.building:
            build_cell_name = self.building.get_action_cell_name()
            if build_cell_name == None: return False
            unit, teleport_building = UMgr.get_free_nearest_unit( self.building, build_cell_name, 'builder' )# можно попробовать возвращать путь, чтобы не считать его заново
            if unit == None:
                return False
            removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            unit.set_current_subtype('builder') # нам нужен строитель
            _c = unit.get('cell')
            UMgr.teleport_unit(unit, teleport_building)
            unit.remove_at_target_msg()
            success = unit.run_to_target(self.building, None, build_cell_name)
            if not success:
                trace('BuildingTask::try_call_unit: run_to_target failed', unit.get('uid'))
                unit.teleport_to_cell(_c)   # return unit back
                if removed: UMgr.add_free_unit(unit)
                return False
            # - нужно как-то зарезервировать указанную ячейку, чтобы в следующий раз
            # в нее не прибежал другой юнит
            # - не забыть потом эту ячейку разрезервировать!
            self.building.reserve_action_cell(build_cell_name)
            #unit.task = self
            #unit.task_type = self.type
            self.apply_start_task(unit)
            self._units.append(unit)
            self.create_unit_signal()   # позже позовем еще одного
            return True
        return False
# ============================================================
    def _complete_task(self):
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace('BuildingTask task complete')
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            unit.remove_at_target_msg()
            unit.apply_stop_task(False)
            #unit.task = None
            #unit.task_type = ''
            UMgr.add_free_unit(unit)    # учесть, что это юнит строитель!
        b = self.building
        b.stop_task_participation(self)
        self.building = None
        b.FSM.HandleEvent('prenormal_event')
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-строителем при достижении цели'''
        if not self.is_building_started:
            self.building.FSM.HandleEvent('build_event', True)
        self.is_building_started = True
        sender.FSM.HandleEvent('build_event')
        sender.Drawer.PlayAnimationOnce('build', 'build_hit_complete', '', 50, 'hit_moment')        # ударим еще раз
# ============================================================
    def on_update(self, ms):
        if self.is_building_started: # если строительство уже началось, то:
            # нужно готовность здания увеличивать пропорционально кол-ву работающих юнитов
            b = self.building
            count = self._get_building_units_count()
            quant = ms * count * b.get_boost_multiplier()
            b.build_progress += quant
            if b.build_progress >= b.build_time:
                self.complete_task()
            else:
                b.Drawer.SetFramesProgress('build', b.build_progress/b.build_time) # сменяет кадры постройки, если изображение указано как анимированное
# ============================================================
    def cancel(self, force=False):
        '''Нужно вызывать при отмене задания'''
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        b = self.building
        if b: b.show_clock(False)  # песочные часы ожидания
        if not self.is_building_started:        # если юнит еще не начал действие, то
            UStats.AddRes(self.resources_need)  # нужно возвращать стоимость апгрейда
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace("BuildingTask task canceled")
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            #unit.task = None
            #unit.task_type = ''
            unit.apply_stop_task(False)
            UMgr.add_free_unit(unit)
        if b:
            b.unreserve_action_cells()
            #b.SM.SetState('normal')
            #b.switch_level()   # переключаемся на следующий вариант
            b.stop_task_participation(self)
            if not self.is_building_predefined:
                b.on_cancel_build()
                BMgr.RemoveItemFromMap(b)
        self.building = None
# ============================================================
    def _get_building_units_count(self):
        count = 0
        for unit in self._units:
            _state = unit.FSM.GetStateID()
            if _state == 'build':
                count += 1
        return count
# ============================================================
    def on_external_signal(self, signal_id):
        if signal_id == 'try_more_units':
            if self.try_call_unit():
                self.create_unit_signal()   # если нужно, то позже позовем еще одного
            else:   # юнита позвать не удалось
                self.create_unit_signal(500)   # попробоуем еще раз через полсекунды
        pass
# ============================================================
    def on_bump_need(self):
        if self.building:
            self.building.scale_bump()
# ============================================================
# #######################################################
# #######################################################
# #######################################################
