# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes


# #######################################################
# #######################################################
class BuildingPlaceTask(task_classes.Task):
    '''Класс-задание для постройки здания, которое имеет предустановленное место'''
    def __init__(self, type = '', map_place_element = None):
        task_classes.Task.__init__(self, type)
        self.ser_name = 'BuildingPlaceTask'

        self.priority = 50
        self.building = map_place_element
        self.is_building_started = False
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append('free_unit') # триггеры, которые отрабатывает данный таск
        self._units = []
# ============================================================
    def do_pre_action(self):
        '''Проверка необходимых условий начала таска, а так же минимальная инициализации при достаточности условий'''
        b = self.building
        self.resources_need = b.GetNeedsRes('level_1')
        b.FSM.HandleEvent('build_wait_event')
        b.start_task_participation(self)
        return True
# ============================================================
    def try_do_action(self):
        if self.try_call_unit():
            trace('BuildingPlaceTask: started', self.building.get('name'))   #unit.get('name'),
            #mbox(to_str("task:", self.uid, self._units))
            return True
        else:
            return False
# ============================================================
    def try_call_unit(self):
        if self.building:
            build_cell_name = self.building.get_action_cell_name()
            if build_cell_name == None: return False
            unit, teleport_building = UMgr.get_free_nearest_unit( self.building, build_cell_name, 'builder' )
            if unit == None:
                return False
            removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            unit.set_current_subtype('builder') # нам нужен строитель
            _c = unit.get('cell')
            UMgr.teleport_unit(unit, teleport_building)
            success = unit.run_to_target(self.building, None, build_cell_name)
            if not success:
                trace('BuildingPlaceTask::try_call_unit: run_to_target failed', unit.get('uid'))
                unit.teleport_to_cell(_c)   # return unit back
                if removed: UMgr.add_free_unit(unit)
                return False
            self.building.reserve_action_cell(build_cell_name)
            #unit.task = self
            #unit.task_type = self.type
            self.apply_start_task(unit)
            self._units.append(unit)
            trace('BuildingPlaceTask: started', unit.get('name'), self.building.get('name'))
            #mbox(to_str("task:", self.uid, self._units))
            self.create_unit_signal()   # позже позовем еще одного
            return True
        return False
# ============================================================
    def _complete_task(self):
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace('BuildingPlaceTask task complete')
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            #unit.task = None
            #unit.task_type = ''
            unit.remove_at_target_msg()
            unit.apply_stop_task(False)
            UMgr.add_free_unit(unit)
        b = self.building
        b.stop_task_participation(self)
        self.building = None
        b.FSM.HandleEvent('prenormal_event')
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-строителем при достижении цели'''
        if not self.is_building_started:
            self.building.FSM.HandleEvent('build_event', True)
        self.is_building_started = True
        sender.FSM.HandleEvent('build_event')
        sender.Drawer.PlayAnimationOnce('build', 'build_hit_complete', '', 50, 'hit_moment')        # ударим еще раз
# ============================================================
    def on_update(self, ms):
        if self.is_building_started: # если строительство уже началось, то:
            # нужно готовность здания увеличивать пропорционально кол-ву работающих юнитов
            b = self.building
            count = self._get_building_units_count()
            quant = ms * count * b.get_boost_multiplier()
            b.build_progress += quant
            if b.build_progress >= b.build_time:
                self.complete_task()
            else:
                b.Drawer.SetFramesProgress('build', b.build_progress/b.build_time) # сменяет кадры постройки, если изображение указано как анимированное
# ============================================================
    def cancel(self, force=False):
        '''Нужно вызывать при отмене задания'''
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        b = self.building
        b.FSM.HandleEvent('cancel_event')
        if not self.is_building_started:        # если юнит еще не начал действие, то
            UStats.AddRes(self.resources_need)  # нужно возвращать стоимость апгрейда
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace('BuildingPlaceTask task canceled')
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            #unit.task = None
            #unit.task_type = ''
            unit.apply_stop_task(False)
            UMgr.add_free_unit(unit)
        b.stop_task_participation(self)
        self.building = None
# ============================================================
    def _get_building_units_count(self):
        count = 0
        for unit in self._units:
            _state = unit.FSM.GetStateID()
            if _state == 'build':
                count += 1
        return count
# ============================================================
    def on_external_signal(self, signal_id):
        if signal_id == 'try_more_units':
            if self.try_call_unit():
                self.create_unit_signal()   # если нужно, то позже позовем еще одного
            else:   # юнита позвать не удалось
                self.create_unit_signal(500)   # попробоуем еще раз через полсекунды
        pass
# ============================================================
    def on_bump_need(self):
        if self.building:
            self.building.scale_bump()
# #######################################################
# #######################################################
# #######################################################
