# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# #######################################################
# #######################################################
class CallWorkerTask(task_classes.Task):
    def __init__(self, building = None):
        type = 'call_worker_to_building'
        task_classes.Task.__init__(self, type)
        self.ser_name = 'CallWorkerTask'

        self.priority = 50
        self.building = building
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append('free_unit') # триггеры, которые отрабатывает данный таск
        self._units = []
# ============================================================
    def do_pre_action(self):
        self.building.show_clock()  # песочные часы ожидания
        b = self.building
        _is_participating = b.is_task_participating(self.type)
        if _is_participating:
            trace('CallWorkerTask::do_pre_action cancel (participating)')
            return False
        b.start_task_participation(self)
        return True
# ============================================================
    def try_do_action(self):
        if self.try_call_unit():
            trace('CallUnitTask: started', self.building.get('name'), self.building.get('uid'))   #unit.get('name'),
            for unit in self._units:
                trace('        ', unit)   #unit.get('name'),
            #self.is_started = True
            return True
        else:
            return False
# ============================================================
    def try_call_unit(self):
        '''Вызывает юнита на работу'''
        if self.building:
            map_index = self.building.get('locind')
            if UMgr.get_free_real_units_count(map_index) <= 1:    # последнего юнита нельзя отправить на работу в здание
                return False
            in_cell_name = 'start_cell'  #self.building.get_action_cell_name()
            is_passable = self.building.is_cell_passable(in_cell_name)
            if not is_passable:
                return False
            _unit_subtype = ''
            b_stype = self.building.get('stype')
            if b_stype == 'watchtower':
                _unit_subtype = 'soldier'
            #elif b_stype == 'woodcutter':
            #    _unit_subtype = 'dt_wcutter'
            #elif b_stype == 'quarry':
            #    _unit_subtype = 'quarryman'
            else:
                _unit_subtype = 'simple'
            unit, teleport_building = UMgr.get_free_nearest_unit( self.building, in_cell_name, _unit_subtype )
            if unit == None:
                return False
            UMgr.add_building_worker(unit)  # увеличим число занятых зданиями работников
            removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            unit.set_current_subtype(_unit_subtype)
            if _unit_subtype == 'soldier':
                unit.SetNeedToTeleport()        # чтобы солдат бежал через телепортацию
            _c = unit.get('cell')
            UMgr.teleport_unit(unit, teleport_building)
            success = unit.run_to_target(self.building, None, in_cell_name)
            if not success:
                trace('CallWorkerTask::try_call_unit: run_to_target failed', unit.get('uid'))
                unit.teleport_to_cell(_c)   # return unit back
                if removed: UMgr.add_free_unit(unit)
                return False
            self.building.reserve_action_cell(in_cell_name)
            self.apply_start_task(unit)
            #unit.task = self
            #unit.task_type = self.type
            self._units.append(unit)
            return True
        return False
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-работником при достижении цели'''
        self.building.show_clock(False)  # песочные часы ожидания
        b = self.building
        unit = sender
        unit.stop_walk()
        unit.set_home(b)
        self.complete_task()
        b.on_worker_in(unit)
# ============================================================
    def _complete_task(self):
        self.is_complete = True
        trace('task_callworker complete')
        b = self.building
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            unit.apply_stop_task(False)
            #unit.task = None
            #unit.task_type = ''
            b.worker_units.append(unit)
        b.stop_task_participation(self)
        self.building = None
# ============================================================
    def on_update(self, ms):
        pass
# ============================================================
    def cancel(self, force=False):
        '''Нужно вызывать при отмене задания, но данное задание отменить нельзя'''
        self.is_complete = True
        self.is_started = False
        trace('task "call_worker" cancel')
        self.building.show_clock(False)  # песочные часы ожидания
        self.building.stop_task_participation(self)
        if self._units:
            unit = self._units[0]
            unit.apply_stop_task()
            UMgr.remove_building_worker(unit)
            UMgr.add_free_unit(unit)
            self._units = []
        self.building = None
        self.is_unique_task = False
# ============================================================
    #def on_external_signal(self, signal_id):
    #    pass
# ============================================================
    def process_removing_item(self, r_uid, r_mi):
        if self.building:
            _uid = self.building.get('uid')
            _mi = self.building.get('locind')
            if _uid == r_uid and _mi == r_mi:
                trace('_PRI_')
                self.cancel()
# #######################################################
# #######################################################
# #######################################################
