# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# #######################################################
# #######################################################
class HealSubtask(task_classes.Subtask):
    def __init__(self, parent_task = None, actor = None):
        task_classes.Subtask.__init__(self, parent_task, actor)
        self.ser_name = 'HealSubtask'
    # ============================================================
    def start_action(self, item):
        self.is_subtask_started = True
        self.actor.stop_walk()
        #self.actor.sick_unit = item
        self.actor.FSM.HandleEvent('start_curing_event', True)
        item.set_check(False)
        #item.is_deactivated = True  # чтобы нельзя было кликнуть
        item.Signals.Restart('curing_process', 2000)    # чтобы отобразить прогрессбар лечения
        #self.actor.action_hit_item(item)
        item.switch_animation()
        return True
    # ============================================================
    def on_action_complete(self):
        #item.is_deactivated = False
        task_classes.Subtask.on_action_complete(self)
# ============================================================
# ============================================================
class DoctorTask(task_classes.Task):
    def __init__(self, sick_unit = None):
        type = 'run_doctor'
        task_classes.Task.__init__(self, type)
        self.ser_name = 'DoctorTask'
        self.priority = 1000
        self.sick_unit = sick_unit
        self.from_map_index = -1
        self.to_map_index = -1
        self.doctor = None
        self.is_event_task = False
        #self.is_event_task = True   # EVENT ONLY
        #self._event_triggers.append("free_unit") # триггеры, которые отрабатывает данный таск
# ============================================================
    def do_pre_action(self):
        _doctor = UMgr.get_doctor()
        if not _doctor:
            return False
        self.doctor = _doctor
        # - нужно проверить наличие рабочего здания с доктором
        if self.sick_unit:
            _is_sick = self.sick_unit.FSM.GetStateID() == 'sick'
            if not _is_sick:
                return False
            self.sick_unit.task = self
            self.sick_unit.set_check(True) #???
        return True
# ============================================================
    def try_do_action(self):
        trace('Trying "DoctorTask"...', self.uid)
        if not self.doctor or not self.sick_unit:
            #mbox(str(self.unit) + str(self.stone))   #@test
            trace('No doctor or sick')
            return False
        if self.doctor.task:
            trace('Doctor already has a task:', self.doctor.task)
            trace('Task UID:', self.uid, 'Prev task:', self.doctor.task.uid)
            return False
        # - нужно проверить наличие рабочего здания с доктором

        _doctor = self.doctor
        self.from_map_index = _doctor.get('locind')
        self.to_map_index = self.sick_unit.get('locind')
        # u.show()    # нужно показать юнита перед началом действия
        if _doctor.parent_home:
            _unit_unused = _doctor.parent_home.pop_doctor() #если заняли юнита делом, то удалим его из списка свободных
        if self.from_map_index != self.to_map_index:
            # get transition points
            _points = BMgr.get_submap_exit(self.to_map_index, self.from_map_index)
            _len = len(_points)
            if _len == 0:
                mbox('No submap exits found')
            trace('Submap exits count =', _len)
            _transition_obj = _points[0]
            _transition_start_cell = _doctor.get('cell')
            BMgr.transfer_unit(_doctor, self.to_map_index)
            _cell = _transition_obj.GetAttrCellFixed('action_cell')
            _doctor.set('cell', _cell)
            _doctor.transition_start_cell = _transition_start_cell
            _doctor.transition_object = _transition_obj
        _doctor.SetNeedToTeleport()
        success = _doctor.run_to_target(self.sick_unit, None, self.sick_unit.get_heal_cell_name())
        if success:
            _subtask = self.sick_unit.get_subtask(self, _doctor)
            self.apply_start_task(_doctor, _subtask)
            trace('task "DoctorTask" started')
            return True
        return False
# ============================================================
    def _complete_task(self):
        self.is_complete = True
        trace("task 'DoctorTask' completed")
        self.doctor.apply_stop_task()
        _doctor = self.doctor
        self.doctor = None
        #>>>> UMgr.add_free_unit(_unit)
        _doctor.append_self_to_parent_home()
        if self.sick_unit: self.sick_unit.show_arrow(False)   # отключим стрелку, если была, на всякий случай
        self.sick_unit.FSM.HandleEvent('heal_event')
        self.sick_unit.task = None
        self.sick_unit = None
        _doctor.go_home() # отправим юнита домой
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом при достижении цели'''
        if self.subtask != None:
            self.subtask.start_action(self.sick_unit)
        else:
            self.complete_task()
# ============================================================
    def on_update(self, ms):
        #if self.is_subtask_started:
        pass
# ============================================================
    def cancel(self, force=False):
        '''Нужно вызывать при отмене задания'''
        _was_complete = self.is_complete
        self.is_complete = True
        trace("task 'DoctorTask' canceled")
        if self.sick_unit:
            self.sick_unit.set_check(False)        #???
            self.sick_unit.task = None
        self.sick_unit = None
        # нельзя напрямую отменять действие генерала, если сейчас данный таск не обрабатывается!
        if self.is_started:
            if not _was_complete:   #@?
                self.doctor.apply_stop_task(False)
                _doctor = self.doctor
                _doctor.go_home()
                #self.doctor = None
# ============================================================
    def on_callback(self, result, extra_dict):
        if self.subtask != None:
            self.subtask.on_callback(result, extra_dict)
# ============================================================
    def process_removing_item(self, r_uid, r_mi):
        if self.doctor:
            _item = self.doctor.parent_home
            if _item:
                _uid = _item.get('uid')
                _mi = _item.get('locind')
                if _uid == r_uid and _mi == r_mi:
                    trace('_PRI_', _uid, _mi)
                    self.cancel()
# ============================================================
# #######################################################
# #######################################################
# #######################################################
