# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# #######################################################
# #######################################################
class TakeResTask(task_classes.Task):
    '''Забирает ресурс у здания и несет его либо в склад,
    либо к зданию, которое сейчас ожидает данный ресурс (зависимое)'''
    def __init__(self, from_building = None, from_cell = None):
        task_classes.Task.__init__(self, 'take_res')
        self.ser_name = 'TakeResTask'
        self.is_unique_task = True
        if from_building:
            self.unique_id = 'take_res_from' + ':' + from_building.get('uid')
            self.map_index = from_building.get('locind')
            #self.extra = 'TakeResTask'
            #self.extra += ' ' + from_building.get('name') + ':' + from_building.get('uid')

        self.from_building = from_building      # забрать ресурс у этого здания
        self.from_cell = from_cell
        self.to_building = None
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append('free_unit') # триггеры, которые отрабатывает данный таск
        #self._event_triggers.append("res_taken") # триггеры, которые отрабатывает данный таск
        self._unit = None
        self.is_resource_taken = False
        self._generate_extra_desc()
# ============================================================
    def _generate_extra_desc(self, text = ''):
        self.extra = 'TakeResTask'
        if self.from_building:
            self.extra += ' ' + self.from_building.get('name') + ':' + self.from_building.get('uid')
            if self._unit: self.extra += ' u:' + self._unit.get('uid')
        if text:
            self.extra += ' ^ff0000^' + text
# ============================================================
    def do_pre_action(self):
        #if self.type == "take_res":    #если строительство здания
        return True
# ============================================================
    def try_do_action(self):
        units = UMgr.get_free_units(self.from_building.get('locind'), 'simple')   # только обычные юниты подходят
        if not units:
            _from_name = ''
            _from_uid = ''
            if True:#self.from_building:
                _from_name = self.from_building.get('name')
                _from_uid = self.from_building.get('uid')
            trace('TakeRes: try_do_action - no free units', _from_name, _from_uid)
            self._generate_extra_desc('(no units)')
            return False
        res_cell = 'res_cell'
        is_passable = self.from_building.is_cell_passable(res_cell)
        if not is_passable:
            return False
        unit, teleport_building = UMgr.get_free_nearest_unit( self.from_building, res_cell, 'simple' ) # ищем ближайшего свободного юнита
        if unit == None:
            _from_name = ''
            if self.from_building: _from_name = self.from_building.get('name')
            trace('TakeRes: try_do_action - path was not found', _from_name)
            self._generate_extra_desc('(no path)')
            return False # сюда не должны приходить
        #if self.from_building.is_task_participating(self.type + 'from'):
        #    return False   # а зачем? Если из здания нечего нести, то всеравно нечего нести
        self.from_building.start_task_participation(self, 'from')   #было 'from'
        removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
        #unit.set_current_subtype('simple')
        _c = unit.get('cell')
        UMgr.teleport_unit(unit, teleport_building)
        success = unit.run_to_target(self.from_building, None, res_cell)    # отправим юнита к зданию
        if not success:
            trace('TakeResTask::try_call_unit: run_to_target failed', unit.get('uid'))
            unit.teleport_to_cell(_c)   # return unit back
            if removed: UMgr.add_free_unit(unit)
            return False
        self._unit = unit
        self.apply_start_task(unit)
        self._generate_extra_desc()
        trace('TakeResTask: started', unit.get('name'), unit.get('uid'), self.from_building.__str__())
        return True
# ============================================================
    def _complete_task(self):
        self.is_complete = True
        trace('Task "take_res" complete')
        if self.from_building: trace('   task "take_res" from building:', self.from_building.get('name'), self.from_building.get('uid'))
        if self.to_building: trace('   task "take_res" to building:', self.to_building.get('name'), self.to_building.get('uid'))
        unit = self._unit
        unit.apply_stop_task(False)
        UMgr.add_free_unit(unit)
        self._unit = None
        self.from_building = None
        self.to_building.stop_task_participation(self, 'to')
        self.is_unique_task = False
        self.to_building = None
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-исполнителем при достижении цели'''
        # - получить Resource-объект с кол-вом ресурса
        # - юниту переключить анимацию на соответствующую ресурсу
        # - отправить юнита в следующее ближайшее здание (нужна какая-то таблица соответствий ресурсов зданиям)
        res_cell = 'res_cell'
        trace("Task 'take_res' target_reached")
        if self.from_building: trace("   task 'take_res' from building:", self.from_building.get('name'), self.from_building.get('uid'))
        if self.to_building: trace("   task 'take_res' to building:", self.to_building.get('name'), self.to_building.get('uid'))

        if not self.is_resource_taken:
            # нужно отдать ресурс зданию
            # - нужно проверить - есть ли здания указанного типа (если это не __store__ конечно)
            # - если есть, то
            #   - отнести ресурс туда
            #   - иначе нести в __store__
            _res = self.from_building.give_resource()
            self._unit.resource = _res
            _single_count = _res.GetSingleValue()
            self.is_resource_taken = True   # ресурс взяли

            if self.from_building.to_building_subtype == '__store__' or _single_count != 1: # если нужно нести сразу на склад
                buildings = BMgr.get_buildings(self.map_index, '__store__', self.type, ['broken', 'place'])
                self.to_building = BMgr.find_nearest(buildings, res_cell, self.from_building, res_cell) # ищем ближайшее здание, принимающее ресурсы
                if self.to_building:
                    self.to_building.start_task_participation(self, 'to')
            else:                   # если нужно нести в здание переработчик
                buildings = BMgr.get_buildings(self.map_index, self.from_building.to_building_subtype, self.type + 'to', ['broken', 'place', 'map_place'])    # фикс, чтобы по двое не бегали в одно здание
                trace('TaskRes >>> buildings count('+ self.from_building.to_building_subtype +'):', str(len(buildings)))
                trace('TaskRes: from_building.to_building_subtype:', self.from_building.to_building_subtype)
                filtered_buildings = []
                for b in buildings:
                    if not b.clock_shown:
                        _has_generating_res_progress = b.has_generating_res_progress()  # отсекаем те, у которых генерится или есть незабранный ресурс
                        if not _has_generating_res_progress:
                            filtered_buildings.append(b)
                trace('TaskRes >>> filtered buildings count('+ self.from_building.to_building_subtype +'):', str(len(filtered_buildings)))
                self.to_building = BMgr.find_nearest(filtered_buildings, res_cell, self.from_building, res_cell) # ищем ближайшее здание, принимающее ресурсы
                if not self.to_building:    # если такого свободного здания не оказалось, то
                    # нужно отнести ресурс в склад
                    buildings = BMgr.get_buildings(self.map_index, '__store__', self.type, ['broken', 'place'])
                    filtered_buildings = []
                    #for b in buildings: if not b.clock_shown: filtered_buildings.append(b)
                    self.to_building = BMgr.find_nearest(buildings, res_cell, self.from_building, res_cell) # ищем ближайшее здание, принимающее ресурсы
                if self.to_building:
                    self.to_building.start_task_participation(self, 'to')
                    self._generate_extra_desc('(running to building ' + self.to_building.get('name') + self.to_building.get('uid') +')')
                else:
                    _from_name = ''
                    if self.from_building: _from_name = self.from_building.get('name')
                    trace('TakeRes ERROR: self.to_building is None. From:', _from_name)
                    self._generate_extra_desc('(to_building is None)')
            self._unit.run_to_target(self.to_building, None, res_cell)    # отправим юнита к зданию
            self.from_building.stop_task_participation(self, 'from') #было 'from'    # освободим, чтобы к нему мог выбежать другой юнит, если нужно
            self.is_unique_task = False
            self._generate_extra_desc()
            self.from_building = None   # здание свободно
        else:
            # нужно отдать ресурс зданию
            _b = self.to_building
            _is_broken = _b.is_broken()
            _fsm_state = _b.FSM.GetStateID()
            _b_stype = _b.get('stype')
            _need_return_res_to_store = False
            if _is_broken:
                trace('TaskRes: broken building on at_target (1)', _b.get('name'), _b.get('uid'))
                _need_return_res_to_store = True
                self._generate_extra_desc('(broken at target)')
            if _b.is_under_attack:
                trace('TaskRes: is_under_attack building on at_target', _b.get('name'), _b.get('uid'))
                _need_return_res_to_store = True
                self._generate_extra_desc('(broken at target)')
            #if _b_state == 'upgrade':
            if _fsm_state == 'upgrading':
                if _b_stype not in ['warehouse', 'hq', 'hq_small']:
                    trace('TaskRes: upgrading building on at_target', _b.get('name'), _b.get('uid'))
                    _need_return_res_to_store = True
                    self._generate_extra_desc('(upgrading at target)')
            if _b.clock_shown:
                if _b_stype not in ['warehouse', 'hq', 'hq_small']:
                    trace('TaskRes: awaiting building on at_target', _b.get('name'), _b.get('uid'))
                    _need_return_res_to_store = True
                    self._generate_extra_desc('(awaiting at target)')
            if _need_return_res_to_store:    # если со зданием что-то не так и нужно вернуть ресурс в склад
                self.is_unique_task = False # спокойствия ради
                self.to_building.stop_task_participation(self, 'to')
                self.to_building = None
                buildings = BMgr.get_buildings(self.map_index, '__store__', self.type, ['broken', 'place'])
                _buildings = []
                for b in buildings:     # фильтруем поломанные и прочие неактивные здания
                    _is_functional = b.is_functional_non_attacked()
                    _has_access = b.HasAccess()
                    if _is_functional and _has_access:
                        _buildings.append(b)
                self.to_building = BMgr.find_nearest(_buildings, res_cell, _b, res_cell) # ищем ближайшее здание, принимающее ресурсы
                if self.to_building:
                    self.to_building.start_task_participation(self, 'to')
                    self._unit.run_to_target(self.to_building, None, res_cell)    # отправим юнита к зданию
                else:
                    self._generate_extra_desc('(no STORE! Bad)')
            else:
                trace('TakeResTask: building takes resource', self.to_building.get('name'), self.to_building.get('uid'))
                self._generate_extra_desc('(resource at building! Good)')
                self.to_building.res_unit = self._unit
                self.to_building.FSM.HandleEvent('res_unit_in_event', True)
                #self.to_building.take_resource(self._unit.resource)
                #self._unit.resource = None
                self.complete_task()   # считаем задание выполненным
                trace('>>> task_res >>')
# ============================================================
    def on_update(self, ms):
        if self.is_started:
            self.ticks += 1
            if not self.ticks%5: # каждый пятый тик проверяем
                if self.to_building:
                    _fsm_state = self.to_building.FSM.GetStateID()
                    if _fsm_state in self.res_cancel_states_on_remove:
                        self.cancel_action()
                    elif _fsm_state in self.res_cancel_states_other:
                        self.cancel_action()
                if self.from_building:
                    _fsm_state = self.from_building.FSM.GetStateID()
                    if _fsm_state in self.res_cancel_states_on_remove:
                        self.cancel_action()
# ============================================================
    def cancel_action(self):
        self.is_complete = True
        self.is_started = False
        trace('task "take_res" cancel')
        if self.from_building: trace('   task "take_res" from building:', self.from_building.get('name'), self.from_building.get('uid'))
        if self.to_building: trace('   task "take_res" to building:', self.to_building.get('name'), self.to_building.get('uid'))
        if self._unit:
            unit = self._unit
            unit.apply_stop_task(False)
            UMgr.add_free_unit(unit)
            self._unit = None
        if self.from_building:
            self.from_building.stop_task_participation(self, 'from')
        self.from_building = None
        if self.to_building:
            self.to_building.stop_task_participation(self, 'to')
            #self.to_building.is_wait_res_unit = False
            self.to_building._restask_self_cancel()
        self.is_unique_task = False
        self.to_building = None
# ============================================================
    def process_removing_item(self, r_uid, r_mi):
        if self.from_building:
            _uid = self.from_building.get('uid')
            _mi = self.from_building.get('locind')
            if _uid == r_uid and _mi == r_mi:
                trace('_PRI_ 1')
                self.cancel_action()
        if self.to_building:
            _uid = self.to_building.get('uid')
            _mi = self.to_building.get('locind')
            if _uid == r_uid and _mi == r_mi:
                trace('_PRI_ 2')
                self.cancel_action()
# #######################################################
# #######################################################
# #######################################################
