# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# - посчитать - есть ли необходимый ресурс у игрока
# - найти ближайшего юнита в складе/главном здании
# - вызвать юнита с ресурсами (не более необходимого для производства)
# - вычесть ресурс из имеющихся
# - "навесить" ресурс на юнита
# - профит.

# #######################################################
# #######################################################
class TakeResStoreTask(task_classes.Task):
    '''Запрос ресурса зданием у склада'''
    def __init__(self, caller_building = None, caller_cell = None, res_name = None):
        task_classes.Task.__init__(self, 'take_res')    # тип оставить этот, чтобы здание считалось занятым
        self.ser_name = 'TakeResStoreTask'
        self.is_unique_task = True
        if caller_building:
            self.unique_id = 'take_res_store_to' + ':' + caller_building.get('uid')
            self.map_index = caller_building.get('locind')
            #self.extra = 'TakeResStoreTask'
            #self.extra += ' ' + caller_building.get('name') + ':' + caller_building.get('uid')

        self.caller_building = caller_building
        self.caller_cell = caller_cell
        self.res_name = res_name
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append('free_unit') # триггеры, которые отрабатывает данный таск
        self._event_triggers.append('resource_stored') # триггеры, которые отрабатывает данный таск
        #self._event_triggers.append("res_taken") # триггеры, которые отрабатывает данный таск
        self._unit = None
        self.is_resource_taken = False
        if caller_building:
            self._generate_extra_desc()
        trace('TakeResStoreTask created')
# ============================================================
    def _generate_extra_desc(self, text = ''):
        self.extra = 'TakeResStoreTask'
        if self.caller_building:
            self.extra += ' ' + self.caller_building.get('name') + ':' + self.caller_building.get('uid')
            if self._unit: self.extra += ' u:' + self._unit.get('uid')
        if text:
            self.extra += ' ^ff0000^' + text
# ============================================================
    def do_pre_action(self):
        b = self.caller_building
        if b.is_generating_res:
            trace('TakeResStoreTask::do_pre_action cancel (generating)')
            return False    # отменяем таск
        if b.is_generated_res:
            trace('TakeResStoreTask::do_pre_action cancel (generated)')
            return False    # отменяем таск
        _is_participating = b.is_task_participating(self.type + 'to')
        if _is_participating:
            trace('TakeResStoreTask::do_pre_action cancel (participating)')
            return False
        trace('TakeResStoreTask do_pre_action True', b.get('name'))
        self.is_wait_res_unit = True    # потому что добавили запрос
        return True
# ============================================================
    def try_do_action(self):
        b = self.caller_building
        trace('TakeResStoreTask trying action', b.get('name'), b.get('uid'))
        _is_participating = b.is_task_participating(self.type + 'to')
        # --- ПРОВЕРКИ >>> ---
        if _is_participating:
            self.cancel_action()    # отменяем таск
            trace('TakeResStoreTask canceled (already participating):', b.get('name'))
            return False
        if b.is_generating_res:
            self.cancel_action()    # отменяем таск
            trace('TakeResStoreTask canceled (generating res):', b.get('name'))
            return False            # отменяем таск
        if b.is_generated_res:
            self.cancel_action()    # отменяем таск
            trace('TakeResStoreTask canceled (generated res):', b.get('name'))
            return False
        #_b_state = b.SM.GetState()
        #if _b_state == 'upgrade':
        _fsm_state = b.FSM_GetStateID()
        if _fsm_state == 'upgrading':
            self.cancel_action()    # отменяем таск
            trace('TakeResStoreTask canceled (building upgrading):', b.get('name'))
            return False
        _is_broken = b.is_broken()
        if not _is_broken: _is_broken = b.is_under_attack
        if _is_broken:    # если здание успели сломать к этому моменту
            self.cancel_action()    # отменяем таск
            trace('TakeResStoreTask canceled (building broken):', b.get('name'))
            return False
        # --- ПРОВЕРКИ <<< ---

        _cur_res = UStats.GetCurRes()
        trace('TakeResStoreTask.res_name =', self.res_name)
        if _cur_res:
            _resval = _cur_res.Get(self.res_name)
            trace('TakeResStoreTask _cur_res count =', _resval)
        _res_count = _cur_res.Get(self.res_name)
        if _res_count == 0: # если нет необходимых ресурсов
            trace('TakeResStoreTask try_do_action (no needed res):', b.get('name'))
            self._generate_extra_desc('(not enough res)')
            return False    # попробуем в другой раз
        # ---
        # ---
        res_cell = 'res_cell'
        is_passable = self.caller_building.is_cell_passable(res_cell)
        if not is_passable:
            return False
        unit = None
        stores = BMgr.get_buildings(self.map_index, '__store__', '', ['broken', 'place', 'map_place'])
        nearest_b = BMgr.find_nearest(stores, res_cell, self.caller_building, res_cell)
        if not nearest_b:
            trace('ERROR: BMgr.find_nearest returned NOT a building')
            self._generate_extra_desc('(not a building)')
            return False
        _new_unit_cell = None
        _need_teleport_unit = nearest_b.IsSubTypeOf('warehouse')    # хинтово делаем так, чтобы юнит выбегал из склада
        if not _need_teleport_unit:                                 # а эта проверка потому что в здании-складе юнитов может не быть
            _building_units_count = nearest_b.get_units_count()  # кол-во юнитов данного здания
            _need_teleport_unit = (_building_units_count == 0)      # если юнитов здания нет
        if _need_teleport_unit:   # если ближайшим является склад (либо в здании нет юнитов, т.е. в главном здании)
            units = UMgr.get_free_units_at_home(b.get('locind'), 'simple')   # найдем ближайшего юнита в доме (чтобы обратно не слишком далеко бежал)
            if not units:
                trace('TakeResStoreTask no units to teleport')
                self._generate_extra_desc('(no units)')
                return False    # попробуем в другой раз
            unit, teleport_building = UMgr.get_free_nearest_unit( self.caller_building, res_cell, 'simple', units ) # ищем ближайшего свободного юнита
            if unit == None:
                trace('TakeResStoreTask: try_do_action - path was not found')
                self._generate_extra_desc('(no path)')
                return False # сюда не должны приходить
            trace('TakeResStoreTask teleporting unit')
            _new_unit_cell = nearest_b.GetAttrCellFixed('res_cell') # юнита нужно стартовать от склада
        else:
            unit = nearest_b.get_first_unit()  # возьмем одного юнита из нашего здания
        # ---
        if unit.task:
            trace('ERROR: unit already has a task', unit.task.type, unit.task.uid, unit.get('uid'))  # дебаговая затычка
        # ---
        b.start_task_participation(self, 'to')   # фикс, было 'from', но ведь бежит от склада!
        removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
        if _need_teleport_unit:
            self._generate_extra_desc('(teleporting)')
            unit.set('cell', _new_unit_cell)    # чтобы бежал от склада
        success = unit.run_to_target(self.caller_building, None, res_cell)    # отправим юнита к зданию
        if not success:
            trace('TakeResStoreTask::try_call_unit: run_to_target failed', unit.get('uid'))
            if removed: UMgr.add_free_unit(unit)
            return False
        self.apply_start_task(unit)
        #! нужно дать юниту ресурс!
        #res = b.GetRes("normal_gives_1")
        # #_single_resval = res.GetSingleValue()
        # #b.move_global_res_to_unit(unit, self.res_name, _single_resval)
        b.move_global_res_to_unit(unit, self.res_name, 1)
        self._unit = unit
        self._generate_extra_desc()
        trace('TakeResStoreTask: started', unit.get('name'), self.caller_building.__str__())
        return True
# ============================================================
    def _complete_task(self):
        self.is_complete = True
        if self.caller_building:
            _b = self.caller_building
            trace("task 'take_res_store' complete", _b.get('name'), _b.get('uid'))
        else:
            trace("task 'take_res_store' complete")
        if self.caller_building: trace("   task 'take_res_store' caller:", self.caller_building.get('name'))
        unit = self._unit
        unit.apply_stop_task(False)
        UMgr.add_free_unit(unit)
        self._unit = None
        if self.caller_building:
            self.caller_building.stop_task_participation(self, 'to')
            self.caller_building.is_wait_res_unit = False   # нужно ли это?
            self.is_unique_task = False
            self.caller_building = None   # здание свободно
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-исполнителем при достижении цели'''
        res_cell = 'res_cell'
        _b = self.caller_building
        trace('TakeResStoreTask: target reached', _b.get('name'), _b.get('uid'))
        _b_state = _b.FSM.GetStateID()
        _b_stype = _b.get('stype')
        _is_broken = _b.is_broken()
        _is_building_ok = True
        if not _is_broken: _is_broken = _b.is_under_attack
        if _is_broken:    # если здание успели сломать к этому моменту
            _is_building_ok = False
            trace('TakeResStoreTask: broken building on at_target', _b.get('name'), _b.get('uid'))
            self._generate_extra_desc('(broken at target)')
            _b.stop_task_participation(self, 'to')
            _b._restask_self_cancel()
            self.is_unique_task = False
            self.caller_building = None
            buildings = BMgr.get_buildings(self.map_index, '__store__', self.type, ['broken', 'place', 'map_place'])
            _building = BMgr.find_nearest(buildings, res_cell, _b, res_cell) # ищем ближайшее здание, принимающее ресурсы
            if _building:
                _building.start_task_participation(self, 'to')
                self._unit.run_to_target(_building, None, res_cell)    # отправим юнита обратно к зданию
                self.caller_building = _building
            #self.cancel_action()    # отменяем таск
        elif _b_state == 'upgrading' or _b_state == 'repairing': # если здание апгрейдится, то отдавать ему ничего не нужно
            if _b_stype not in ['warehouse', 'hq', 'hq_small']:
                _is_building_ok = False
                trace('TakeResStoreTask: upgrading building on at_target', _b.get('name'), _b.get('uid'))
                self._generate_extra_desc('(upgrading at target)')
                _b.stop_task_participation(self, 'to')
                _b._restask_self_cancel()
                self.is_unique_task = False
                self.caller_building = None
                buildings = BMgr.get_buildings(self.map_index, '__store__', self.type, ['broken', 'place', 'map_place'])
                _building = BMgr.find_nearest(buildings, res_cell, _b, res_cell) # ищем ближайшее здание, принимающее ресурсы
                if _building:
                    _building.start_task_participation(self, 'to')
                    self._unit.run_to_target(_building, None, res_cell)    # отправим юнита обратно к зданию
                    self.caller_building = _building
                #self.cancel_action()    # отменяем таск
        if _is_building_ok:
            trace('TakeResStoreTask: building takes resource', _b.get('name'), _b.get('uid'))
            self._generate_extra_desc('(resource at building! Good)')
            self.caller_building.res_unit = self._unit
            self.caller_building.FSM.HandleEvent('res_unit_in_event', True)
            #self.caller_building.take_resource(self._unit.resource)
            #self._unit.resource = None
            self.complete_task()   # считаем задание выполненным
            trace('>>> task_res_store >>')
# ============================================================
    def cancel_action(self):        # пока не совсем ясно в какой ситуации должно вызываться, но юнитов нужно "деинициализить"
        '''Отменяет задание'''
        trace('TakeResStoreTask: cancel_action')
        if self.caller_building:
            trace('   TakeResStoreTask caller:', self.caller_building.get('name'), self.caller_building.get('uid'))
            #self.caller_building.is_wait_res_unit = False
            self.caller_building._restask_self_cancel()
        self.is_started = False
        if self._unit:
            self._unit.apply_stop_task(False)
            UMgr.add_free_unit(self._unit)
            self._unit = None
        #if stop_participating and self.caller_building and self.caller_building.is_task_participating(self.type + 'to'):   # fix - не было " +'to' "
        #    self.caller_building.stop_task_participation(self, 'to')    # fix - было 'from', но ведь бежит от склада!
        if self.caller_building:
            _is_participating = self.caller_building.is_task_participating(self.type + 'to')
            if _is_participating:
                self.caller_building.stop_task_participation(self, 'to')
        self.is_unique_task = False
        self.caller_building = None
        self.is_complete = True
# ============================================================
    #@override
    def on_update(self, ms):
        if self.is_started:
            self.ticks += 1
            if not self.ticks%5: # каждый пятый тик проверяем
                if self.caller_building:
                    _fsm_state = self.caller_building.FSM.GetStateID()
                    if _fsm_state in self.res_cancel_states_on_remove:
                        self.cancel_action()
                    elif _fsm_state in self.res_cancel_states_other:
                        self.cancel_action()
# ============================================================
    def process_removing_item(self, r_uid, r_mi):
        if self.caller_building:
            _uid = self.caller_building.get('uid')
            _mi = self.caller_building.get('locind')
            if _uid == r_uid and _mi == r_mi:
                trace('_PRI_')
                self.cancel_action()
# ============================================================
# #######################################################
# #######################################################
# #######################################################
