# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# #######################################################
# #######################################################
class UpgradeTask(task_classes.Task):
    def __init__(self, building = None):
        type = 'upgrade_building'
        task_classes.Task.__init__(self, type)
        self.ser_name = 'UpgradeTask'
        if building:
            self.extra = building.get('name')

            self.priority = 50
            if not UMgr.is_free_real_units_enough(building.get('locind')):    # если мало юнитов, то:
                _stype = building.get('stype')
                if _stype in ['hq', 'hq_small', 'house']:
                    self.priority = 100

            self.is_digger = building.is_digger_upgrade
        self.extra += ' p=' + str(self.priority)
        self.building = building
        self.is_building_started = False
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append("free_unit") # триггеры, которые отрабатывает данный таск
        self._units = []
# ============================================================
    def do_pre_action(self):
        b = self.building
        if b.is_next_level_possible():
            b.show_clock()  # песочные часы ожидания
            next_level_name = b.get_next_level_name()   # название варианта следующего уровня
            self.resources_need = b.GetNeedsRes(next_level_name)   # необходимые ресурсы
            UStats.SubRes(self.resources_need)      # вычитаем затраченные ресурсы
            b.on_before_building_complete()
            b.start_task_participation(self)
            #self.building.SM.SetState('awaiting_action')
            return True
        return False
# ============================================================
    def try_do_action(self):
        if self.try_call_unit():
            trace('UpgradeTask: started', self.building.get('name'))   #unit.get('name'),
            #mbox(to_str("task:", self.uid, self._units))
            return True
        else:
            return False
# ============================================================
    def try_call_unit(self):
        '''Вызывает юнита на апгрейд'''
        if self.building:
            build_cell_name = self.building.get_action_cell_name()
            if build_cell_name == None: return False
            actor_type = 'builder'
            if self.is_digger: actor_type = 'simple'
            unit, teleport_building = UMgr.get_free_nearest_unit( self.building, build_cell_name, actor_type )
            if unit == None:
                return False
            removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            unit.set_current_subtype(actor_type) # нужен строитель
            _c = unit.get('cell')
            UMgr.teleport_unit(unit, teleport_building)
            unit.remove_at_target_msg()
            success = unit.run_to_target(self.building, None, build_cell_name)
            if not success:
                trace('UpgradeTask::try_call_unit: run_to_target failed', unit.get('uid'))
                unit.teleport_to_cell(_c)   # return unit back
                if removed: UMgr.add_free_unit(unit)
                return False
            self.building.reserve_action_cell(build_cell_name)
            self.apply_start_task(unit)
            self._units.append(unit)
            trace('UpgradeTask: started try_call_unit', unit.get('name'), self.building.get('name'))
            #mbox(to_str("task:", self.uid, self._units))
            self.create_unit_signal()   # позже позовем еще одного
            return True
        return False
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-строителем при достижении цели'''
        if not self.is_building_started:
            _fsm_state = self.building.FSM.GetStateID()
            if _fsm_state == 'repair_wait':
                self.building.FSM.HandleEvent('repair_event', True)
            else:
                if self.building.is_upgrade_units_waiting:
                    self.building.FSM.HandleEvent('upgrade_event', True)
        #    self.create_unit_signal()   # если нужно, то позже позовем еще одного
        self.is_building_started = True
        animation = 'build'
        if self.is_digger:
            animation = 'dig'
            sender.FSM.HandleEvent('dig_event') # юнита переключаем в режим молотка
        else:
            sender.FSM.HandleEvent('build_event') # юнита переключаем в режим молотка
        sender.Drawer.PlayAnimationOnce(animation, 'build_hit_complete', '', 50, 'hit_moment')        # ударим еще раз
        if self.building.status == 'normal':
            if self.building.hack_for_upgrade:
                sender.hide()
# ============================================================
    def _complete_task(self):
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace('task complete')
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            unit.remove_at_target_msg()
            unit.show()
            unit.apply_stop_task(False)
            UMgr.add_free_unit(unit)
        b = self.building
        b.show_unit = False
        b.FSM.HandleEvent('prenormal_event')
        #b.SM.SetState('normal')
        #_prev_status = self.building.status
        #if _prev_status == 'broken':
        #    b.switch_level('repaired')   # переключаемся на следующий вариант (после того, как состояние здания перевели в normal !)
        #    trace('Building repair complete')
        #else:
        #    b.switch_level('upgraded')
        #    trace('Building upgrade complete')
        b.stop_task_participation(self)
        self.building = None
# ============================================================
    def on_update(self, ms):
        if self.is_building_started: # если строительство уже началось, то:
            # нужно готовность здания увеличивать пропорционально кол-ву работающих юнитов
            b = self.building
            count = self._get_building_units_count()
            quant = ms * count * b.get_boost_multiplier()
            b.build_progress += quant
            if b.build_progress >= b.build_time:
                self.complete_task()
            else:
                b.Drawer.SetFramesProgress('build', b.build_progress/b.build_time) # сменяет кадры постройки, если изображение указано как анимированное
# ============================================================
    def cancel(self, force=False):
        '''Нужно вызывать при отмене задания'''
        trace('Cancel building upgrade')
        EMgr.remove_signal(self, 'try_more_units')    # удалим лишние сигналы
        self.building.show_clock(False)  # песочные часы ожидания
        if not self.is_building_started:        # если юнит еще не начал действие, то
            UStats.AddRes(self.resources_need)  # нужно возвращать стоимость апгрейда
        self.is_complete = True
        self.is_building_started = False    # на всякий случай
        trace("task canceled")
        units_copy = self._units[:]
        del self._units[:]
        for unit in units_copy:
            unit.apply_stop_task(False)
            UMgr.add_free_unit(unit)
        b = self.building
        b.unreserve_action_cells()
        #b.SM.SetState("normal")
        #b.switch_level()   # переключаемся на следующий вариант
        b.stop_task_participation(self)
        b.on_upgrade_cancel()
        self.building = None
# ============================================================
    def _get_building_units_count(self):
        count = 0
        for unit in self._units:
            _state = unit.FSM.GetStateID()
            if _state in ['build', 'dig']:
                count += 1
        return count
# ============================================================
# ============================================================
    def on_external_signal(self, signal_id):
        if signal_id == 'try_more_units':
            if not self.try_call_unit():
                #self.create_unit_signal()   # если нужно, то позже позовем еще одного
            #else:   # юнита позвать не удалось
                self.create_unit_signal(500)   # попробуем еще раз через полсекунды
        pass
# ============================================================
    def on_bump_need(self):
        if self.building:
            if not self.is_digger:
                self.building.scale_bump()
# #######################################################
# #######################################################
# #######################################################
