# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# #######################################################
# #######################################################
class WalkGeneralTask(task_classes.Task):
    def __init__(self, item = None, is_loot = False):
        task_classes.Task.__init__(self, 'walk_general')
        self.ser_name = 'WalkGeneralTask'
        self.priority = 1000
        self.is_event_task = True   # EVENT ONLY
        self._event_triggers.append('free_general') # триггеры, которые отрабатывает данный таск
        self.general = None
        self.item = item
        if item:
            self.map_index = self.item.get('locind')
        self.subtask = None
        self.is_canceled = False
        self.is_loot = is_loot
        self.loot_pause = 250
        self.loot_ready_to_start = False
        if is_loot:
            self.priority = 100000
            self._event_triggers.append('try_general_task')
# ============================================================
    def do_pre_action(self):
        #if self.type == "take_res":    #если строительство здания
        _general = UMgr.get_cluster(self.map_index).general
        _is_general_sick = _general.is_in_sick_states()
        if _is_general_sick:
            return False
        if not self.is_loot:
            self.item.set_check(True)
        else:
            self.item.allow_clicks = False
        return True
# ============================================================
    def try_do_action(self):
        trace('trying WalkGeneralTask. uid:', self.uid)
        if self.is_canceled:
            return False
        self.general = UMgr.get_cluster(self.map_index).general
        #if self.general and not self.general.task:       # is free
        if self.general and not self.general.is_busy():
            _is_general_sick = self.general.is_in_sick_states()
            if _is_general_sick:
                trace('General is sick. Task UID:', self.uid)
                return False
            _ready_to_start = not self.is_loot
            if not _ready_to_start: _ready_to_start = self.loot_ready_to_start
            if _ready_to_start:
                cell_name = self.item.get_action_cell_name()
                if cell_name == None:
                    trace('>>>> ### >>> cell_name == None')
                    self.cancel()   # отменяем задание, если не удается его запустить, чтобы генерал не завис
                    return False
                self.general.stop_walk()
                self.general.remove_at_target_msg() # попытка поправить баг
                success = self.general.run_to_target(self.item, None, cell_name)
                if success:    # отправим генерала к горшку
                    _extra_check_allow = self.item.is_extra_check_allow()
                    if not _extra_check_allow:
                        trace('--> DiffItem (intask) extra check disallow', self.item.get('uid'))
                        self.cancel()
                        return False
                    _subtask = self.item.get_subtask(self, self.general)
                    self.apply_start_task(self.general, _subtask)
                    trace('task "WalkG" started')
                    _is_different_type = self.item.IsTypeOf('different')
                    if _is_different_type: self.item.had_action = True
                    return True
            trace('>>>> ### >>> final False')
            self.cancel()   # отменяем задание, если не удается его запустить, чтобы генерал не завис
        else:
            if self.general:
                trace('General already has a task or busy:', self.general.task, self.general.unit_is_talking, self.general.need_magic)
                if self.general.task:
                    trace('Task UID:', self.uid, 'Prev task:', self.general.task.uid)
                else:
                    trace('Task UID:', self.uid)
        return False
# ============================================================
    def _complete_task(self):
        self.is_complete = True
        trace('task "WalkG" completed. uid:', self.uid)
        _item = self.item
        _general = self.general
        if self.general:
            self.general.attack_state = ''
            self.general.apply_stop_task()
            self.general = None
        if self.item:
            #@! опасный код, объекта может уже не быть, а ссылка останется!!!
            self.item.set_check(False)
            self.item.task = None   # ##
        self.item = None
        if _item:
            _state = _general.FSM.GetStateID()
            if _state != 'sick':
                _loots = _item.get_loot()
                if _loots:
                    _loots = _loots[::-1]   # reverse
                    for _loot in _loots:
                        if 'no_autoloot' in _loot:
                            if _loot.no_autoloot == True:
                                continue
                        _loot_task = WalkGeneralTask(_loot, True)
                        TMgr.add_task(_loot_task, True)
            else:
                trace('     task_walkg::_complete_task: general is sick')
        TMgr.send_event('free_general', None)
        trace('>> task_wg dbg end <<<')
# ============================================================
    def target_reached(self, sender):
        '''Вызывается юнитом-исполнителем при достижении цели'''
        self.general.attack_state = 'attack'
        if self.subtask != None:
            if not self.is_action_started:
                if not self.subtask.start_action(self.item):
                    self.complete_task()        # на случай, если квест нельзя запустить
                else:
                    self.subtask.actor.current_action_animation = self.subtask.actor_action_animation
                    self.is_action_started = True
                    if self.item:
                        self.item.on_before_action_started()    # перед тем, как начнется действие над объектом
                    self.on_subtask_started()
        else:
            self.complete_task()
        #self.general.stop_walk()
        #self.general.action_hit_item(self.item)     # генерал сам доломает горшок, BUG: возможен, если отмена произойдет в момент ломания
# ============================================================
    #@override
    def on_subtask_started(self):
        if self.item:  self.item.set_check(False)
        EMgr.SendGlobalMsg('action_started')
# ============================================================
    def on_update(self, ms):
        if self.is_loot:
            if not self.loot_ready_to_start:
                self.loot_pause -= ms
                if self.loot_pause <= 0:
                    self.loot_ready_to_start = True
                    TMgr.send_event('try_general_task', None)
# ============================================================
    def cancel(self, force=False):
        '''Вызывается, если игрок отменил задание'''
        # делать нужно на основе _complete_task!!!
        if self.is_loot: return # не отменяемо
        self.is_complete = True
        self.is_canceled = True
        trace('task "WalkG" canceled. uid:', self.uid)
        # нельзя напрямую отменять действие генерала, если сейчас данный таск не обрабатывается!
        if self.is_started:
            self.general.attack_state = ''
            self.general.cancel_hits()
            self.general.apply_stop_task()
            self.general = None
        if self.item:
            self.item.set_check(False)
            self.item.FSM.HandleEvent('cancel_interaction_event')
            if not force:
                tools.PlaySound('SND_CLICK_TASK_CANCEL')
            self.item.task = None
            self.item = None
        if self.is_started:
            TMgr.send_event('free_general', None)
# ============================================================
    def on_callback(self, result, extra_dict):
        if self.subtask != None:
            self.subtask.on_callback(result, extra_dict)
# ============================================================
    def process_removing_item(self, r_uid, r_mi):
        if self.item:
            if not self.is_action_started:  # если действие еще не запустили
                _uid = self.item.get('uid')
                _mi = self.item.get('locind')
                if _uid == r_uid and _mi == r_mi:
                    trace('_PRI_ WalkGeneralTask, obj:', _uid, _mi)
                    self.cancel()
# ============================================================

