# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import task_classes

# #######################################################
# #######################################################
class WalkWorkerTask(task_classes.Task):
    def __init__(self, item = None, unit = None):
        type = 'walk_worker'
        task_classes.Task.__init__(self, type)
        self.ser_name = 'WalkWorkerTask'
        self.priority = 50
        self.item = item
        self.unit = unit
        self.is_event_task = True   # EVENT ONLY
        self.non_canceleable = False
        self._event_triggers.append('free_unit') # триггеры, которые отрабатывает данный таск
        if item:
            self.action_cost_res = self.item.res_action_cost
            UStats.SubRes(self.action_cost_res) # вычтем стоимость действия
# ============================================================
    def do_pre_action(self):
        if not self.non_canceleable:
            if self.item:
                self.item.task = self
                self.item.set_check(True)
        return True
# ============================================================
    def try_do_action(self):
        unit = None
        _action_cell_name = self.item.get_action_cell_name()
        if self.item.IsSubTypeOf('enter'):
            _action_cell_name = self.item.get_first_free_action_cell_name()
        if _action_cell_name == None: return False
        if not self.unit:
            _unit_subtype = self.item.get_unit_subtype()
            #@!!! здесь может быть разный unit_subtype - нужно это учитывать!!!
            unit, teleport_building = UMgr.get_free_nearest_unit( self.item, _action_cell_name, _unit_subtype )   # можно бы еще и путь возвращать
            if unit == None:
                return False
            trace('WWT::try_do_action 1', _unit_subtype)
            removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            unit.set_current_subtype(_unit_subtype)    #@!!! может быть другой subtype!!!
            _c = unit.get('cell')
            UMgr.teleport_unit(unit, teleport_building)
            unit.SetNeedToTeleport()
            success = unit.run_to_target(self.item, None, _action_cell_name)
            if not success:
                trace('WalkWorkerTask::try_call_unit: run_to_target failed', unit.get('uid'))
                unit.teleport_to_cell(_c)   # return unit back
                if removed: UMgr.add_free_unit(unit)
                return False
        else:
            unit = self.unit
            trace('WWT::try_do_action 2')
            removed = UMgr.remove_free_unit(unit) #если заняли юнита делом, то сообщим об этом
            success = unit.run_to_target(self.item, None, _action_cell_name)
            if not success:
                trace('WalkWorkerTask::try_call_unit: run_to_target failed 2', unit.get('uid'))
                if removed: UMgr.add_free_unit(unit)
                return False
        self.unit = unit
        _subtask = self.item.get_subtask(self, unit)
        self.apply_start_task(unit, _subtask)
        trace('WalkWorkerTask: started', unit.get('name'), self.item.get('name'))
        if not self.non_canceleable:
            _is_different_type = self.item.IsTypeOf('different')
            if _is_different_type: self.item.had_action = True
        return True
# ============================================================
    def _complete_task(self):
        self.is_complete = True
        trace('task "WalkW" completed')
        _unit = None
        if not self.non_canceleable:
            if self.item:
                #@! опасный код, объекта может уже не быть, а ссылка останется!!!
                self.item.set_check(False)
        _is_unit_dead = True    # для спасения выживших в подземельях
        if self.unit:
            _is_unit_dead = self.unit.get('is_dead')
            self.unit.apply_stop_task()
            _unit = self.unit
            self.unit = None
        if not _is_unit_dead:
            if _unit:
                #if not _unit.resource or _unit.resource.Empty():
                _is_unit_free = False
                if not _unit.resource:
                    _is_unit_free = True
                else:
                    _is_unit_free = _unit.resource.Empty()
                if _is_unit_free:
                    UMgr.add_free_unit(_unit)
                elif not _unit.task and not _unit.is_in_home:     # если задания нет и юнит еще не дома
                    _unit.go_home()    # то отправим юнита домой
        if self.item:
            #@! опасный код, объекта может уже не быть, а ссылка останется!!!
            self.item.task = None
            self.item = None
# ============================================================
    #@override
    def on_subtask_started(self):
        if not self.non_canceleable:
            if self.item:
                self.item.set_check(False)
        EMgr.SendGlobalMsg('action_started')
# ============================================================
    def on_update(self, ms):
        #if self.is_subtask_started:
        pass
# ============================================================
    def cancel(self, force=False):
        '''Нужно вызывать при отмене задания'''
        if self.non_canceleable:
            return
        self.is_complete = True
        trace('task "WalkW" canceled')
        if self.item:
            self.item.set_check(False)
            if not force:
                tools.PlaySound('SND_CLICK_TASK_CANCEL')
            self.item.task = None
        self.item = None
        if self.is_started:
            #self.unit.stop_walk()
            self.unit.apply_stop_task(False)
            _unit = self.unit
            self.unit = None
            UMgr.add_free_unit(_unit)
        if not self.is_action_started:
            if self.action_cost_res:        # если юнит еще не начал действие, то
                UStats.AddRes(self.action_cost_res)  # вернем стоимость действия
                self.action_cost_res = None
# ============================================================
    def on_callback(self, result, extra_dict):
        if self.subtask != None:
            self.subtask.on_callback(result, extra_dict)
# ============================================================
    def process_removing_item(self, r_uid, r_mi):
        if self.item:
            if not self.is_action_started:  # если действие еще не запустили
                _uid = self.item.get('uid')
                _mi = self.item.get('locind')
                if _uid == r_uid and _mi == r_mi:
                    trace('_PRI_', _uid, _mi)
                    self.cancel()
# #######################################################
# #######################################################
# #######################################################
