# -*- coding: utf-8 -*-

import sys
if not 'tinypy' in sys.version:
    from _globals import *

import base_unit
import toolz

#@statemachine
class UnitBarbarian(base_unit.BaseUnit):
    states_anim_dict = {'action': 'hit', 'hit': 'hit', 'run': 'default', 'stay': 'walk' }
# ============================================================
    def __init__(self, cdata):
        base_unit.BaseUnit.__init__(self, cdata)
        self.ser_name = 'UnitBarbarian'
        self.resource = None
        self.energy = 100 #percent
        self.hits_count = 5
        self.is_attacking = False
        self.is_going_home = False
        self.home = None            # nest
        self.kick_effect_name = self.GetAttrStr('kick_effect')
        self.is_first = False       # если это самый первый дикарь, то нужно показывать стрелку над ним
        self.snd_voice = self.GetAttrStr('snd_voice', 'SND_BARB_ATTACK_VOICE')
        self.snd_attack_hit = self.GetAttrStr('snd_attack_hit', 'SND_BARB_ATTACK_HIT')
        self.snd_hit = self.GetAttrStr('snd_hit', 'SND_BARBARIAN_HIT')
        self.current_action_animation = 'run'   # данное поле выставляется при взаимодействии с тем или иным объектом
        _is_raptor = self.get('name') == 'raptor1'
        if _is_raptor:
            default_unit_speed = Globals.GFloat('DEFAULT_RAPTOR_SPEED') # default speed
            cur_unit_speed = Globals.GFloat('RAPTOR_SPEED')
        else:
            default_unit_speed = Globals.GFloat('DEFAULT_BARB_SPEED') # default speed
            cur_unit_speed = Globals.GFloat('BARB_SPEED')
        self.Drawer.SetAnimBoost(['walk', 'default'],\
                                    1000*cur_unit_speed/default_unit_speed)
# ============================================================
    def extra_init(self):
        '''Вызывается только при открытии уровня/создании нового юнита, но НЕ при загрузке сохранения'''
        base_unit.BaseUnit.extra_init(self)
        self.StopMove()
        self.FSM.HandleEvent('stay_event')  #@!<<<
        self.SetMouseActive(True)
# ============================================================
    def on_message(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'hit_moment':
            #self.PlaySound('SND_HAMMER_HIT')
            if self.task:
                self.task.on_bump_need()
        elif msg_type == 'at_target':
            if self.is_going_home:
                if self.home:
                    self.home.hide_barbarian(self)
            else:
                trace(self.task_type)
                if self.task:
                    self.task.target_reached(self)
            return
# ============================================================
    def run_home(self):
        if self.home:
            self.is_attacking = False
            # найти ближайую точку входа
            home_cells = self.home.get_cells()
            result_index = BMgr.FindNearestCell(self, '', home_cells)
            self.cell_index = result_index + 1
            # отправить туда варвара
            self.TryRunTo(self.home, 'barb_cell' + str(self.cell_index), 0)
# ============================================================
    def on_signal(self, signal):
        #base_unit.BaseUnit.on_signal(self, signal)
        pass
# ============================================================
    def has_access(self):
        return True
# ============================================================
    def on_click(self, btn_name):
        click_power = Globals.GFloat('CLICK_POWER')
        self.attack(click_power)
# ============================================================
    def attack(self, force):
        if self.is_first:
            self.is_first = False
            self.switch_animation()
        if self.energy > 0:
            self.energy -= force
            # add effect
            if self.task:
                #self.AddEffect(self.kick_effect_name, self.get('position'))
                self.PlaySound(self.snd_hit)
                self.Drawer.Blink(['default', 'hit', 'walk'], 1, 0xffff8888, 0.5)
            if self.energy <= 0:
                # нужно отменить атаку и бежать в нору
                if self.action_item:
                    self.action_item.FSM.HandleEvent('attack_stop_event')
                if self.task:
                    self.task.cancel()
                EMgr.send_event('barbarian_attack_stopped', None)
# ============================================================
    def on_create(self):
        self.init_fsm()
        #self.SetDirection(self.GetAttrInt('start_direction'))
        self.switch_animation()
# ============================================================
# ============================================================
    # @override
    def action_hit_item(self, item):
        self.action_item = item
        self.is_attacking = True
        self.FSM.HandleEvent('action_event')    #@fixit  здесь нужно еще разобраться какое состояние выставлять
        self.SendMsg('action_hit_item')
        self.PlaySound(self.snd_voice)  # чтобы первую анимацию озвучить
# ============================================================
    # @override
    def cancel_action_hits(self):
        self.action_item = None
        self.is_attacking = False
        #self. SM .SetState('stay')
# ============================================================
    # @override
    def on_action_hit_moment(self):
        if self.action_item:
            self.action_item.action_hit_me()      # удар по объекту
            self.PlaySound(self.snd_attack_hit)
        self.hits_count -= 1
# ============================================================
    # @override
    def on_action_hit_complete(self):
        if self.action_item:      # проверяем на случай, если отменили задание
            if self.action_item.energy > 0 or self.hits_count > 0 or\
                self.action_item.is_under_attack > 0 or self.action_item.status != 'broken':   # объект еще не разрушен
                self.Drawer.PlayAnimationOnce(self.current_action_animation, 'action_hit_complete', '', 50, 'action_hit_moment')        # ударим еще раз
                self.PlaySound(self.snd_voice)  # по факту на начало анимации чтобы игралось
            else:
                #self.action_item.play_complete_sound()
                self.action_item = None
                if self.task:
                    self.task.subtask.on_action_complete()
                #self.go_home()      # для всех, кроме работников - они через free_unit -> TMgr убегают назад
                # нужно бежать назад к норе
                self.switch_animation()
# ============================================================
    def __str__(self):
        return base_unit.BaseUnit.__str__(self)
# ============================================================
# ============================================================
# ============================================================
    def switch_animation(self):
        #new_animation_name = 'run'
        _fsm_state = self.FSM.GetStateID()
        dir = self.GetDirection()
        self.Drawer.ShowImages('', False)
        self.Drawer.ShowImages('shadow', True)
        new_image_type = 'idle'
        if _fsm_state == 'action':
            new_image_type = self.current_action_animation
        elif _fsm_state in self.states_anim_dict:
            new_image_type = self.states_anim_dict[_fsm_state]
        if self.is_first:
            self.Drawer.SetSequence('arrow_tut', 0)
            self.Drawer.ShowImages('arrow_tut')
        self.Drawer.ShowImages(new_image_type, True)
        self.Drawer.SetSequence(new_image_type, dir)
# ============================================================
    def Get_tip_text(self, by_click = False):
        '''Возвращает LIST, содержащий текстовые ID'''
        result = []
        if not self.is_shown:
            return result
        if not self._get_tips_extra('before_name', result, by_click): return result
        item_name = self.GetAttrStr('item_name')
        if not item_name or item_name == 'NO_TIPS':
            return result
        result.append( ('text', 'COLOR_HEADER') )
        result.append( ('text', item_name) )
        if not self._get_tips_extra('after_name', result, by_click): return result
        if not self.is_active():
            return result
        return result
# ============================================================
    def _get_tips_extra(self, place, result, by_click):
        '''Предназначается для переопределения в потомках
        place: метка позиции вставки типса
        result: список значений, в который нужно добавлять данные
        by_click: типс по клику или нет
        returns: False - для завершения типса, т.е. после него ни одной строки не прибавится'''
        if place == 'after_name':
            if self.task:
                if self.energy >= 100:
                    result.append('newline')
                    result.append( ('text', 'COLOR_LINES') )
                    result.append( ('text', 'CLICK_TO_ATTACK') )
                else:
                    self._append_lifebar(result)
        return True
# ============================================================
    def _append_lifebar(self, result):
        _places_count = 10
        _filled_places = _places_count * self.energy
        _filled_places /= 100   # потому что self.energy в процентах
        _filled_places = int(_filled_places)
        if _filled_places > _places_count: _filled_places = _places_count
        elif _filled_places < 0: _filled_places = 0
        result.append('newline')
        result.append( ('text', 'COLOR_LINES') )
        result.append( ('text', 'TIP_LIFE') )
        result.append('newline')
        result.append( ('text', 'TIP_LIFEBAR_L') )
        for i in range(_filled_places):
            result.append( ('text', 'TIP_LIFEBAR_FULL') )
        for i in range(_places_count-_filled_places):
            result.append( ('text', 'TIP_LIFEBAR_EMPTY') )
        result.append( ('text', 'TIP_LIFEBAR_R') )
# ============================================================




