# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_unit
import task_walke
import task_classes

# ============================================================
# ============================================================
# ============================================================
class UnitEnemy(base_unit.BaseUnit):
    def __init__(self, cdata):
        base_unit.BaseUnit.__init__(self, cdata)
        self.ser_name = 'UnitEnemy'
        self.is_indian = True       # чтобы отличать индейцев от не индейцев
        self.is_going_for_action = True
        self.target = None
        self.start_cell = None
        self.snd_hit = self.GetAttrStr('snd_hit')
        self.snd_hit_percent = self.GetAttrInt('snd_hit_percent')
        if self.snd_hit:
            self.Drawer.SetAnimationEvent('default', self.snd_hit_percent, 'need_hit_sound')
# ============================================================
    # @override
    def dispose(self):
        pass
# ============================================================
    #@override
    def on_signal(self, signal):
        base_unit.BaseUnit.on_signal(self, signal)
        if signal == 'face_to_screen':
            self.set_face_direction()   # чтобы лицом к экрану стоял
        elif signal == 'go_back':
            self.go_to(self.start_cell)
            UI.SendMsg('hide_enemy_face')
        elif signal == 'go_to_signal':
            self.go_to(self.target)
# ============================================================
    def on_message(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'at_target':
            if self.is_going_for_action:
                #self. SM .SetStateNow('talk')
                self.FSM.HandleEvent('talk_event', True)
                self.set_face_direction()
                self.Signals.Restart('face_to_screen', 250)
                Camera.MoveTo(self)
                EMgr.SendGlobalMsg('enemy_came')
                UI.SendMsg('show_enemy_face', 'IMG_I_COMPETITOR_PORT', self.get('uid'))
                UI.show_tut_text('ENEMY_CAME_TEXT', 2, self.get('uid'))  # 2 - рамка для врага, 3 - для раптора/варвара
                tools.PlaySound('SND_ENEMY_LAUGH')
                self.is_going_for_action = False
                self.Signals.Restart('go_back', 2030)
            else:
                EMgr.SendGlobalMsg('enemy_gone')
                self.Drawer.Hide(True)
        elif msg_type == 'transition_complete':
            _next_variant = self.GetAttrStr('next_variant')
            self.switch_variant(_next_variant)
        elif msg_type == 'need_hit_sound':
            self.PlaySound(self.snd_hit)
# ============================================================
    def on_click(self, btn_name):
        '''Вызывается при клике по объекту'''
        LBoard.CrossNoToLastClick()
# ============================================================
# ============================================================
    def on_create(self):
        #_talk_quest_started = UStats.GetLevelValue('talk_quest_was_started') > 0        # если запущен квест на разговор с юнитом до создания подгружаемого уровня
        #if _talk_quest_started:
        #    self.try_show_talk_sign()
        self.init_fsm()
        self.switch_animation()
# ============================================================
    def __str__(self):
        return base_unit.BaseUnit.__str__(self)
# ============================================================
# ============================================================
    indian_animation_dict = {'run' : 'default', 'idle' : 'default', 'talk':'talk'}
# ============================================================
    def switch_animation(self):
        drawer = self.Drawer
        _is_partial_fog = drawer.IsPartiallyInFog()
        if _is_partial_fog:
            drawer.ShowImages('', False)
            return
        if not self.is_shown:
            drawer.ShowImages('', False)
            return
        state_name = self.FSM.GetStateID()
        drawer.ShowImages('', False)
        new_animation_name = 'default'
        if state_name in self.indian_animation_dict:
            new_animation_name = self.indian_animation_dict[state_name]
        if self.is_transition:
            new_animation_name = 'transition'
        drawer.ShowImages([new_animation_name,'shadow'], True)
        drawer.ShowImages('select', self.is_mouse_over and not self.is_deactivated)
        drawer.ShowImages('check', self.has_check)
        drawer.ShowImages('arrow', self.arrow_shown)
# ============================================================
    # @override
    def is_active(self):
        '''Возможны ли действия над объектом в данный момент. ПЕРЕОПРЕДЕЛЯЕМ'''
        if not self.non_selectable:
            if not self.is_deactivated:
                if self.is_shown:
                    if self.unique_name:
                        return self.show_talk_sign
        return False
# ============================================================
    def has_sign(self):
        if not self.non_selectable:
            if not self.is_deactivated:
                if self.is_shown:
                    if self.unique_name:
                        return self.show_talk_sign
        return False
# ============================================================
    def is_enough_params(self):
        return True
# ============================================================
    def get_not_enough_param(self):
        '''Возвращает имя параметра, которого недостаточно'''
        return ''
# ============================================================
    def get_subtask(self, parent_task, actor):
        return base_unit.BaseUnit.get_subtask(self, parent_task, actor)
# ============================================================
    def try_start_transition(self, force=False):
        if force:
            self.is_transition = True
            self.switch_animation()
            self.Drawer.StopAnimation('transition', 0)
            self.Drawer.PlayAnimationOnce('transition', 'transition_complete')
# ============================================================
# ============================================================
# ============================================================
    def is_not_ready_to_talk(self):
        return False        # пока отключено
        if self.is_deactivated:
        #if not _is_active:
            if self.unit_is_talking:
                if self.unique_name:
                    return True
        return False
# ============================================================
    #@override
    def is_green_state(self):
        '''Цвет типса/селекта'''
        _not_ready_to_talk = self.is_not_ready_to_talk()
        if _not_ready_to_talk:
            return True
        _has_access = self.has_access()
        if self.is_deactivated:
            if _has_access: return True
            else:           return False
        if not _has_access: return False
        _is_enough_params = self.is_enough_params()
        if not _is_enough_params:
            return False
        _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
        if _has_barbarians: return False
        _is_active = self.is_active()
        return base_unit.BaseUnit.is_green_state(self)
# ============================================================
    def Get_tip_text(self, by_click = False):
        '''Возвращает LIST, содержащий текстовые ID'''
        # нужно писать что нужно индейцу, например: красноречие, сила, ресурсы
        result = []
        if not self.is_shown:
            return result
        item_name = self.GetAttrStr('item_name')
        if not item_name:
            return result
        if self.is_tip_frozen:  # для индейцев "болванчиков"
            result.append( ('text', 'COLOR_HEADER') )
            result.append( ('text', item_name) )
            return result
        _is_debug = tools.IsDebug()
        if _is_debug:
            result.append( ('text', 'COLOR_HEADER') )
            result.append( ('debug', str(self.is_active())) )
            result.append( ('debug', str(self.is_deactivated)) )
            #result.append( ('debug', str(self.is_not_ready_to_talk())) )
            result.append('newline')
        if not by_click:
            result.append( ('text', 'COLOR_HEADER') )
            result.append( ('text', item_name) )

            # дальше хитрое шаманство недостающих параметров и ресурсов:
            result_params = []
            self.get_params_not_enough_text(result_params, [('text', 'COLOR_LINES')])
            result_need_res = []
            result_ne_res = []
            parts = []
            parts.append(result_params)
            parts.append(result_need_res)
            needs_combined = self._combine_not_enough_texts(parts)
            if needs_combined:
                result.append('newline')
                result.append( ('text', 'COLOR_LINES') )
                result.append( ('text', 'NEED_FOR_ACTION') )
                result.extend(needs_combined)

            result_params = []
            self.get_params_not_enough_text(result_params,\
                                            [('text', 'COLOR_RED')], True )
            parts = []
            parts.append(result_params)
            parts.append(result_ne_res)
            not_enough_combined = self._combine_not_enough_texts(parts)
            if not_enough_combined:
                result.extend( ['newline', ('text', 'COLOR_RED'), ('text', 'NOT_ENOUGH')] )
                result.extend(not_enough_combined)
            self._append_need_boat_text(result, False)
        else:   # by click:
            _has_access = self.has_access()
            _no_time_to_talk = False
            _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
            if _has_barbarians: _no_time_to_talk = True
            _is_active = self.is_active()
            _skip_extra_tips = False
            if _is_active:
                if self.task:
                    _skip_extra_tips = True
            if _has_access:
                if not _skip_extra_tips:
                    if _no_time_to_talk:
                        result.append( ('text', 'COLOR_LINES') )
                        result.append( ('text', 'YOU_HAVE_NO_TIME_TO_TALK') )
                        return result
                elif _has_barbarians:  # спец.типсы пропускаем, но варварские нужно показать
                    result.append( ('text', 'COLOR_LINES') )
                    result.append( ('text', 'YOU_HAVE_NO_TIME_TO_TALK') )
                    return result
            if not self.non_selectable:
                #if not self.is_deactivated:
                if _has_access:
                    if self.is_shown:
                        if self.unique_name:
                            if not _has_barbarians:
                                _talk_quest_started = UStats.GetLevelValue('talk_quest_was_started') > 0
                                if not _talk_quest_started:
                                    result.append( ('text', 'COLOR_LINES') )
                                    result.append( ('text', ('NOT_READY_TO_TALK','NOT_READY_TO_TALK_W')[self.is_woman == True]) )
                                    return result
            if True:
                result = []
                if not _has_access:
                    result.append( ('text', 'COLOR_RED') )
                    result.append( ('text', 'NO_PATH') )
                    self._append_need_boat_text(result, True)
                    return result
            result_ne_res = []
            result_params = []
            self.get_params_not_enough_text(result_params,\
                                            [('text', 'COLOR_RED')], True )
            parts = []
            parts.append( result_params )
            parts.append( result_ne_res )
            combined = self._combine_not_enough_texts(parts)
            result = []
            _main_tasks = self._get_main_tasks_incomplete_text()
            if not combined:
                if not _main_tasks:
                    return self.Get_tip_text(False)
            else:
                result.extend( [('text', 'COLOR_RED'), ('text', 'NOT_ENOUGH')] )
                result.extend( combined )
            self._append_need_boat_text(result, True)
        return result
# ============================================================
    def _get_res_not_enough_text(self, result, show_red = False):
        return False
# ============================================================
    #@override
    def get_ui_subtext(self):
        '''Текст, который должен быть написан в типсе возле панели с
        лицом персонажа второй строкой'''
        if not self.non_selectable and not self.is_deactivated and self.is_shown:
            _is_active = self.is_active()
            if self.unique_name:
                if _is_active:
                    return ([('text', ('FACES_READY_TO_SPEAK','FACES_READY_TO_SPEAK_W')[self.is_woman == True])], True)
        return [[], True]
# ============================================================
    def _combine_not_enough_texts(self, parts):
        _len = 0
        filtered_parts = []
        for part in parts:
            if part:    # учитываем только не пустые записи
                _len += 1
                filtered_parts.append(part)
        result = []
        if _len == 1:
            result.append('space')  # пробел-разделитель после двоеточия
            result.extend( filtered_parts[0] )
        else:
            for part in filtered_parts:
                result.append('newline')  # перевод строки-разделитель после двоеточия и после предыдущего пункта
                result.extend( part )
        return result
# ============================================================
    def go_to(self, target):
        trace('ENEMY go_to call')
        self.target = target
        _has_barb = UStats.GetLevelValue('barbarian_attack') > 0
        if _has_barb:
            self.Signals.Restart('go_to_signal', 1000)
            trace('ENEMY got_to call restart')
            return
        if self.is_going_for_action:
            self.is_shown = True
            LBoard.RemoveItemPassability(self)
            UStats.SetLevelValue('enemy_go', 1)
        else:
            UStats.SetLevelValue('enemy_go', 0)
        self.start_cell = self.get('cell')  # будет перетираться методом, но в нашем случае это, вроде, не важно
        # need to show if hidden
        self.Drawer.Hide(False)
        _task = task_walke.WalkEnemyTask(target, self)
        TMgr.add_task( _task )
        self.switch_animation()
        trace('ENEMY got_to call complete')
# ============================================================
# ============================================================
