# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_unit


class UnitFirefighter(base_unit.BaseUnit):
    def __init__(self, cdata):
        base_unit.BaseUnit.__init__(self, cdata)
        self.ser_name = 'UnitFirefighter'
        self.current_subtype = 'firefighter'
        self.work_boost = 1
        self.resource = None
        self.parent_home = None    # здание, к которому привязан юнит
        self.transition_object = None # объект перехода между подуровнями
        self.transition_start_cell = None   # точка, с которой был начат переход на другой подуровень
        self.need_transition = False
        self.building = None
        self.current_action_animation = 'action_extinguish'
        default_unit_speed = Globals.GFloat('DEFAULT_UNIT_SPEED') # default speed
        cur_unit_speed = Globals.GFloat('UNIT_SPEED')
        self.run_boost = cur_unit_speed/default_unit_speed
        self.Drawer.SetAnimBoost(['run', 'fast_run'], 1000*self.run_boost)
# ============================================================
    def extra_init(self):
        '''Вызывается только при открытии уровня, но НЕ при загрузке сохранения'''
        base_unit.BaseUnit.extra_init(self)
        self.FSM.HandleEvent('stay_event')   #@?
# ============================================================
    # @override
    def on_message(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'at_target':
            trace(self.task_type)
            if self.task:
                self.task.target_reached(self)
                self.PlaySound('SND_BUILDING_EXTINGUISH')
            elif self.need_transition:
                BMgr.transfer_unit(self, self.transition_object.GetAttrInt('sublevel'))
                #_cell = self.transition_object.GetAttrCellFixed('action_cell')
                self.set('cell', self.transition_start_cell)
                self.transition_object = None
                self.need_transition = False
                self.go_home()
            elif self.home_building: # running home and already at home
                if 'col' not in self.target:
                    if 'row' not in self.target:
                        _home = self.target
                        if _home:
                            trace('UnitFirefighter: ',_home.get('name'))
                        else:
                            trace('UnitFirefighter: _home = None')
                        _home.in_unit(self)    # hide unit in building
# ============================================================
    # @override
    def __str__(self):
        return base_unit.BaseUnit.__str__(self)

# ============================================================
# ============================================================
# ============================================================
    _firefighter_anim_dict = {'extinguishing': 'action_extinguish', 'run': 'run', 'stay': 'run', 'fast_run': 'fast_run'}
    def switch_animation(self):
        _state = self.FSM.GetStateID()
        #dir = self.GetDirection()
        if _state == 'run':
            _run_coeff = self.GetRunFinalCoeff()
            if _run_coeff >= 1.25: _state = 'fast_run'
        dir = self.GetDirection()
        self.Drawer.ShowImages('', False)
        self.Drawer.ShowImages('shadow', True)
        self.Drawer.ShowImages('arrow', self.arrow_shown)
        new_image_type = 'run'
        if _state in self._firefighter_anim_dict:
            new_image_type = self._firefighter_anim_dict[_state]
        #if _state == 'extinguishing':
        #    self.Drawer.SetAnimBoost(new_image_type, self.work_boost*1000)
        if new_image_type == 'action_extinguish':
            dir = int(dir / 2)  # only 4 directions
        self.Drawer.ShowImages(new_image_type, True)
        self.Drawer.SetSequence(new_image_type, dir)
# ============================================================
    _final_removing_fsm_states = ['removing', 'removing_final', '_dead_']
    # @override
    def go_home(self):
        '''Отправляет юнита домой. Иницализирует необходимые переменные
        После события "free_unit" TMgr вызывает у юнита данный метод'''
        if not self.parent_home:
            if self.transition_object:
                self.need_transition = True
                self.run_to_target(self.transition_object, None, self.transition_object.get_action_cell_name())
                return
            base_unit.BaseUnit.go_home(self)    # затычка на случай, когда отменили задание и нужно бежать в штаб/домик
            return
        if self.home_building:
            if self.is_in_home:  # уже дома
                return
        self._run_to_waterpump()
# ============================================================
    def _run_to_waterpump(self):
        if self.transition_object:
            self.need_transition = True
            self.run_to_target(self.transition_object, None, self.transition_object.get_action_cell_name())
            return
        b = self.parent_home
        _b_state = b.FSM.GetStateID()
        if _b_state in self._final_removing_fsm_states:
            b._internal_free_units.remove(self)
            self.parent_home = None # чтобы юнит бежал в штаб/домик
            UMgr.add_free_unit(self)
            b.firefighter = None
        else:
            self.home_building = b
            b.add_walking_home_unit(self)
            self.run_to_target( b.start_cell_pos, b, '', True )
# ============================================================
    # @override
    def set_home(self, building):
        '''Помещает юнита в здание. Инициализирует необходимые переменные, прячет юнита, выставляет координаты'''
        self.home_building = None   # ?
        base_unit.BaseUnit.set_home(self, building)
        self.current_subtype = 'firefighter'
        self.parent_home = building
        building.is_unit_inside = True
# ============================================================

