# -*- coding: utf-8 -*-

import sys
if not "tinypy" in sys.version:
    from extra import *

import base_unit
import fsm_state
import task_doctor


class UnitGeneral(base_unit.BaseUnit):
    states_anim_dict = {'hit': 'hit', 'run': 'run', 'stay': 'idle', 'pickup':'pickup',\
                        'magic': 'magic', 'talk':'talk',\
                        'fast_run':'fast_run',
                        'boat_stay':'boat_idle', 'boat_run': 'boat_run',
                        'sick':'sick', 'sick_standup':'standup' }

    def __init__(self, cdata):
        base_unit.BaseUnit.__init__(self, cdata)
        self.ser_name = 'UnitGeneral'
        self.resource = None
        self.attacked_item = None
        self.attack_state = ''
        self.last_transport = ''    # последний, использованный транспорт, чтобы показывать верные типсы
        self.need_magic = False         # затычка для генерала, чтобы показать магию при взятии gems
        default_unit_speed = Globals.GFloat('DEFAULT_UNIT_SPEED') # default speed
        cur_unit_speed = Globals.GFloat('GENERAL_SPEED')
        self.Drawer.SetAnimBoost('run', 1000*cur_unit_speed/default_unit_speed)
        self.Drawer.SetAnimBoost('fast_run', 1000*cur_unit_speed/default_unit_speed)
        self.Drawer.SetAnimationEvent('boat_run', 30, 'on_boat_move')
# ============================================================
    def extra_init(self):
        '''Вызывается только при открытии уровня, но НЕ при загрузке сохранения'''
        base_unit.BaseUnit.extra_init(self)
        self.init_listeners()
# ============================================================
    def deserialize_init(self):
        self.init_listeners()
# ============================================================
    def init_listeners(self):
        EMgr.add_concrete_event_listener('doctor_building_event', self.on_doctor_event)
# ============================================================
    # @override
    def dispose(self):
        EMgr.remove_concrete_event_listener('doctor_building_event', self.on_doctor_event)
# ============================================================
    def on_message(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'at_target':
            #self. SM .SetStateNow('stay')
            self.FSM.HandleEvent('general_attarget_stay_event', True)
            return
        elif msg_type == 'hit_item':
            #self.Drawer.PlayAnimationOnce(self.current_action_animation, 'hit_complete')
            self.Drawer.PlayAnimationOnce(self.current_action_animation, 'hit_anim_complete', '', 50, 'hit_complete') # ударим еще раз
        elif msg_type == 'hit_anim_complete':
            trace('general hit_anim_complete')
            self.on_hit_anim_complete()
        elif msg_type == 'hit_complete':
            trace('general hit_complete')
            self.on_hit_complete()
        elif msg_type == 'on_boat_move':
            self.PlaySound('SND_BOAT')
# ============================================================
    def on_signal(self, signal):
        if signal == 'offscreen':
            _state_name = self.FSM.GetStateID()
            if _state_name == 'stay':   # idle state
                self.set_face_direction()
        elif signal == 'step_aside':
            self.step_aside_now()
        elif signal == 'try_call_doctor':
            self.try_call_doctor()
        else:
            base_unit.BaseUnit.on_signal(self, signal)
# ============================================================
    def on_click(self, btn_name):
        if self.FSM.GetStateID() == 'sick':
            self.on_sick_click()
        else:
            pass    # на клики не реагирует
# ============================================================
    def on_create(self):
        self.init_fsm()
        self.SetDirection(self.GetAttrInt('start_direction'))
        self.switch_animation()
# ============================================================
    def __str__(self):
        return base_unit.BaseUnit.__str__(self)
# ============================================================
    def on_doctor_event(self, event):
        self.switch_animation()
# ============================================================
# ============================================================
# ============================================================
    def switch_animation(self):
        state_name = self.FSM.GetStateID()
        dir = self.GetDirection()
        drawer = self.Drawer
        drawer.ShowImages('', False)
        drawer.ShowImages('shadow', True)
        drawer.ShowImages('arrow', self.arrow_shown)
        new_image_type = 'idle'
        _is_on_boat = self.IsOnBoat()
        if _is_on_boat: # если на лодке, то используем другие анимации:
            if state_name == 'stay': state_name = 'boat_stay'
            if state_name == 'run': state_name = 'boat_run'
            if state_name in ['hit', 'pickup']: state_name = 'boat_stay' # затычка, чтобы сразу анимация менялась
        if state_name == 'run':
            _run_coeff = self.GetRunFinalCoeff()
            if _run_coeff >= 1.25: state_name = 'fast_run'
        if state_name == 'hit':
            new_image_type = self.current_action_animation
        elif state_name in self.states_anim_dict:
            new_image_type = self.states_anim_dict[state_name]

        drawer.ShowImages(new_image_type, True)
        if new_image_type == 'magic':
            dir = int(dir / 2)
        if new_image_type == 'sick':
            dir = int(dir / 4)
        if new_image_type == 'standup':
            dir = int(dir / 4)
        drawer.SetSequence(new_image_type, dir)
        _bonus_v = UStats.GetLevelValue('IS_GENERAL_SPEEDBONUS_ACTIVE')
        drawer.ShowImages('bonus', (_bonus_v > 0))
        _is_sick = self.FSM.GetStateID() == 'sick'
        if _is_sick:
            _has_doctor = UMgr.has_doctor()
            drawer.ShowImages('sign_sick', _has_doctor)
            drawer.ShowImages('sign_sick_red', not _has_doctor)
            if self.task:
                if self.task.subtask:
                    if self.task.subtask.is_subtask_started:
                        if dir == 0:
                            drawer.ShowImages('heal_effect_r')
                        else:
                            drawer.ShowImages('heal_effect_l')
# ============================================================
    # @override
#    def stop_walk(self):
#        self.StopMove()     # событие сработает не мгновенно, а остановиться нужно сразу!
#        self.FSM.HandleEvent('stay_event')

        #_need_magic = self.need_magic
        #if _need_magic:
        #    self.show_magic()
        #self.StopMove()     # событие сработает не мгновенно, а остановиться нужно сразу!
        #state_name = self.FSM.GetStateID()
        #if state_name == 'stay':  # уже стоим   #@!<<<
        #    return
        #if not _need_magic:
        #    self.FSM.HandleEvent('stay_event')
        #    self.switch_animation() # не поломает ли чего? не должно вроде
# ============================================================
    # @override
    def action_hit_item(self, item):
        self.attacked_item = item
        self.FSM.HandleEvent('hit_event')
        self.SendMsg('hit_item')
# ============================================================
    def cancel_hits(self):
        self.attacked_item = None
        self.StopMove()
        self.FSM.HandleEvent('cancel_hits_event')
# ============================================================
    def on_hit_complete(self):
        trace('on_hit_complete 1')
        if self.attacked_item:      # проверяем на случай, если отменили задание
            _is_pickup = self.attacked_item.GetAttrBool('is_pickup')
            trace('on_hit_complete 2')
            self.attacked_item.action_hit_me()      # удар по объекту
            trace('general hit complete:', self.attacked_item)
            if self.attacked_item:  # в некоторых случаях объект может быть уничтожен
                _is_action_complete = self.attacked_item.is_action_complete()
                if not _is_action_complete:   # объект еще не разрушен
                    trace('on_hit_complete 3')
                    self.attacked_item.play_hit_sound()
                    self.attacked_item.scale_bump()     # чтобы объект "дергался"
                else:
                    if not _is_pickup:
                        self.attacked_item.play_hit_sound()
                    self.attacked_item.play_complete_sound()
                    trace('on_hit_complete 4')
                    self.attack_state = 'complete'
            else:
                self.attacked_item.play_hit_sound()
# ============================================================
    def on_hit_anim_complete(self):
        if self.attacked_item:      # проверяем на случай, если отменили задание
            self.attacked_item.action_hit_anim_complete()
            if self.attack_state == 'attack' and not self.attacked_item.is_action_complete():   # объект еще не разрушен
                self.Drawer.PlayAnimationOnce(self.current_action_animation, 'hit_anim_complete', '', 50, 'hit_complete') # ударим еще раз
            else:
                self.attack_state = ''
                self.StopMove()
                self.FSM.HandleEvent('stay_event')
                #self.switch_animation()
                self.attacked_item = None
                if self.task:
                    self.task.subtask.on_action_complete()
# ============================================================
    def need_show_magic(self):
        self.need_magic = True
# ============================================================
    #@override
#    def show_magic(self):
#        self.FSM.HandleEvent('magic_event')
# ============================================================
    #@override
    def is_busy(self):
        if self.task != None or self.need_magic or self.unit_is_talking:    #2check
            return True
        else:
            _state = self.FSM.GetStateID()
            if _state == 'magic':
                return True
            return False
# ============================================================
    def step_aside(self):
        self.Signals.Restart('step_aside', 2000)
# ============================================================
    def step_aside_now(self):
        if not self.task:   # если нет задания, то отходим, иначе будут баги
            _is_moving = self.IsMoving()
            if not _is_moving:      # проверка, что в данный момент юнит никуда не движется:
                _cell = self.get('cell')
                trace('step_aside_now center cell:', _cell)
                _destCell = UMgr.GetStepAsideCell( self, _cell, 5 ) # 7 - должно хватить, а то и много
                if _destCell:
                    trace('step_aside_now:', _destCell)
                    self.run_to_target(_destCell)
# ============================================================
    #@override
    def Get_tip_image(self):
        _fsm_state = self.FSM.GetStateID()
        if _fsm_state == 'sick':
            _has_doctor = UMgr.has_doctor() # а есть ли доктор и госпиталь?
            if not _has_doctor:
                return 'IMG_I_TIP_BG_RED'
        return 'IMG_I_TIP_BG'
# ============================================================
    def _get_tips_extra(self, place, result, by_click):
        if place == 'before_access_check_click':
            if self.FSM.GetStateID() == 'sick':
                self._append_sick_general(result, True)
            return False
        elif place == 'after_lifebar':
            if self.FSM.GetStateID() == 'sick':
                self._append_sick_general(result)
        return True
# ============================================================
    def attacked_by_explosion(self):
        self.FSM.HandleEvent('sick_event', True)
        if LBoard.Core.episode == 1:
            if LBoard.Core.level in [1,2]:
                return  # do not show at these levels
        _has_doctors = UStats.GetLevelValue('HAS_DOCTORS')
        if _has_doctors:
            return
        result = BMgr.get_hospitals_simple_allmaps()
        _h_count = len(result)
        if _h_count > 0:
            return
        UI.show_tut_text('TUT_HELP_BOMB')
# ============================================================
    def try_call_doctor(self):
        _task = self.task
        if not _task:
            #self.task = task_doctor.DoctorTask(self)
            _task = task_doctor.DoctorTask(self)
            result = TMgr.add_task( _task )
            if not result:
                self.task = None
# ============================================================
    def Run_to_water_cell(self, row, column):
        '''Отправляет генерала к точке на воде'''
        _cell = {'row':row, 'col':column}
        _cur_cell = self.get('cell')
        if _cell == _cur_cell:  # если клик по той же ячейке
            return
        result = self.TryRunTo(_cell, '', 2)
        if result == False:
            trace('run_to_target failed', self.get('uid'), _cell)
            return
        #self.show()
        return
# ============================================================
    def on_boat_on(self, _boat):
        self.SetOnBoat(True)
        _boat_cell = _boat.get('cell')
        BMgr.RemoveItemFromMap(_boat)
        self.set('cell', _boat_cell)
        self.ClearGeneralAccess()
        self.CheckGeneralAccess()
        self.last_transport = 'boat'
# ============================================================
    def on_teleport_in(self, _teleport):
        #self.SetOnBoat(True)
        _tele_cell = _teleport.GetAttrCellFixed('action_cell')
        self.set('cell', _tele_cell)
        self.ClearGeneralAccess()
        self.CheckGeneralAccess()
        self.last_transport = 'teleport'
        Camera.MoveTo(self)
# ============================================================
    def On_boat_off(self, _boat_row, _boat_col):
        map_index = self.get('locind')
        self.ClearGeneralAccess()   # сначала чистим доступность - после метода 'On_boat_off' вызывается CheckGeneralAccess()
        _boat_cell = {'row':_boat_row, 'col':_boat_col}
        _boat = BMgr.AddItemToMap(map_index, 'boat1', '', _boat_cell, True)
        if _boat:
            _is_passable = self.IsCellPassable(self.get('cell'))
            if _is_passable:
                _boat.SetAccessible(True)
            else:
                _boat.SetGeneralAccessible(True)
        self.SetOnBoat(False)
        self.CheckGeneralAccess()
        self.last_transport = 'boat'
# ============================================================
# ============================================================
# ============================================================
    # @override
    def init_fsm(self):
        base_unit.BaseUnit.init_fsm(self)
        self.fsm_methods.states[StateGeneralStay.name] = StateGeneralStay(self)
# ============================================================
# ============================================================
class StateGeneralStay(fsm_state.FSMState):
    name = 'stay'

    def __init__(self, item = None):
        fsm_state.FSMState.__init__(self, item)
        self.ser_name = 'StateGeneralStay'

    def on_enter(self, evID):
        u = self.item
        if evID == 'attarget_stay_event':
            u.Signals.Restart('face_to_screen', 250)
        elif evID == 'general_attarget_stay_event':
            # at target
            trace(u.task_type)
            if u.task:
                u.task.target_reached(u)
        # ---
        if u.need_magic:
            u.FSM.HandleEvent('magic_event')
        # ---
        u.switch_animation()
        TMgr.send_event('free_general', None)

    def can_exit(self, evID):
        if evID == 'general_attarget_stay_event':
            return False
        return True

    def on_handle(self, evID):
        if evID == 'general_attarget_stay_event':
            # at target
            u = self.item
            trace(u.task_type)
            if u.task:
                u.task.target_reached(u)
            TMgr.send_event('free_general', None)
# ============================================================
