# -*- coding: utf-8 -*-

import sys
if not 'tinypy' in sys.version:
    from _globals import *
import base_unit


class UnitWorker(base_unit.BaseUnit):
    resource_animation_dict = {
        '': 'run',
        '_wood': 'run_wood',
        'boards': 'run_boards',
        '_stones': 'run_stone',
        '_ore': 'run_ore',
        '_fish': 'run_fish',
        '_fish_sm': 'run_fish',
        'coins': 'run_coin',
        'bricks': 'run_brick',
        '_berries': 'run_basket',
        '_bread': 'run_basket',
        'food': 'run_basket',   # для подстраховки
        '_granite': 'run_stone',
        '_flour': 'run_bag',
        'gold': 'run_bag'
        }
    run_animations = ['run', 'run_w_cut', 'run_w_wood', 'run_bag', 'run_basket',\
                        'run_boards', 'run_brick', 'run_coin', 'run_fish',\
                        'run_ore', 'run_stone', 'run_wood', 'run_builder',\
                        'run_quarry', 'run_soldier',\
                        'fast_run', 'fast_run_builder', 'fast_run_quarry']
# ============================================================
    def __init__(self, cdata):
        base_unit.BaseUnit.__init__(self, cdata)
        self.ser_name = 'UnitWorker'
        #self.StopMove()
        self.current_action_animation = 'run'   # данное поле выставляется при взаимодействии с тем или иным объектом
        default_unit_speed = Globals.GFloat('DEFAULT_UNIT_SPEED') # default speed
        cur_unit_speed = Globals.GFloat('UNIT_SPEED')
        self.Drawer.SetAnimBoost(self.run_animations,\
                                    1000*cur_unit_speed/default_unit_speed)
# ============================================================
    def extra_init(self):
        '''Вызывается только при открытии уровня, но НЕ при загрузке сохранения'''
        base_unit.BaseUnit.extra_init(self)
        self.FSM.HandleEvent('stay_event')  #@???
        self.SetDirection(self.GetAttrInt('start_direction'))
        self.init_listeners()
# ============================================================
    def deserialize_init(self):
        self.init_listeners()
# ============================================================
    def init_listeners(self):
        EMgr.add_concrete_event_listener('barbarian_attack_started', self.on_barbarians)
        EMgr.add_concrete_event_listener('barbarian_attack_completed', self.on_barbarians)
        EMgr.add_concrete_event_listener('barbarian_attack_canceled', self.on_barbarians)
# ============================================================
    # @override
    def dispose(self):
        EMgr.remove_concrete_event_listener('barbarian_attack_started', self.on_barbarians)
        EMgr.remove_concrete_event_listener('barbarian_attack_completed', self.on_barbarians)
        EMgr.remove_concrete_event_listener('barbarian_attack_canceled', self.on_barbarians)
# ============================================================
    def on_message(self, msg_type, msg_param1='', msg_param2='', msg_dict={}):
        if msg_type == 'hit_moment':
            if self.task:
                self.task.on_bump_need()
                state_name = self.FSM.GetStateID()
                if state_name == 'dig':
                    tools.PlaySound('SND_WATER_SPADE')
            else:
                self.PlaySoundRandom(['SND_HAMMER_HIT_1', 'SND_HAMMER_HIT_2', 'SND_HAMMER_HIT_3'])
        elif msg_type == 'build_hit_complete':
            state_name = self.FSM.GetStateID()
            if state_name not in ['run', 'fast_run']:
                animation = 'build'
                if state_name == 'dig': animation = 'dig'
                self.Drawer.PlayAnimationOnce(animation, 'build_hit_complete', '', 50, 'hit_moment')        # ударим еще раз
        elif msg_type == 'at_target':
            if self.home_building: # running home and already at home
                # if not istype(self.target, "list"): # and self.target.__class__ == "base_building.BaseBuiding":
                # if "__class__" in self.target: # and self.target.__class__ == "base_building.BaseBuiding":
                if 'col' not in self.target:
                    if 'row' not in self.target: # and self.target.__class__ == "base_building.BaseBuiding":
                        _is_target_home = self.target.is_home()
                        if _is_target_home:
                            self.target.in_unit(self)    # hide unit in building
                        else:
                            self.give_resource(self.target) # отдадим ресурс зданию
                            self.target.remove_walking_home_unit(self)  # освободимся от здания
                            self.go_home()  # теперь можно идти домомй
                self.switch_animation()
                return
            else:       # at target
                trace('at_target, task:', self.task_type)
                if self.task:
                    self.task.target_reached(self)  #!bug is here... somehow
                #else:
                #    self.go_home()   # hack - just for test - running to the nearest vacant home
                self.switch_animation()
                return
# ============================================================
    def on_signal(self, signal):
        base_unit.BaseUnit.on_signal(self, signal)
# ============================================================
    def on_barbarians(self, event):
        self.switch_animation()
# ============================================================
    def on_create(self):
        self.init_fsm()
        self.switch_animation()
# ============================================================
    def __str__(self):
        return base_unit.BaseUnit.__str__(self)
# ============================================================
# ============================================================
# ============================================================
    simple_dict = {'build': 'build', 'run': 'run', 'fast_run': 'fast_run', 'stay': 'stay'}
    builder_animation_dict = {'run' : 'run_builder', 'fast_run': 'fast_run_builder', 'build' : 'build', 'action' : 'build', 'stay':'run_builder'}#<<
    breaker_animation_dict = {'run' : 'run_breaker', 'fast_run': 'run_breaker', 'action' : 'action_breaker', 'stay':'run_breaker'}#<<
    woodcutter_animation_dict = {'run': 'run_w_cut', 'fast_run': 'run_w_cut', 'action': 'action_w_cut', 'stay': 'run_w_cut'}
    soldier_animation_dict = {'run': 'run_soldier', 'fast_run': 'run_soldier', 'action': 'run_soldier', 'stay': 'run_soldier'}
    quarry_animation_dict = {'run': 'run_quarry', 'fast_run': 'fast_run_quarry', 'action': 'action_quarry', 'stay': 'run_quarry'}#<<
# ============================================================
    def switch_animation(self):
        drawer = self.Drawer
        _ShowImages = drawer.ShowImages
        _is_partial_fog = drawer.IsPartiallyInFog()
        if _is_partial_fog:     # для выживших?
            _ShowImages('', False)
            return
        if self.is_hidden:
            _ShowImages('', False)
            return
        new_animation_name = 'run'
        state_name = self.FSM.GetStateID()
        if state_name == 'run':
            _run_coeff = self.GetRunFinalCoeff()
            if _run_coeff >= 1.25: state_name = 'fast_run'
        #if self.resource and not self.action_item:
        _unit_with_res = False
        _has_res = self.has_resource()
        if _has_res:
            if not self.action_item:
                _unit_with_res = True
        if _unit_with_res:
            _res_name = self.resource.GetSingleName()
            new_animation_name = self.resource_animation_dict[_res_name]
            if new_animation_name == 'run_wood':
                if self.current_subtype == 'dt_wcutter':    # если лесоруб с бревном
                    new_animation_name = 'run_w_wood'
        elif self.current_subtype == 'builder': # строитель с молотком
            if state_name in self.builder_animation_dict:
                new_animation_name = self.builder_animation_dict[state_name]
        elif self.current_subtype == 'dt_wcutter': # с топором
            if state_name in self.woodcutter_animation_dict:
                new_animation_name = self.woodcutter_animation_dict[state_name]
        elif self.current_subtype == 'soldier':
            if state_name in self.soldier_animation_dict:
                new_animation_name = self.soldier_animation_dict[state_name]
        elif self.current_subtype == 'quarryman':   # ломатель киркой
            if state_name in self.quarry_animation_dict:
                new_animation_name = self.quarry_animation_dict[state_name]
        elif self.current_subtype == 'breaker': # ломатель киркой
            if state_name in self.breaker_animation_dict:
                new_animation_name = self.breaker_animation_dict[state_name]
        elif self.current_subtype == 'simple':  # обычный юнит
            if state_name in self.simple_dict:
                new_animation_name = self.simple_dict[state_name]
            elif state_name == 'action':
                # нужно выбирать анимацию в зависимости от объекта и действия
                new_animation_name = self.current_action_animation
        if state_name == 'dig':
            new_animation_name = 'dig'
        dir = self.GetDirection()
        _ShowImages('', False)
        _ShowImages(new_animation_name, True)
        drawer.SetSequence(new_animation_name, dir)
        _ShowImages('shadow', True)
        _ShowImages('arrow', self.arrow_shown)
        if self.is_survived:
            _ShowImages('check', self.has_check)
        _has_barbarians = UStats.GetLevelValue('barbarian_attack') > 0
        if not _has_barbarians:
            _has_access = self.HasAccess()
            if _has_access:
                if self.is_survived:
                    if not self.is_deactivated:
                        if self.GetAttrBool('is_talking'):
                            _ShowImages('sign_e', True)
        _bonus_v = UStats.GetLevelValue('IS_UNIT_SPEEDBONUS_ACTIVE')
        _ShowImages('bonus', (_bonus_v > 0))
# ============================================================
    def has_sign(self):
        return (self.is_survived and not self.is_deactivated and self.GetAttrBool('is_talking'))
# ============================================================




